@@ -80,14 +80,16 @@ bool AddImplicitArgs::runOnModule(Module &M)
         if (func.isDeclaration()) continue;
         // skip non-entry functions
         if (m_pMdUtils->findFunctionsInfoItem(&func) == m_pMdUtils->end_FunctionsInfo()) continue;
-        // Skip functions called from function marked with IndirectlyCalled attribute
-        // TODO: This function verifies only parent caller, in the future we may need to
-        //       also verify all the callers up to the tree.
-        if (hasIndirectlyCalledParent(&func)) continue;
-        // No implicit arg support for stack calls
-        if (func.hasFnAttribute("visaStackCall")) continue;
+        // Skip functions called from functions marked with stackcall attribute
+        // Also skip the dummy kernel if one was created
+        if (hasStackCallInCG(&func) || IGC::isIntelSymbolTableVoidProgram(&func))
+        {
+            FunctionInfoMetaDataHandle funcInfo = m_pMdUtils->getFunctionsInfoItem(&func);
+            funcInfo->clearImplicitArgInfoList();
+            continue;
+        }
 
-        // see the detail in StatelessToStatefull.cpp.
+        // see the detail in StatelessToStateful.cpp.
         // If SToSProducesPositivePointer is true, do not generate implicit arguments.
         if (IGC_IS_FLAG_DISABLED(SToSProducesPositivePointer) &&
             (ctx->getModuleMetaData()->compOpt.HasBufferOffsetArg ||
@@ -129,7 +131,7 @@ bool AddImplicitArgs::runOnModule(Module &M)
         }
     }
 
-    if (IGC_GET_FLAG_VALUE(FunctionControl) != FLAG_FCALL_FORCE_INLINE)
+    if (!IGC::ForceAlwaysInline())
     {
         for (auto I : funcsMappingForReplacement)
         {
@@ -180,14 +182,17 @@ bool AddImplicitArgs::runOnModule(Module &M)
     return true;
 }
 
-bool AddImplicitArgs::hasIndirectlyCalledParent(const Function* F)
+bool AddImplicitArgs::hasStackCallInCG(const Function* F)
 {
+    if (F->hasFnAttribute("visaStackCall") || F->hasFnAttribute("referenced-indirectly"))
+        return true;
+
     for (auto u = F->user_begin(), e = F->user_end(); u != e; u++)
     {
         if (const CallInst* call = dyn_cast<CallInst>(*u))
         {
             const Function* parent = call->getParent()->getParent();
-            if (parent->hasFnAttribute("referenced-indirectly"))
+            if (hasStackCallInCG(parent))
                 return true;
         }
     }
@@ -216,9 +221,9 @@ void AddImplicitArgs::updateNewFuncArgs(llvm::Function* pFunc, llvm::Function* p
     // the new arguments, also transferring over the names as well.
     std::vector<std::pair<llvm::Instruction*, unsigned int>> newAddr;
     bool fullDebugInfo = false;
-    IF_DEBUG_INFO(bool lineNumbersOnly = false;)
-    IF_DEBUG_INFO(CodeGenContext* ctx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();)
-    IF_DEBUG_INFO(DebugMetadataInfo::hasAnyDebugInfo(ctx, fullDebugInfo, lineNumbersOnly);)
+    bool lineNumbersOnly = false;
+    CodeGenContext* ctx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
+    DebugMetadataInfo::hasAnyDebugInfo(ctx, fullDebugInfo, lineNumbersOnly);
 
     if (fullDebugInfo)
     {
@@ -271,16 +276,16 @@ void AddImplicitArgs::updateNewFuncArgs(llvm::Function* pFunc, llvm::Function* p
     // storage location for the variable.
     for (auto toReplace : newAddr)
     {
-        IF_DEBUG_INFO(auto d = dyn_cast<DbgDeclareInst>(toReplace.first);)
+        auto d = dyn_cast<DbgDeclareInst>(toReplace.first);
 
-        IF_DEBUG_INFO(llvm::DIBuilder Builder(*pNewFunc->getParent()));
-        IF_DEBUG_INFO(auto DIVar = d->getVariable();)
-        IF_DEBUG_INFO(auto DIExpr = d->getExpression();)
+        llvm::DIBuilder Builder(*pNewFunc->getParent());
+        auto DIVar = d->getVariable();
+        auto DIExpr = d->getExpression();
 
         IGC_ASSERT(toReplace.second < pNewFunc->arg_size());
-        IF_DEBUG_INFO(Value* v = pNewFunc->arg_begin() + toReplace.second);
-        IF_DEBUG_INFO(Builder.insertDeclare(v, DIVar, DIExpr, d->getDebugLoc().get(), d);)
-        IF_DEBUG_INFO(d->eraseFromParent();)
+        Value* v = pNewFunc->arg_begin() + toReplace.second;
+        Builder.insertDeclare(v, DIVar, DIExpr, d->getDebugLoc().get(), d);
+        d->eraseFromParent();
     }
 
     // Set implicit argument names
@@ -493,7 +498,7 @@ bool BuiltinCallGraphAnalysis::runOnModule(Module &M)
     m_pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
     CallGraph &CG = getAnalysis<CallGraphWrapperPass>().getCallGraph();
 
-    if (IGC_GET_FLAG_VALUE(FunctionControl) == FLAG_FCALL_FORCE_INLINE)
+    if (IGC::ForceAlwaysInline())
     {
         return false;
     }
@@ -533,13 +538,12 @@ bool BuiltinCallGraphAnalysis::pruneCallGraphForStackCalls(CallGraph& CG)
                     Function* pFuncOnPath = IT.getPath(i)->getFunction();
                     if (pFuncOnPath)
                     {
-                        if (pFuncOnPath->hasFnAttribute("forceRecurse"))
+                        if (pFuncOnPath->hasFnAttribute("hasRecursion"))
                         {
                             IGC_ASSERT_MESSAGE(0, "Cannot inline for recursion!");
                             return false;
                         }
                         PrunedFuncs.insert(pFuncOnPath);
-                        changed = true;
                     }
                 }
             }
@@ -547,9 +551,18 @@ bool BuiltinCallGraphAnalysis::pruneCallGraphForStackCalls(CallGraph& CG)
     }
     for (auto pF : PrunedFuncs)
     {
+        // We can only remove the "visaStackCall" attribute if the function isn't called indirectly,
+        // since these attributes are always coupled together.
+        if (pF->hasFnAttribute("referenced-indirectly"))
+        {
+            IGC_ASSERT_MESSAGE(0, "Cannot force inline indirect calls! Requires ForceInlineStackCallWithImplArg=0 and IA buffer support.");
+            continue;
+        }
+
         pF->removeFnAttr("visaStackCall");
         pF->removeFnAttr(llvm::Attribute::NoInline);
         pF->addFnAttr(llvm::Attribute::AlwaysInline);
+        changed = true;
     }
     return changed;
 }
@@ -719,19 +732,29 @@ void BuiltinCallGraphAnalysis::combineTwoArgDetail(
 
 void BuiltinCallGraphAnalysis::writeBackAllIntoMetaData(const ImplicitArgumentDetail& data, Function * f)
 {
-    if (f->hasFnAttribute("referenced-indirectly"))
-        return;
-
     FunctionInfoMetaDataHandle funcInfo = m_pMdUtils->getFunctionsInfoItem(f);
     funcInfo->clearImplicitArgInfoList();
 
+    bool isEntry = isEntryFunc(m_pMdUtils, f);
+
     for (const auto& A : data.ArgsMaps)
     {
+        // For the implicit args that have GenISAIntrinsic support used in subroutines,
+        // they do not require to be added as explicit arguments other than in the caller kernel.
+        // Always add metadata for stackcalls to provide info for inlining. They won't be added
+        // to function argument list.
         ImplicitArg::ArgType argId = A.first;
+        if (!isEntry && ImplicitArgs::hasIntrinsicSupport(argId))
+        {
+            bool isStackCall = f->hasFnAttribute("visaStackCall");
+            if (!isStackCall && IGC_IS_FLAG_ENABLED(EnableImplicitArgAsIntrinsic))
+            {
+                continue;
+            }
+        }
         if (argId < ImplicitArg::ArgType::STRUCT_START)
         {
             // unique implicit argument, add it on metadata
-
             ArgInfoMetaDataHandle argMD = ArgInfoMetaDataHandle(ArgInfoMetaData::get());
             argMD->setArgId(argId);
             funcInfo->addImplicitArgInfoListItem(argMD);
