@@ -18,7 +18,7 @@ SPDX-License-Identifier: MIT
 #include <llvm/IR/Function.h>
 #include "llvm/IR/InstIterator.h"
 #include <llvm/Transforms/Utils/BasicBlockUtils.h>
-#include <llvm/Transforms/Utils/Cloning.h>
+#include <llvmWrapper/Transforms/Utils/Cloning.h>
 #include "common/LLVMWarningsPop.hpp"
 #include "common/Types.hpp"
 #include "Probe/Assertion.h"
@@ -71,8 +71,9 @@ LegalizeFunctionSignatures::LegalizeFunctionSignatures()
 
 //****************************************************************//
 
-static const unsigned int MAX_STACKCALL_RETVAL_SIZE_IN_BITS = 64;
-static const unsigned int MAX_STRUCT_ARGUMENT_SIZE_IN_BITS = 128;
+static const unsigned int MAX_RETVAL_SIZE_IN_BITS = 64;
+static const unsigned int MAX_STRUCT_SIZE_IN_BITS = 128;
+static const unsigned int MAX_SUBROUTINE_STRUCT_SIZE_IN_BITS = 512;
 
 bool LegalizeFunctionSignatures::runOnModule(Module& M)
 {
@@ -89,16 +90,16 @@ bool LegalizeFunctionSignatures::runOnModule(Module& M)
     return true;
 }
 
-// IGC stackcall ABI requires return values to be <= 64bits, since we don't support return value on stack.
-// Stackcalls with return values > 64bits will need to be changed to pass-by-ref.
-inline bool isLegalReturnType(Type* ty)
+// If the return type size is greater than the allowed size, we convert the return value to pass-by-pointer
+inline bool isLegalReturnType(const Type* ty)
 {
     // check return type size
-    return ty->getPrimitiveSizeInBits() <= MAX_STACKCALL_RETVAL_SIZE_IN_BITS;
+    //return ty->getPrimitiveSizeInBits() <= MAX_RETVAL_SIZE_IN_BITS;
+    return true; // allow all return sizes
 }
 
 // Check if an int or int-vector argument type is a power of two
-inline bool isLegalIntVectorType(Module& M, Type* ty)
+inline bool isLegalIntVectorType(const Module& M, Type* ty)
 {
     if (ty->isIntOrIntVectorTy())
     {
@@ -117,7 +118,7 @@ inline bool isLegalIntVectorType(Module& M, Type* ty)
     return true;
 }
 
-inline Type* LegalizedIntVectorType(Module& M, Type* ty)
+inline Type* LegalizedIntVectorType(const Module& M, Type* ty)
 {
     IGC_ASSERT(ty && ty->isIntOrIntVectorTy());
 
@@ -133,31 +134,34 @@ inline Type* LegalizedIntVectorType(Module& M, Type* ty)
 
     return ty->isIntegerTy() ?
         cast<Type>(IntegerType::get(M.getContext(), newSize)) :
-        IGCLLVM::FixedVectorType::get(IntegerType::get(M.getContext(), newSize), (unsigned)cast<VectorType>(ty)->getNumElements());
+        IGCLLVM::FixedVectorType::get(IntegerType::get(M.getContext(), newSize), (unsigned)cast<IGCLLVM::FixedVectorType>(ty)->getNumElements());
 }
 
 // Returns true for small structures that only contain primitive types
-inline bool isPromotableStructType(Module& M, Type* ty, bool isReturnValue = false)
+inline bool isPromotableStructType(const Module& M, const Type* ty, bool isStackCall, bool isReturnValue = false)
 {
     if (IGC_IS_FLAG_DISABLED(EnableByValStructArgPromotion))
         return false;
 
-    // We can separate promoting argument and return value sizes.
-    // Return value is limited to 64-bits due to vISA stackcall conventions.
-    // Argument is not limited to 64-bits, but can be adjusted to minimize spill.
-    const unsigned int maxSize = isReturnValue ? MAX_STACKCALL_RETVAL_SIZE_IN_BITS : MAX_STRUCT_ARGUMENT_SIZE_IN_BITS;
+    const unsigned int maxSize = isStackCall ? MAX_STRUCT_SIZE_IN_BITS
+        : MAX_SUBROUTINE_STRUCT_SIZE_IN_BITS;
+
     const DataLayout& DL = M.getDataLayout();
 
     if (ty->isPointerTy())
     {
         StructType* sTy = dyn_cast<StructType>(ty->getPointerElementType());
-        if (sTy && (unsigned)DL.getStructLayout(sTy)->getSizeInBits() < maxSize)
+        if (sTy && DL.getStructLayout(sTy)->getSizeInBits() <= maxSize)
         {
             for (const auto* EltTy : sTy->elements())
             {
                 // Check if all elements are primitive types
                 if (!EltTy->isSingleValueType() || EltTy->isVectorTy())
                     return false;
+                // Avoid int64 and fp64 because of unimplemented InstExpander::visitInsertValue
+                // and InstExpander::visitExtractValue in the Emu64Ops pass.
+                if (EltTy->isIntegerTy(64) || EltTy->isDoubleTy())
+                    return false;
             }
             return true;
         }
@@ -166,12 +170,12 @@ inline bool isPromotableStructType(Module& M, Type* ty, bool isReturnValue = fal
 }
 
 // Check if a function's first argument has the "sret" attribute and is a promotable struct type
-inline bool FunctionHasPromotableSRetArg(Module& M, Function* F)
+inline bool FunctionHasPromotableSRetArg(const Module& M, const Function* F)
 {
     if (F->getReturnType()->isVoidTy() &&
         !F->arg_empty() &&
         F->arg_begin()->hasStructRetAttr() &&
-        isPromotableStructType(M, F->arg_begin()->getType(), true))
+        isPromotableStructType(M, F->arg_begin()->getType(), F->hasFnAttribute("visaStackCall"), true))
     {
         return true;
     }
@@ -179,7 +183,7 @@ inline bool FunctionHasPromotableSRetArg(Module& M, Function* F)
 }
 
 // Promotes struct pointer to struct type
-inline Type* PromotedStructValueType(Module& M, Type* ty)
+inline Type* PromotedStructValueType(const Module& M, const Type* ty)
 {
     IGC_ASSERT(ty->isPointerTy() && ty->getPointerElementType()->isStructTy());
     return cast<StructType>(ty->getPointerElementType());
@@ -235,7 +239,9 @@ void LegalizeFunctionSignatures::FixFunctionSignatures(Module& M)
 
         // For binary linking, calling a function outside the module is possible, so declaration
         // signatures has to be fixed as well
-        if (pFunc->isDeclaration() && !pFunc->hasFnAttribute("referenced-indirectly"))
+        if (pFunc->isDeclaration() &&
+            !pFunc->hasFnAttribute("referenced-indirectly") &&
+            !pFunc->hasFnAttribute("invoke_simd_target"))
         {
             continue;
         }
@@ -256,7 +262,7 @@ void LegalizeFunctionSignatures::FixFunctionSignatures(Module& M)
             legalizeReturnType = true;
             argTypes.push_back(PointerType::get(pFunc->getReturnType(), 0));
         }
-        else if (isStackCall && FunctionHasPromotableSRetArg(M, pFunc))
+        else if (FunctionHasPromotableSRetArg(M, pFunc))
         {
             promoteSRetType = true;
             ai++; // Skip adding the first arg
@@ -269,9 +275,8 @@ void LegalizeFunctionSignatures::FixFunctionSignatures(Module& M)
                 fixArgType = true;
                 argTypes.push_back(LegalizedIntVectorType(M, ai->getType()));
             }
-            else if (isStackCall &&
-                ai->hasByValAttr() &&
-                isPromotableStructType(M, ai->getType()))
+            else if (ai->hasByValAttr() &&
+                isPromotableStructType(M, ai->getType(), isStackCall))
             {
                 fixArgType = true;
                 argTypes.push_back(PromotedStructValueType(M, ai->getType()));
@@ -296,6 +301,7 @@ void LegalizeFunctionSignatures::FixFunctionSignatures(Module& M)
         Function* pNewFunc = Function::Create(signature, pFunc->getLinkage(), pFunc->getName(), pFunc->getParent());
         pNewFunc->takeName(pFunc);
         pNewFunc->setCallingConv(pFunc->getCallingConv());
+        pNewFunc->setAttributes(pFunc->getAttributes());
 
         // Since we need to pass in pointers to be dereferenced by the new function, remove the "readnone" attribute
         // Also we need to create allocas for these pointers, so set the flag to true
@@ -334,7 +340,7 @@ void LegalizeFunctionSignatures::FixFunctionBody(Module& M)
                 legalizeReturnType = true;
                 ++NewArgIt; // Skip first argument that we added.
             }
-            else if (isStackCall && FunctionHasPromotableSRetArg(M, pFunc)) {
+            else if (FunctionHasPromotableSRetArg(M, pFunc)) {
                 promoteSRetType = true;
             }
 
@@ -347,6 +353,7 @@ void LegalizeFunctionSignatures::FixFunctionBody(Module& M)
                 {
                     // Create a temp alloca to map the old argument. This will be removed later by SROA.
                     tempAllocaForSRetPointer = builder.CreateAlloca(PromotedStructValueType(M, OldArgIt->getType()));
+                    tempAllocaForSRetPointer = builder.CreateAddrSpaceCast(tempAllocaForSRetPointer, OldArgIt->getType());
                     VMap[&*OldArgIt] = tempAllocaForSRetPointer;
                     continue;
                 }
@@ -358,9 +365,8 @@ void LegalizeFunctionSignatures::FixFunctionBody(Module& M)
                     Value* trunc = builder.CreateTrunc(&*NewArgIt, OldArgIt->getType());
                     VMap[&*OldArgIt] = trunc;
                 }
-                else if (isStackCall &&
-                    OldArgIt->hasByValAttr() &&
-                    isPromotableStructType(M, OldArgIt->getType()))
+                else if (OldArgIt->hasByValAttr() &&
+                    isPromotableStructType(M, OldArgIt->getType(), isStackCall))
                 {
                     // remove "byval" attrib since it is now pass-by-value
                     NewArgIt->removeAttr(llvm::Attribute::ByVal);
@@ -377,7 +383,7 @@ void LegalizeFunctionSignatures::FixFunctionBody(Module& M)
             }
 
             // Clone the old function body into the new
-            CloneFunctionInto(pNewFunc, pFunc, VMap, true, Returns);
+            IGCLLVM::CloneFunctionInto(pNewFunc, pFunc, VMap, true, Returns);
 
             // Merge the BB for when extra instructions were created
             BasicBlock* ClonedEntryBB = cast<BasicBlock>(VMap[&*pFunc->begin()]);
@@ -435,19 +441,20 @@ void LegalizeFunctionSignatures::FixFunctionUsers(Module& M)
         Function* pFunc = it.first;
         Function* pNewFunc = it.second;
 
-        for (auto ui = pFunc->user_begin(), ei = pFunc->user_end(); ui != ei; ++ui)
+        std::vector<User*> pFuncUses(pFunc->user_begin(), pFunc->user_end());
+        for (auto ui : pFuncUses)
         {
-            CallInst* callInst = dyn_cast<CallInst>(*ui);
+            CallInst* callInst = dyn_cast<CallInst>(ui);
             if (callInst && callInst->getCalledFunction() == pFunc)
             {
                 // Find the callers of the transformed functions
                 callsToFix.push_back(callInst);
             }
-            else if (Instruction* inst = dyn_cast<Instruction>(*ui))
+            else if (Instruction* inst = dyn_cast<Instruction>(ui))
             {
                 // Any other uses can be replaced with a pointer cast
                 IGCLLVM::IRBuilder<> builder(inst);
-                Value* pCast = builder.CreatePointerCast(pNewFunc, pFunc->getFunctionType());
+                Value* pCast = builder.CreatePointerCast(pNewFunc, pFunc->getType());
                 inst->replaceUsesOfWith(pFunc, pCast);
             }
         }
@@ -505,11 +512,10 @@ void LegalizeFunctionSignatures::FixCallInstruction(Module& M, CallInst* callIns
         ArgAttrVec.push_back(retAttrib);
         legalizeReturnType = true;
     }
-    else if (isStackCall &&
-        callInst->getType()->isVoidTy() &&
+    else if (callInst->getType()->isVoidTy() &&
         callInst->getNumArgOperands() > 0 &&
         callInst->paramHasAttr(0, llvm::Attribute::StructRet) &&
-        isPromotableStructType(M, callInst->getArgOperand(0)->getType(), true /* retval */))
+        isPromotableStructType(M, callInst->getArgOperand(0)->getType(), isStackCall, true /* retval */))
     {
         opNum++; // Skip the first call operand
         promoteSRetType = true;
@@ -528,9 +534,8 @@ void LegalizeFunctionSignatures::FixCallInstruction(Module& M, CallInst* callIns
             ArgAttrVec.push_back(AttributeSet());
             fixArgType = true;
         }
-        else if (isStackCall &&
-            callInst->paramHasAttr(opNum, llvm::Attribute::ByVal) &&
-            isPromotableStructType(M, arg->getType()))
+        else if (callInst->paramHasAttr(opNum, llvm::Attribute::ByVal) &&
+            isPromotableStructType(M, arg->getType(), isStackCall))
         {
             // Map the new operand to the loaded value of the struct pointer
             IGCLLVM::IRBuilder<> builder(callInst);
