@@ -10,7 +10,7 @@ SPDX-License-Identifier: MIT
 
 #include "common/LLVMWarningsPush.hpp"
 #include <llvm/IR/InstVisitor.h>
-#include <llvm/IR/IRBuilder.h>
+#include <llvmWrapper/IR/IRBuilder.h>
 #include "common/LLVMWarningsPop.hpp"
 #include "../Compiler/IGCPassSupport.h"
 
@@ -40,7 +40,7 @@ public:
   void ResolveExtractValue( llvm::ExtractValueInst* extractVal );
   void ResolveStoreInst( llvm::StoreInst *st, llvm::Type *ty,llvm::SmallVector<unsigned, 8> &index );
   llvm::Value* ResolveValue( llvm::Instruction *st,llvm::Value* arg,llvm::SmallVector<unsigned,8> &index );
-  llvm::Value* CreateGEP( llvm::IRBuilder<> &builder,llvm::Value*  ptr,llvm::SmallVector<unsigned,8> &indices );
+  llvm::Value* CreateGEP( IGCLLVM::IRBuilder<> &builder,llvm::Value*  ptr,llvm::SmallVector<unsigned,8> &indices );
   llvm::AllocaInst* CreateAlloca( llvm::Instruction *phi );
 
   static char ID;
