@@ -1,28 +1,11 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
+============================= end_copyright_notice ===========================*/
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
 #include "IGC/common/igc_debug.h"
 #include "../sp/sp_types.h"
 
@@ -60,7 +43,7 @@ Helper functions
 inline bool IsPlatformValid(
     const PLATFORM productID)
 {
-    if (productID.eRenderCoreFamily <= IGFX_GEN12LP_CORE)
+    if (productID.eRenderCoreFamily <= IGFX_XE_HPC_CORE)
     {
         return true;
     }
@@ -75,7 +58,7 @@ void DebugSurfaceStateCommand(
       const PLATFORM productID,
       std::string &output )
 {
-#if ( defined( _DEBUG ) || defined( _INTERNAL ) || defined( _RELEASE_INTERNAL ) )
+#if ( defined( _DEBUG ) || defined( _INTERNAL ) || defined( _RELEASE_INTERNAL )   || defined(ICBE_LINUX) || defined(_LINUX) || defined(LINUX))
     if (IGC_IS_FLAG_ENABLED(DumpPatchTokens)) {
         SSharedStateSurfaceState* p3DStateSurfaceState =
             (SSharedStateSurfaceState*)pLinearAddress;
@@ -499,7 +482,7 @@ void DebugInterfaceDescriptorDataCommand(
     const PLATFORM productID,
     std::string &output )
 {
-#if ( defined( _DEBUG ) || defined( _INTERNAL ) || defined( _RELEASE_INTERNAL ) )
+#if ( defined( _DEBUG ) || defined( _INTERNAL ) || defined( _RELEASE_INTERNAL )   || defined(ICBE_LINUX) || defined(_LINUX) || defined(LINUX))
     SMediaStateInterfaceDescriptorData* pInterfaceDescriptorData =
         (SMediaStateInterfaceDescriptorData*)pLinearAddress;
 
@@ -938,7 +921,7 @@ void DebugBindingTableStateCommand(
     const PLATFORM productID,
     std::string &output)
 {
-#if ( defined( _DEBUG ) || defined( _INTERNAL ) || defined( _RELEASE_INTERNAL ) )
+#if ( defined( _DEBUG ) || defined( _INTERNAL ) || defined( _RELEASE_INTERNAL )   || defined(ICBE_LINUX) || defined(_LINUX) || defined(LINUX))
     SSharedStateBindingTableState* pBindingTableState =
         (SSharedStateBindingTableState*)pLinearAddress;
 
