@@ -1,28 +1,10 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
+============================= end_copyright_notice ===========================*/
 
 #ifndef SPP_G8_H
 #define SPP_G8_H
@@ -44,26 +26,16 @@ namespace IGC
     struct SProgramOutput;
 };
 
-namespace cmc
-{
-    class CMKernel;
-};
-
 namespace iOpenCL
 {
 
-struct KernelData
-{
-    Util::BinaryStream* kernelBinary = nullptr;
-    Util::BinaryStream* kernelDebugData = nullptr;
-};
-
 // This is the base class to create an OpenCL ELF binary with patch tokens.
 // It owns BinaryStreams allocated.
 class CGen8OpenCLProgramBase : DisallowCopy {
 public:
     explicit CGen8OpenCLProgramBase(PLATFORM platform,
-                                    const CGen8OpenCLStateProcessor::IProgramContext& PI);
+                                    const CGen8OpenCLStateProcessor::IProgramContext& PI,
+                                    const WA_TABLE& WATable);
     virtual ~CGen8OpenCLProgramBase();
 
     /// GetProgramBinary - getting legacy (Patch token based) binary format
@@ -72,8 +44,12 @@ public:
     /// CGen8OpenCLProgram::CreateKernelBinaries or CGen8CMProgram::CreateKernelBinaries
     RETVAL GetProgramBinary(Util::BinaryStream& programBinary,
         unsigned pointerSizeInBytes);
+    /// GetProgramDebugDataSize - get size of debug info patch token
+    RETVAL GetProgramDebugDataSize(size_t& totalDbgInfoBufferSize);
     /// GetProgramDebugData - get debug data binary for legacy (Patch token based)
     /// binary format
+    RETVAL GetProgramDebugData(char* dstBuffer, size_t dstBufferSize);
+    /// GetProgramDebugData - get program debug data API used by VC.
     RETVAL GetProgramDebugData(Util::BinaryStream& programDebugData);
     /// CreateProgramScopePatchStream - get program scope patch token for legacy
     /// (Patch token based) binary format
@@ -128,6 +104,9 @@ private:
         bool needsSystemKernel() const  override;
         bool isProgramDebuggable() const override;
         bool hasProgrammableBorderColor() const override;
+        bool useBindlessMode() const override;
+        bool useBindlessLegacyMode() const override;
+
 
     private:
        const IGC::OpenCLProgramContext& m_Context;
@@ -137,43 +116,6 @@ private:
     CLProgramCtxProvider m_ContextProvider;
 };
 
-#if !defined(WDDM_LINUX) && (!defined(IGC_VC_DISABLED) || !IGC_VC_DISABLED)
-class CGen8CMProgram : public CGen8OpenCLProgramBase {
-public:
-    class CMProgramCtxProvider : public CGen8OpenCLStateProcessor::IProgramContext {
-    public:
-        CMProgramCtxProvider() {}
-
-        ShaderHash getProgramHash() const override  { return {}; }
-        bool needsSystemKernel() const  override { return false; }
-        bool isProgramDebuggable() const override  { return IsDebuggable; }
-        bool hasProgrammableBorderColor() const override { return false ; }
-
-        void updateDebuggableStatus(bool Debuggable) {
-            IsDebuggable = Debuggable;
-        }
-    private:
-        bool IsDebuggable = false;
-    };
-
-    explicit CGen8CMProgram(PLATFORM platform);
-    ~CGen8CMProgram();
-
-    // Produce the final ELF binary with the given CM kernels
-    // in OpenCL format.
-    void CreateKernelBinaries();
-    void GetZEBinary(llvm::raw_pwrite_stream& programBinary,
-        unsigned pointerSizeInBytes) override;
-
-    // CM kernel list.
-    std::vector<cmc::CMKernel*> m_kernels;
-
-    // Data structure to create patch token based binaries.
-    std::unique_ptr<IGC::SOpenCLProgramInfo> m_programInfo;
-
-    CMProgramCtxProvider m_ContextProvider;
-};
-#endif // !defined(WDDM_LINUX) && (!defined(IGC_VC_DISABLED) || !IGC_VC_DISABLED)
 }
 
 #endif //SPP_G8_H
