@@ -516,9 +516,25 @@ _SPIRV_OP(TypeTokenINTEL, 6113)
 _SPIRV_OP(ConvertFToBF16INTEL, 6116)
 _SPIRV_OP(ConvertBF16ToFINTEL, 6117)
 // SPV_INTEL_matrix
-_SPIRV_OP(TypeMatrixINTEL, 6119)
-_SPIRV_OP(MatrixLoadINTEL, 6120)
-_SPIRV_OP(MatrixStoreINTEL, 6121)
-_SPIRV_OP(MatrixMadINTEL, 6122)
+_SPIRV_OP(TypeJointMatrixINTEL, 6119)
+_SPIRV_OP(JointMatrixLoadINTEL, 6120)
+_SPIRV_OP(JointMatrixStoreINTEL, 6121)
+_SPIRV_OP(JointMatrixMadINTEL, 6122)
+_SPIRV_OP(JointMatrixSUMadINTEL, 6128)
+_SPIRV_OP(JointMatrixUSMadINTEL, 6129)
+_SPIRV_OP(JointMatrixUUMadINTEL, 6130)
+_SPIRV_OP(JointMatrixWorkItemLengthINTEL, 6410)
 // SPV_INTEL_arithmetic_fence
 _SPIRV_OP(ArithmeticFenceINTEL, 6145)
+// SPV_KHR_uniform_group_instructions
+_SPIRV_OP(GroupIMul, 6401)
+_SPIRV_OP(GroupFMul, 6402)
+_SPIRV_OP(GroupBitwiseAnd, 6403)
+_SPIRV_OP(GroupBitwiseOr, 6404)
+_SPIRV_OP(GroupBitwiseXor, 6405)
+_SPIRV_OP(GroupLogicalAnd, 6406)
+_SPIRV_OP(GroupLogicalOr, 6407)
+_SPIRV_OP(GroupLogicalXor, 6408)
+// SPV_INTEL_split_barrier
+_SPIRV_OP(ControlBarrierArriveINTEL, 6142)
+_SPIRV_OP(ControlBarrierWaitINTEL, 6143)
