@@ -353,5 +353,48 @@ void SPIRVTypeForwardPointer::decode(std::istream& I) {
   Decoder >> PointerId >> SC;
 }
 
+unsigned SPIRVTypeJointMatrixINTEL::getLayout() const {
+  return (unsigned)get<SPIRVConstant>(Layout)->getZExtIntValue();
+}
+
+unsigned SPIRVTypeJointMatrixINTEL::getRows() const {
+  return (unsigned)get<SPIRVConstant>(Rows)->getZExtIntValue();
+}
+
+unsigned SPIRVTypeJointMatrixINTEL::getColumns() const {
+  return (unsigned)get<SPIRVConstant>(Columns)->getZExtIntValue();
+}
+
+unsigned SPIRVTypeJointMatrixINTEL::getScope() const {
+  return (unsigned)get<SPIRVConstant>(Scope)->getZExtIntValue();
+}
+
+std::string SPIRVTypeJointMatrixINTEL::getMangledName() const {
+    std::string name;
+    switch (getLayout()) {
+      case SPIRVTypeJointMatrixINTEL::LayoutPackedA:
+        name += "packedA_";
+        break;
+      case SPIRVTypeJointMatrixINTEL::LayoutPackedB:
+        name += "packedB_";
+        break;
+      case SPIRVTypeJointMatrixINTEL::LayoutRowMajor:
+      case SPIRVTypeJointMatrixINTEL::LayoutColumnMajor:
+        name += "acc_";
+        break;
+    }
+    name += std::to_string(getRows());
+    name += "x";
+    name += std::to_string(getColumns());
+    name += "_";
+
+    if (ElemType->isTypeFloat()) {
+        name += "f";
+    } else {
+        name += "i";
+    }
+    name += std::to_string(ElemType->getBitWidth());
+    return std::move(name);
+}
 }
 
