@@ -406,6 +406,7 @@ public:
       CV.push_back(SPIRVCapabilityKind::CapabilityImageMipmap);
     return CV;
   }
+  SPIRVType* getSampledType() const { return get<SPIRVType>(SampledType); }
 protected:
   _SPIRV_DEF_DEC9(Id, SampledType, Desc.Dim, Desc.Depth,
       Desc.Arrayed, Desc.MS, Desc.Sampled, Desc.Format, Acc)
@@ -676,6 +677,62 @@ protected:
         }
 };
 
+class SPIRVTypeJointMatrixINTEL : public SPIRVType {
+public:
+    const static Op OC = OpTypeJointMatrixINTEL;
+    const static SPIRVWord FixedWC = 7;
+    // Complete constructor
+    SPIRVTypeJointMatrixINTEL(SPIRVModule *M, SPIRVId TheId, SPIRVType *ElemType,
+                              SPIRVId Rows, SPIRVId Columns,
+                              SPIRVId Layout, SPIRVId Scope)
+        : SPIRVType(M, FixedWC, OC, TheId), ElemType(ElemType),
+          Rows(Rows), Columns(Columns), Layout(Layout), Scope(Scope) {
+        validate();
+    }
+
+    // Incomplete constructor
+    SPIRVTypeJointMatrixINTEL()
+        : SPIRVType(OC), ElemType(0), Rows(0), Columns(0),
+          Layout(0), Scope(0) {
+    }
+
+    CapVec getRequiredCapability() const override {
+        return getVec(SPIRVCapabilityKind::CapabilityJointMatrixINTEL);
+    }
+
+    SPIRVType *getElemType() const { return ElemType; }
+
+    unsigned getLayout() const;
+    unsigned getRows() const;
+    unsigned getColumns() const;
+    unsigned getScope() const;
+    std::string getMangledName() const;
+
+    enum {
+        LayoutColumnMajor = 0,
+        LayoutRowMajor = 1,
+        LayoutPackedA = 2,
+        LayoutPackedB = 3,
+        LayoutMAX
+    };
+
+protected:
+    _SPIRV_DEF_DEC6(Id, ElemType, Rows, Columns, Layout, Scope)
+    void validate() const override {
+        SPIRVEntry::validate();
+        ElemType->validate();
+        IGC_ASSERT_EXIT_MESSAGE(getRows()    <= 64, "Unsupported rows size.");
+        IGC_ASSERT_EXIT_MESSAGE(getColumns() <= 64, "Unsupported columns size.");
+        IGC_ASSERT_EXIT_MESSAGE(getLayout() < LayoutMAX, "Unsupported layout.");
+    }
+
+private:
+    SPIRVType *ElemType;
+    SPIRVId Rows;
+    SPIRVId Columns;
+    SPIRVId Layout;
+    SPIRVId Scope;
+};
 
 
 template<typename T2, typename T1>
@@ -725,24 +782,24 @@ protected:
 class SPIRVTypeSubgroupINTEL;
 template<> inline void
 SPIRVMap<std::string, Op, SPIRVTypeSubgroupINTEL>::init() {
-#define _SPIRV_OP(x,y) \
-  add("struct.intel_sub_group_avc_"#x, OpTypeAvc##y##INTEL);
-_SPIRV_OP(mce_payload_t, McePayload)
-_SPIRV_OP(mce_result_t, MceResult)
-_SPIRV_OP(sic_payload_t, SicPayload)
-_SPIRV_OP(sic_result_t, SicResult)
-_SPIRV_OP(ime_result_single_reference_streamout_t, ImeResultSingleReferenceStreamout)
-_SPIRV_OP(ime_result_dual_reference_streamout_t, ImeResultDualReferenceStreamout)
-_SPIRV_OP(ime_single_reference_streamin_t, ImeSingleReferenceStreamin)
-_SPIRV_OP(ime_dual_reference_streamin_t, ImeDualReferenceStreamin)
-_SPIRV_OP(ime_payload_t, ImePayload)
-_SPIRV_OP(ime_result_t, ImeResult)
-_SPIRV_OP(ref_payload_t, RefPayload)
-_SPIRV_OP(ref_result_t, RefResult);
+#define _SPIRV_OP(x) \
+  add("spirv.Avc"#x, OpTypeAvc##x##INTEL);
+_SPIRV_OP(McePayload)
+_SPIRV_OP(MceResult)
+_SPIRV_OP(SicPayload)
+_SPIRV_OP(SicResult)
+_SPIRV_OP(ImeResultSingleReferenceStreamout)
+_SPIRV_OP(ImeResultDualReferenceStreamout)
+_SPIRV_OP(ImeSingleReferenceStreamin)
+_SPIRV_OP(ImeDualReferenceStreamin)
+_SPIRV_OP(ImePayload)
+_SPIRV_OP(ImeResult)
+_SPIRV_OP(RefPayload)
+_SPIRV_OP(RefResult);
 #undef _SPIRV_OP
 }
 typedef SPIRVMap<std::string, Op, SPIRVTypeSubgroupINTEL>
-OCLSubgroupINTELTypeOpCodeMap;
+SPIRVSubgroupINTELTypeOpCodeMap;
 
 class SPIRVTypeSubgroupAvcINTEL : public SPIRVType {
 public:
