@@ -1,28 +1,10 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
+============================= end_copyright_notice ===========================*/
 
 #pragma once
 
@@ -193,6 +175,10 @@ struct STB_TranslateInputArgs
     const uint32_t* pSpecConstantsIds;    // user-defined spec constants ids
     const uint64_t* pSpecConstantsValues; // spec constants values to be translated
     uint32_t        SpecConstantsSize;    // number of specialization constants
+    const char**    pVISAAsmToLinkArray;  // array of additional visa assembly in text format
+                                          // that should be linked with compiled module.
+                                          // Used e.g. for "sginvoke" functionality.
+    uint32_t        NumVISAAsmsToLink;
 
     STB_TranslateInputArgs()
     {
@@ -209,6 +195,8 @@ struct STB_TranslateInputArgs
         pSpecConstantsIds     = NULL;
         pSpecConstantsValues  = NULL;
         SpecConstantsSize     = 0;
+        pVISAAsmToLinkArray   = NULL;
+        NumVISAAsmsToLink     = 0;
     }
 };
 
