@@ -26,6 +26,7 @@ SPDX-License-Identifier: MIT
 #include "common/SystemThread.h"
 
 #include "cif/macros/enable.h"
+#include "version.h"
 
 namespace IGC
 {
@@ -94,6 +95,14 @@ CIF_DECLARE_INTERFACE_PIMPL(IgcOclDeviceCtx) : CIF::PimplBase
         return true;
     }
 
+    const char* GetIGCRevision() {
+#ifdef IGC_REVISION
+      return IGC_REVISION;
+#else
+      return "";
+#endif
+    }
+
     struct MiscOptions
     {
         MiscOptions()
