@@ -1,28 +1,11 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
+============================= end_copyright_notice ===========================*/
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
 /*****************************************************************************\
 Abstract:   Contains common patch structure definitions
 \*****************************************************************************/
@@ -33,7 +16,7 @@ Abstract:   Contains common patch structure definitions
 
 namespace iOpenCL
 {
-const uint32_t CURRENT_ICBE_VERSION = 1069;
+const uint32_t CURRENT_ICBE_VERSION = 1081;
 const uint32_t MAGIC_CL = 0x494E5443;      // 'I', 'N', 'T', 'C'
 const uint32_t INVALID_INDEX = 0xFFFFFFFF;
 
@@ -150,12 +133,14 @@ enum PATCH_TOKEN
     PATCH_TOKEN_PROGRAM_RELOCATION_TABLE,                         // 54  @SPatchFunctionTableInfo@
     PATCH_TOKEN_MEDIA_VFE_STATE_SLOT1,                            // 55  @SPatchMediaVFEState of slot1@
     PATCH_TOKEN_ALLOCATE_SYNC_BUFFER,                             // 56  @SPatchAllocateSyncBuffer@
+    PATCH_TOKEN_GLOBAL_HOST_ACCESS_TABLE,                         // 57  @SPatchHostAccessTableInfo@
+    PATCH_TOKEN_ALLOCATE_RT_GLOBAL_BUFFER,                        // 58  @SPatchAllocateRTGlobalBuffer@
 
     NUM_PATCH_TOKENS
 };
 
 // Update CURRENT_ICBE_VERSION when modifying the patch list
-static_assert( NUM_PATCH_TOKENS == 57, "NUM_PATCH_TOKENS has invalid value");
+static_assert( NUM_PATCH_TOKENS == 59, "NUM_PATCH_TOKENS has invalid value");
 
 /*****************************************************************************\
 ENUM: IMAGE_MEMORY_OBJECT_TYPE
@@ -263,11 +248,14 @@ enum DATA_PARAMETER_TOKEN
     DATA_PARAMETER_FLAT_IMAGE_WIDTH,                                // 45
     DATA_PARAMETER_FLAT_IMAGE_HEIGHT,                               // 46
     DATA_PARAMETER_FLAT_IMAGE_PITCH,                                // 47
+    DATA_PARAMETER_RT_STACK_ID,                                     // 48
+    DATA_PARAMETER_IMPL_ARG_BUFFER,                                 // 49
+    DATA_PARAMETER_LOCAL_ID_BUFFER,                                 // 50
     NUM_DATA_PARAMETER_TOKENS
 };
 
 // Update CURRENT_ICBE_VERSION when modifying the patch list
-static_assert(NUM_DATA_PARAMETER_TOKENS == 48, "NUM_DATA_PARAMETER_TOKENS has invalid value");
+static_assert(NUM_DATA_PARAMETER_TOKENS == 51, "NUM_DATA_PARAMETER_TOKENS has invalid value");
 
 /*****************************************************************************\
 ENUM: CONSTANT_BUFFER_TYPE
