@@ -1541,6 +1541,12 @@ long8  __attribute__((overloadable)) intel_bfi(long8  base, long8  insert, uint8
 long16 __attribute__((overloadable)) intel_bfi(long16 base, long16 insert, uint16 offset, uint16 count);
 
 // sbfe i8
+char   __attribute__((overloadable)) intel_sbfe(uchar   base, uint   offset, uint   count);
+char2  __attribute__((overloadable)) intel_sbfe(uchar2  base, uint2  offset, uint2  count);
+char3  __attribute__((overloadable)) intel_sbfe(uchar3  base, uint3  offset, uint3  count);
+char4  __attribute__((overloadable)) intel_sbfe(uchar4  base, uint4  offset, uint4  count);
+char8  __attribute__((overloadable)) intel_sbfe(uchar8  base, uint8  offset, uint8  count);
+char16 __attribute__((overloadable)) intel_sbfe(uchar16 base, uint16 offset, uint16 count);
 char   __attribute__((overloadable)) intel_sbfe(char   base, uint   offset, uint   count);
 char2  __attribute__((overloadable)) intel_sbfe(char2  base, uint2  offset, uint2  count);
 char3  __attribute__((overloadable)) intel_sbfe(char3  base, uint3  offset, uint3  count);
@@ -1549,6 +1555,12 @@ char8  __attribute__((overloadable)) intel_sbfe(char8  base, uint8  offset, uint
 char16 __attribute__((overloadable)) intel_sbfe(char16 base, uint16 offset, uint16 count);
 
 // sbfe i16
+short   __attribute__((overloadable)) intel_sbfe(ushort   base, uint   offset, uint   count);
+short2  __attribute__((overloadable)) intel_sbfe(ushort2  base, uint2  offset, uint2  count);
+short3  __attribute__((overloadable)) intel_sbfe(ushort3  base, uint3  offset, uint3  count);
+short4  __attribute__((overloadable)) intel_sbfe(ushort4  base, uint4  offset, uint4  count);
+short8  __attribute__((overloadable)) intel_sbfe(ushort8  base, uint8  offset, uint8  count);
+short16 __attribute__((overloadable)) intel_sbfe(ushort16 base, uint16 offset, uint16 count);
 short   __attribute__((overloadable)) intel_sbfe(short   base, uint   offset, uint   count);
 short2  __attribute__((overloadable)) intel_sbfe(short2  base, uint2  offset, uint2  count);
 short3  __attribute__((overloadable)) intel_sbfe(short3  base, uint3  offset, uint3  count);
@@ -1557,6 +1569,12 @@ short8  __attribute__((overloadable)) intel_sbfe(short8  base, uint8  offset, ui
 short16 __attribute__((overloadable)) intel_sbfe(short16 base, uint16 offset, uint16 count);
 
 // sbfe i32
+int   __attribute__((overloadable)) intel_sbfe(uint   base, uint   offset, uint   count);
+int2  __attribute__((overloadable)) intel_sbfe(uint2  base, uint2  offset, uint2  count);
+int3  __attribute__((overloadable)) intel_sbfe(uint3  base, uint3  offset, uint3  count);
+int4  __attribute__((overloadable)) intel_sbfe(uint4  base, uint4  offset, uint4  count);
+int8  __attribute__((overloadable)) intel_sbfe(uint8  base, uint8  offset, uint8  count);
+int16 __attribute__((overloadable)) intel_sbfe(uint16 base, uint16 offset, uint16 count);
 int   __attribute__((overloadable)) intel_sbfe(int   base, uint   offset, uint   count);
 int2  __attribute__((overloadable)) intel_sbfe(int2  base, uint2  offset, uint2  count);
 int3  __attribute__((overloadable)) intel_sbfe(int3  base, uint3  offset, uint3  count);
@@ -1565,6 +1583,12 @@ int8  __attribute__((overloadable)) intel_sbfe(int8  base, uint8  offset, uint8
 int16 __attribute__((overloadable)) intel_sbfe(int16 base, uint16 offset, uint16 count);
 
 // sbfe i64
+long   __attribute__((overloadable)) intel_sbfe(ulong   base, uint   offset, uint   count);
+long2  __attribute__((overloadable)) intel_sbfe(ulong2  base, uint2  offset, uint2  count);
+long3  __attribute__((overloadable)) intel_sbfe(ulong3  base, uint3  offset, uint3  count);
+long4  __attribute__((overloadable)) intel_sbfe(ulong4  base, uint4  offset, uint4  count);
+long8  __attribute__((overloadable)) intel_sbfe(ulong8  base, uint8  offset, uint8  count);
+long16 __attribute__((overloadable)) intel_sbfe(ulong16 base, uint16 offset, uint16 count);
 long   __attribute__((overloadable)) intel_sbfe(long   base, uint   offset, uint   count);
 long2  __attribute__((overloadable)) intel_sbfe(long2  base, uint2  offset, uint2  count);
 long3  __attribute__((overloadable)) intel_sbfe(long3  base, uint3  offset, uint3  count);
@@ -1580,6 +1604,12 @@ uchar3  __attribute__((overloadable)) intel_ubfe(uchar3  base, uint3  offset, ui
 uchar4  __attribute__((overloadable)) intel_ubfe(uchar4  base, uint4  offset, uint4  count);
 uchar8  __attribute__((overloadable)) intel_ubfe(uchar8  base, uint8  offset, uint8  count);
 uchar16 __attribute__((overloadable)) intel_ubfe(uchar16 base, uint16 offset, uint16 count);
+uchar   __attribute__((overloadable)) intel_ubfe(char   base, uint   offset, uint   count);
+uchar2  __attribute__((overloadable)) intel_ubfe(char2  base, uint2  offset, uint2  count);
+uchar3  __attribute__((overloadable)) intel_ubfe(char3  base, uint3  offset, uint3  count);
+uchar4  __attribute__((overloadable)) intel_ubfe(char4  base, uint4  offset, uint4  count);
+uchar8  __attribute__((overloadable)) intel_ubfe(char8  base, uint8  offset, uint8  count);
+uchar16 __attribute__((overloadable)) intel_ubfe(char16 base, uint16 offset, uint16 count);
 
 // ubfe i16
 ushort   __attribute__((overloadable)) intel_ubfe(ushort   base, uint   offset, uint   count);
@@ -1588,6 +1618,12 @@ ushort3  __attribute__((overloadable)) intel_ubfe(ushort3  base, uint3  offset,
 ushort4  __attribute__((overloadable)) intel_ubfe(ushort4  base, uint4  offset, uint4  count);
 ushort8  __attribute__((overloadable)) intel_ubfe(ushort8  base, uint8  offset, uint8  count);
 ushort16 __attribute__((overloadable)) intel_ubfe(ushort16 base, uint16 offset, uint16 count);
+ushort   __attribute__((overloadable)) intel_ubfe(short   base, uint   offset, uint   count);
+ushort2  __attribute__((overloadable)) intel_ubfe(short2  base, uint2  offset, uint2  count);
+ushort3  __attribute__((overloadable)) intel_ubfe(short3  base, uint3  offset, uint3  count);
+ushort4  __attribute__((overloadable)) intel_ubfe(short4  base, uint4  offset, uint4  count);
+ushort8  __attribute__((overloadable)) intel_ubfe(short8  base, uint8  offset, uint8  count);
+ushort16 __attribute__((overloadable)) intel_ubfe(short16 base, uint16 offset, uint16 count);
 
 // ubfe i32
 uint   __attribute__((overloadable)) intel_ubfe(uint   base, uint   offset, uint   count);
@@ -1596,6 +1632,12 @@ uint3  __attribute__((overloadable)) intel_ubfe(uint3  base, uint3  offset, uint
 uint4  __attribute__((overloadable)) intel_ubfe(uint4  base, uint4  offset, uint4  count);
 uint8  __attribute__((overloadable)) intel_ubfe(uint8  base, uint8  offset, uint8  count);
 uint16 __attribute__((overloadable)) intel_ubfe(uint16 base, uint16 offset, uint16 count);
+uint   __attribute__((overloadable)) intel_ubfe(int   base, uint   offset, uint   count);
+uint2  __attribute__((overloadable)) intel_ubfe(int2  base, uint2  offset, uint2  count);
+uint3  __attribute__((overloadable)) intel_ubfe(int3  base, uint3  offset, uint3  count);
+uint4  __attribute__((overloadable)) intel_ubfe(int4  base, uint4  offset, uint4  count);
+uint8  __attribute__((overloadable)) intel_ubfe(int8  base, uint8  offset, uint8  count);
+uint16 __attribute__((overloadable)) intel_ubfe(int16 base, uint16 offset, uint16 count);
 
 // ubfe i64
 ulong   __attribute__((overloadable)) intel_ubfe(ulong   base, uint   offset, uint   count);
@@ -1604,6 +1646,12 @@ ulong3  __attribute__((overloadable)) intel_ubfe(ulong3  base, uint3  offset, ui
 ulong4  __attribute__((overloadable)) intel_ubfe(ulong4  base, uint4  offset, uint4  count);
 ulong8  __attribute__((overloadable)) intel_ubfe(ulong8  base, uint8  offset, uint8  count);
 ulong16 __attribute__((overloadable)) intel_ubfe(ulong16 base, uint16 offset, uint16 count);
+ulong   __attribute__((overloadable)) intel_ubfe(long   base, uint   offset, uint   count);
+ulong2  __attribute__((overloadable)) intel_ubfe(long2  base, uint2  offset, uint2  count);
+ulong3  __attribute__((overloadable)) intel_ubfe(long3  base, uint3  offset, uint3  count);
+ulong4  __attribute__((overloadable)) intel_ubfe(long4  base, uint4  offset, uint4  count);
+ulong8  __attribute__((overloadable)) intel_ubfe(long8  base, uint8  offset, uint8  count);
+ulong16 __attribute__((overloadable)) intel_ubfe(long16 base, uint16 offset, uint16 count);
 
 // bfrev i8
 uchar   __attribute__((overloadable)) intel_bfrev(uchar   base);
@@ -1612,6 +1660,12 @@ uchar3  __attribute__((overloadable)) intel_bfrev(uchar3  base);
 uchar4  __attribute__((overloadable)) intel_bfrev(uchar4  base);
 uchar8  __attribute__((overloadable)) intel_bfrev(uchar8  base);
 uchar16 __attribute__((overloadable)) intel_bfrev(uchar16 base);
+char   __attribute__((overloadable)) intel_bfrev(char   base);
+char2  __attribute__((overloadable)) intel_bfrev(char2  base);
+char3  __attribute__((overloadable)) intel_bfrev(char3  base);
+char4  __attribute__((overloadable)) intel_bfrev(char4  base);
+char8  __attribute__((overloadable)) intel_bfrev(char8  base);
+char16 __attribute__((overloadable)) intel_bfrev(char16 base);
 
 // bfrev i16
 ushort   __attribute__((overloadable)) intel_bfrev(ushort   base);
@@ -1620,6 +1674,12 @@ ushort3  __attribute__((overloadable)) intel_bfrev(ushort3  base);
 ushort4  __attribute__((overloadable)) intel_bfrev(ushort4  base);
 ushort8  __attribute__((overloadable)) intel_bfrev(ushort8  base);
 ushort16 __attribute__((overloadable)) intel_bfrev(ushort16 base);
+short   __attribute__((overloadable)) intel_bfrev(short   base);
+short2  __attribute__((overloadable)) intel_bfrev(short2  base);
+short3  __attribute__((overloadable)) intel_bfrev(short3  base);
+short4  __attribute__((overloadable)) intel_bfrev(short4  base);
+short8  __attribute__((overloadable)) intel_bfrev(short8  base);
+short16 __attribute__((overloadable)) intel_bfrev(short16 base);
 
 // bfrev i32
 uint   __attribute__((overloadable)) intel_bfrev(uint   base);
@@ -1628,6 +1688,12 @@ uint3  __attribute__((overloadable)) intel_bfrev(uint3  base);
 uint4  __attribute__((overloadable)) intel_bfrev(uint4  base);
 uint8  __attribute__((overloadable)) intel_bfrev(uint8  base);
 uint16 __attribute__((overloadable)) intel_bfrev(uint16 base);
+int   __attribute__((overloadable)) intel_bfrev(int   base);
+int2  __attribute__((overloadable)) intel_bfrev(int2  base);
+int3  __attribute__((overloadable)) intel_bfrev(int3  base);
+int4  __attribute__((overloadable)) intel_bfrev(int4  base);
+int8  __attribute__((overloadable)) intel_bfrev(int8  base);
+int16 __attribute__((overloadable)) intel_bfrev(int16 base);
 
 // bfrev i64
 ulong   __attribute__((overloadable)) intel_bfrev(ulong   base);
@@ -1636,8 +1702,214 @@ ulong3  __attribute__((overloadable)) intel_bfrev(ulong3  base);
 ulong4  __attribute__((overloadable)) intel_bfrev(ulong4  base);
 ulong8  __attribute__((overloadable)) intel_bfrev(ulong8  base);
 ulong16 __attribute__((overloadable)) intel_bfrev(ulong16 base);
+long   __attribute__((overloadable)) intel_bfrev(long   base);
+long2  __attribute__((overloadable)) intel_bfrev(long2  base);
+long3  __attribute__((overloadable)) intel_bfrev(long3  base);
+long4  __attribute__((overloadable)) intel_bfrev(long4  base);
+long8  __attribute__((overloadable)) intel_bfrev(long8  base);
+long16 __attribute__((overloadable)) intel_bfrev(long16 base);
 #endif // defined(cl_intel_bit_instructions)
 
 #ifdef cl_intel_subgroups_ballot
 uint intel_sub_group_ballot(bool p);
 #endif
+
+
+#ifdef cl_khr_fp16
+#pragma OPENCL EXTENSION cl_khr_fp16 : enable
+#endif
+
+#ifdef cl_intel_subgroup_matrix_multiply_accumulate
+
+/**** SIMD8 ****/
+
+// 8-bit and 8-bit matrices:
+int   __attribute__((overloadable)) intel_sub_group_i8_i8_matrix_mad_k32(int   a, int8  b, int  acc);
+int2  __attribute__((overloadable)) intel_sub_group_i8_i8_matrix_mad_k32(int2  a, int8  b, int2 acc);
+int4  __attribute__((overloadable)) intel_sub_group_i8_i8_matrix_mad_k32(int4  a, int8  b, int4 acc);
+int8  __attribute__((overloadable)) intel_sub_group_i8_i8_matrix_mad_k32(int8  a, int8  b, int8 acc);
+
+int   __attribute__((overloadable)) intel_sub_group_i8_u8_matrix_mad_k32(int   a, uint8 b, int  acc);
+int2  __attribute__((overloadable)) intel_sub_group_i8_u8_matrix_mad_k32(int2  a, uint8 b, int2 acc);
+int4  __attribute__((overloadable)) intel_sub_group_i8_u8_matrix_mad_k32(int4  a, uint8 b, int4 acc);
+int8  __attribute__((overloadable)) intel_sub_group_i8_u8_matrix_mad_k32(int8  a, uint8 b, int8 acc);
+
+int   __attribute__((overloadable)) intel_sub_group_u8_i8_matrix_mad_k32(uint  a, int8  b, int  acc);
+int2  __attribute__((overloadable)) intel_sub_group_u8_i8_matrix_mad_k32(uint2 a, int8  b, int2 acc);
+int4  __attribute__((overloadable)) intel_sub_group_u8_i8_matrix_mad_k32(uint4 a, int8  b, int4 acc);
+int8  __attribute__((overloadable)) intel_sub_group_u8_i8_matrix_mad_k32(uint8 a, int8  b, int8 acc);
+
+int   __attribute__((overloadable)) intel_sub_group_u8_u8_matrix_mad_k32(uint  a, uint8 b, int  acc);
+int2  __attribute__((overloadable)) intel_sub_group_u8_u8_matrix_mad_k32(uint2 a, uint8 b, int2 acc);
+int4  __attribute__((overloadable)) intel_sub_group_u8_u8_matrix_mad_k32(uint4 a, uint8 b, int4 acc);
+int8  __attribute__((overloadable)) intel_sub_group_u8_u8_matrix_mad_k32(uint8 a, uint8 b, int8 acc);
+
+// bfloat16 and bfloat16 matrices:
+float   __attribute__((overloadable)) intel_sub_group_bf16_bf16_matrix_mad_k16(int  a, int8 b, float  acc);
+float2  __attribute__((overloadable)) intel_sub_group_bf16_bf16_matrix_mad_k16(int2 a, int8 b, float2 acc);
+float4  __attribute__((overloadable)) intel_sub_group_bf16_bf16_matrix_mad_k16(int4 a, int8 b, float4 acc);
+float8  __attribute__((overloadable)) intel_sub_group_bf16_bf16_matrix_mad_k16(int8 a, int8 b, float8 acc);
+
+// fp16 and fp16 matrices:
+float   __attribute__((overloadable)) intel_sub_group_f16_f16_matrix_mad_k16(int  a, int8 b, float  acc);
+float2  __attribute__((overloadable)) intel_sub_group_f16_f16_matrix_mad_k16(int2 a, int8 b, float2 acc);
+float4  __attribute__((overloadable)) intel_sub_group_f16_f16_matrix_mad_k16(int4 a, int8 b, float4 acc);
+float8  __attribute__((overloadable)) intel_sub_group_f16_f16_matrix_mad_k16(int8 a, int8 b, float8 acc);
+
+
+/**** SIMD16 ****/
+
+// 8-bit and 8-bit matrices:
+int  __attribute__((overloadable)) intel_sub_group_i8_i8_matrix_mad_k32(short a, int8 b, int acc);   // M = 1
+int2 __attribute__((overloadable)) intel_sub_group_i8_i8_matrix_mad_k32(short2 a, int8 b, int2 acc); // M = 2
+int4 __attribute__((overloadable)) intel_sub_group_i8_i8_matrix_mad_k32(short4 a, int8 b, int4 acc); // M = 4
+int8 __attribute__((overloadable)) intel_sub_group_i8_i8_matrix_mad_k32(short8 a, int8 b, int8 acc); // M = 8
+
+int  __attribute__((overloadable)) intel_sub_group_i8_u8_matrix_mad_k32(short a, uint8 b, int acc); // ...
+int2 __attribute__((overloadable)) intel_sub_group_i8_u8_matrix_mad_k32(short2 a, uint8 b, int2 acc);
+int4 __attribute__((overloadable)) intel_sub_group_i8_u8_matrix_mad_k32(short4 a, uint8 b, int4 acc);
+int8 __attribute__((overloadable)) intel_sub_group_i8_u8_matrix_mad_k32(short8 a, uint8 b, int8 acc);
+
+int  __attribute__((overloadable)) intel_sub_group_u8_i8_matrix_mad_k32(ushort a, int8 b, int acc);
+int2 __attribute__((overloadable)) intel_sub_group_u8_i8_matrix_mad_k32(ushort2 a, int8 b, int2 acc);
+int4 __attribute__((overloadable)) intel_sub_group_u8_i8_matrix_mad_k32(ushort4 a, int8 b, int4 acc);
+int8 __attribute__((overloadable)) intel_sub_group_u8_i8_matrix_mad_k32(ushort8 a, int8 b, int8 acc);
+
+int  __attribute__((overloadable)) intel_sub_group_u8_u8_matrix_mad_k32(ushort a, uint8 b, int acc);
+int2 __attribute__((overloadable)) intel_sub_group_u8_u8_matrix_mad_k32(ushort2 a, uint8 b, int2 acc);
+int4 __attribute__((overloadable)) intel_sub_group_u8_u8_matrix_mad_k32(ushort4 a, uint8 b, int4 acc);
+int8 __attribute__((overloadable)) intel_sub_group_u8_u8_matrix_mad_k32(ushort8 a, uint8 b, int8 acc);
+
+// bfloat16 and bfloat16 matrices,
+// float acc, float return type:
+float  __attribute__((overloadable)) intel_sub_group_bf16_bf16_matrix_mad_k16(short a, int8 b, float acc);
+float2 __attribute__((overloadable)) intel_sub_group_bf16_bf16_matrix_mad_k16(short2 a, int8 b, float2 acc);
+float4 __attribute__((overloadable)) intel_sub_group_bf16_bf16_matrix_mad_k16(short4 a, int8 b, float4 acc);
+float8 __attribute__((overloadable)) intel_sub_group_bf16_bf16_matrix_mad_k16(short8 a, int8 b, float8 acc);
+
+// fp16 and fp16 matrices,
+// float acc, float return type:
+float  __attribute__((overloadable)) intel_sub_group_f16_f16_matrix_mad_k16(short a, int8 b, float acc);
+float2 __attribute__((overloadable)) intel_sub_group_f16_f16_matrix_mad_k16(short2 a, int8 b, float2 acc);
+float4 __attribute__((overloadable)) intel_sub_group_f16_f16_matrix_mad_k16(short4 a, int8 b, float4 acc);
+float8 __attribute__((overloadable)) intel_sub_group_f16_f16_matrix_mad_k16(short8 a, int8 b, float8 acc);
+
+#ifdef cl_intel_subgroup_matrix_multiply_accumulate_tf32
+
+// A: half of tfloat32 B: tfloat32 ACC: float DST: float
+float  __attribute__((overloadable)) intel_sub_group_tf32_tf32_matrix_mad_k8_f32(short  a, int8 b, float  acc);
+float2 __attribute__((overloadable)) intel_sub_group_tf32_tf32_matrix_mad_k8_f32(short2 a, int8 b, float2 acc);
+float4 __attribute__((overloadable)) intel_sub_group_tf32_tf32_matrix_mad_k8_f32(short4 a, int8 b, float4 acc);
+float8 __attribute__((overloadable)) intel_sub_group_tf32_tf32_matrix_mad_k8_f32(short8 a, int8 b, float8 acc);
+
+#endif // cl_intel_subgroup_matrix_multiply_accumulate_tf32
+#endif // cl_intel_subgroup_matrix_multiply_accumulate
+
+#ifdef cl_intel_subgroup_split_matrix_multiply_accumulate
+
+/// split matrix (dpasw, simd8 only)
+int2 __attribute__((overloadable)) intel_sub_group_i8_i8_split_matrix_mad_k32(int   a, int8  b, int2 acc);  // M = 2
+int4 __attribute__((overloadable)) intel_sub_group_i8_i8_split_matrix_mad_k32(int2  a, int8  b, int4 acc);  // M = 4
+int8 __attribute__((overloadable)) intel_sub_group_i8_i8_split_matrix_mad_k32(int4  a, int8  b, int8 acc);  // M = 8
+
+int2 __attribute__((overloadable)) intel_sub_group_i8_u8_split_matrix_mad_k32(int   a, uint8 b, int2 acc);
+int4 __attribute__((overloadable)) intel_sub_group_i8_u8_split_matrix_mad_k32(int2  a, uint8 b, int4 acc);
+int8 __attribute__((overloadable)) intel_sub_group_i8_u8_split_matrix_mad_k32(int4  a, uint8 b, int8 acc);
+
+int2 __attribute__((overloadable)) intel_sub_group_u8_i8_split_matrix_mad_k32(uint  a, int8  b, int2 acc);
+int4 __attribute__((overloadable)) intel_sub_group_u8_i8_split_matrix_mad_k32(uint2 a, int8  b, int4 acc);
+int8 __attribute__((overloadable)) intel_sub_group_u8_i8_split_matrix_mad_k32(uint4 a, int8  b, int8 acc);
+
+int2 __attribute__((overloadable)) intel_sub_group_u8_u8_split_matrix_mad_k32(uint  a, uint8 b, int2 acc);
+int4 __attribute__((overloadable)) intel_sub_group_u8_u8_split_matrix_mad_k32(uint2 a, uint8 b, int4 acc);
+int8 __attribute__((overloadable)) intel_sub_group_u8_u8_split_matrix_mad_k32(uint4 a, uint8 b, int8 acc);
+
+// bfloat16 and bfloat16 matrices:
+float2 __attribute__((overloadable)) intel_sub_group_bf16_bf16_split_matrix_mad_k16(int  a, int8 b, float2 acc);
+float4 __attribute__((overloadable)) intel_sub_group_bf16_bf16_split_matrix_mad_k16(int2 a, int8 b, float4 acc);
+float8 __attribute__((overloadable)) intel_sub_group_bf16_bf16_split_matrix_mad_k16(int4 a, int8 b, float8 acc);
+
+// fp16 and fp16 matrices:
+float2 __attribute__((overloadable)) intel_sub_group_f16_f16_split_matrix_mad_k16(int  a, int8 b, float2 acc);
+float4 __attribute__((overloadable)) intel_sub_group_f16_f16_split_matrix_mad_k16(int2 a, int8 b, float4 acc);
+float8 __attribute__((overloadable)) intel_sub_group_f16_f16_split_matrix_mad_k16(int4 a, int8 b, float8 acc);
+
+#endif // cl_intel_subgroup_split_matrix_multiply_accumulate
+
+//atomic fadd/fsub
+#if defined(cl_intel_global_float_atomics)
+float __attribute__((overloadable)) atom_add(volatile __global float *p, float val);
+float __attribute__((overloadable)) atomic_add(volatile __global float *p, float val);
+float __attribute__((overloadable)) atom_sub(volatile __global float *p, float val);
+float __attribute__((overloadable)) atomic_sub(volatile __global float *p, float val);
+
+#if (__OPENCL_C_VERSION__ >= CL_VERSION_2_0)
+float __attribute__((overloadable)) atomic_fetch_add(volatile __global atomic_float *object, float operand);
+float __attribute__((overloadable)) atomic_fetch_add_explicit(volatile __global atomic_float *object, float operand, memory_order order);
+float __attribute__((overloadable)) atomic_fetch_add_explicit(volatile __global atomic_float *object, float operand, memory_order order, memory_scope scope);
+float __attribute__((overloadable)) atomic_fetch_sub(volatile __global atomic_float *object, float operand);
+float __attribute__((overloadable)) atomic_fetch_sub_explicit(volatile __global atomic_float *object, float operand, memory_order order);
+float __attribute__((overloadable)) atomic_fetch_sub_explicit(volatile __global atomic_float *object, float operand, memory_order order, memory_scope scope);
+#endif // CL_VERSION_2_0
+#endif //defined(cl_intel_global_float_atomics)
+
+#if defined(cl_intel_pvc_rt_validation)
+
+struct rtglobals_t;
+typedef __global struct rtglobals_t *rtglobals_t;
+struct rtfence_t;
+typedef __private struct rtfence_t *rtfence_t;
+
+void *intel_get_rt_stack(rtglobals_t rt_dispatch_globals);
+
+void *intel_get_thread_btd_stack(rtglobals_t rt_dispatch_globals);
+
+void *intel_get_global_btd_stack(rtglobals_t rt_dispatch_globals);
+
+rtfence_t intel_dispatch_trace_ray_query(
+    rtglobals_t rt_dispatch_globals, uint bvh_level, uint traceRayCtrl);
+
+void intel_rt_sync(rtfence_t fence);
+
+global void* intel_get_implicit_dispatch_globals();
+
+#endif // defined(cl_intel_pvc_rt_validation)
+
+#ifdef cl_intel_subgroup_extended_block_read
+ushort2  intel_subgroup_block_read_u8_m1k32v2(__global void *base_address, int width, int height, int pitch, int2 coord);
+ushort4  intel_subgroup_block_read_u8_m2k32v2(__global void *base_address, int width, int height, int pitch, int2 coord);
+ushort8  intel_subgroup_block_read_u8_m4k32v2(__global void *base_address, int width, int height, int pitch, int2 coord);
+ushort16 intel_subgroup_block_read_u8_m8k32v2(__global void *base_address, int width, int height, int pitch, int2 coord);
+ushort2  intel_subgroup_block_read_u16_m1k16v2(__global void *base_address, int width, int height, int pitch, int2 coord);
+ushort4  intel_subgroup_block_read_u16_m2k16v2(__global void *base_address, int width, int height, int pitch, int2 coord);
+ushort8  intel_subgroup_block_read_u16_m4k16v2(__global void *base_address, int width, int height, int pitch, int2 coord);
+ushort16 intel_subgroup_block_read_u16_m8k16v2(__global void *base_address, int width, int height, int pitch, int2 coord);
+uint8    intel_subgroup_block_read_transform_u8_k32(__global void *base_address, int width, int height, int pitch, int2 coord);
+uint8    intel_subgroup_block_read_transform_u16_k16(__global void *base_address, int width, int height, int pitch, int2 coord);
+uint8    intel_subgroup_block_read_transpose_u32_k8(__global void *base_address, int width, int height, int pitch, int2 coord);
+
+#endif //defined(cl_intel_subgroup_extended_block_read)
+
+void global_barrier();
+
+//
+// Named Barriers definitions
+//
+#if (__OPENCL_C_VERSION__ >= CL_VERSION_2_0)
+#ifndef NAMED_BARRIER_STRUCT_TYPE
+#define NAMED_BARRIER_STRUCT_TYPE
+typedef struct
+{
+    int count;
+    int orig_count;
+    int inc;
+} __namedBarrier;
+#endif // NAMED_BARRIER_STRUCT_TYPE
+typedef __namedBarrier NamedBarrier_t;
+
+local NamedBarrier_t* __attribute__((overloadable)) named_barrier_init(int count);
+
+void __attribute__((overloadable)) work_group_named_barrier(local NamedBarrier_t *barrier, cl_mem_fence_flags flags);
+
+void __attribute__((overloadable)) work_group_named_barrier(local NamedBarrier_t *barrier, cl_mem_fence_flags flags, memory_scope scope);
+#endif // __OPENCL_C_VERSION__ >= CL_VERSION_2_0
