@@ -1,29 +1,12 @@
 
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
+============================= end_copyright_notice ===========================*/
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
 #include "Compiler/CISACodeGen/CheckInstrTypes.hpp"
 #include "Compiler/CISACodeGen/helper.h"
 #include "Compiler/CodeGenPublic.h"
@@ -51,19 +34,18 @@ IGC_INITIALIZE_PASS_END(CheckInstrTypes, PASS_FLAG, PASS_DESCRIPTION, PASS_CFG_O
 
 char CheckInstrTypes::ID = 0;
 
-CheckInstrTypes::CheckInstrTypes(IGC::SInstrTypes* instrList) : FunctionPass(ID), g_InstrTypes(instrList)
+CheckInstrTypes::CheckInstrTypes(IGC::SInstrTypes* instrList, IGCMetrics::IGCMetric* metrics) : FunctionPass(ID), g_InstrTypes(instrList)
 {
     initializeLoopInfoWrapperPassPass(*PassRegistry::getPassRegistry());
     initializeCheckInstrTypesPass(*PassRegistry::getPassRegistry());
 
+    g_metrics = metrics;
     instrList->CorrelatedValuePropagationEnable = false;
-    instrList->hasLoop = false;
     instrList->hasMultipleBB = false;
     instrList->hasCmp = false;
     instrList->hasSwitch = false;
     instrList->hasPhi = false;
     instrList->hasLoadStore = false;
-    instrList->hasCall = false;
     instrList->hasIndirectCall = false;
     instrList->hasInlineAsm = false;
     instrList->hasInlineAsmPointerAccess = false;
@@ -73,7 +55,10 @@ CheckInstrTypes::CheckInstrTypes(IGC::SInstrTypes* instrList) : FunctionPass(ID)
     instrList->hasPointer = false;
     instrList->hasGenericAddressSpacePointers = false;
     instrList->hasLocalLoadStore = false;
-    instrList->hasBufferStore = false;
+    instrList->hasGlobalLoad = false;
+    instrList->hasGlobalStore = false;
+    instrList->hasStorageBufferLoad = false;
+    instrList->hasStorageBufferStore = false;
     instrList->hasSubroutines = false;
     instrList->hasPrimitiveAlloca = false;
     instrList->hasNonPrimitiveAlloca = false;
@@ -83,20 +68,33 @@ CheckInstrTypes::CheckInstrTypes(IGC::SInstrTypes* instrList) : FunctionPass(ID)
     instrList->psHasSideEffect = false;
     instrList->hasDebugInfo = false;
     instrList->hasAtomics = false;
-    instrList->hasBarrier = false;
     instrList->hasDiscard = false;
+    instrList->hasTypedRead = false;
     instrList->hasTypedwrite = false;
     instrList->mayHaveIndirectOperands = false;
+    instrList->mayHaveIndirectResources = false;
     instrList->hasUniformAssumptions = false;
-    instrList->hasWaveIntrinsics = false;
+    instrList->numCall = 0;
+    instrList->numBarrier = 0;
+    instrList->numTypedReadWrite = 0;
+    instrList->numLoadStore = 0;
+    instrList->numAtomics = 0;
+    instrList->numWaveIntrinsics = 0;
+    instrList->numAllInsts = 0;
     instrList->numPsInputs = 0;
     instrList->numSample = 0;
     instrList->numBB = 0;
     instrList->numLoopInsts = 0;
     instrList->numOfLoop = 0;
     instrList->numInsts = 0;
+    instrList->numAllocaInsts = 0;
+    instrList->numGlobalInsts = 0;
+    instrList->numLocalInsts = 0;
     instrList->sampleCmpToDiscardOptimizationPossible = false;
     instrList->sampleCmpToDiscardOptimizationSlot = 0;
+    instrList->hasPullBary = false;
+    instrList->hasDynamicGenericLoadStore = false;  /* may use hasGenericAddressSpacePointers instead */
+    instrList->hasUnmaskedRegion = false;
 }
 
 void CheckInstrTypes::SetLoopFlags(Function& F)
@@ -121,21 +119,45 @@ void CheckInstrTypes::SetLoopFlags(Function& F)
 bool CheckInstrTypes::runOnFunction(Function& F)
 {
     LI = &getAnalysis<LoopInfoWrapperPass>().getLoopInfo();
-    g_InstrTypes->hasLoop |= !(LI->empty());
+    if (g_metrics != nullptr)
+        g_metrics->CollectLoops(LI);
 
     // check if module has debug info
     g_InstrTypes->hasDebugInfo = F.getParent()->getNamedMetadata("llvm.dbg.cu") != nullptr;
 
+    for (auto BI = F.begin(), BE = F.end(); BI != BE; BI++)
+    {
+        g_InstrTypes->numBB++;
+        g_InstrTypes->numAllInsts += BI->size();
+    }
+
     visit(F);
     SetLoopFlags(F);
     return false;
 }
 
+void CheckInstrTypes::checkGlobalLocal(llvm::Instruction& I)
+{
+    BasicBlock* dBB = I.getParent();
+
+    for (auto U : I.users()) {
+        auto UI = dyn_cast<Instruction>(U);
+        BasicBlock* uBB = UI->getParent();
+        if (uBB != dBB)
+        {
+            g_InstrTypes->numGlobalInsts++;
+            return;
+        }
+    }
+    g_InstrTypes->numLocalInsts++;
+}
+
 void CheckInstrTypes::visitInstruction(llvm::Instruction& I)
 {
     if (!llvm::isa<llvm::DbgInfoIntrinsic>(&I))
     {
         g_InstrTypes->numInsts++;
+        checkGlobalLocal(I);
     }
 
     if (I.getOpcode() == Instruction::FRem)
@@ -153,7 +175,8 @@ void CheckInstrTypes::visitInstruction(llvm::Instruction& I)
 void CheckInstrTypes::visitCallInst(CallInst& C)
 {
     g_InstrTypes->numInsts++;
-    g_InstrTypes->hasCall = true;
+    checkGlobalLocal(C);
+    g_InstrTypes->numCall++;
 
     Function* calledFunc = C.getCalledFunction();
 
@@ -223,22 +246,28 @@ void CheckInstrTypes::visitCallInst(CallInst& C)
         case GenISAIntrinsic::GenISA_floatatomicrawA64:
         case GenISAIntrinsic::GenISA_floatatomicstructured:
             g_InstrTypes->hasAtomics = true;
+            g_InstrTypes->numAtomics++;
             break;
         case GenISAIntrinsic::GenISA_discard:
             g_InstrTypes->hasDiscard = true;
             break;
         case GenISAIntrinsic::GenISA_WaveShuffleIndex:
             g_InstrTypes->mayHaveIndirectOperands = true;
-            g_InstrTypes->hasWaveIntrinsics = true;
+            g_InstrTypes->numWaveIntrinsics++;
             break;
         case GenISAIntrinsic::GenISA_threadgroupbarrier:
-            g_InstrTypes->hasBarrier = true;
+            g_InstrTypes->numBarrier++;
             break;
         case GenISAIntrinsic::GenISA_is_uniform:
             g_InstrTypes->hasUniformAssumptions = true;
             break;
+        case GenISAIntrinsic::GenISA_typedread:
+            g_InstrTypes->hasTypedRead = true;
+            g_InstrTypes->numTypedReadWrite++;
+            break;
         case GenISAIntrinsic::GenISA_typedwrite:
             g_InstrTypes->hasTypedwrite = true;
+            g_InstrTypes->numTypedReadWrite++;
             break;
         case GenISAIntrinsic::GenISA_WaveAll:
         case GenISAIntrinsic::GenISA_WaveBallot:
@@ -248,50 +277,98 @@ void CheckInstrTypes::visitCallInst(CallInst& C)
         case GenISAIntrinsic::GenISA_WaveClustered:
         case GenISAIntrinsic::GenISA_QuadPrefix:
         case GenISAIntrinsic::GenISA_simdShuffleDown:
-            g_InstrTypes->hasWaveIntrinsics = true;
+            g_InstrTypes->numWaveIntrinsics++;
             break;
         case GenISAIntrinsic::GenISA_DCL_inputVec:
         case GenISAIntrinsic::GenISA_DCL_ShaderInputVec:
             g_InstrTypes->numPsInputs++;
             break;
+        case GenISAIntrinsic::GenISA_PullSampleIndexBarys:
+        case GenISAIntrinsic::GenISA_PullSnappedBarys:
+        case GenISAIntrinsic::GenISA_PullCentroidBarys:
+            g_InstrTypes->hasPullBary = true;
+            break;
+        case GenISAIntrinsic::GenISA_ldraw_indexed:
+        case GenISAIntrinsic::GenISA_ldrawvector_indexed:
+        {
+            BufferType bufferType = DecodeBufferType(
+                CI->getArgOperand(0)->getType()->getPointerAddressSpace());
+            if (bufferType == UAV || bufferType == BINDLESS)
+            {
+                g_InstrTypes->hasStorageBufferLoad = true;
+            }
+            break;
+        }
+        case GenISAIntrinsic::GenISA_storeraw_indexed:
+        case GenISAIntrinsic::GenISA_storerawvector_indexed:
+        {
+            BufferType bufferType = DecodeBufferType(
+                CI->getArgOperand(0)->getType()->getPointerAddressSpace());
+            if (bufferType == UAV || bufferType == BINDLESS)
+            {
+                g_InstrTypes->hasStorageBufferStore = true;
+            }
+            break;
+        }
         default:
             break;
         }
+
+        Value* resourcePtr = GetBufferOperand(CI);
+        if (resourcePtr == nullptr)
+        {
+            Value* samplerPtr = nullptr;
+            getTextureAndSamplerOperands(CI, resourcePtr, samplerPtr);
+        }
+        if (resourcePtr &&
+            resourcePtr->getType()->isPointerTy() &&
+            isStatefulAddrSpace(resourcePtr->getType()->getPointerAddressSpace()) &&
+            !IsDirectIdx(resourcePtr->getType()->getPointerAddressSpace()))
+        {
+            g_InstrTypes->mayHaveIndirectResources = true;
+        }
     }
 }
 
 void CheckInstrTypes::visitBranchInst(BranchInst& I)
 {
     g_InstrTypes->numInsts++;
+    checkGlobalLocal(I);
 }
 
 void CheckInstrTypes::visitSwitchInst(SwitchInst& I)
 {
     g_InstrTypes->numInsts++;
+    checkGlobalLocal(I);
     g_InstrTypes->hasSwitch = true;
 }
 
 void CheckInstrTypes::visitIndirectBrInst(IndirectBrInst& I)
 {
     g_InstrTypes->numInsts++;
+    checkGlobalLocal(I);
     g_InstrTypes->hasIndirectBranch = true;
 }
 
 void CheckInstrTypes::visitICmpInst(ICmpInst& I)
 {
     g_InstrTypes->numInsts++;
+    checkGlobalLocal(I);
     g_InstrTypes->hasCmp = true;
 }
 
 void CheckInstrTypes::visitFCmpInst(FCmpInst& I)
 {
     g_InstrTypes->numInsts++;
+    checkGlobalLocal(I);
     g_InstrTypes->hasCmp = true;
 }
 
 void CheckInstrTypes::visitAllocaInst(AllocaInst& I)
 {
     g_InstrTypes->numInsts++;
+    checkGlobalLocal(I);
+    g_InstrTypes->numAllocaInsts++;
     if (I.isArrayAllocation() ||
         I.getAllocatedType()->isArrayTy() ||
         I.getAllocatedType()->isStructTy() ||
@@ -319,55 +396,140 @@ void CheckInstrTypes::visitAllocaInst(AllocaInst& I)
 void CheckInstrTypes::visitLoadInst(LoadInst& I)
 {
     g_InstrTypes->numInsts++;
+    checkGlobalLocal(I);
     g_InstrTypes->hasLoadStore = true;
-    if (I.getPointerAddressSpace() == ADDRESS_SPACE_LOCAL)
+    g_InstrTypes->numLoadStore++;
+    uint as = I.getPointerAddressSpace();
+    switch (as)
     {
+    case ADDRESS_SPACE_LOCAL:
         g_InstrTypes->hasLocalLoadStore = true;
-    }
-    if (I.getPointerAddressSpace() == ADDRESS_SPACE_GENERIC)
-    {
+        break;
+    case ADDRESS_SPACE_GENERIC:
         g_InstrTypes->hasGenericAddressSpacePointers = true;
+        g_InstrTypes->hasDynamicGenericLoadStore = true;
+        break;
+    case ADDRESS_SPACE_GLOBAL:
+        g_InstrTypes->hasGlobalLoad = true;
+        break;
+    default:
+    {
+        BufferType bufferType = DecodeBufferType(as);
+        switch (bufferType)
+        {
+        case IGC::UAV:
+        case IGC::BINDLESS:
+        case IGC::SSH_BINDLESS:
+            g_InstrTypes->hasStorageBufferLoad = true;
+            break;
+        case IGC::STATELESS:
+            g_InstrTypes->hasGlobalLoad = true;
+            break;
+        case IGC::CONSTANT_BUFFER:
+        case IGC::RESOURCE:
+        case IGC::SLM:
+        case IGC::POINTER:
+        case IGC::BINDLESS_CONSTANT_BUFFER:
+        case IGC::BINDLESS_TEXTURE:
+        case IGC::SAMPLER:
+        case IGC::BINDLESS_SAMPLER:
+        case IGC::RENDER_TARGET:
+        case IGC::STATELESS_READONLY:
+        case IGC::STATELESS_A32:
+        case IGC::SSH_BINDLESS_CONSTANT_BUFFER:
+        case IGC::SSH_BINDLESS_TEXTURE:
+        case IGC::BUFFER_TYPE_UNKNOWN:
+            break;
+        }
+        if (isStatefulAddrSpace(as) && !IsDirectIdx(as))
+        {
+            g_InstrTypes->mayHaveIndirectResources = true;
+        }
+        break;
+    }
     }
 }
 
 void CheckInstrTypes::visitStoreInst(StoreInst& I)
 {
     g_InstrTypes->numInsts++;
+    checkGlobalLocal(I);
     g_InstrTypes->hasLoadStore = true;
+    g_InstrTypes->numLoadStore++;
+
     uint as = I.getPointerAddressSpace();
     if (as != ADDRESS_SPACE_PRIVATE)
     {
         g_InstrTypes->psHasSideEffect = true;
     }
-    if (as == ADDRESS_SPACE_LOCAL)
+    switch (as)
     {
+    case ADDRESS_SPACE_LOCAL:
         g_InstrTypes->hasLocalLoadStore = true;
-    }
-    if (as == ADDRESS_SPACE_GENERIC)
-    {
+        break;
+    case ADDRESS_SPACE_GENERIC:
         g_InstrTypes->hasGenericAddressSpacePointers = true;
-    }
-    if (as == ADDRESS_SPACE_GLOBAL)
+        g_InstrTypes->hasDynamicGenericLoadStore = true;
+        break;
+    case ADDRESS_SPACE_GLOBAL:
+        g_InstrTypes->hasGlobalStore = true;
+        break;
+    default:
     {
-        g_InstrTypes->hasBufferStore = true;
+        BufferType bufferType = DecodeBufferType(as);
+        switch (bufferType)
+        {
+        case IGC::UAV:
+        case IGC::BINDLESS:
+        case IGC::SSH_BINDLESS:
+            g_InstrTypes->hasStorageBufferStore = true;
+            break;
+        case IGC::STATELESS:
+            g_InstrTypes->hasGlobalStore = true;
+            break;
+        case IGC::CONSTANT_BUFFER:
+        case IGC::RESOURCE:
+        case IGC::SLM:
+        case IGC::POINTER:
+        case IGC::BINDLESS_CONSTANT_BUFFER:
+        case IGC::BINDLESS_TEXTURE:
+        case IGC::SAMPLER:
+        case IGC::BINDLESS_SAMPLER:
+        case IGC::RENDER_TARGET:
+        case IGC::STATELESS_READONLY:
+        case IGC::STATELESS_A32:
+        case IGC::SSH_BINDLESS_CONSTANT_BUFFER:
+        case IGC::SSH_BINDLESS_TEXTURE:
+        case IGC::BUFFER_TYPE_UNKNOWN:
+            break;
+        }
+        if (isStatefulAddrSpace(as) && !IsDirectIdx(as))
+        {
+            g_InstrTypes->mayHaveIndirectResources = true;
+        }
+        break;
+    }
     }
 }
 
 void CheckInstrTypes::visitPHINode(PHINode& PN)
 {
     g_InstrTypes->numInsts++;
+    checkGlobalLocal(PN);
     g_InstrTypes->hasPhi = true;
 }
 
 void CheckInstrTypes::visitSelectInst(SelectInst& I)
 {
     g_InstrTypes->numInsts++;
+    checkGlobalLocal(I);
     g_InstrTypes->hasSel = true;
 }
 
 void CheckInstrTypes::visitGetElementPtrInst(llvm::GetElementPtrInst& I)
 {
     g_InstrTypes->numInsts++;
+    checkGlobalLocal(I);
     if (I.getPointerAddressSpace() == ADDRESS_SPACE_GENERIC)
     {
         g_InstrTypes->hasGenericAddressSpacePointers = true;
