@@ -1,28 +1,10 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
+============================= end_copyright_notice ===========================*/
 
 //===-------- CoalescingEngine.cpp - Pass and analysis for coalescing-------===//
 //
@@ -223,8 +205,8 @@ namespace IGC
         if (SampleIntrinsic * sample = dyn_cast<SampleIntrinsic>(inst))
         {
             Value* sampler = sample->getSamplerValue();
-            uint bufferIndex;
-            bool directIndexing;
+            uint bufferIndex = 0;
+            bool directIndexing = false;
             if (sampler->getType()->isPointerTy())
             {
                 BufferType bufType = DecodeAS4GFXResource(
@@ -471,7 +453,47 @@ namespace IGC
         bool interferes = false;
         if (ccTuple->HasNonHomogeneousElements())
         {
-            if (offsetDiff == 0 && ccTuple->GetNumElements() == numOperands)
+            // Finding a supremum instruction for a homogeneous part is implemented
+            // only for render target write instructions (RTWritIntrinsic).
+            // An only other possible combination for non-homogeneous instructions comprises
+            // RTWritIntrinsic and RTDualBlendSourceIntrinsic.
+            // In some cases there is an opportunity to coalesce their payloads but there exists
+            // a danger that they are compiled in different SIMD modes so there is a safe assumption
+            // that they cannot be coalesced.
+            // A comparison of the payload of RTWritIntrinsic and RTDualBlendSourceIntrinsic:
+            // +----------------------------+----------------------------+
+            // | RTWritIntrinsic            | RTDualBlendSourceIntrinsic |
+            // +----------------------------+----------------------------+
+            // | src0 Alpha (optional)      | src0 Alpha (unavailable)*  |
+            // +----------------------------+----------------------------+
+            // | output mask (optional)     | output mask (optional)     |
+            // +----------------------------+----------------------------+
+            // | -                          | R0 channel                 |
+            // +----------------------------+----------------------------+
+            // | -                          | G0 channel                 |
+            // +----------------------------+----------------------------+
+            // | -                          | B0 channel                 |
+            // +----------------------------+----------------------------+
+            // | -                          | A0 channel                 |
+            // +----------------------------+----------------------------+
+            // | R0 channel                 | R1 channel                 |
+            // +----------------------------+----------------------------+
+            // | G0 channel                 | G1 channel                 |
+            // +----------------------------+----------------------------+
+            // | B0 channel                 | B1 channel                 |
+            // +----------------------------+----------------------------+
+            // | A0 channel                 | A1 channel                 |
+            // +----------------------------+----------------------------+
+            // | Depth (optional)           | Depth (optional)           |
+            // +----------------------------+----------------------------+
+            // | Stencil (optional)         | Stencil (optional)         |
+            // +----------------------------+----------------------------+
+            // All optional fields except for src0 Alpha are consistent for both instruction called in a fragment shader.
+            // * RTDualBlendSourceIntrinsic doesn't have such an argument but it is defined in its payload.
+            if (offsetDiff == 0 &&
+                ccTuple->GetNumElements() == numOperands &&
+                llvm::isa<llvm::RTWritIntrinsic>(ccTuple->GetRoot()) &&
+                llvm::isa<llvm::RTWritIntrinsic>(tupleGeneratingInstruction))
             {
                 if (m_PayloadMapping.HasNonHomogeneousPayloadElements(tupleGeneratingInstruction))
                 {
@@ -1118,11 +1140,13 @@ namespace IGC
         SIMDMode simdMode,
         const DataLayout* pDL,
         llvm::Instruction* inst,
-        int& payloadOffsetInBytes
-    )
+        int& payloadOffsetInBytes)
     {
         SetCurrentPart(inst, 0);
         const uint numOperands = m_PayloadMapping.GetNumPayloadElements(inst);
+        const uint grfSize = outProgram->GetContext()->platform.getGRFSize();
+        const uint numSubVarsPerOperand = numLanes(simdMode) / (grfSize / 4);
+        IGC_ASSERT(numSubVarsPerOperand == 1 || numSubVarsPerOperand == 2);
         CoalescingEngine::CCTuple* ccTuple = nullptr;
         int payloadToCCTupleRelativeOffset = 0;
         Value* dummyValPtr = nullptr;
@@ -1168,7 +1192,8 @@ namespace IGC
 
                 CVariable* dataElement = outProgram->GetSymbol(val);
                 //FIXME: need to do additional checks for size
-                encoder->SetDstSubVar(payloadToCCTupleRelativeOffset + index);
+                uint subVar = payloadToCCTupleRelativeOffset + index * numSubVarsPerOperand;
+                encoder->SetDstSubVar(subVar);
                 payload = outProgram->GetCCTupleToVariableMapping(ccTuple);
 
                 if (touchedValuesSet.count(val)) {
@@ -1208,14 +1233,15 @@ namespace IGC
         }
         else
         {
-            payload = outProgram->GetNewVariable(
-                numOperands * numLanes(simdMode), ISA_TYPE_F, outProgram->GetContext()->platform.getGRFSize() == 64 ? EALIGN_32WORD : EALIGN_HWORD, CName::NONE);
+            payload = outProgram->GetNewVariable(numOperands * numLanes(simdMode), ISA_TYPE_F,
+                outProgram->GetContext()->platform.getGRFSize() == 64 ? EALIGN_32WORD : EALIGN_HWORD,
+                "CEExplicitPayload");
 
             for (uint i = 0; i < numOperands; i++)
             {
                 Value* val = m_PayloadMapping.GetPayloadElementToValueMapping(inst, i);
                 CVariable* data = outProgram->GetSymbol(val);
-                encoder->SetDstSubVar(i);
+                encoder->SetDstSubVar(i * numSubVarsPerOperand);
                 encoder->Copy(payload, data);
                 encoder->Push();
             }
@@ -1224,6 +1250,83 @@ namespace IGC
         return payload;
     }
 
+    // Prepares payload for the uniform URB Write messages that are used in
+    // mesh and task shader stages.
+    // On DG2 platform a uniform URB write is a simd1 message that writes data
+    // from the fist DWORDs of each GRF of payload, e.g.:
+    //   (W)     mov (1|M0)               r16.0<1>:ud   0x0:ud
+    //   (W)     mov (1|M0)               r17.0<1>:f   -1.0:f
+    //   (W)     mov (1|M0)               r18.0<1>:f    0.0:f
+    //   (W)     mov (1|M0)               r19.0<1>:f    1.0:f
+    //   (W)     send.urb (1|M0)          null     r2   r16:4  0x0  0x020827F7
+    CVariable* CoalescingEngine::PrepareUniformUrbWritePayload(
+        CShader* shader,
+        CEncoder* encoder,
+        llvm::GenIntrinsicInst* inst)
+    {
+        IGC_ASSERT(inst->getIntrinsicID() == GenISAIntrinsic::GenISA_URBWrite);
+        CVariable* payload = nullptr;
+        const uint numOperands = m_PayloadMapping.GetNumPayloadElements(inst);
+        const uint grfSize = shader->GetContext()->platform.getGRFSize();
+        {
+            // Payload spans multiple GRFs, only the first DWORD of each GRF
+            // will be populated with (uniform) data.
+            const uint elemsPerGrf = grfSize / CVariable::GetCISADataTypeSize(ISA_TYPE_F);
+            payload = shader->GetNewVariable(
+                numOperands * elemsPerGrf, ISA_TYPE_F,
+                (grfSize == 64 ? EALIGN_32WORD : EALIGN_HWORD),
+                "CEExplicitPayload");
+        }
+        for (uint i = 0; i < numOperands; i++)
+        {
+            Value* val = m_PayloadMapping.GetPayloadElementToValueMapping(inst, i);
+            CVariable* data = shader->GetSymbol(val);
+            IGC_ASSERT(isUniform(val) && data->IsUniform());
+            encoder->SetNoMask();
+            encoder->SetSimdSize(SIMDMode::SIMD1);
+            {
+                encoder->SetDstSubVar(i);
+            }
+            encoder->Copy(payload, data);
+        }
+        return payload;
+    }
+
+    // Returns URB write payload for a split simd8 message in a simd16 mesh/task
+    // shader.
+    // TODO: This method does not use the CoalescingEngine currently and mearly
+    // copies payload to a newly created variable.
+    CVariable* CoalescingEngine::PrepareSplitUrbWritePayload(
+        CShader* outProgram,
+        CEncoder* encoder,
+        SIMDMode  simdMode, ///< must be SIMD16
+        uint32_t  splitPartNo, ///< can be either 0 - lower quarter, or 1 upper quarter
+        llvm::Instruction* inst)
+    {
+        SetCurrentPart(inst, 0);
+        IGC_ASSERT(outProgram->m_SIMDSize == SIMDMode::SIMD16);
+        IGC_ASSERT(splitPartNo < 2);
+        const GenIntrinsicInst* const intrinsicInst = dyn_cast<GenIntrinsicInst>(inst);
+        IGC_ASSERT(nullptr != intrinsicInst);
+        IGC_ASSERT(intrinsicInst->getIntrinsicID() == GenISAIntrinsic::GenISA_URBWrite);
+
+        const uint numOperands = m_PayloadMapping.GetNumPayloadElements(inst);
+        CVariable* payload =
+            outProgram->GetNewVariable(numOperands * numLanes(SIMDMode::SIMD8), ISA_TYPE_F, outProgram->GetContext()->platform.getGRFSize() == 64 ? EALIGN_32WORD : EALIGN_HWORD, CName::NONE);
+
+        for (uint i = 0; i < numOperands; i++)
+        {
+            Value* val = m_PayloadMapping.GetPayloadElementToValueMapping(inst, i);
+            CVariable* data = outProgram->GetSymbol(val);
+            encoder->SetSimdSize(SIMDMode::SIMD8);
+            encoder->SetMask((splitPartNo == 0) ? EMASK_Q1 : EMASK_Q2);
+            encoder->SetSrcSubVar(0, data->IsUniform() ? 0 : splitPartNo);
+            encoder->SetDstSubVar(i);
+            encoder->Copy(payload, data);
+            encoder->Push();
+        }
+        return payload;
+    }
 
     CoalescingEngine::ElementNode*
         CoalescingEngine::ElementNode::getLeader() {
