@@ -40,11 +40,15 @@ namespace IGC
         bool        HasSLM() const { return m_hasSLM; }
         bool        HasFullDispatchMask() override;
 
+        bool        passNOSInlineData() override;
+        bool        loadThreadPayload() override;
+
     protected:
         uint                   m_numberOfUntypedAccess;
         uint                   m_numberOfTypedAccess;
         uint                   m_num1DAccesses;
         uint                   m_num2DAccesses;
+        uint                   m_numSLMAccesses;
 
         bool                   m_dispatchAlongY;
         bool                   m_disableMidThreadPreemption;
