@@ -68,6 +68,7 @@ See LICENSE.TXT for details.
 #include "llvmWrapper/IR/Instructions.h"
 #include <llvm/IR/InstIterator.h>
 #include <llvm/IR/InlineAsm.h>
+#include <llvmWrapper/IR/DerivedTypes.h>
 #include "common/LLVMWarningsPop.hpp"
 #include <algorithm>
 #include "Probe/Assertion.h"
@@ -312,7 +313,8 @@ bool DeSSA::runOnFunction(Function& MF)
 {
     m_F = &MF;
     CurrColor = 0;
-    MetaDataUtils* pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
+    MetaDataUtils* pMdUtils = nullptr;
+    pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
     if (pMdUtils->findFunctionsInfoItem(&MF) == pMdUtils->end_FunctionsInfo())
     {
         return false;
@@ -424,10 +426,10 @@ bool DeSSA::runOnFunction(Function& MF)
     //  so it is likely for the algorithm to coalesce the phi's dst and the
     //  other src that is used in the loop, and therefore remove mov instrutions
     //  in the loop.
-  //
-  //  Note that isolating a value introduce additional copy, thus a threshold
-  //  is used here as a heuristic to try to make sure that a benefit is more
-  //  than the cost.
+    //
+    //  Note that isolating a value introduce additional copy, thus a threshold
+    //  is used here as a heuristic to try to make sure that a benefit is more
+    //  than the cost.
     enum { PHI_SRC_USE_THRESHOLD = 3 };  // arbitrary number
     DenseMap<Value*, int> PHILoopPreHeaderSrcs;
 
@@ -1020,7 +1022,7 @@ int getPartialWriteSource(Value *Inst)
             SmallVector<StringRef, 8> constraints;
             constraintStr.split(constraints, ',');
             for (int i = 0; i < (int)constraints.size(); i++) {
-                unsigned destID;
+                unsigned destID = 0;
                 if (constraints[i].getAsInteger(10, destID) == 0) {
                     // constraint-string indicates that source(i-1) and
                     // destination should be the same vISA variable
@@ -1302,7 +1304,7 @@ int DeSSA::checkInsertElementAlias(
     //
     // If found, return the actual vector size;
     // otherwise, return 0.
-    VectorType* VTy = cast<VectorType>(IEI->getType());
+    IGCLLVM::FixedVectorType* VTy = cast<IGCLLVM::FixedVectorType>(IEI->getType());
     IGC_ASSERT(nullptr != VTy);
     int nelts = (int)VTy->getNumElements();
     AllIEIs.resize(nelts, nullptr);
@@ -1491,4 +1493,4 @@ bool DeSSA::alignInterfere(e_alignment a1, e_alignment a2)
         return true;
     }
     return false;
-}
\ No newline at end of file
+}
