@@ -1,28 +1,10 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
+============================= end_copyright_notice ===========================*/
 
 #include "Compiler/CISACodeGen/helper.h"
 #include "Compiler/CISACodeGen/ShaderUnits.hpp"
@@ -133,7 +115,8 @@ namespace IGC
 
     bool DomainShaderLowering::runOnFunction(llvm::Function& F)
     {
-        MetaDataUtils* pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
+        MetaDataUtils* pMdUtils = nullptr;
+        pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
         if (!isEntryFunc(pMdUtils, &F))
         {
             return false;
@@ -153,7 +136,7 @@ namespace IGC
         Value* zero = builder.getInt32(0);
 
         llvm::GlobalVariable* pGlobal = m_pModule->getGlobalVariable("MaxNumOfInputSignatureEntries");
-        auto pCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
+        IGC::CodeGenContext* pCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
         uint maxInputSignatureCount =
             int_cast<uint>(llvm::cast<llvm::ConstantInt>(pGlobal->getInitializer())->getZExtValue());
         pGlobal = m_pModule->getGlobalVariable("MaxNumOfPatchConstantSignatureEntries");
@@ -368,7 +351,8 @@ namespace IGC
 
     void DomainShaderLowering::CalculateVertexHeaderSize(Function& F)
     {
-        IGC::CodeGenContext* context = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
+        IGC::CodeGenContext* context = nullptr;
+        context = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
         if (context->getModuleMetaData()->URBInfo.hasVertexHeader)
         {
             if (context->getModuleMetaData()->URBInfo.has64BVertexHeaderOutput)
@@ -448,7 +432,7 @@ namespace IGC
         for (unsigned int i = 0; i < 4; i++)
         {
             Value* index = builder.getInt32(currentElementIndex + i);
-            auto pCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
+            IGC::CodeGenContext* pCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
             const e_interpolation interpolant =
                 DSDualPatchEnabled(pCtx) ?
                 EINTERPOLATION_UNDEFINED : EINTERPOLATION_CONSTANT;
