@@ -52,6 +52,9 @@ namespace IGC
         /// The driver supports using stateless space to store the private memory
         /// Driver must be able to use at least one way to store the private memory: either "scratch space" or "stateless space"
         /// and by default, driver only supports one of them.
+        /// NOTE: This method should only be used for XeHP and above to avoid changes to legacy GENs
+        ///         And this is the only place telling if one API supports statelesspvtmem or not.
+        ///         If this API doesn't support statelesspvtmem, IGC will error out if pvtmemusage > 256k in PrivateMemoryResolution
         virtual bool supportsStatelessSpacePrivateMemory() const { return !supportsScratchSpacePrivateMemory(); }
 
         /// The driver requires to align each entry (a workgroup item) of private scratch memory in a stateless
@@ -90,6 +93,9 @@ namespace IGC
         /// The driver doesn't clear the vertex header so it needs to be done in the compiler
         virtual bool NeedClearVertexHeader() const { return false; }
 
+        /// Do Fastest Stage1 only for 3D
+        virtual bool SupportFastestStage1() const { return true; }
+
         /// do code sinking before CFGSimplification, helps some workloads
         virtual bool CodeSinkingBeforeCFGSimplification() const { return false; }
 
@@ -155,6 +161,9 @@ namespace IGC
         /// Turn on vISA pre-RA scheduler. Not tested on all APIs
         virtual bool enableVISAPreRAScheduler() const { return false; }
 
+        /// Turn on vISA pre-RA scheduler for retry
+        virtual bool enableVISAPreRASchedulerForRetry() const { return false; }
+
         /// Configure vISA pre-RA scheduler. Not tested on all APIs
         virtual unsigned getVISAPreRASchedulerCtrl() const { return 4; }
 
@@ -168,7 +177,7 @@ namespace IGC
         virtual bool AlwaysEnableSimd32() const { return false; }
 
         /// Driver supports promoting buffers to bindful
-        virtual bool SupportsStatelessToStatefullBufferTransformation() const { return false; }
+        virtual bool SupportsStatelessToStatefulBufferTransformation() const { return false; }
 
         /// Need emulation of 64bits type for HW not supporting it natively
         virtual bool Enable64BitEmu() const { return false; }
@@ -233,6 +242,9 @@ namespace IGC
         /// WA for failures with HS with push constants
         virtual bool WaDisablePushConstantsForHS() const { return false; }
 
+        /// WA for failures with push constants and no pushed attributes
+        virtual bool WaDisablePushConstantsWithNoPushedAttributes() const { return false; }
+
         /// Check if we have to worry about stack overflow while recursing in loop analysis
         virtual bool HasSmallStack() const { return false; }
 
@@ -298,8 +310,54 @@ namespace IGC
         // Determines whether the PAYLOAD_HEADER implicit arg must be present
         virtual bool RequirePayloadHeader() const { return true; }
 
+        virtual bool supportsAutoGRFSelection() const { return autoGRFSelection; }
+        virtual void setAutoGRFSelection(bool value) { autoGRFSelection = value; }
+        virtual bool UseScratchSpaceForATSPlus() const { return false; }
+        /// Enables HWGenerateThreadID from API level. To help debug, we must enable it from both API level AND IGC Core level.
+        virtual bool SupportHWGenerateTID() const { return false; }
+        // Enables the use of simple push constants when on platforms with local (device) memory
+        virtual bool supportsSimplePushForLocalMem() const { return false; }
+        // disable dual8 with discard
+        virtual bool DisableDual8WithDiscard() const { return false; }
+        // support force routing to HDC and LCS caching options
+        virtual bool SupportForceRouteAndCache() const { return false; }
+        // If enabled, IGC must provide the corresponding UMD info on how much
+        // memory to allocate for the RTGlobals + global root signature.
+        virtual bool supportsExpandedRTGlobals() const { return false; }
+        // If enabled, UMD must support setting up threadgroup according to
+        // RayTracingCustomTileXDim* and RayTracingCustomTileYDim*. If you want
+        // to experiment with non-power-of-2 x dimensions, you also have to
+        // support filling local IDs in the indirect state for the shader to
+        // read (not required otherwise).
+        virtual bool supportsRaytracingTiling() const { return false; }
+        // Enables the use of scratch space in raytracing shaders when possible
+        virtual bool supportsRTScratchSpace() const { return false; }
+        // Enables Raytracing printf
+        virtual bool SupportsRTPrintf() const { return false; }
+        // enables stateful accesses to the RTAsyncStack, SWHotZone, SWStack and RTSyncStack
+        virtual bool supportsRaytracingStatefulAccesses() const { return false; }
+        // To support this, the compiler output must be able to express a
+        // raygen shader identifier with continuation KSPs after it.
+        virtual bool supportsRaytracingContinuationPromotion() const { return false; }
+
+        // Enable LSC on DG2 for the following:
+        //   GenISAIntrinsic::GenISA_ldraw_indexed
+        //   GenISAIntrinsic::GenISA_ldrawvector_indexed
+        //   GenISAIntrinsic::GenISA_storeraw_indexed
+        //   GenISAIntrinsic::GenISA_storerawvector_indexed
+        // todo: remove when all APIs enable LSC
+        virtual bool EnableLSCForLdRawAndStoreRawOnDG2() const { return false; }
+        // Check SLM limit on compute shader to select SIMD8
+        virtual bool SupportCSSLMLimit() const { return false; }
 
 
+        // When dual-source blending is enabled, enable sending the
+        // single-source RTW message (with data for the second color) after the
+        // dual-source blending RTW message. The second message must be send
+        // when the state of dual-source blending is not known at compile time.
+        virtual bool sendSingleSourceRTWAfterDualSourceRTW() const { return true; }
+protected:
+    bool autoGRFSelection = false;
     };
 
 }//namespace IGC
