@@ -78,6 +78,8 @@ namespace {
 
         SmallPtrSet<Instruction*, 32> DeadInsts;
 
+        SmallPtrSet<Instruction*, 16> Int64Insts;
+        bool doNotDeleteInstr = false;
     public:
 
         static char ID;
@@ -137,7 +139,17 @@ namespace {
 
         void copyKnownMetadata(Instruction* NewI, Instruction* OldI) const
         {
-            // Nothing needed yet
+            unsigned LscCacheCtrlID =
+                OldI->getContext().getMDKindID("lsc.cache.ctrl");
+            SmallVector<std::pair<unsigned, MDNode*>, 8> MD;
+            OldI->getAllMetadata(MD);
+            for (const auto& MDPair : MD)
+            {
+                unsigned ID = MDPair.first;
+                MDNode* N = MDPair.second;
+                if (ID == LscCacheCtrlID)
+                    NewI->setMetadata(ID, N);
+            }
         }
 
         void dupMemoryAttribute(LoadInst* NewLD, LoadInst* RefLD, unsigned Off) const {
@@ -165,6 +177,9 @@ namespace {
         bool expandInsts(Function& F);
         bool populatePHIs(Function& F);
         bool removeDeadInsts();
+        bool hasOpcodeInt64Support(unsigned int opcode) const;
+        void addInstToInt64InstrList(Instruction* inst) { Int64Insts.insert(inst); }
+        bool eraseInstrFromInt64InstrList(Instruction* inst) { return Int64Insts.erase(inst); }
     };
 
     class InstExpander : public InstVisitor<InstExpander, bool> {
@@ -246,6 +261,8 @@ namespace {
         bool visitLandingPad(LandingPadInst&);
 
         Value* convertUIToFP32(Type* DstTy, Value* Lo, Value* Hi, Instruction* Pos);
+        bool hasHWSupport(BinaryOperator&);
+        bool needsLoHiSplitting(Instruction&);
     };
 
     class Preprocessor {
@@ -456,7 +473,8 @@ IGC_INITIALIZE_PASS_END(Emu64Ops, PASS_FLAG, PASS_DESC, PASS_CFG_ONLY, PASS_ANAL
 
 bool Emu64Ops::runOnFunction(Function& F) {
     // Skip non-kernel function.
-    MetaDataUtils* MDU = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
+    MetaDataUtils* MDU = nullptr;
+    MDU = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
     IGC_ASSERT(nullptr != MDU);
     auto FII = MDU->findFunctionsInfoItem(&F);
     if (FII == MDU->end_FunctionsInfo())
@@ -478,6 +496,7 @@ bool Emu64Ops::runOnFunction(Function& F) {
     ValueMap.clear();
     Arg64Casts.clear();
     DeadInsts.clear();
+    Int64Insts.clear();
 
     bool Changed = false;
     Changed |= ThePreprocessor.preprocess(F);
@@ -492,7 +511,7 @@ bool Emu64Ops::runOnFunction(Function& F) {
 
 ValuePair Emu64Ops::getExpandedValues(Value* V) {
     ValueMapTy::iterator VMI;
-    bool New;
+    bool New = false;
     std::tie(VMI, New) = ValueMap.insert(std::make_pair(V, ValuePair()));
     if (!New)
         return VMI->second;
@@ -582,15 +601,19 @@ bool Emu64Ops::expandInsts(Function& F) {
     for (auto& BB : RPOT) {
         for (auto BI = BB->begin(), BE = BB->end(); BI != BE; /*EMPTY*/) {
             Instruction* I = &(*BI++);
-
+            doNotDeleteInstr = false;
+            CGC->metrics.StatBeginEmuFunc(I);
             bool LocalChanged = Expander->expand(I);
             Changed |= LocalChanged;
 
             if (LocalChanged) {
+                CGC->metrics.StatEndEmuFunc(I);
                 BI = std::next(BasicBlock::iterator(I));
                 BE = I->getParent()->end();
                 DeadInsts.insert(I);
             }
+            if (doNotDeleteInstr)
+                BE = I->getParent()->end();
         }
     }
 
@@ -633,6 +656,15 @@ bool Emu64Ops::removeDeadInsts() {
     return Changed;
 }
 
+bool Emu64Ops::hasOpcodeInt64Support(unsigned int opcode) const
+{
+    // PVC-B0 only
+    return ((opcode == llvm::Instruction::Shl) ||
+            (opcode == llvm::Instruction::LShr) ||
+            (opcode == llvm::Instruction::AShr) ||
+            (CGC->platform.hasQWAddSupport() && (opcode == llvm::Instruction::Add)));
+}
+
 bool InstExpander::expand(Instruction* I) {
     IRB->SetInsertPoint(I);
 
@@ -674,6 +706,11 @@ bool InstExpander::visitAdd(BinaryOperator& BinOp) {
     if (!Emu->isInt64(&BinOp))
         return false;
 
+    if (Emu->CGC->platform.hasQWAddSupport()) {
+        if (hasHWSupport(BinOp))
+            return false;
+    }
+
     Value* L0 = nullptr, * H0 = nullptr;
     std::tie(L0, H0) = Emu->getExpandedValues(BinOp.getOperand(0));
     Value* L1 = nullptr, * H1 = nullptr;
@@ -761,6 +798,9 @@ bool InstExpander::visitShl(BinaryOperator& BinOp) {
     if (!Emu->isInt64(&BinOp))
         return false;
 
+    if (hasHWSupport(BinOp))
+        return false;
+
     Value* Lo = nullptr, * Hi = nullptr;
     std::tie(Lo, Hi) = Emu->getExpandedValues(BinOp.getOperand(0));
     Value* ShAmt = nullptr;
@@ -769,10 +809,11 @@ bool InstExpander::visitShl(BinaryOperator& BinOp) {
     BasicBlock* OldBB = BinOp.getParent();
     BasicBlock* InnerTBB = nullptr;
     BasicBlock* InnerFBB = nullptr;
-    Value* InnerResLo = nullptr;
-    Value* InnerResHi = nullptr;
+    PHINode* InnerResLo = nullptr;
+    PHINode* InnerResHi = nullptr;
     Value* ResLo = nullptr;
     Value* ResHi = nullptr;
+    DebugLoc BinOpDebugLoc = BinOp.getDebugLoc();
 
     Value* Cond = nullptr;
 
@@ -786,15 +827,19 @@ bool InstExpander::visitShl(BinaryOperator& BinOp) {
         Value* NE = IRB->CreateICmpNE(ShAmt, Constant::getNullValue(ShAmt->getType()));
         BasicBlock* JointBB = OldBB->splitBasicBlock(&BinOp);
         ResLo = PHINode::Create(IRB->getInt32Ty(), 2, ".shl.outer.merge.lo", &BinOp);
+        cast<Instruction>(ResLo)->setDebugLoc(BinOpDebugLoc);
         ResHi = PHINode::Create(IRB->getInt32Ty(), 2, ".shl.outer.merge.hi", &BinOp);
+        cast<Instruction>(ResHi)->setDebugLoc(BinOpDebugLoc);
 
         BasicBlock* TrueBB = BasicBlock::Create(*Emu->getContext(),
             ".shl.outer.true.branch");
         TrueBB->insertInto(Emu->getFunction(), JointBB);
         Instruction* TrueJmp = BranchInst::Create(JointBB, TrueBB);
+        TrueJmp->setDebugLoc(BinOpDebugLoc);
 
         OldBB->getTerminator()->eraseFromParent();
-        BranchInst::Create(TrueBB, JointBB, NE, OldBB);
+        Instruction* TempOldBBBranchInst = BranchInst::Create(TrueBB, JointBB, NE, OldBB);
+        TempOldBBBranchInst->setDebugLoc(BinOpDebugLoc);
 
         // Create the inner branch.
         IRB->SetInsertPoint(&(*TrueBB->begin()));
@@ -804,18 +849,23 @@ bool InstExpander::visitShl(BinaryOperator& BinOp) {
         // than 32 (true branch) or otherwise (false branch).
         BasicBlock* InnerJBB = TrueBB->splitBasicBlock(TrueJmp);
         InnerResLo = PHINode::Create(IRB->getInt32Ty(), 2, ".shl.merge.inner.lo", TrueJmp);
+        InnerResLo->setDebugLoc(BinOpDebugLoc);
         InnerResHi = PHINode::Create(IRB->getInt32Ty(), 2, ".shl.merge.inner.hi", TrueJmp);
+        InnerResHi->setDebugLoc(BinOpDebugLoc);
 
         InnerTBB = BasicBlock::Create(*Emu->getContext(), ".shl.inner.true.branch");
         InnerTBB->insertInto(Emu->getFunction(), InnerJBB);
-        BranchInst::Create(InnerJBB, InnerTBB);
+        Instruction* TempInnerTBBBranchInst = BranchInst::Create(InnerJBB, InnerTBB);
+        TempInnerTBBBranchInst->setDebugLoc(BinOpDebugLoc);
 
         InnerFBB = BasicBlock::Create(*Emu->getContext(), ".shl.inner.false.branch");
         InnerFBB->insertInto(Emu->getFunction(), InnerJBB);
-        BranchInst::Create(InnerJBB, InnerFBB);
+        Instruction* TempInnerFBBBranchInst = BranchInst::Create(InnerJBB, InnerFBB);
+        TempInnerFBBBranchInst->setDebugLoc(BinOpDebugLoc);
 
         TrueBB->getTerminator()->eraseFromParent();
-        BranchInst::Create(InnerTBB, InnerFBB, Cond, TrueBB);
+        Instruction* TempTrueBBBranchInst = BranchInst::Create(InnerTBB, InnerFBB, Cond, TrueBB);
+        TempTrueBBBranchInst->setDebugLoc(BinOpDebugLoc);
 
         // The result is the same as the source if ShAmt is 0, i.e. NE is
         // false.
@@ -840,8 +890,8 @@ bool InstExpander::visitShl(BinaryOperator& BinOp) {
         if (InnerTBB) {
             IGC_ASSERT(isa<PHINode>(InnerResLo));
             IGC_ASSERT(isa<PHINode>(InnerResHi));
-            cast<PHINode>(InnerResLo)->addIncoming(L, InnerTBB);
-            cast<PHINode>(InnerResHi)->addIncoming(H, InnerTBB);
+            InnerResLo->addIncoming(L, InnerTBB);
+            InnerResHi->addIncoming(H, InnerTBB);
         }
         else {
             ResLo = L;
@@ -859,8 +909,8 @@ bool InstExpander::visitShl(BinaryOperator& BinOp) {
         if (InnerFBB) {
             IGC_ASSERT(isa<PHINode>(InnerResLo));
             IGC_ASSERT(isa<PHINode>(InnerResHi));
-            cast<PHINode>(InnerResLo)->addIncoming(L, InnerFBB);
-            cast<PHINode>(InnerResHi)->addIncoming(H, InnerFBB);
+            InnerResLo->addIncoming(L, InnerFBB);
+            InnerResHi->addIncoming(H, InnerFBB);
         }
         else {
             ResLo = L;
@@ -877,6 +927,9 @@ bool InstExpander::visitLShr(BinaryOperator& BinOp) {
     if (!Emu->isInt64(&BinOp))
         return false;
 
+    if (hasHWSupport(BinOp))
+        return false;
+
     Value* Lo = nullptr, * Hi = nullptr;
     std::tie(Lo, Hi) = Emu->getExpandedValues(BinOp.getOperand(0));
     Value* ShAmt = nullptr;
@@ -885,10 +938,11 @@ bool InstExpander::visitLShr(BinaryOperator& BinOp) {
     BasicBlock* OldBB = BinOp.getParent();
     BasicBlock* InnerTBB = nullptr;
     BasicBlock* InnerFBB = nullptr;
-    Value* InnerResLo = nullptr;
-    Value* InnerResHi = nullptr;
+    PHINode* InnerResLo = nullptr;
+    PHINode* InnerResHi = nullptr;
     Value* ResLo = nullptr;
     Value* ResHi = nullptr;
+    DebugLoc BinOpDebugLoc = BinOp.getDebugLoc();
 
     Value* Cond = nullptr;
 
@@ -902,15 +956,19 @@ bool InstExpander::visitLShr(BinaryOperator& BinOp) {
         Value* NE = IRB->CreateICmpNE(ShAmt, Constant::getNullValue(ShAmt->getType()));
         BasicBlock* JointBB = OldBB->splitBasicBlock(&BinOp);
         ResLo = PHINode::Create(IRB->getInt32Ty(), 2, ".lshr.outer.merge.lo", &BinOp);
+        cast<Instruction>(ResLo)->setDebugLoc(BinOpDebugLoc);
         ResHi = PHINode::Create(IRB->getInt32Ty(), 2, ".lshr.outer.merge.hi", &BinOp);
+        cast<Instruction>(ResHi)->setDebugLoc(BinOpDebugLoc);
 
         BasicBlock* TrueBB = BasicBlock::Create(*Emu->getContext(),
             ".lshr.outer.true.branch");
         TrueBB->insertInto(Emu->getFunction(), JointBB);
         Instruction* TrueJmp = BranchInst::Create(JointBB, TrueBB);
+        TrueJmp->setDebugLoc(BinOpDebugLoc);
 
         OldBB->getTerminator()->eraseFromParent();
-        BranchInst::Create(TrueBB, JointBB, NE, OldBB);
+        Instruction* TempOldBBBranchInst = BranchInst::Create(TrueBB, JointBB, NE, OldBB);
+        TempOldBBBranchInst->setDebugLoc(BinOpDebugLoc);
 
         // Create the inner branch.
         IRB->SetInsertPoint(&(*TrueBB->begin()));
@@ -920,18 +978,23 @@ bool InstExpander::visitLShr(BinaryOperator& BinOp) {
         // than 32 (true branch) or otherwise (false branch).
         BasicBlock* InnerJBB = TrueBB->splitBasicBlock(TrueJmp);
         InnerResLo = PHINode::Create(IRB->getInt32Ty(), 2, ".lshr.merge.inner.lo", TrueJmp);
+        InnerResLo->setDebugLoc(BinOpDebugLoc);
         InnerResHi = PHINode::Create(IRB->getInt32Ty(), 2, ".lshr.merge.inner.hi", TrueJmp);
+        InnerResHi->setDebugLoc(BinOpDebugLoc);
 
         InnerTBB = BasicBlock::Create(*Emu->getContext(), ".lshr.inner.true.branch");
         InnerTBB->insertInto(Emu->getFunction(), InnerJBB);
-        BranchInst::Create(InnerJBB, InnerTBB);
+        Instruction* TempInnerTBBBranchInst = BranchInst::Create(InnerJBB, InnerTBB);
+        TempInnerTBBBranchInst->setDebugLoc(BinOpDebugLoc);
 
         InnerFBB = BasicBlock::Create(*Emu->getContext(), ".lshr.inner.false.branch");
         InnerFBB->insertInto(Emu->getFunction(), InnerJBB);
-        BranchInst::Create(InnerJBB, InnerFBB);
+        Instruction* TempInnerFBBBranchInst = BranchInst::Create(InnerJBB, InnerFBB);
+        TempInnerFBBBranchInst->setDebugLoc(BinOpDebugLoc);
 
         TrueBB->getTerminator()->eraseFromParent();
-        BranchInst::Create(InnerTBB, InnerFBB, Cond, TrueBB);
+        Instruction* TempTrueBBBranchInst = BranchInst::Create(InnerTBB, InnerFBB, Cond, TrueBB);
+        TempTrueBBBranchInst->setDebugLoc(BinOpDebugLoc);
 
         // The result is the same as the source if ShAmt is 0, i.e. NE is
         // false.
@@ -959,10 +1022,8 @@ bool InstExpander::visitLShr(BinaryOperator& BinOp) {
         L = IRB->CreateOr(L, T0);
 
         if (InnerTBB) {
-            IGC_ASSERT(isa<PHINode>(InnerResLo));
-            IGC_ASSERT(isa<PHINode>(InnerResHi));
-            cast<PHINode>(InnerResLo)->addIncoming(L, InnerTBB);
-            cast<PHINode>(InnerResHi)->addIncoming(H, InnerTBB);
+            InnerResLo->addIncoming(L, InnerTBB);
+            InnerResHi->addIncoming(H, InnerTBB);
         }
         else {
             ResLo = L;
@@ -982,10 +1043,8 @@ bool InstExpander::visitLShr(BinaryOperator& BinOp) {
         Value* L = IRB->CreateLShr(Hi, Amt);
 
         if (InnerFBB) {
-            IGC_ASSERT(isa<PHINode>(InnerResLo));
-            IGC_ASSERT(isa<PHINode>(InnerResHi));
-            cast<PHINode>(InnerResLo)->addIncoming(L, InnerFBB);
-            cast<PHINode>(InnerResHi)->addIncoming(H, InnerFBB);
+            InnerResLo->addIncoming(L, InnerFBB);
+            InnerResHi->addIncoming(H, InnerFBB);
         }
         else {
             ResLo = L;
@@ -1002,6 +1061,9 @@ bool InstExpander::visitAShr(BinaryOperator& BinOp) {
     if (!Emu->isInt64(&BinOp))
         return false;
 
+    if (hasHWSupport(BinOp))
+        return false;
+
     Value* Lo = nullptr, * Hi = nullptr;
     std::tie(Lo, Hi) = Emu->getExpandedValues(BinOp.getOperand(0));
     Value* ShAmt = nullptr;
@@ -1010,10 +1072,11 @@ bool InstExpander::visitAShr(BinaryOperator& BinOp) {
     BasicBlock* OldBB = BinOp.getParent();
     BasicBlock* InnerTBB = nullptr;
     BasicBlock* InnerFBB = nullptr;
-    Value* InnerResLo = nullptr;
-    Value* InnerResHi = nullptr;
+    PHINode* InnerResLo = nullptr;
+    PHINode* InnerResHi = nullptr;
     Value* ResLo = nullptr;
     Value* ResHi = nullptr;
+    DebugLoc BinOpDebugLoc = BinOp.getDebugLoc();
 
     Value* Cond = nullptr;
 
@@ -1027,15 +1090,19 @@ bool InstExpander::visitAShr(BinaryOperator& BinOp) {
         Value* NE = IRB->CreateICmpNE(ShAmt, Constant::getNullValue(ShAmt->getType()));
         BasicBlock* JointBB = OldBB->splitBasicBlock(&BinOp);
         ResLo = PHINode::Create(IRB->getInt32Ty(), 2, ".ashr.outer.merge.lo", &BinOp);
+        cast<Instruction>(ResLo)->setDebugLoc(BinOpDebugLoc);
         ResHi = PHINode::Create(IRB->getInt32Ty(), 2, ".ashr.outer.merge.hi", &BinOp);
+        cast<Instruction>(ResHi)->setDebugLoc(BinOpDebugLoc);
 
         BasicBlock* TrueBB = BasicBlock::Create(*Emu->getContext(),
             ".ashr.outer.true.branch");
         TrueBB->insertInto(Emu->getFunction(), JointBB);
         Instruction* TrueJmp = BranchInst::Create(JointBB, TrueBB);
+        TrueJmp->setDebugLoc(BinOpDebugLoc);
 
         OldBB->getTerminator()->eraseFromParent();
-        BranchInst::Create(TrueBB, JointBB, NE, OldBB);
+        Instruction* TempOldBBBranchInst = BranchInst::Create(TrueBB, JointBB, NE, OldBB);
+        TempOldBBBranchInst->setDebugLoc(BinOpDebugLoc);
 
         // Create the inner branch.
         IRB->SetInsertPoint(&(*TrueBB->begin()));
@@ -1045,18 +1112,23 @@ bool InstExpander::visitAShr(BinaryOperator& BinOp) {
         // than 32 or otherwise.
         BasicBlock* InnerJBB = TrueBB->splitBasicBlock(TrueJmp);
         InnerResLo = PHINode::Create(IRB->getInt32Ty(), 2, ".ashr.merge.inner.lo", TrueJmp);
+        InnerResLo->setDebugLoc(BinOpDebugLoc);
         InnerResHi = PHINode::Create(IRB->getInt32Ty(), 2, ".ashr.merge.inner.hi", TrueJmp);
+        InnerResHi->setDebugLoc(BinOpDebugLoc);
 
         InnerTBB = BasicBlock::Create(*Emu->getContext(), ".ashr.inner.true.branch");
         InnerTBB->insertInto(Emu->getFunction(), InnerJBB);
-        BranchInst::Create(InnerJBB, InnerTBB);
+        Instruction* TempInnerTBBBranchInst = BranchInst::Create(InnerJBB, InnerTBB);
+        TempInnerTBBBranchInst->setDebugLoc(BinOpDebugLoc);
 
         InnerFBB = BasicBlock::Create(*Emu->getContext(), ".ashr.inner.false.branch");
         InnerFBB->insertInto(Emu->getFunction(), InnerJBB);
-        BranchInst::Create(InnerJBB, InnerFBB);
+        Instruction* TempInnerFBBBranchInst = BranchInst::Create(InnerJBB, InnerFBB);
+        TempInnerFBBBranchInst->setDebugLoc(BinOp.getDebugLoc());
 
         TrueBB->getTerminator()->eraseFromParent();
-        BranchInst::Create(InnerTBB, InnerFBB, Cond, TrueBB);
+        Instruction* TempTrueBBBranchInst = BranchInst::Create(InnerTBB, InnerFBB, Cond, TrueBB);
+        TempTrueBBBranchInst->setDebugLoc(BinOpDebugLoc);
 
         // The result is the same as the source if ShAmt is 0, i.e. NE is
         // false.
@@ -1079,10 +1151,8 @@ bool InstExpander::visitAShr(BinaryOperator& BinOp) {
         L = IRB->CreateOr(L, T0);
 
         if (InnerTBB) {
-            IGC_ASSERT(isa<PHINode>(InnerResLo));
-            IGC_ASSERT(isa<PHINode>(InnerResHi));
-            cast<PHINode>(InnerResLo)->addIncoming(L, InnerTBB);
-            cast<PHINode>(InnerResHi)->addIncoming(H, InnerTBB);
+            InnerResLo->addIncoming(L, InnerTBB);
+            InnerResHi->addIncoming(H, InnerTBB);
         }
         else {
             ResLo = L;
@@ -1098,10 +1168,8 @@ bool InstExpander::visitAShr(BinaryOperator& BinOp) {
         Value* L = IRB->CreateAShr(Hi, Amt);
 
         if (InnerFBB) {
-            IGC_ASSERT(isa<PHINode>(InnerResLo));
-            IGC_ASSERT(isa<PHINode>(InnerResHi));
-            cast<PHINode>(InnerResLo)->addIncoming(L, InnerFBB);
-            cast<PHINode>(InnerResHi)->addIncoming(H, InnerFBB);
+            InnerResLo->addIncoming(L, InnerFBB);
+            InnerResHi->addIncoming(H, InnerFBB);
         }
         else {
             ResLo = L;
@@ -1252,6 +1320,9 @@ bool InstExpander::visitSExt(SExtInst& SEI) {
     if (!Emu->isInt64(&SEI))
         return false;
 
+    if (!needsLoHiSplitting(SEI))
+        return false;
+
     Value* Src = SEI.getOperand(0);
     Type* SrcTy = SEI.getSrcTy();
     IGC_ASSERT(nullptr != SrcTy);
@@ -1272,6 +1343,9 @@ bool InstExpander::visitZExt(ZExtInst& ZEI) {
     if (!Emu->isInt64(&ZEI))
         return false;
 
+    if (!needsLoHiSplitting(ZEI))
+        return false;
+
     Value* Src = ZEI.getOperand(0);
     Type* SrcTy = ZEI.getSrcTy();
     IGC_ASSERT(nullptr != SrcTy);
@@ -1425,18 +1499,9 @@ bool InstExpander::visitFPToSI(FPToSIInst& F2S) {
     return true;
 }
 
-// Note: This method uses splitBasciBlock() which moves Pos (and all
-// instructions following it till the end of basic block) to a new basic block.
-// If IRB's insert point is set to Pos or an instruction after Pos in the same
-// basic block than the insert basic block value in IRB will be invalid after
-// this method completes. A correct insert point should be set in IRB after this
-// method is called.
-//
 Value* InstExpander::convertUIToFP32(Type* DstTy, Value* Lo, Value* Hi, Instruction* Pos) {
     IGC_ASSERT(nullptr != Pos);
     IGC_ASSERT(nullptr != IRB);
-    BuilderType::InsertPointGuard Guard(*IRB);
-    IRB->SetInsertPoint(Pos);
 
     IGCLLVM::Intrinsic IID;
     IID = Intrinsic::ctlz;
@@ -1449,7 +1514,13 @@ Value* InstExpander::convertUIToFP32(Type* DstTy, Value* Lo, Value* Hi, Instruct
     BasicBlock* OldBB = Pos->getParent();
     IGC_ASSERT(nullptr != OldBB);
     BasicBlock* JointBB = OldBB->splitBasicBlock(Pos);
+
+    // Set insert point to Pos in the new block JointBB
+    IRB->SetInsertPoint(Pos);
+    DebugLoc PosDebugLoc = Pos->getDebugLoc();
+
     PHINode* Res = PHINode::Create(IRB->getInt32Ty(), 2, ".u2f.outer.merge", Pos);
+    Res->setDebugLoc(PosDebugLoc);
 
     {
         BuilderType::InsertPointGuard Guard(*IRB);
@@ -1457,9 +1528,11 @@ Value* InstExpander::convertUIToFP32(Type* DstTy, Value* Lo, Value* Hi, Instruct
         BasicBlock* TrueBB = BasicBlock::Create(*Emu->getContext(), ".u2f.outer.true.branch");
         TrueBB->insertInto(Emu->getFunction(), JointBB);
         Instruction* TrueJmp = BranchInst::Create(JointBB, TrueBB);
+        TrueJmp->setDebugLoc(PosDebugLoc);
 
         OldBB->getTerminator()->eraseFromParent();
-        BranchInst::Create(TrueBB, JointBB, NE, OldBB);
+        Instruction* TempOldBBBranchInst = BranchInst::Create(TrueBB, JointBB, NE, OldBB);
+        TempOldBBBranchInst->setDebugLoc(PosDebugLoc);
 
         IRB->SetInsertPoint(&(*TrueBB->begin()));
         // Check ShAmt == 0
@@ -1467,14 +1540,18 @@ Value* InstExpander::convertUIToFP32(Type* DstTy, Value* Lo, Value* Hi, Instruct
 
         BasicBlock* InnerJBB = TrueBB->splitBasicBlock(TrueJmp);
         PHINode* InnerResHi = PHINode::Create(IRB->getInt32Ty(), 2, ".u2f.inner.merge.hi", TrueJmp);
+        InnerResHi->setDebugLoc(PosDebugLoc);
         PHINode* InnerResLo = PHINode::Create(IRB->getInt32Ty(), 2, ".u2f.inner.merge.lo", TrueJmp);
+        InnerResLo->setDebugLoc(PosDebugLoc);
 
         BasicBlock* InnerTBB = BasicBlock::Create(*Emu->getContext(), ".u2f.inner.true.branch");
         InnerTBB->insertInto(Emu->getFunction(), InnerJBB);
-        BranchInst::Create(InnerJBB, InnerTBB);
+        Instruction* TempInnerTBBBranchInst = BranchInst::Create(InnerJBB, InnerTBB);
+        TempInnerTBBBranchInst->setDebugLoc(PosDebugLoc);
 
         TrueBB->getTerminator()->eraseFromParent();
-        BranchInst::Create(InnerTBB, InnerJBB, NE, TrueBB);
+        Instruction* TempTrueBBBranchInst = BranchInst::Create(InnerTBB, InnerJBB, NE, TrueBB);
+        TempTrueBBBranchInst->setDebugLoc(PosDebugLoc);
 
         IRB->SetInsertPoint(&(*InnerTBB->begin()));
         // 0 < ShAmt < 32
@@ -1496,13 +1573,16 @@ Value* InstExpander::convertUIToFP32(Type* DstTy, Value* Lo, Value* Hi, Instruct
 
         BasicBlock* RoundingJBB = InnerJBB->splitBasicBlock(InnerJmp);
         PHINode* RoundingRes = PHINode::Create(IRB->getInt32Ty(), 2, ".u2f.rounding.merge.hi", InnerJmp);
+        RoundingRes->setDebugLoc(PosDebugLoc);
 
-        BasicBlock* RoundingBB = BasicBlock::Create(*Emu->getContext(), ".u2f.roudning.branch");
+        BasicBlock* RoundingBB = BasicBlock::Create(*Emu->getContext(), ".u2f.rounding.branch");
         RoundingBB->insertInto(Emu->getFunction(), RoundingJBB);
-        BranchInst::Create(RoundingJBB, RoundingBB);
+        Instruction* TempRoundingBBBranchInst = BranchInst::Create(RoundingJBB, RoundingBB);
+        TempRoundingBBBranchInst->setDebugLoc(PosDebugLoc);
 
         InnerJBB->getTerminator()->eraseFromParent();
-        BranchInst::Create(RoundingBB, RoundingJBB, NE, InnerJBB);
+        Instruction* TempInnerJBBBranchInst = BranchInst::Create(RoundingBB, RoundingJBB, NE, InnerJBB);
+        TempInnerJBBBranchInst->setDebugLoc(PosDebugLoc);
 
         // Rounding
         IRB->SetInsertPoint(&(*RoundingBB->begin()));
@@ -1555,7 +1635,6 @@ bool InstExpander::visitUIToFP(UIToFPInst& U2F) {
     }
     else {
         NewVal = convertUIToFP32(IRB->getFloatTy(), Lo, Hi, &U2F);
-        IRB->SetInsertPoint(&U2F);
         // It's OK to apply the same approach in `convertUIToFP32` to convert 64-bit
         // integer into half. But, it would introduce a little more instructions to
         // properly round the remaining 48 bits into the high 16 bits. Instead, that
@@ -1617,7 +1696,6 @@ bool InstExpander::visitSIToFP(SIToFPInst& S2F) {
         Hi = IRB->CreateExtractValue(V, 1);
 
         NewVal = convertUIToFP32(IRB->getFloatTy(), Lo, Hi, &S2F);
-        IRB->SetInsertPoint(&S2F);
         NewVal = IRB->CreateBitCast(NewVal, IRB->getInt32Ty());
         Sign = IRB->CreateAnd(Sign, IRB->getInt32(0x80000000));
         NewVal = IRB->CreateOr(NewVal, Sign);
@@ -1945,7 +2023,7 @@ bool InstExpander::visitExtractElement(ExtractElementInst& EEI) {
     // later.
 
     Value* V = EEI.getVectorOperand();
-    unsigned NumElts = (unsigned)cast<VectorType>(V->getType())->getNumElements();
+    unsigned NumElts = (unsigned)cast<IGCLLVM::FixedVectorType>(V->getType())->getNumElements();
     V = IRB->CreateBitCast(V, Emu->getV2Int32Ty(NumElts));
     // Re-calculate indices to Lo and Hi parts.
     Value* Idx = EEI.getIndexOperand();
@@ -1982,7 +2060,7 @@ bool InstExpander::visitInsertElement(InsertElementInst& IEI) {
 
     // Create the emulated vector.
     Value* NewVal = IEI.getOperand(0);
-    unsigned NumElts = (unsigned)cast<VectorType>(NewVal->getType())->getNumElements();
+    unsigned NumElts = (unsigned)cast<IGCLLVM::FixedVectorType>(NewVal->getType())->getNumElements();
     NewVal = IRB->CreateBitCast(NewVal, Emu->getV2Int32Ty(NumElts));
     // Re-calculate indices to Lo and Hi parts.
     Value* Idx = IEI.getOperand(2);
@@ -2008,7 +2086,7 @@ bool InstExpander::visitExtractValue(ExtractValueInst& EVI) {
 bool InstExpander::visitInsertValue(InsertValueInst& IVI) {
     // TODO: Add i64 emulation support.
     IGC_ASSERT(nullptr != Emu);
-    IGC_ASSERT(0 < IVI.getNumOperands());
+    IGC_ASSERT(1 < IVI.getNumOperands());
     IGC_ASSERT_MESSAGE(false == Emu->isInt64(IVI.getOperand(1)), "TODO: NOT IMPLEMENTED YET!");
     return false;
 }
@@ -2016,7 +2094,103 @@ bool InstExpander::visitInsertValue(InsertValueInst& IVI) {
 bool InstExpander::visitLandingPad(LandingPadInst& LPI) {
     // TODO: Add i64 emulation support.
     IGC_ASSERT(nullptr != Emu);
-    IGC_ASSERT(0 < LPI.getNumOperands());
+    IGC_ASSERT(1 < LPI.getNumOperands());
     IGC_ASSERT_MESSAGE(false == Emu->isInt64(LPI.getOperand(1)), "TODO: NOT IMPLEMENTED YET!");
     return false;
 }
+
+bool InstExpander::hasHWSupport(BinaryOperator& BinOp) {
+
+    if (Emu->CGC->platform.hasPartialInt64Support())
+    {
+        if (Emu->eraseInstrFromInt64InstrList(&BinOp))
+        {
+            // No need to reconstruct the 64b operands
+            // the former instruction SExt or ZExt has not been changed.
+
+            // Just do not delete an original instruction
+            // and split the result into Lo and Hi part.
+        }
+        else
+        {
+            // reconstruct the 64b operands
+            for (uint operandId = 0; operandId < 2; operandId++)
+            {
+                Value* Lo = nullptr, * Hi = nullptr;
+                std::tie(Lo, Hi)   = Emu->getExpandedValues(BinOp.getOperand(operandId));
+                Type* V2I32Ty = Emu->getV2Int32Ty();
+                Value* reconstrut64bOperands = UndefValue::get(V2I32Ty);
+                reconstrut64bOperands = IRB->CreateInsertElement(reconstrut64bOperands, Lo, IRB->getInt32(0));
+                reconstrut64bOperands = IRB->CreateInsertElement(reconstrut64bOperands, Hi, IRB->getInt32(1));
+                reconstrut64bOperands = IRB->CreateBitCast(reconstrut64bOperands, IRB->getInt64Ty());
+                BinOp.setOperand(operandId, reconstrut64bOperands);
+            }
+        }
+
+        // Leave, do not delete an original instruction
+        BasicBlock::iterator BI = std::next(BasicBlock::iterator(&BinOp));
+        IRB->SetInsertPoint(&*BI);
+
+        // Split the result into Lo and Hi part.
+        Value* V = IRB->CreateBitCast(&BinOp, Emu->getV2Int32Ty());
+        Value* outputLo = IRB->CreateExtractElement(V, IRB->getInt32(0));
+        Value* outputHi = IRB->CreateExtractElement(V, IRB->getInt32(1));
+        Emu->setExpandedValues(&BinOp, outputLo, outputHi);
+
+        Emu->doNotDeleteInstr = true;
+        return true;
+    }
+    return false;
+}
+
+bool InstExpander::needsLoHiSplitting(Instruction& inst) {
+
+    bool splittingRequired = true;
+    if (Emu->CGC->platform.hasPartialInt64Support())
+    {
+        bool doNextStep = true;
+
+        // Do not split into Lo and Hi parts if the result of CastInst:
+        // (A) is used only by the opcode with HW native Int64 support, AND
+        //      eg. %9 = shl nsw i64 %conv.i1, 1  ;  % 5 = add i64 % 4, % conv.i1
+        // (B) is used by the binary instr w/ HW Int64 support and all operands come from SExt,ZExt or
+        //     one is a constant value
+        //      eg. %1 = shl nsw i64 %conv.i1, 3  ;  %shr.i.i = lshr i64 % cond.i9.i, % shr.mask.i.i
+
+        for (auto* U : inst.users())
+        {
+            if (Instruction * _instr = dyn_cast<Instruction>(U))
+            {
+                if (!Emu->hasOpcodeInt64Support(_instr->getOpcode())) // (A)
+                {
+                    doNextStep = false;
+                    break;
+                }
+
+                for (auto& Op : _instr->operands())
+                {
+                    if (!isa<ConstantInt>(Op.get()) &&
+                        !isa<SExtInst>(Op.get()) &&
+                        !isa<ZExtInst>(Op.get())) // (B)
+                    {
+                        doNextStep = false;
+                        break;
+                    }
+                }
+            }
+        }
+
+        if (doNextStep)
+        {
+            for (auto* U : inst.users())
+            {
+                if (Instruction * binaryInstr = dyn_cast<BinaryOperator>(U))
+                {
+                    Emu->addInstToInt64InstrList(binaryInstr);
+                    splittingRequired = false;
+                }
+            }
+        }
+    }
+    return splittingRequired;
+}
