@@ -1,28 +1,11 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
+============================= end_copyright_notice ===========================*/
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
 #include "common/LLVMWarningsPush.hpp"
 #include <llvm/Pass.h>
 #include <llvm/Analysis/PostDominators.h>
@@ -252,9 +235,9 @@ CVariable* CGeometryShader::GetURBInputHandle(CVariable* pVertexIndex)
 
         if (!m_properties.Input().HasInstancing())
         {
-            // offset = vertexIndex * 32 since offset needs to be in bytes, not grf numbers
+            // offset = vertexIndex * 32 or 64(based on simdsize) since offset needs to be in bytes, not grf numbers
             CVariable* pTemp = GetNewVariable(pOffset);
-            encoder.Shl(pTemp, pVertexIndexWord, ImmToVariable(5, ISA_TYPE_UW));
+            encoder.Shl(pTemp, pVertexIndexWord, ImmToVariable(iSTD::Log2(getGRFSize()), ISA_TYPE_UW));
             encoder.Push();
 
             // offset = temp + perLaneOffsets
@@ -364,9 +347,9 @@ void CGeometryShader::FillProgram(SGeometryShaderKernelProgram* pKernelProgram)
     }
     pKernelProgram->RenderTargetLoaded = m_renderTargetLoaded;
 
-    pKernelProgram->NOSBufferSize = m_NOSBufferSize / getGRFSize(); // in 256 bits
+    pKernelProgram->NOSBufferSize = m_NOSBufferSize / getMinPushConstantBufferAlignmentInBytes();
 
-    pKernelProgram->MaxNumberOfThreads = m_Platform->getMaxGeometryShaderThreads();
+    pKernelProgram->MaxNumberOfThreads = m_Platform->getMaxGeometryShaderThreads() / GetShaderThreadUsageRate();
     pKernelProgram->hasControlFlow = m_numBlocks > 1 ? true : false;
 
     // Gen 7 specific Compiler Output
