@@ -17,6 +17,7 @@ SPDX-License-Identifier: MIT
 #include <llvm/IR/PassManager.h>
 #include <llvm/IR/IRBuilder.h>
 #include <llvm/IR/Verifier.h>
+#include <llvmWrapper/IR/DerivedTypes.h>
 #include "common/LLVMWarningsPop.hpp"
 #include "Compiler/InitializePasses.h"
 #include "Probe/Assertion.h"
@@ -157,7 +158,8 @@ GeometryShaderLowering::GeometryShaderLowering()
 
 bool GeometryShaderLowering::runOnFunction(llvm::Function& function)
 {
-    MetaDataUtils* pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
+    MetaDataUtils* pMdUtils = nullptr;
+    pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
     if (!isEntryFunc(pMdUtils, &function))
     {
         return false;
@@ -444,7 +446,7 @@ void GeometryShaderLowering::AddURBRead(
     {
         Value* vec = UndefValue::get(inst->getType());
         IRBuilder<> builder(inst);
-        for (unsigned int i = 0; i < cast<VectorType>(inst->getType())->getNumElements(); i++)
+        for (unsigned int i = 0; i < cast<IGCLLVM::FixedVectorType>(inst->getType())->getNumElements(); i++)
         {
             Value* vecElement = builder.CreateExtractElement(urbRead, builder.getInt32(i));
             vec = builder.CreateInsertElement(vec, vecElement, builder.getInt32(i));
@@ -630,7 +632,7 @@ void GeometryShaderLowering::LowerControlHeader(llvm::Instruction* inst)
     // Add write to vertex count field for non-static number of output vertices
     IRBuilder<> irb(inst);
     Value* undef = llvm::UndefValue::get(Type::getFloatTy(m_pModule->getContext()));
-    auto pCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
+    IGC::CodeGenContext* pCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
     if (pConstVertexIndex == nullptr || pCtx->platform.disableStaticVertexCount())
     {
         IGC_ASSERT(m_gsProps->GetProperties().Output().HasNonstaticVertexCount());
