@@ -1,28 +1,10 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
+============================= end_copyright_notice ===========================*/
 
 #include "Compiler/CISACodeGen/HullShaderCodeGen.hpp"
 #include "Compiler/CISACodeGen/EmitVISAPass.hpp"
@@ -286,9 +268,9 @@ namespace IGC
         }
         pKernelProgram->RenderTargetLoaded = m_renderTargetLoaded;
 
-        pKernelProgram->NOSBufferSize = m_NOSBufferSize / getGRFSize(); // in 256 bits
+        pKernelProgram->NOSBufferSize = m_NOSBufferSize / getMinPushConstantBufferAlignmentInBytes();
 
-        pKernelProgram->MaxNumberOfThreads = m_Platform->getMaxHullShaderThreads();
+        pKernelProgram->MaxNumberOfThreads = m_Platform->getMaxHullShaderThreads() / GetShaderThreadUsageRate();
         pKernelProgram->IncludeVertexHandles = NeedVertexHandle();
         pKernelProgram->URBAllocationSize = GetURBAllocationSize();
         pKernelProgram->PatchConstantURBSize = GetPatchConstantURBSize();
@@ -350,7 +332,7 @@ namespace IGC
             else
             {
                 // Eight Patch
-                pConst = ImmToVariable(0x05, ISA_TYPE_UW);
+                pConst = ImmToVariable(iSTD::Log2(getGRFSize()), ISA_TYPE_UW);
             }
 
             encoder.Shl(pPerLaneOffsetsReg, pVertexIndexWord, pConst);
