@@ -81,7 +81,8 @@ namespace IGC
 
     bool LowerGSInterface::runOnFunction(llvm::Function& F)
     {
-        MetaDataUtils* pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
+        MetaDataUtils* pMdUtils = nullptr;
+        pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
         if (pMdUtils->findFunctionsInfoItem(&F) == pMdUtils->end_FunctionsInfo())
         {
             return false;
@@ -236,7 +237,7 @@ namespace IGC
     ///     cutBits[index] |= orVal
     void LowerGSInterface::updateCutVariables(llvm::GenIntrinsicInst& I)
     {
-        llvm::IRBuilder<> b(&I);
+        IGCLLVM::IRBuilder<> b(&I);
         Value* emitCount = I.getArgOperand(0);
         Value* zeroValue = b.getInt32(0);
 
@@ -326,7 +327,7 @@ namespace IGC
         Value* streamID = I.getArgOperand(0);
         Value* emitCount = I.getArgOperand(1);
 
-        llvm::IRBuilder<> builder(&I);
+        IGCLLVM::IRBuilder<> builder(&I);
         // %emitCountMulTwo_s = shl i32 %emitCount, 1
         Value* emitCountMulTwo = builder.CreateShl(
             emitCount,
@@ -401,7 +402,7 @@ namespace IGC
     /// Adds instructions calling intrinsic that will get translated to writing control data header.
     void LowerGSInterface::writeControlHeaderCuts(llvm::ReturnInst& I)
     {
-        llvm::IRBuilder<> builder(&I);
+        IGCLLVM::IRBuilder<> builder(&I);
         Module* M = I.getParent()->getParent()->getParent();
         Function* pFunc = llvm::GenISAIntrinsic::getDeclaration(M, llvm::GenISAIntrinsic::GenISA_GsCutControlHeader);
         IGC_ASSERT(pFunc);
@@ -432,7 +433,7 @@ namespace IGC
     /// Adds instructions calling intrinsic that will get translated to writing control data header.
     void LowerGSInterface::writeControlHeaderStreams(llvm::ReturnInst& I)
     {
-        llvm::IRBuilder<> builder(&I);
+        IGCLLVM::IRBuilder<> builder(&I);
         Module* M = I.getParent()->getParent()->getParent();
         Function* pFunc = llvm::GenISAIntrinsic::getDeclaration(M, llvm::GenISAIntrinsic::GenISA_GsStreamHeader);
         IGC_ASSERT(pFunc);
