@@ -19,7 +19,7 @@ SPDX-License-Identifier: MIT
 #include <llvm/IR/DataLayout.h>
 #include <llvm/IR/Function.h>
 #include <llvm/IR/GlobalAlias.h>
-#include <llvm/IR/IRBuilder.h>
+#include <llvmWrapper/IR/IRBuilder.h>
 #include <llvm/Pass.h>
 #include <llvmWrapper/Support/Alignment.h>
 #include <llvmWrapper/IR/DerivedTypes.h>
@@ -67,6 +67,7 @@ namespace {
         TargetLibraryInfo* TLI;
 
         bool AllowNegativeSymPtrsForLoad = false;
+        bool AllowVector8LoadStore = false;
 
         // Map of profit vector lengths per scalar type. Each entry specifies the
         // profit vector length of a given scalar type.
@@ -81,9 +82,11 @@ namespace {
     public:
         static char ID;
 
-        MemOpt(bool AllowNegativeSymPtrsForLoad = false) :
+        MemOpt(bool AllowNegativeSymPtrsForLoad = false, bool AllowVector8LoadStore = false) :
             FunctionPass(ID), DL(nullptr), AA(nullptr), SE(nullptr), WI(nullptr),
-            CGC(nullptr), AllowNegativeSymPtrsForLoad(AllowNegativeSymPtrsForLoad) {
+            CGC(nullptr), AllowNegativeSymPtrsForLoad(AllowNegativeSymPtrsForLoad),
+            AllowVector8LoadStore(AllowVector8LoadStore)
+        {
             initializeMemOptPass(*PassRegistry::getPassRegistry());
         }
 
@@ -108,7 +111,7 @@ namespace {
             MemRefListTy& MemRefs, TrivialMemRefListTy& ToOpt);
 
         unsigned getNumElements(Type* Ty) const {
-            return Ty->isVectorTy() ? (unsigned)cast<VectorType>(Ty)->getNumElements() : 1;
+            return Ty->isVectorTy() ? (unsigned)cast<IGCLLVM::FixedVectorType>(Ty)->getNumElements() : 1;
         }
 
         MemoryLocation getLocation(Instruction* I) const {
@@ -177,7 +180,7 @@ namespace {
         bool isSafeToMergeLoad(const LoadInst* Ld,
             const SmallVectorImpl<Instruction*>& checkList) const;
         bool isSafeToMergeStores(
-            const SmallVectorImpl<std::tuple<StoreInst*, int64_t, MemRefListTy::iterator, uint32_t>>& Stores,
+            const SmallVectorImpl<std::tuple<StoreInst*, int64_t, MemRefListTy::iterator>>& Stores,
             const SmallVectorImpl<Instruction*>& checkList) const;
 
         bool shouldSkip(const Value* Ptr) const {
@@ -187,8 +190,8 @@ namespace {
             if (PtrTy->getPointerAddressSpace() != ADDRESS_SPACE_PRIVATE) {
                 if (CGC->type != ShaderType::OPENCL_SHADER) {
                     // For non-OpenCL shader, skip constant buffer accesses.
-                    bool DirectIndex;
-                    unsigned BufID;
+                    bool DirectIndex = false;
+                    unsigned BufID = 0;
                     BufferType BufTy = DecodeAS4GFXResource(AS, DirectIndex, BufID);
                     if (BufTy == CONSTANT_BUFFER &&
                         UsesTypedConstantBuffer(CGC, BufTy))
@@ -216,7 +219,7 @@ namespace {
 
         template <typename AccessInstruction>
         bool checkAlignmentBeforeMerge(const AccessInstruction* inst,
-            SmallVector<std::tuple<AccessInstruction*, int64_t, MemRefListTy::iterator, uint32_t>, 8> & AccessIntrs,
+            SmallVector<std::tuple<AccessInstruction*, int64_t, MemRefListTy::iterator>, 8> & AccessIntrs,
             unsigned& NumElts)
         {
             if (inst->getAlignment() < 4 && !WI->isUniform(inst))
@@ -327,20 +330,10 @@ namespace {
 
         // getConstantOffset - Return the constant offset between two memory
         // locations.
-        bool getConstantOffset(const SymbolicPointer& Other, int64_t& Off, int64_t &ArrayElem) {
+        bool getConstantOffset(const SymbolicPointer& Other, int64_t& Off) {
             if (!BasePtr || !Other.BasePtr)
                 return true;
 
-            ArrayElem = 1;
-            // look up ArrayElem=144 in this example
-            //   getelementptr [144 x float], [144 x float] addrspace(3)* %1184, ...
-            if (BasePtr->getType()->isPointerTy())
-            {
-                llvm::Type* BaseType = BasePtr->getType()->getPointerElementType();
-                if (BaseType->isArrayTy())
-                    ArrayElem = BaseType->getArrayNumElements();
-            }
-
             if (BasePtr != Other.BasePtr &&
                 (!isa<ConstantPointerNull>(BasePtr) ||
                     !isa<ConstantPointerNull>(Other.BasePtr)))
@@ -377,8 +370,8 @@ namespace {
     };
 }
 
-FunctionPass* createMemOptPass(bool AllowNegativeSymPtrsForLoad) {
-    return new MemOpt(AllowNegativeSymPtrsForLoad);
+FunctionPass* createMemOptPass(bool AllowNegativeSymPtrsForLoad, bool AllowVector8LoadStore) {
+    return new MemOpt(AllowNegativeSymPtrsForLoad, AllowVector8LoadStore);
 }
 
 #define PASS_FLAG     "igc-memopt"
@@ -398,6 +391,11 @@ char MemOpt::ID = 0;
 
 void MemOpt::buildProfitVectorLengths(Function& F) {
     ProfitVectorLengths.clear();
+    if (AllowVector8LoadStore)
+    {
+        ProfitVectorLengths[64].push_back(4);
+        ProfitVectorLengths[32].push_back(8);
+    }
 
     // 64-bit integer
     ProfitVectorLengths[64].push_back(2);
@@ -423,7 +421,8 @@ void MemOpt::buildProfitVectorLengths(Function& F) {
 
 bool MemOpt::runOnFunction(Function& F) {
     // Skip non-kernel function.
-    MetaDataUtils* MDU = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
+    MetaDataUtils* MDU = nullptr;
+    MDU = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
     auto FII = MDU->findFunctionsInfoItem(&F);
     if (FII == MDU->end_FunctionsInfo())
         return false;
@@ -526,8 +525,8 @@ bool MemOpt::mergeLoad(LoadInst* LeadingLoad,
     // there's no way to trace down.
     bool isUniformLoad = (CGC->type == ShaderType::OPENCL_SHADER) && (WI->isUniform(LeadingLoad));
     if (isUniformLoad) {
-        unsigned C = IGC_GET_FLAG_VALUE(UniformMemOptLimit);
-        if (C == 0) C = 256;
+        unsigned C = IGC_GET_FLAG_VALUE(UniformMemOpt4OW);
+        C = (C == 1) ? 512 : 256;
         C /= TypeSizeInBits;
         for (; C >= 2; --C)
             profitVec.push_back(C);
@@ -550,9 +549,9 @@ bool MemOpt::mergeLoad(LoadInst* LeadingLoad,
         return false;
 
     // LoadInst, Offset, MemRefListTy::iterator, LeadingLoad's int2PtrOffset
-    SmallVector<std::tuple<LoadInst*, int64_t, MemRefListTy::iterator, uint32_t>, 8>
+    SmallVector<std::tuple<LoadInst*, int64_t, MemRefListTy::iterator>, 8>
         LoadsToMerge;
-    LoadsToMerge.push_back(std::make_tuple(LeadingLoad, 0, MI, 0));
+    LoadsToMerge.push_back(std::make_tuple(LeadingLoad, 0, MI));
 
     // Loads to be merged is scanned in the program order and will be merged into
     // the leading load. So two edges of that consecutive region are checked
@@ -570,11 +569,9 @@ bool MemOpt::mergeLoad(LoadInst* LeadingLoad,
     // list.
 
     // Two edges of the region where loads are merged into.
-    int64_t HighestOffset = LdSize, HighestOffset4Transpose = 0;
+    int64_t HighestOffset = LdSize;
     int64_t LowestOffset = 0;
 
-    bool bCheckNext = true;
-
     // List of instructions need dependency check.
     SmallVector<Instruction*, 8> CheckList;
 
@@ -622,7 +619,6 @@ bool MemOpt::mergeLoad(LoadInst* LeadingLoad,
             continue;
 
         int64_t Off = 0;
-        int64_t ArrayElem = 1;
         const SCEVConstant* Offset
             = dyn_cast<SCEVConstant>(SE->getMinusSCEV(NextPtr, LeadingPtr));
         // Skip load with non-constant distance.
@@ -634,7 +630,7 @@ bool MemOpt::mergeLoad(LoadInst* LeadingLoad,
                 LeadingSymPtr, CGC) ||
                 SymbolicPointer::decomposePointer(NextLoad->getPointerOperand(),
                     NextSymPtr, CGC) ||
-                NextSymPtr.getConstantOffset(LeadingSymPtr, Off, ArrayElem)) {
+                NextSymPtr.getConstantOffset(LeadingSymPtr, Off)) {
                 continue;
             }
             else {
@@ -648,121 +644,29 @@ bool MemOpt::mergeLoad(LoadInst* LeadingLoad,
 
         unsigned NextLoadSize = unsigned(DL->getTypeStoreSize(NextLoadType));
 
-        bool enableTransposeSLM = false;
-        uint32_t LeadInt2PtrOffset = 0;
-
-        // detect if we can merge non-sequential SLM loads
-        // Example
-        //   %1184 = inttoptr i32 4608 to[144 x float] addrspace(3) *
-        //   %1185 = getelementptr[144 x float], [144 x float] addrspace(3) * %1184, i32 0, i32 % 1179
-        //   %1186 = inttoptr i32 5184 to[144 x float] addrspace(3) *
-        //   %1187 = getelementptr[144 x float], [144 x float] addrspace(3) * %1186, i32 0, i32 % 1179
-        //   %1188 = inttoptr i32 5760 to[144 x float] addrspace(3) *
-        //   %1189 = getelementptr[144 x float], [144 x float] addrspace(3) * %1188, i32 0, i32 % 1179
-        //   %1190 = load float, float addrspace(3) * %1185, align 4
-        //   %1191 = load float, float addrspace(3) * %1187, align 4
-        //   %1192 = load float, float addrspace(3) * %1189, align 4
-        if (IGC_IS_FLAG_ENABLED(EnableMergeTransposeSLM)) {
-            unsigned int resourceIndex = 0;
-            bool direct = false;
-            BufferType bufType = IGC::DecodeAS4GFXResource(
-                LeadingLoad->getPointerAddressSpace(), direct, resourceIndex);
-
-            // initialize for first check in the loop
-            if (!HighestOffset4Transpose)
-                HighestOffset4Transpose = NextLoadSize * ArrayElem;
-
-            if (SLM == bufType &&
-                (Off > ArrayElem) && !(Off % ArrayElem))
-            {
-                GetElementPtrInst* LeadGEP = dyn_cast<GetElementPtrInst>(LeadingLoad->getOperand(0));
-                GetElementPtrInst* NextGEP = dyn_cast<GetElementPtrInst>(NextLoad->getOperand(0));
-
-                if (LeadGEP && NextGEP &&
-                    (LeadGEP->getOperand(1) == NextGEP->getOperand(1)) &&
-                    (LeadGEP->getOperand(2) == NextGEP->getOperand(2)) &&
-                    isa<IntToPtrInst>(LeadGEP->getPointerOperand()))
-                {
-                    IntToPtrInst* Int2Ptr = dyn_cast<IntToPtrInst>(LeadGEP->getPointerOperand());
-                    if (const ConstantInt* CI =
-                        dyn_cast<ConstantInt>(Int2Ptr->getOperand(0)))
-                    {
-                        if (CI->getType()->isIntegerTy())
-                        {
-                            LeadInt2PtrOffset = (uint32_t)CI->getZExtValue();
-                            enableTransposeSLM = true;
-                        }
-                    } //  if (const ConstantInt* CI = ..
-                } // if (LeadGEP && NextGEP ...
-
-            } //if (SLM == bufType && ..
-        } // if (IGC_IS_FLAG_ENABLED(EnableMergeTransposeSLM)
-
         // By assuming dead load elimination always works correctly, if the load on
         // the same location is observed again, that is probably because there is
         // an instruction with global effect between them. Bail out directly.
         if (Off == 0 && LdSize == NextLoadSize)
             break;
 
-        int64_t newHighestOffset;
-        int64_t newLowestOffset;
-        uint64_t newNumElts;
+        int64_t newHighestOffset = std::max(Off + NextLoadSize, HighestOffset);
+        int64_t newLowestOffset = std::min(Off, LowestOffset);
+        uint64_t newNumElts = uint64_t((newHighestOffset - newLowestOffset) /
+            LdScalarSize);
 
-        if (IGC_IS_FLAG_ENABLED(EnableMergeTransposeSLM) && enableTransposeSLM) {
-            if (!bCheckNext)
-                continue;
+        // Ensure that the total size read evenly divides the element type.
+        // For example, we could have a packed struct <{i64, i32, i64}> that
+        // would compute a size of 20 but, without this guard, would set
+        // 'NumElts' to 2 as if the i32 wasn't present.
+        if (uint64_t(newHighestOffset - newLowestOffset) % LdScalarSize != 0)
+            continue;
 
-            newHighestOffset = std::max(Off + ArrayElem * NextLoadSize, HighestOffset4Transpose);
-            newLowestOffset = std::min(Off, LowestOffset);
-            newNumElts = uint64_t((newHighestOffset - newLowestOffset) /
-                LdScalarSize) / ArrayElem;
+        // Bail out if the resulting vector load is already not profitable.
+        if (newNumElts > profitVec[0])
+            continue;
 
-            // Update HighestOffset4Transpose for each iteration and
-            // check against the next expected in the sequence
-            // Example: Off = 576 when checking 'i32 5184' entry
-            //   (Offset   0)  %1184 = inttoptr i32 4608 to[144 x float] addrspace(3) *
-            //                 %1185 = getelementptr[144 x float], [144 x float] addrspace(3) * %1184, i32 0, i32 % 1179
-            //   (Offset 576)  %1186 = inttoptr i32 5184 to[144 x float] addrspace(3) *
-            if (Off != HighestOffset4Transpose) {
-                bCheckNext = false; // abort enableTransposeSLM checking
-                continue;
-            }
-
-            // Ensure that the total size read evenly divides the element type.
-            // For example, we could have a packed struct <{i64, i32, i64}> that
-            // would compute a size of 20 but, without this guard, would set
-            // 'NumElts' to 2 as if the i32 wasn't present.
-            if (uint64_t(newHighestOffset - newLowestOffset) % (LdScalarSize * ArrayElem) != 0)
-                continue;
-
-            // Limit to 3 entries for merging
-            if (newNumElts > 3)
-                continue;
-        }
-        else {
-            newHighestOffset = std::max(Off + NextLoadSize, HighestOffset);
-            newLowestOffset = std::min(Off, LowestOffset);
-            newNumElts = uint64_t((newHighestOffset - newLowestOffset) /
-                LdScalarSize);
-
-            // Ensure that the total size read evenly divides the element type.
-            // For example, we could have a packed struct <{i64, i32, i64}> that
-            // would compute a size of 20 but, without this guard, would set
-            // 'NumElts' to 2 as if the i32 wasn't present.
-            if (uint64_t(newHighestOffset - newLowestOffset) % LdScalarSize != 0)
-                continue;
-
-            // Bail out if the resulting vector load is already not profitable.
-            if (newNumElts > profitVec[0])
-                continue;
-        }
-
-        if (enableTransposeSLM) {
-            HighestOffset4Transpose = newHighestOffset;
-        }
-        else {
-            HighestOffset = newHighestOffset;
-        }
+        HighestOffset = newHighestOffset;
         LowestOffset = newLowestOffset;
 
         NumElts = static_cast<unsigned>(newNumElts);
@@ -775,14 +679,14 @@ bool MemOpt::mergeLoad(LoadInst* LeadingLoad,
         if (!isSafeToMergeLoad(NextLoad, CheckList))
             break;
 
-        LoadsToMerge.push_back(std::make_tuple(NextLoad, Off, MI, LeadInt2PtrOffset));
+        LoadsToMerge.push_back(std::make_tuple(NextLoad, Off, MI));
     }
 
     unsigned s = LoadsToMerge.size();
     if (s < 2)
         return false;
 
-    IRBuilder<> Builder(LeadingLoad);
+    IGCLLVM::IRBuilder<> Builder(LeadingLoad);
 
     // Start to merge loads.
     IGC_ASSERT_MESSAGE(1 < NumElts, "It's expected to merge into at least 2-element vector!");
@@ -853,42 +757,6 @@ bool MemOpt::mergeLoad(LoadInst* LeadingLoad,
             Ptr = Builder.CreateGEP(Ptr, Idx);
     }
 
-    uint32_t ArrayElem = 1;
-    uint32_t newInt2PtrOffset = std::get<3>(LoadsToMerge.back());
-
-    // lookup 144
-    //   %55 = getelementptr [144 x float], [144 x float] addrspace(3)* %54, i32 0, i32 %36
-    if (GetElementPtrInst* GEP =
-        dyn_cast<GetElementPtrInst>(FirstLoad->getPointerOperand())) {
-        Value* GEPptr = GEP->getPointerOperand();
-        if (GEPptr->getType()->isPointerTy()) {
-            Type* GEPElemType = GEPptr->getType()->getPointerElementType();
-            if (GEPElemType->isArrayTy())
-                ArrayElem = (uint32_t)GEPElemType->getArrayNumElements();
-        }
-    }
-    // If newInt2PtrOffset is non-zero, that means enableTransposeSLM is set
-    // Prepare new instructions
-    if (newInt2PtrOffset && ArrayElem) {
-        Type* newArrayType = PointerType::get(
-            ArrayType::get(LeadingLoadScalarType, ArrayElem * NumElts),
-            LeadingLoad->getPointerAddressSpace());
-
-        Value* NewInt2Ptr = Builder.getInt32(std::get<3>(LoadsToMerge.back()));
-
-        NewInt2Ptr = createBitOrPointerCast(NewInt2Ptr, newArrayType, Builder);
-        GetElementPtrInst* LeadGEP =
-            dyn_cast<GetElementPtrInst>(FirstLoad->getPointerOperand());
-
-        // We need to adjust the buffer index by multiply by 3
-        // From x0, x1, ... y0, y1, ... z0, z1, ...
-        // To   x0, y0, z0, x1, y1, z1, ....
-        Value* NewGEPOffset =
-            Builder.CreateMul(LeadGEP->getOperand(2), Builder.getInt32(3));
-        Value* GEPArg[] = { LeadGEP->getOperand(1), NewGEPOffset };
-        Ptr = Builder.CreateGEP(NewInt2Ptr, GEPArg);
-    } // if (newInt2PtrOffset && ArrayElem)
-
     Type* NewLoadType = IGCLLVM::FixedVectorType::get(LeadingLoadScalarType, NumElts);
     Type* NewPointerType =
         PointerType::get(NewLoadType, LeadingLoad->getPointerAddressSpace());
@@ -907,7 +775,6 @@ bool MemOpt::mergeLoad(LoadInst* LeadingLoad,
     for (auto& I : LoadsToMerge) {
         Type* Ty = std::get<0>(I)->getType();
         Type* ScalarTy = Ty->getScalarType();
-        uint32_t newInt2PtrOffset = std::get<3>(LoadsToMerge.back());
         IGC_ASSERT(hasSameSize(ScalarTy, LeadingLoadScalarType));
 
         mdLoadInv = std::get<0>(I)->getMetadata(LLVMContext::MD_invariant_load);
@@ -916,13 +783,10 @@ bool MemOpt::mergeLoad(LoadInst* LeadingLoad,
             allInvariantLoads = false;
         }
 
-        if (IGC_IS_FLAG_ENABLED(EnableMergeTransposeSLM) && newInt2PtrOffset)
-            Pos = unsigned((std::get<1>(I) - FirstOffset) / LdScalarSize / ArrayElem);
-        else
-            Pos = unsigned((std::get<1>(I) - FirstOffset) / LdScalarSize);
+        Pos = unsigned((std::get<1>(I) - FirstOffset) / LdScalarSize);
 
         if (Ty->isVectorTy()) {
-            if (Pos + cast<VectorType>(Ty)->getNumElements() > NumElts) {
+            if (Pos + cast<IGCLLVM::FixedVectorType>(Ty)->getNumElements() > NumElts) {
                 // This implies we're trying to extract an element from our new load
                 // with an index > the size of the new load.  If this happens,
                 // we'll generate correct code if it does since we don't remove the
@@ -930,7 +794,7 @@ bool MemOpt::mergeLoad(LoadInst* LeadingLoad,
                 continue;
             }
             Value* Val = UndefValue::get(Ty);
-            for (unsigned i = 0, e = (unsigned)cast<VectorType>(Ty)->getNumElements(); i != e; ++i) {
+            for (unsigned i = 0, e = (unsigned)cast<IGCLLVM::FixedVectorType>(Ty)->getNumElements(); i != e; ++i) {
                 Value* Ex = Builder.CreateExtractElement(NewLoad, Builder.getInt32(Pos + i));
                 Ex = createBitOrPointerCast(Ex, ScalarTy, Builder);
                 Val = Builder.CreateInsertElement(Val, Ex, Builder.getInt32(i));
@@ -1019,10 +883,10 @@ bool MemOpt::mergeStore(StoreInst* LeadingStore,
         return false;
 
     // StoreInst, Offset, MemRefListTy::iterator, LeadingStore's int2PtrOffset
-    SmallVector<std::tuple<StoreInst*, int64_t, MemRefListTy::iterator, uint32_t>, 8>
+    SmallVector<std::tuple<StoreInst*, int64_t, MemRefListTy::iterator>, 8>
         StoresToMerge;
 
-    StoresToMerge.push_back(std::make_tuple(LeadingStore, 0, MI, 0));
+    StoresToMerge.push_back(std::make_tuple(LeadingStore, 0, MI));
 
     // Stores to be merged are scanned in the program order from the leading store
     // but need to be merged into the tailing store. So two edges of that
@@ -1090,7 +954,6 @@ bool MemOpt::mergeStore(StoreInst* LeadingStore,
             continue;
 
         int64_t Off = 0;
-        int64_t ArrayElem = 1; // default gap between elements is 1 entry
         const SCEVConstant* Offset
             = dyn_cast<SCEVConstant>(SE->getMinusSCEV(NextPtr, LeadingPtr));
         // Skip store with non-constant distance.
@@ -1102,7 +965,7 @@ bool MemOpt::mergeStore(StoreInst* LeadingStore,
                 LeadingStore->getPointerOperand(), LeadingSymPtr, CGC) ||
                 SymbolicPointer::decomposePointer(NextStore->getPointerOperand(),
                     NextSymPtr, CGC) ||
-                NextSymPtr.getConstantOffset(LeadingSymPtr, Off, ArrayElem))
+                NextSymPtr.getConstantOffset(LeadingSymPtr, Off))
                 continue;
         }
         else
@@ -1116,76 +979,15 @@ bool MemOpt::mergeStore(StoreInst* LeadingStore,
 
         unsigned NextStoreSize = unsigned(DL->getTypeStoreSize(NextStoreType));
 
-        bool enableTransposeSLM = false;
-        // LeadInt2PtrOffset is non-zero for enableTransposeSLM case so we can re-create
-        // new inttoptr inst.
-        uint32_t LeadInt2PtrOffset = 0;
-
-        // detect if we can merge non-sequential SLM stores
-        // Example
-        //   %54 = inttoptr i32 4608 to[144 x float] addrspace(3) *
-        //   %55 = getelementptr[144 x float], [144 x float] addrspace(3) * %54, i32 0, i32 % 36
-        //   %56 = inttoptr i32 5184 to[144 x float] addrspace(3) *
-        //   %57 = getelementptr[144 x float], [144 x float] addrspace(3) * %56, i32 0, i32 % 36
-        //   %58 = inttoptr i32 5760 to[144 x float] addrspace(3) *
-        //   %59 = getelementptr[144 x float], [144 x float] addrspace(3) * %58, i32 0, i32 % 36
-        //   store float% 51, float addrspace(3)*% 55, align 4
-        //   store float% 52, float addrspace(3)*% 57, align 4
-        //   store float% 53, float addrspace(3)*% 59, align 4
-        if (IGC_IS_FLAG_ENABLED(EnableMergeTransposeSLM)) {
-            unsigned int resourceIndex = 0;
-            bool direct = false;
-            BufferType bufType = IGC::DecodeAS4GFXResource(
-                LeadingStore->getPointerAddressSpace(), direct, resourceIndex);
-            if (SLM == bufType && (abs(Off) > ArrayElem)) {
-
-                if ((Off > 0 && Off != LastToLeading4Transpose + ArrayElem * NextStoreSize) ||
-                    (Off < 0 && (-Off) != (LeadingToFirst + ArrayElem * NextStoreSize)))
-                    continue;
-                else { // check if it matches the pattern for enableTransposeSLM
-                    GetElementPtrInst* LeadGEP = dyn_cast<GetElementPtrInst>(LeadingStore->getOperand(1));
-                    GetElementPtrInst* NextGEP = dyn_cast<GetElementPtrInst>(NextStore->getOperand(1));
-                    if (!LeadGEP || !NextGEP)
-                        continue;
-
-                    if ((LeadGEP->getOperand(1) != NextGEP->getOperand(1)) ||
-                        (LeadGEP->getOperand(2) != NextGEP->getOperand(2)))
-                        continue;
-
-                    if (!isa<IntToPtrInst>(LeadGEP->getPointerOperand()))
-                        continue;
-
-                    if (IntToPtrInst* Int2Ptr =
-                        dyn_cast<IntToPtrInst>(LeadGEP->getPointerOperand())) {
-                        if (const ConstantInt* CI =
-                            dyn_cast<ConstantInt>(Int2Ptr->getOperand(0))) {
-                            if (CI->getType()->isIntegerTy()) {
-                                LeadInt2PtrOffset = (uint32_t)CI->getZExtValue();
-                                enableTransposeSLM = true;
-                            }
-                        }
-                    } //  if (IntToPtrInst* Int2Ptr
-
-                    if (!enableTransposeSLM)
-                        continue;
-
-                    NumElts += getNumElements(NextStoreType);
-
-                    // Limit to 3 entries for merging
-                    if (NumElts > 3)
-                        break;
-                } // else { // check if it matches the pattern for enableTransposeSLM
-            } // if (SLM == bufType && ...
-        } else if ((Off > 0 && Off != LastToLeading) ||
-                   (Off < 0 && (-Off) != (LeadingToFirst + NextStoreSize)))
+        if ((Off > 0 && Off != LastToLeading) ||
+            (Off < 0 && (-Off) != (LeadingToFirst + NextStoreSize)))
             // Check it's consecutive to the current stores to be merged.
             continue;
-        else {
-            NumElts += getNumElements(NextStoreType);
-            // Bail out if the resulting vector store is already not profitable.
-            if (NumElts > profitVec[0])
-                break;
-        }
+
+        NumElts += getNumElements(NextStoreType);
+        // Bail out if the resulting vector store is already not profitable.
+        if (NumElts > profitVec[0])
+            break;
 
         // This store is to be merged. Remove it from check list.
         CheckList.pop_back();
@@ -1198,7 +1000,7 @@ bool MemOpt::mergeStore(StoreInst* LeadingStore,
         // Clear check list.
         CheckList.clear();
 
-        StoresToMerge.push_back(std::make_tuple(NextStore, Off, MI, LeadInt2PtrOffset));
+        StoresToMerge.push_back(std::make_tuple(NextStore, Off, MI));
 
         if (Off > 0) {
             LastToLeading = Off + NextStoreSize;
@@ -1218,7 +1020,7 @@ bool MemOpt::mergeStore(StoreInst* LeadingStore,
 
     // Tailing store is always the last one in the program order.
     StoreInst* TailingStore = std::get<0>(StoresToMerge.back());
-    IRBuilder<> Builder(TailingStore);
+    IGCLLVM::IRBuilder<> Builder(TailingStore);
 
     // Start to merge stores.
     NumElts = 0;
@@ -1267,44 +1069,6 @@ bool MemOpt::mergeStore(StoreInst* LeadingStore,
     // store values, extracting and inserting is necessary to avoid tracking uses
     // of each element in the original vector store value.
     unsigned Pos = 0;
-
-    Value* BitCastPtr = FirstStore->getPointerOperand();
-    uint32_t newInt2PtrOffset = std::get<3>(StoresToMerge.back());
-
-    // lookup 144
-    //   %55 = getelementptr [144 x float], [144 x float] addrspace(3)* %54, i32 0, i32 %36
-    uint32_t ArrayElem = 1;
-    if (GetElementPtrInst* GEP =
-        dyn_cast<GetElementPtrInst>(FirstStore->getPointerOperand())) {
-        Value* GEPptr = GEP->getPointerOperand();
-        if (GEPptr->getType()->isPointerTy()) {
-            Type* GEPElemType = GEPptr->getType()->getPointerElementType();
-            if (GEPElemType->isArrayTy())
-                ArrayElem = (uint32_t)GEPElemType->getArrayNumElements();
-        }
-    }
-
-    // If newInt2PtrOffset is non-zero, that means enableTransposeSLM is set
-    // Prepare new instructions
-    if (newInt2PtrOffset && ArrayElem) {
-        Type* newArrayType = PointerType::get(
-            ArrayType::get(LeadingStoreScalarType, ArrayElem * NumElts),
-            LeadingStore->getPointerAddressSpace());
-
-        Value* NewInt2Ptr = Builder.getInt32(newInt2PtrOffset);
-        NewInt2Ptr = createBitOrPointerCast(NewInt2Ptr, newArrayType, Builder);
-        GetElementPtrInst* LeadGEP =
-            dyn_cast<GetElementPtrInst>(FirstStore->getOperand(1));
-
-        // We need to adjust the buffer index by multiply by 3
-        // From x0, x1, ... y0, y1, ... z0, z1, ...
-        // To   x0, y0, z0, x1, y1, z1, ....
-        Value* NewGEPOffset =
-            Builder.CreateMul(LeadGEP->getOperand(2), Builder.getInt32(3));
-        Value* GEPArg[] = { LeadGEP->getOperand(1), NewGEPOffset };
-        BitCastPtr = Builder.CreateGEP(NewInt2Ptr, GEPArg);
-    }
-
     for (auto& I : StoresToMerge) {
         Value* Val = std::get<0>(I)->getValueOperand();
         Type* Ty = Val->getType();
@@ -1312,7 +1076,7 @@ bool MemOpt::mergeStore(StoreInst* LeadingStore,
         IGC_ASSERT(hasSameSize(ScalarTy, LeadingStoreScalarType));
 
         if (Ty->isVectorTy()) {
-            for (unsigned i = 0, e = (unsigned)cast<VectorType>(Ty)->getNumElements(); i != e; ++i) {
+            for (unsigned i = 0, e = (unsigned)cast<IGCLLVM::FixedVectorType>(Ty)->getNumElements(); i != e; ++i) {
                 Value* Ex = Builder.CreateExtractElement(Val, Builder.getInt32(i));
                 Ex = createBitOrPointerCast(Ex, LeadingStoreScalarType, Builder);
                 NewStoreVal = Builder.CreateInsertElement(NewStoreVal, Ex,
@@ -1355,7 +1119,7 @@ bool MemOpt::mergeStore(StoreInst* LeadingStore,
     Type* NewPointerType =
         PointerType::get(NewStoreType, LeadingStore->getPointerAddressSpace());
     Value* NewPointer =
-        Builder.CreateBitCast(BitCastPtr, NewPointerType);
+        Builder.CreateBitCast(FirstStore->getPointerOperand(), NewPointerType);
     StoreInst* NewStore =
         Builder.CreateAlignedStore(NewStoreVal, NewPointer,
             IGCLLVM::getAlign(FirstStore->getAlignment()));
@@ -1416,7 +1180,7 @@ bool MemOpt::isSafeToMergeLoad(const LoadInst* Ld,
 /// specified store set to any one in the check list, which may read/write to
 /// that location.
 bool MemOpt::isSafeToMergeStores(
-    const SmallVectorImpl<std::tuple<StoreInst*, int64_t, MemRefListTy::iterator, uint32_t> >& Stores,
+    const SmallVectorImpl<std::tuple<StoreInst*, int64_t, MemRefListTy::iterator> >& Stores,
     const SmallVectorImpl<Instruction*>& CheckList) const {
     // Arrange CheckList as the outer loop to favor the case where there are
     // back-to-back stores only.
@@ -1760,7 +1524,7 @@ SymbolicPointer::decomposePointer(const Value* Ptr, SymbolicPointer& SymPtr,
             return false;
         }
 
-        if (Op->getOpcode() == Instruction::BitCast) {
+        if (Op->getOpcode() == Instruction::BitCast || Op->getOpcode() == Instruction::AddrSpaceCast) {
             Ptr = Op->getOperand(0);
             continue;
         }
