@@ -1,28 +1,11 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
+============================= end_copyright_notice ===========================*/
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
 #pragma once
 #include "Compiler/CISACodeGen/CISACodeGen.h"
 #include "Compiler/CISACodeGen/helper.h"
@@ -87,6 +70,9 @@ namespace IGC
     {
         virtual void Emit(EmitPass* pass, const DstModifier& modifier) = 0;
         virtual ~Pattern() {}
+
+        // Does the pattern allow destination saturation?
+        virtual bool supportsSaturate() { return true; }
     };
 
     struct SDAG
@@ -171,6 +157,7 @@ namespace IGC
         void visitFPToUIInst(llvm::FPToUIInst& I);
         void visitDbgInfoIntrinsic(llvm::DbgInfoIntrinsic& I);
         void visitExtractValueInst(llvm::ExtractValueInst& I);
+        void visitInsertValueInst(llvm::InsertValueInst& I);
         void visitBranchInst(llvm::BranchInst& I);
 
     public:
@@ -189,6 +176,7 @@ namespace IGC
 
         bool MatchFMA(llvm::IntrinsicInst& I);
         bool MatchFrc(llvm::BinaryOperator& I);
+        bool MatchFloor(llvm::BinaryOperator& I);
         bool MatchPredAdd(llvm::BinaryOperator& I);
         bool MatchSimpleAdd(llvm::BinaryOperator& I);
         bool MatchMad(llvm::BinaryOperator& I);
@@ -211,7 +199,12 @@ namespace IGC
         bool MatchPow(llvm::IntrinsicInst& I);
         bool MatchCondModifier(llvm::CmpInst& I);
         bool MatchBoolOp(llvm::BinaryOperator& I);
-        bool MatchRotate(llvm::Instruction& I);
+        bool MatchFunnelShiftRotate(llvm::IntrinsicInst& I);
+        bool MatchImmOffsetLSC(llvm::Instruction& I);
+        bool MatchBfn(llvm::Instruction& I);
+        bool MatchCmpSelect(llvm::SelectInst& I);
+        bool MatchAdd3(llvm::Instruction& I);
+        bool MatchDpas(llvm::GenIntrinsicInst& I);
         bool MatchDp4a(llvm::GenIntrinsicInst& I);
         bool MatchLogicAlu(llvm::BinaryOperator& I);
         bool MatchRsqrt(llvm::BinaryOperator& I);
@@ -242,6 +235,9 @@ namespace IGC
 
         bool MatchUnmaskedRegionBoundary(llvm::Instruction& I, bool start);
 
+        bool MatchInsertToStruct(llvm::InsertValueInst*);
+        bool MatchExtractFromStruct(llvm::ExtractValueInst*);
+
         void AddPattern(Pattern* P)
         {
             m_currentPattern = P;
@@ -273,7 +269,7 @@ namespace IGC
 
         bool isUniform(const llvm::Value* V) const
         {
-            return m_WI && (m_WI->whichDepend(V) == WIAnalysis::UNIFORM);
+            return m_WI && (m_WI->isUniform(V));
         };
 
     public:
@@ -319,12 +315,15 @@ namespace IGC
         // Find bool values that will be emitted as uniform variables.
         // Otherwise they will be expanded to the SIMD size, by default.
         void gatherUniformBools(llvm::Value* Val);
+
+        // Return true if it is dbg related instruction
+        bool IsDbgInst(llvm::Instruction& inst) const;
     };
 
     //helper
     bool GetModifier(llvm::Instruction& modifier, e_modifier& mod, llvm::Value*& source);
     bool GetModifier(llvm::Value& modifier, e_modifier& mod, llvm::Value*& source);
-    bool IsNegate(llvm::BinaryOperator& modifier, llvm::Value*& negateSource);
+    bool IsNegate(llvm::Instruction* modifier, llvm::Value*& negateSource);
     bool IsZero(llvm::Value* v);
     bool isAbs(llvm::Value* abs, e_modifier& mod, llvm::Value*& source);
     bool isSat(llvm::Instruction* sat, llvm::Value*& source, bool& isUnsigned);
