@@ -145,10 +145,6 @@ void CPixelShader::AllocatePSPayload()
         for (uint i = 0; i < GetR1()->GetNumberInstance(); i++)
         {
             AllocateInput(GetR1(), offset, i, forceLiveOut);
-            for (auto R1Lo: GetR1Lo()) {
-                AllocateInput(R1Lo, offset, i, forceLiveOut);
-            }
-
             offset += getGRFSize();
         }
     }
@@ -268,6 +264,24 @@ void CPixelShader::AllocatePSPayload()
     // allocate space for NOS constants and pushed constants
     AllocateConstants3DShader(offset);
 
+    // RPC0 and subsequent regs: per-primitive constant data
+    for (uint index = 0; index < perPrimitiveSetup.size(); index++)
+    {
+        CVariable* var = perPrimitiveSetup[index];
+        if (var && var->GetAlias() == NULL)
+        {
+            AllocateInput(var, offset);
+        }
+
+        if (IsDualSIMD8())
+        {
+            offset += 2 * getGRFSize();
+        }
+        else
+        {
+            offset += getGRFSize();
+        }
+    }
 
     // Allocate size for attributes coming from VS
     IGC_ASSERT(offset % getGRFSize() == 0);
@@ -285,7 +299,7 @@ void CPixelShader::AllocatePSPayload()
                 }
             }
 
-            AllocateInput(setup[i], offset + subRegOffset);
+            AllocateInput(setup[i], offset + subRegOffset, 0, forceLiveOut);
 
             if (m_Signature)
             {
@@ -342,7 +356,6 @@ void CPixelShader::AllocatePSPayload()
         return;
     }
 
-    offset = payloadEnd;
 
     // create output registers for coarse phase
     for (const auto& it : m_CoarseOutput)
@@ -507,6 +520,37 @@ CVariable* CPixelShader::GetBaryRegLoweredFloat(e_interpolation mode)
 }
 
 
+CVariable* CPixelShader::GetPerPrimitiveSetupVar(uint inputIndex)
+{
+
+    // A single setupVar holds up to GRF of per-primitive data.
+    const uint grfSizeInDwords = getGRFSize() / sizeof(DWORD);
+    const uint setupIndex = inputIndex / grfSizeInDwords;
+    if (setupIndex >= perPrimitiveSetup.size())
+    {
+        perPrimitiveSetup.resize(setupIndex + 1, nullptr);
+    }
+
+    CVariable* inputVar = perPrimitiveSetup[setupIndex];
+    if (inputVar == nullptr)
+    {
+        if (IsDualSIMD8())
+        {
+            // In dual-simd8 mode per-primitive data is interleaved.
+            // Each piece of 32 bytes of data for Primitive0 is followed by
+            // 32 bytes of data for Primitve1.
+
+            // Use 2 GRFs per 32 bytes of data.
+            inputVar = GetNewVariable(16, ISA_TYPE_F, EALIGN_GRF, CName::NONE);
+        }
+        else
+        {
+            inputVar = GetNewVariable(grfSizeInDwords, ISA_TYPE_F, EALIGN_GRF, CName::NONE);
+        }
+        perPrimitiveSetup[setupIndex] = inputVar;
+    }
+    return inputVar;
+}
 
 CVariable* CPixelShader::GetInputDelta(uint index, bool loweredInput)
 {
@@ -517,7 +561,7 @@ CVariable* CPixelShader::GetInputDelta(uint index, bool loweredInput)
         {
             if (index % 2 == 0)
             {
-                inputVar = GetNewVariable(8, ISA_TYPE_F, EALIGN_GRF, true, CName::NONE);
+                inputVar = GetNewVariable(8, ISA_TYPE_F, EALIGN_GRF, true, "InputDelta" + std::to_string(index));
                 setup[index + 1] = GetNewAlias(inputVar, ISA_TYPE_F, 16, 4);
             }
             else
@@ -527,7 +571,7 @@ CVariable* CPixelShader::GetInputDelta(uint index, bool loweredInput)
         }
         else
         {
-            inputVar = GetNewVariable(4, ISA_TYPE_F, EALIGN_OWORD, true, CName::NONE);
+            inputVar = GetNewVariable(4, ISA_TYPE_F, EALIGN_OWORD, true, "InputDelta" + std::to_string(index));
         }
         setup[index] = inputVar;
     }
@@ -831,6 +875,8 @@ void CPixelShader::FillProgram(SPixelShaderKernelProgram* pKernelProgram)
 {
     const PixelShaderInfo& psInfo = GetContext()->getModuleMetaData()->psInfo;
 
+    pKernelProgram->m_StagingCtx = GetContext()->m_StagingCtx;
+    pKernelProgram->m_RequestStage2 = RequestStage2(GetContext()->m_CgFlag, GetContext()->m_StagingCtx);
     pKernelProgram->blendToFillEnabled = psInfo.blendToFillEnabled;
     pKernelProgram->forceEarlyZ = psInfo.forceEarlyZ;
 
@@ -874,6 +920,12 @@ void CPixelShader::FillProgram(SPixelShaderKernelProgram* pKernelProgram)
             mdconst::dyn_extract<ConstantInt>(primIdNod->getOperand(0)->getOperand(0))->getZExtValue());
         pKernelProgram->hasPrimID = true;
     }
+    if (NamedMDNode* PointCoordNod = entry->getParent()->getNamedMetadata("PointCoordLocation"))
+    {
+        pKernelProgram->pointCoordLocation = int_cast<uint>(
+            mdconst::dyn_extract<ConstantInt>(PointCoordNod->getOperand(0)->getOperand(0))->getZExtValue());
+        pKernelProgram->hasPointCoord = true;
+    }
 
     pKernelProgram->posXYOffsetEnable = m_pPositionXYOffset ? true : false;
     pKernelProgram->VectorMask = m_VectorMask;
@@ -1261,6 +1313,8 @@ bool CPixelShader::CompileSIMDSize(SIMDMode simdMode, EmitPass& EP, llvm::Functi
     {
         if (simdMode == SIMDMode::SIMD32)
             return false;
+        if (EP.m_ShaderDispatchMode == ShaderDispatchMode::DUAL_SIMD8)
+            return false;
     }
 
     if (m_HasoStencil && !ctx->platform.supportsStencil(simdMode))
@@ -1486,24 +1540,23 @@ void CodeGen(PixelShaderContext* ctx)
         pMdUtils = ctx->getMetaDataUtils();
     }
 
-    if (coarsePhase && pixelPhase)
-   {
-        CShaderProgram::KernelShaderMap coarseShaders;
-        CShaderProgram::KernelShaderMap pixelShaders;
+    bool codegenDone = false;
 
+    CShaderProgram::KernelShaderMap coarseShaders;
+    CShaderProgram::KernelShaderMap pixelShaders;
+
+    if (coarsePhase && pixelPhase)
+    {
         // Cancelling staged compilation for multi stage PS.
         ctx->m_CgFlag = FLAG_CG_ALL_SIMDS;
 
         //Multi stage PS, need to do separate compiler and link them
         unsigned int numStage = 2;
         PSSignature signature;
-        SPixelShaderKernelProgram outputs[2];
         FunctionInfoMetaDataHandle coarseFI, pixelFI;
         coarseFI = pMdUtils->getFunctionsInfoItem(coarsePhase);
         pixelFI = pMdUtils->getFunctionsInfoItem(pixelPhase);
 
-        memset(&outputs, 0, 2 * sizeof(SPixelShaderKernelProgram));
-
         for (unsigned int i = 0; i < numStage; i++)
         {
             Function* phaseFunc = (i == 0) ? coarsePhase : pixelPhase;
@@ -1526,6 +1579,8 @@ void CodeGen(PixelShaderContext* ctx)
             }
         }
 
+        codegenDone = true;
+
 
         for (unsigned int i = 0; i < numStage; i++)
         {
@@ -1534,35 +1589,82 @@ void CodeGen(PixelShaderContext* ctx)
                 mdconst::dyn_extract<Function>(pixelNode->getOperand(0)->getOperand(0));
 
             CShaderProgram::KernelShaderMap& shaders = (i == 0) ? coarseShaders : pixelShaders;
-
-            shaders[phaseFunc]->FillProgram(&outputs[i]);
-            COMPILER_SHADER_STATS_PRINT(shaders[phaseFunc]->m_shaderStats, ShaderType::PIXEL_SHADER, ctx->hash, "");
-            COMPILER_SHADER_STATS_SUM(ctx->m_sumShaderStats, shaders[phaseFunc]->m_shaderStats, ShaderType::PIXEL_SHADER);
-            COMPILER_SHADER_STATS_DEL(shaders[phaseFunc]->m_shaderStats);
-            delete shaders[phaseFunc];
+            CPixelShader* simd8Shader = static_cast<CPixelShader*>(shaders[phaseFunc]->GetShader(SIMDMode::SIMD8));
+            CPixelShader* simd16Shader = static_cast<CPixelShader*>(shaders[phaseFunc]->GetShader(SIMDMode::SIMD16));
+            CPixelShader* simd32Shader = static_cast<CPixelShader*>(shaders[phaseFunc]->GetShader(SIMDMode::SIMD32));
+            if (!((simd8Shader && simd8Shader->ProgramOutput()->m_programBin) ||
+                (simd16Shader && simd16Shader->ProgramOutput()->m_programBin) ||
+                (simd32Shader && simd32Shader->ProgramOutput()->m_programBin)
+                ))
+            {
+                shaders[phaseFunc]->DeleteShader(SIMDMode::SIMD8);
+                shaders[phaseFunc]->DeleteShader(SIMDMode::SIMD16);
+                shaders[phaseFunc]->DeleteShader(SIMDMode::SIMD32);
+                if (i == 0)
+                {
+                    delete shaders[coarsePhase];
+                    coarsePhase = nullptr;
+                }
+                else
+                {
+                    delete shaders[pixelPhase];
+                    pixelPhase = nullptr;
+                }
+            }
         }
 
-        linkCPS(outputs, ctx->programOutput, numStage);
-        // Kernels allocated in CISABuilder.cpp (Compile())
-        // are freed in CompilerOutputOGL.hpp (DeleteShaderCompilerOutputOGL())
-        // in case of CPS multistage PS they are separated.
-        // Need to free original kernels here as DeleteShaderCompilerOutputOGL()
-        // will clear new allocations for separated phases in this case.
-        for (unsigned int i = 0; i < numStage; i++)
+        if (coarsePhase && pixelPhase)
         {
-            outputs[i].simd8.Destroy();
-            outputs[i].simd16.Destroy();
-            outputs[i].simd32.Destroy();
+            SPixelShaderKernelProgram outputs[2] = { SPixelShaderKernelProgram(), SPixelShaderKernelProgram() };
+            //memset(&outputs, 0, 2 * sizeof(SPixelShaderKernelProgram));
+
+            for (unsigned int i = 0; i < numStage; i++)
+            {
+                Function* phaseFunc = (i == 0) ?
+                    mdconst::dyn_extract<Function>(coarseNode->getOperand(0)->getOperand(0)) :
+                    mdconst::dyn_extract<Function>(pixelNode->getOperand(0)->getOperand(0));
+
+                CShaderProgram::KernelShaderMap& shaders = (i == 0) ? coarseShaders : pixelShaders;
+
+                shaders[phaseFunc]->FillProgram(&outputs[i]);
+                COMPILER_SHADER_STATS_PRINT(shaders[phaseFunc]->m_shaderStats, ShaderType::PIXEL_SHADER, ctx->hash, "");
+                COMPILER_SHADER_STATS_SUM(ctx->m_sumShaderStats, shaders[phaseFunc]->m_shaderStats, ShaderType::PIXEL_SHADER);
+                COMPILER_SHADER_STATS_DEL(shaders[phaseFunc]->m_shaderStats);
+                delete shaders[phaseFunc];
+            }
+
+            linkCPS(outputs, ctx->programOutput, numStage);
+            // Kernels allocated in CISABuilder.cpp (Compile())
+            // are freed in CompilerOutputOGL.hpp (DeleteShaderCompilerOutputOGL())
+            // in case of CPS multistage PS they are separated.
+            // Need to free original kernels here as DeleteShaderCompilerOutputOGL()
+            // will clear new allocations for separated phases in this case.
+            for (unsigned int i = 0; i < numStage; i++)
+            {
+                outputs[i].simd8.Destroy();
+                outputs[i].simd16.Destroy();
+                outputs[i].simd32.Destroy();
+            }
         }
     }
-    else
+
+    if (!(coarsePhase && pixelPhase))
     {
         CShaderProgram::KernelShaderMap shaders;
+        Function* pFunc = nullptr;
+
+        if (!codegenDone)
+        {
+            // Single PS
+            CodeGen(ctx, shaders);
+            pFunc = getUniqueEntryFunc(ctx->getMetaDataUtils(), ctx->getModuleMetaData());
+        }
+        else
+        {
+            shaders = coarsePhase ? coarseShaders : pixelShaders;
+            pFunc = coarsePhase ? coarsePhase : pixelPhase;
+        }
 
-        // Single PS
-        CodeGen(ctx, shaders);
-        // Assuming single shader information in metadata
-        Function* pFunc = getUniqueEntryFunc(ctx->getMetaDataUtils(), ctx->getModuleMetaData());
         // gather data to send back to the driver
         shaders[pFunc]->FillProgram(&ctx->programOutput);
         COMPILER_SHADER_STATS_PRINT(shaders[pFunc]->m_shaderStats, ShaderType::PIXEL_SHADER, ctx->hash, "");
@@ -1570,7 +1672,6 @@ void CodeGen(PixelShaderContext* ctx)
         COMPILER_SHADER_STATS_DEL(shaders[pFunc]->m_shaderStats);
         delete shaders[pFunc];
     }
-
 }
 
 void CPixelShader::CreatePassThroughVar()
