@@ -14,6 +14,8 @@ SPDX-License-Identifier: MIT
 #include "Probe/Assertion.h"
 #include "common/igc_regkeys.hpp"
 
+#include "../../../skuwa/iacm_g10_rev_id.h"
+#include "../../../skuwa/iacm_g11_rev_id.h"
 
 namespace IGC
 {
@@ -131,6 +133,38 @@ bool supportMSAARateInPayload() const
 bool support16BitImmSrcForMad() const {
     return (m_platformInfo.eRenderCoreFamily >= IGFX_GEN10_CORE);
 }
+
+//This function is for handling special cases,
+//where simple >= comparison between products/cores doesn't produce expected results.
+bool isProductChildOf(PRODUCT_FAMILY product) const
+{
+    bool result;
+    switch (product) {
+    case IGFX_PVC:
+        result = m_platformInfo.eRenderCoreFamily >= IGFX_XE_HPC_CORE;
+        break;
+    case IGFX_XE_HP_SDV:
+                result = m_platformInfo.eProductFamily == IGFX_XE_HP_SDV
+                || m_platformInfo.eProductFamily == IGFX_DG2
+                || m_platformInfo.eProductFamily == IGFX_PVC;
+        break;
+    case IGFX_DG2:
+        result = m_platformInfo.eProductFamily == IGFX_DG2
+                || m_platformInfo.eProductFamily == IGFX_PVC;
+        break;
+    default:
+        result = m_platformInfo.eProductFamily >= product;
+        break;
+    }
+    return result;
+}
+
+bool supports8DWLSCMessage() const {
+    return (SI_WA_FROM(m_platformInfo.usRevId, ACM_G10_GT_REV_ID_B0) && m_platformInfo.eProductFamily == IGFX_DG2)
+        || GFX_IS_DG2_G11_CONFIG(m_platformInfo.usDeviceID)
+        || m_platformInfo.eProductFamily == IGFX_PVC;
+}
+
 PRODUCT_FAMILY GetProductFamily() const { return m_platformInfo.eProductFamily; }
 unsigned short GetDeviceId() const { return m_platformInfo.usDeviceID; }
 unsigned short GetRevId() const { return m_platformInfo.usRevId; }
@@ -295,7 +329,7 @@ bool supportsThreadCombining() const
     return (!(!m_WaTable.WaEnablePooledEuFor2x6 &&
         m_platformInfo.eProductFamily == IGFX_BROXTON &&
         m_GTSystemInfo.SubSliceCount == 2))
-        ;
+        && (m_platformInfo.eRenderCoreFamily < IGFX_GEN12_CORE);
 }
 
 bool enableMaxWorkGroupSizeCalculation() const
@@ -393,7 +427,8 @@ bool disableStaticVertexCount() const
 
 bool hasSamplerSupport() const
 {
-    return true;
+    return (m_platformInfo.eProductFamily != IGFX_PVC) ||
+        (IGC_IS_FLAG_ENABLED(EnableSamplerSupport)); // flag for IGFX_PVC
 }
 
 uint32_t getMinPushConstantBufferAlignment() const
@@ -424,6 +459,10 @@ bool hasDualSubSlices() const
 {
     bool hasDualSS = m_platformInfo.eRenderCoreFamily == IGFX_GEN12_CORE ||
         m_platformInfo.eRenderCoreFamily == IGFX_GEN12LP_CORE;
+    if (m_platformInfo.eRenderCoreFamily == IGFX_XE_HPG_CORE)
+    {
+        hasDualSS = true;
+    }
     return hasDualSS;
 }
 
@@ -452,138 +491,576 @@ bool canForcePrivateToGlobal() const
 
 bool getHWTIDFromSR0() const
 {
-    return isXeHPSDVPlus();
+    return isProductChildOf(IGFX_XE_HP_SDV);
 }
 
 bool supportAdd3Instruction() const
 {
-    return isXeHPSDVPlus();
+    return isProductChildOf(IGFX_XE_HP_SDV);
 }
 
 bool supportBfnInstruction() const
 {
-    return isXeHPSDVPlus();
+    return isProductChildOf(IGFX_XE_HP_SDV);
 }
 
 bool supportDpasInstruction() const
 {
-    return isXeHPSDVPlus();
+    return isProductChildOf(IGFX_XE_HP_SDV);
+}
+
+bool hasPackedRestrictedFloatVector() const
+{
+    return true;
 }
 
 bool supportLoadThreadPayloadForCompute() const
 {
-    return isXeHPSDVPlus();
+    return isProductChildOf(IGFX_XE_HP_SDV);
+}
+
+bool support16BitAtomics() const
+{
+    return m_platformInfo.eProductFamily >= IGFX_GEN12_CORE;
 }
 
 bool Enable32BitIntDivRemEmu() const
 {
-    return isXeHPSDVPlus();
+    return isProductChildOf(IGFX_XE_HP_SDV);
 }
 
 bool support16DWURBWrite() const
 {
-    return IGC_IS_FLAG_ENABLED(Enable16DWURBWrite) && isXeHPSDVPlus();
+    return IGC_IS_FLAG_ENABLED(Enable16DWURBWrite) && isProductChildOf(IGFX_XE_HP_SDV);
 }
 
 bool hasScratchSurface() const
 {
-    return isXeHPSDVPlus();
+    return isProductChildOf(IGFX_XE_HP_SDV);
 }
 
 bool hasAtomicPreDec() const
 {
-    return !isXeHPSDVPlus();
+    return !isProductChildOf(IGFX_XE_HP_SDV);
 }
 
 bool support26BitBSOFormat() const
 {
-    return isXeHPSDVPlus();
+    return isProductChildOf(IGFX_XE_HP_SDV);
 }
 
 bool needsHeaderForAtomicCounter() const
 {
-    return isXeHPSDVPlus();
+    return isProductChildOf(IGFX_XE_HP_SDV);
 }
 
 bool doScalar64bScan() const
 {
-    return isXeHPSDVPlus();
+    return isProductChildOf(IGFX_XE_HP_SDV);
 }
 
 bool hasHWLocalThreadID() const
 {
-    return isXeHPSDVPlus();
+    return isProductChildOf(IGFX_XE_HP_SDV);
 }
 
 unsigned int getOfflineCompilerMaxWorkGroupSize() const
 {
-    if (isXeHPSDVPlus())
+    if (isProductChildOf(IGFX_XE_HP_SDV))
         return 1024;
     return 448;
 }
 
 bool hasFP16AtomicMinMax() const
 {
-    return isXeHPSDVPlus();
+    return isProductChildOf(IGFX_XE_HP_SDV);
 }
 
 bool hasFP32GlobalAtomicAdd() const
 {
-    return isXeHPSDVPlus();
+    return isProductChildOf(IGFX_XE_HP_SDV);
 }
 
 bool has16OWSLMBlockRW() const
 {
-    return IGC_IS_FLAG_ENABLED(Enable16OWSLMBlockRW) && isXeHPSDVPlus();
+    return IGC_IS_FLAG_ENABLED(Enable16OWSLMBlockRW) && isProductChildOf(IGFX_XE_HP_SDV);
 }
 
 bool supportInlineData() const
 {
-    return isXeHPSDVPlus();
+    return isProductChildOf(IGFX_XE_HP_SDV);
+}
+
+// TODO: temporary solution, remove this once it's not needed
+bool supportInlineDataOCL() const
+{
+    if (m_platformInfo.eProductFamily == IGFX_PVC)
+    {
+        return false;
+    }
+    return isProductChildOf(IGFX_XE_HP_SDV);
 }
 
 bool supportsAutoGRFSelection() const
 {
-    return m_platformInfo.eProductFamily == IGFX_XE_HP_SDV;
+    return isProductChildOf(IGFX_PVC) || m_platformInfo.eProductFamily == IGFX_XE_HP_SDV || (m_platformInfo.eProductFamily == IGFX_DG2 && IGC_IS_FLAG_ENABLED(ForceSupportsAutoGRFSelection));
 }
 
-bool isXeHPSDVPlus() const
+float adjustedSpillThreshold() const
 {
-    return m_platformInfo.eProductFamily >= IGFX_XE_HP_SDV;
+    return isProductChildOf(IGFX_DG2) ? 9.0f : 12.0f;
 }
 
-bool supportInlineDataOCL() const
+bool hasLSC() const
 {
-    return isXeHPSDVPlus();
+    return IGC_IS_FLAG_DISABLED(ForceNoLSC) && !WaEnableLSCBackupMode() && isProductChildOf(IGFX_DG2);
 }
 
-bool has64BMediaBlockRW() const
+bool WaEnableLSCBackupMode() const
 {
-    return IGC_IS_FLAG_ENABLED(Enable64BMediaBlockRW) && isXeHPSDVPlus();
+    return (m_WaTable.Wa_14010198302 != 0);
 }
 
-bool hasNoFullI64Support() const
+bool supportQWRotateInstructions() const
 {
-    return hasNoInt64Inst();
+    return m_platformInfo.eProductFamily == IGFX_PVC && IGC_IS_FLAG_ENABLED(EnableQWRotateInstructions);
 }
 
-bool hasNoInt64AddInst() const
+bool loosenSimd32occu() const
 {
-    return hasNoFullI64Support();
+    return (m_platformInfo.eRenderCoreFamily >= IGFX_GEN12_CORE && m_platformInfo.eProductFamily != IGFX_DG2);
+}
+
+// Local memory here refers to memory on the device- e.g. HBM for PVC.
+bool hasLocalMemory() const
+{
+    return m_SkuTable.FtrLocalMemory != 0;
+}
+
+bool enableImmConstantOpt() const
+{
+    return !isProductChildOf(IGFX_XE_HP_SDV);
+}
+
+bool supportsTier2VRS() const
+{
+    return m_platformInfo.eProductFamily == IGFX_DG2;
 }
 
 bool supportsSIMD16TypedRW() const
 {
+    return m_platformInfo.eProductFamily == IGFX_PVC;
+}
+
+bool supportHWGenerateTID() const
+{
+    return IGC_IS_FLAG_ENABLED(EnableHWGenerateThreadID) && isProductChildOf(IGFX_DG2);
+}
+
+bool hasHalfSIMDLSC() const
+{
+    return (m_platformInfo.eProductFamily == IGFX_DG2 && SI_WA_FROM(m_platformInfo.usRevId, ACM_G10_GT_REV_ID_B0)) ||
+        GFX_IS_DG2_G11_CONFIG(m_platformInfo.usDeviceID) ||
+        // false for PVC XL A0 RevID==0x0, true from PVC XT A0 RevID==0x3==REVISION_B
+        (m_platformInfo.eProductFamily == IGFX_PVC && m_platformInfo.usRevId >= REVISION_B);
+}
+
+bool NeedsLSCFenceUGMBeforeEOT() const
+{
+    return getWATable().Wa_22013689345 != 0;
+}
+
+bool hasPartialInt64Support() const
+{
+    // false for PVC XL A0 RevID==0x0, true from PVC XT A0 RevID==0x3==REVISION_B
+    return (m_platformInfo.eProductFamily == IGFX_PVC && m_platformInfo.usRevId >= REVISION_B) ||
+        IGC_IS_FLAG_ENABLED(ForcePartialInt64);
+}
+
+bool hasNoInt64AddInst() const
+{
+    // to be changed for PVC-XT with Add Int64 support;
+    return hasNoFullI64Support() && !hasQWAddSupport();
+}
+
+bool hasQWAddSupport() const
+{
+    return (m_platformInfo.eProductFamily == IGFX_PVC &&
+        ((m_platformInfo.usRevId >= REVISION_D && IGC_IS_FLAG_ENABLED(EnableQWAddSupport))  // true from PVC XT B0 RevID==0x5==REVISION_D
+        || IGC_IS_FLAG_ENABLED(ForceQWAddSupport))); // back door way to enable feature along with ForcePartialInt64 - needed to perform the experiments on PVC-A
+}
+
+bool hasExecSize16DPAS() const
+{
+    return m_platformInfo.eProductFamily == IGFX_PVC;
+}
+
+bool LSCSimd1NeedFullPayload() const
+{
+    // in PVC XL A0 RevID=0x0, SIMD1 reads/writes need full payloads
+    // this causes chaos for vISA (would need 4REG alignment)
+    // and to make extra moves to enable the payload
+    // PVC XT A0 RevID==0x3==REVISION_B gets this feature
+    return (m_platformInfo.eProductFamily == IGFX_PVC &&
+        m_platformInfo.usRevId < REVISION_B);
+}
+
+bool hasNoFullI64Support() const
+{
+    return (hasNoInt64Inst() || hasPartialInt64Support());
+}
+
+// This returns the current maximum size that we recommend for performance.
+// SIMD32 is still allowed and we may relax this in the future.
+SIMDMode getMaxRayQuerySIMDSize() const
+{
+    if (m_platformInfo.eProductFamily <= IGFX_PVC)
+    {
+        return SIMDMode::SIMD16;
+    }
+    else
+    {
+        IGC_ASSERT_MESSAGE(0, "Change code here to support new platform!");
+        return SIMDMode::UNKNOWN;
+    }
+}
+
+SIMDMode getPreferredRayTracingSIMDSize() const
+{
+    switch (m_platformInfo.eProductFamily)
+    {
+    case IGFX_PVC:
+        return SIMDMode::SIMD16;
+    default:
+        return SIMDMode::SIMD8;
+    }
+}
+
+bool supportRayTracing() const
+{
+    return isProductChildOf(IGFX_DG2);
+}
+
+bool isValidNumThreads(uint32_t numThreadsPerEU) const
+{
+    return numThreadsPerEU == 4 || numThreadsPerEU == 8;
+}
+
+bool supports3DAndCubeSampleD() const
+{
+    // Make sure to match hasSupportForSampleDOnCubeTextures LLVM3DBuilder\BuiltinsFrontend.hpp
+    return (
+        m_platformInfo.eProductFamily != IGFX_XE_HP_SDV &&
+        m_platformInfo.eProductFamily != IGFX_DG2
+        ) ||
+        IGC_IS_FLAG_DISABLED(EnableSampleDEmulation);
+}
+
+bool LSCEnabled(SIMDMode m = SIMDMode::UNKNOWN) const
+{
+    if (IGC_IS_FLAG_ENABLED(EnableLSC))
+        return true;
+    if (hasLSC())
+    {
+        switch (m_platformInfo.eProductFamily)
+        {
+        case IGFX_PVC:
+            if (m == SIMDMode::UNKNOWN)
+            {
+                // Must generate LSC from PVC XT A0 RevID==0x3==REVISION_B (not include XL A0)
+                return m_platformInfo.usRevId >= REVISION_B;
+            }
+            return (m == SIMDMode::SIMD16 && m_platformInfo.usRevId > REVISION_A0)
+                || m == SIMDMode::SIMD32;
+        case IGFX_DG2:
+            if (m == SIMDMode::UNKNOWN)
+            {
+                // Must generate LSC after A0 (not include A0)
+                return (SI_WA_FROM(m_platformInfo.usRevId, ACM_G10_GT_REV_ID_B0) || GFX_IS_DG2_G11_CONFIG(m_platformInfo.usDeviceID));
+            }
+            return ((SI_WA_FROM(m_platformInfo.usRevId, ACM_G10_GT_REV_ID_B0) || GFX_IS_DG2_G11_CONFIG(m_platformInfo.usDeviceID))
+                && m == SIMDMode::SIMD8) ||
+                m == SIMDMode::SIMD16;
+        default:
+            return true;
+        }
+    }
     return false;
 }
 
+// Max LSC message block size is 8GRF
+uint32_t getMaxLSCBlockMsgSize(bool isD64 = true) const
+{
+    return (isD64 ? 8 : 4) * getGRFSize();
+}
+
+bool hasURBFence() const
+{
+    return m_platformInfo.eProductFamily == IGFX_DG2;
+}
+
+bool hasMultiTile() const
+{
+    //FIXME: what to do for AOT compile?
+    return m_GTSystemInfo.MultiTileArchInfo.TileCount > 1;
+}
+
+// UGM LSC fence with GPU scope triggers L3 flush
+bool hasL3FlushOnGPUScopeInvalidate() const
+{
+    return m_platformInfo.eProductFamily == IGFX_DG2;
+}
+
+bool L3CacheCoherentCrossTiles() const {
+    return m_platformInfo.eProductFamily == IGFX_PVC;
+}
+
+bool AllowFenceOpt() const
+{
+    return ((m_platformInfo.eProductFamily == IGFX_DG2
+        || m_platformInfo.eProductFamily == IGFX_PVC) &&
+        IGC_IS_FLAG_ENABLED(EnablePlatformFenceOpt));
+}
+
+//if RayTracing fence WA for LSCBackupMode (DG2.A0 only actually) is enabled.
+//In case of RayQuery of DG2.A0, we have to flush L1 after RTWrite and before shader read.
+//The reason is, for RT writes, write - completion must guarantee that
+//subsequent read with LSC bypass can read.This issue can be present
+//even in async mode too but in sync mode read happens rather quickly
+//compared to BTD dispatched thread.
+bool RTFenceWAforBkModeEnabled() const
+{
+    return WaEnableLSCBackupMode();
+}
+
+SIMDMode getMinDispatchMode() const
+{
+    if (m_platformInfo.eProductFamily == IGFX_PVC)
+    {
+        return SIMDMode::SIMD16;
+    }
+    return SIMDMode::SIMD8;
+}
+
+bool hasLSCTypedMessage() const
+{
+    return m_platformInfo.eProductFamily == IGFX_PVC;
+}
+
+SIMDMode getMaxLSCTypedMessageSize() const
+{
+    switch (m_platformInfo.eProductFamily)
+    {
+    case IGFX_DG2:
+        return SIMDMode::SIMD8;
+    case IGFX_PVC:
+        return SIMDMode::SIMD16;
+    default:
+            return SIMDMode::SIMD16;
+    }
+}
+
+unsigned getAccChNumUD() const
+{
+    return m_platformInfo.eProductFamily == IGFX_PVC ? 16 : 8;
+}
+
+bool hasInt64SLMAtomicCAS() const
+{
+    // false for PVC XL A0 RevID==0x0, true from PVC XT A0 RevID==0x3==REVISION_B
+    return m_platformInfo.eProductFamily == IGFX_PVC && m_platformInfo.usRevId >= REVISION_B;
+
+}
+
+bool hasFP64GlobalAtomicAdd() const
+{
+    return m_platformInfo.eProductFamily == IGFX_PVC && m_platformInfo.usRevId > REVISION_A0;
+}
+
+bool supports16BitLdMcs() const
+{
+    return isProductChildOf(IGFX_XE_HP_SDV) && IGC_IS_FLAG_ENABLED(Enable16BitLDMCS);
+}
+
+bool supportsGather4PO() const
+{
+    return !isProductChildOf(IGFX_XE_HP_SDV);
+}
+
+// Typed read supports all renderable image formats, no image data conversion is
+// required.
+bool typedReadSupportsAllRenderableFormats() const
+{
+    bool isChildOfDG2B0 = SI_WA_FROM(m_platformInfo.usRevId, ACM_G10_GT_REV_ID_B0);
+    bool isChildOfDG2C0 = SI_WA_FROM(m_platformInfo.usRevId, ACM_G10_GT_REV_ID_C0);
+    bool isDG2G11Config = GFX_IS_DG2_G11_CONFIG(m_platformInfo.usDeviceID);
+
+    if ((m_platformInfo.eProductFamily == IGFX_DG2 && isChildOfDG2C0) ||
+        (m_platformInfo.eProductFamily == IGFX_DG2 && isDG2G11Config && isChildOfDG2B0) ||
+        (m_platformInfo.eProductFamily == IGFX_PVC))
+    {
+        return IGC_IS_FLAG_DISABLED(ForceFormatConversionDG2Plus);
+    }
+
+    return false;
+}
+
+bool needsWAForThreadsUtilization() const
+{
+    return m_platformInfo.eProductFamily == IGFX_DG2;
+}
+
+bool supportDualSimd8PS() const
+{
+    return IGC_IS_FLAG_ENABLED(EnableDualSIMD8) && (m_platformInfo.eRenderCoreFamily >= IGFX_GEN12_CORE);
+}
+
+bool hasDualSimd8Payload() const
+{
+    return m_platformInfo.eRenderCoreFamily >= IGFX_GEN12_CORE;
+}
+
+unsigned int getMaxMeshShaderThreads() const {
+    return m_caps.MediaShaderThreads - 1;
+}
+
+bool supportDpaswInstruction() const
+{
+    return hasFusedEU() && supportDpasInstruction();
+}
+
+// This represents the max number of logical lanes available for RT,
+// so it is not dependent on compiled SIMD size or "PreferredRayTracingSIMDSize".
+// The calculation here is: (ThreadCount / DualSubSliceCount) * 16,
+// where 16 is max current SIMD lenght for RT.
+unsigned getRTStackDSSMultiplier() const
+{
+    IGC_ASSERT(supportRayTracing());
+    return 2048;
+}
+
+// Max number of hw threads for each workgroup
+unsigned int getMaxNumberHWThreadForEachWG() const
+{
+    if (m_platformInfo.eRenderCoreFamily < IGFX_GEN12_CORE) {
+        // Each WG is dispatched into one subslice for GEN11 and before
+        return getMaxNumberThreadPerSubslice();
+    }
+    else if (m_platformInfo.eRenderCoreFamily <= IGFX_XE_HPC_CORE)
+    {
+        // Each WG is dispatched into one DSS which has 2 Subslices for Gen12 and above
+        if (getWATable().Wa_1609337546 || getWATable().Wa_1609337769) {
+            return 64;
+        }
+        else {
+            return getMaxNumberThreadPerSubslice() * 2;
+        }
+    }
+    else {
+        IGC_ASSERT_MESSAGE(0, "Unsupported platform!");
+    }
+    return 0;
+}
+
+uint32_t getGRFSize() const
+{
+    switch (m_platformInfo.eProductFamily)
+    {
+    case IGFX_PVC:
+        return 64;
+    default:
+        return 32;
+    }
+}
+
+uint32_t getInlineDataSize() const
+{
+    return supportInlineData() ? 32 : 0;
+}
+
+bool hasFusedEU() const
+{
+    return m_platformInfo.eRenderCoreFamily >= IGFX_GEN12_CORE &&
+        m_platformInfo.eProductFamily != IGFX_PVC;
+}
+
+bool hasPartialEmuI64Enabled() const
+{
+    return hasPartialInt64Support() && IGC_IS_FLAG_ENABLED(EnablePartialEmuI64);
+}
+
+
+bool matchImmOffsetsLSC() const
+{
+    enum LscMatchImmMode {
+        OFF = 0,
+        ON = 2,
+    };
+    auto immOffsetMode = (LscMatchImmMode)IGC_GET_FLAG_VALUE(LscImmOffsMatch);
+    return hasLSC() && immOffsetMode >= ON;
+}
+
+bool WaCubeHFPrecisionBug() const
+{
+    return m_WaTable.Wa_18012201914 != 0;
+}
+
+//The max size in bytes of the scratch space per thread.
+//  XeHP_SDV and above are for each physical thread: 256k.
+//  TGLLP and below are for each FFTID: 2M.
+uint32_t maxPerThreadScratchSpace() const
+{
+    return (hasScratchSurface() ? 0x40000 : 0x200000);
+}
+
+bool supportAIParameterCombiningWithLODBiasEnabled() const
+{
+    return IGC_IS_FLAG_ENABLED(EnableAIParameterCombiningWithLODBias) &&
+           (m_platformInfo.eProductFamily == IGFX_DG2 && SI_WA_FROM(m_platformInfo.usRevId, ACM_G10_GT_REV_ID_B0)) ||
+           GFX_IS_DG2_G11_CONFIG(m_platformInfo.usDeviceID);
+}
+
+bool useScratchSpaceForOCL() const
+{
+    // Disable using scratch surface for private memory on XeHP_SDV
+    // because it does not support byte-aligned (byte-scattered) messages.
+    if (hasScratchSurface()) {
+        return LSCEnabled() &&
+               IGC_IS_FLAG_ENABLED(EnableOCLScratchPrivateMemoryForDG2Plus) &&
+               m_platformInfo.eProductFamily == IGFX_PVC;
+    }
+    else {
+        return IGC_IS_FLAG_ENABLED(EnableOCLScratchPrivateMemory);
+    }
+}
+
+
+// Check if byte ALU operations are well supported. If not, promote byte to i16/i32.
+bool supportByteALUOperation() const
+{
+    return m_platformInfo.eProductFamily != IGFX_PVC;
+}
+
+// Platform requires kernel arguments pulling.
+bool HasKernelArguments() const
+{
+    return false;
+}
+
+
+
+bool has64BMediaBlockRW() const
+{
+    return IGC_IS_FLAG_ENABLED(Enable64BMediaBlockRW) && isProductChildOf(IGFX_XE_HP_SDV);
+}
+
 bool supportsStaticRegSharing() const
 {
-    return isXeHPSDVPlus();
+    return isProductChildOf(IGFX_XE_HP_SDV);
 }
 bool emulateByteScraterMsgForSS() const
 {
-    return isXeHPSDVPlus() && (m_platformInfo.usRevId == 0 || IGC_IS_FLAG_ENABLED(EnableUntypedSurfRWofSS));
+    return isProductChildOf(IGFX_XE_HP_SDV) && (m_platformInfo.usRevId == 0 || IGC_IS_FLAG_ENABLED(EnableUntypedSurfRWofSS));
 }
 
 //all the platforms which DONOT support 64 bit int operations
@@ -596,7 +1073,8 @@ bool hasNoInt64Inst() const {
         m_platformInfo.eProductFamily == IGFX_ROCKETLAKE ||
         m_platformInfo.eProductFamily == IGFX_ALDERLAKE_S ||
         m_platformInfo.eProductFamily == IGFX_ALDERLAKE_P ||
-        m_platformInfo.eProductFamily == IGFX_DG1;
+        m_platformInfo.eProductFamily == IGFX_DG1 ||
+        m_platformInfo.eProductFamily == IGFX_DG2;
 }
 
 //all the platforms which DONOT support 64 bit float operations
@@ -609,7 +1087,8 @@ bool hasNoFP64Inst() const {
         m_platformInfo.eProductFamily == IGFX_ROCKETLAKE ||
         m_platformInfo.eProductFamily == IGFX_ALDERLAKE_S ||
         m_platformInfo.eProductFamily == IGFX_ALDERLAKE_P ||
-        m_platformInfo.eProductFamily == IGFX_DG1;
+        m_platformInfo.eProductFamily == IGFX_DG1 ||
+        m_platformInfo.eProductFamily == IGFX_DG2;
 }
 
 //all the platforms which have correctly rounded macros (INVM, RSQRTM, MADM)
@@ -621,10 +1100,10 @@ bool hasCorrectlyRoundedMacros() const {
         m_platformInfo.eProductFamily != IGFX_ROCKETLAKE &&
         m_platformInfo.eProductFamily != IGFX_DG1 &&
         m_platformInfo.eProductFamily != IGFX_ALDERLAKE_S &&
-        m_platformInfo.eProductFamily != IGFX_ALDERLAKE_P;
+        m_platformInfo.eProductFamily != IGFX_ALDERLAKE_P &&
+        m_platformInfo.eProductFamily != IGFX_DG2;
 }
 
-bool hasFusedEU() const { return m_platformInfo.eRenderCoreFamily >= IGFX_GEN12_CORE; }
 bool supportMixMode() const {
     return IGC_IS_FLAG_ENABLED(ForceMixMode) ||
         (IGC_IS_FLAG_DISABLED(DisableMixMode) &&
@@ -633,25 +1112,6 @@ bool supportMixMode() const {
             m_platformInfo.eRenderCoreFamily == IGFX_GEN10_CORE));
 }
 bool DSPrimitiveIDPayloadPhaseCanBeSkipped() const { return false; }
-bool useScratchSpaceForOCL() const
-{
-    return IGC_IS_FLAG_ENABLED(EnableOCLScratchPrivateMemory);
-}
-
-uint32_t getGRFSize() const
-{
-    return 32;
-}
-
-uint32_t maxPerThreadScratchSpace() const
-{
-    return 0x200000;
-}
-
-bool supportByteALUOperation() const
-{
-    return true;
-}
 
 bool NeedsHDCFenceBeforeEOTInPixelShader() const
 {
@@ -663,17 +1123,9 @@ bool canFuseTypedWrite() const
     return false;
 }
 
-unsigned int getMaxNumberHWThreadForEachWG() const
+bool enableSetDefaultTileYWalk() const
 {
-    if (m_platformInfo.eRenderCoreFamily < IGFX_GEN12_CORE)
-    {
-        //each WG is dispatched into one subslice for GEN11 and before
-        return getMaxNumberThreadPerSubslice();
-    }
-    else
-    {
-        return getMaxNumberThreadPerSubslice() * 2;
-    }
+    return false;
 }
 
 // max block size for legacy OWord block messages
@@ -682,16 +1134,6 @@ uint32_t getMaxBlockMsgSize(bool isSLM) const
     return 128;
 }
 
-SIMDMode getMinDispatchMode() const
-{
-    return SIMDMode::SIMD8;
-}
-
-unsigned getAccChNumUD() const
-{
-    return 8;
-}
-
 int getBSOLocInExtDescriptor() const
 {
     return 12;
@@ -783,6 +1225,27 @@ bool WaOverwriteFFID() const
     return m_WaTable.Wa_1409460247 != 0;
 }
 
+bool WaDisableStaticRegSharing() const
+{
+    return m_WaTable.Wa_14012688715 != 0;
+}
+
+bool WaDisablePrimitiveReplicationWithCPS() const
+{
+    return m_WaTable.Wa_18013852970 != 0;
+}
+
+bool supportSystemFence() const
+{
+    return hasLSC()
+        && m_platformInfo.eProductFamily != IGFX_DG2;
+}
+
+
+bool WaGeoShaderURBAllocReduction() const
+{
+    return m_WaTable.Wa_18012660806 != 0;
+}
 
 bool WaDisableSendSrcDstOverlap() const
 {
@@ -820,6 +1283,8 @@ bool enableMultiGRFAccessWA() const
 bool hasSCF() const
 {
     bool doscf = true;
+    // DG2 and PVC still has SCF, but igc will stop using them.
+    doscf = !isProductChildOf(IGFX_DG2);
     return doscf;
 }
 
@@ -832,9 +1297,37 @@ bool supportHeaderRTW() const
 
 bool preemptionSupported() const
 {
+    if (isProductChildOf(IGFX_PVC))
+        return false;
 
     return GetPlatformFamily() >= IGFX_GEN9_CORE;
-};
+}
+
+// platform natively not support DW-DW multiply
+bool noNativeDwordMulSupport() const
+{
+    return m_platformInfo.eProductFamily == IGFX_BROXTON ||
+        m_platformInfo.eProductFamily == IGFX_GEMINILAKE ||
+        m_platformInfo.eProductFamily == IGFX_DG2 ||
+        GetPlatformFamily() == IGFX_GEN11_CORE ||
+        GetPlatformFamily() == IGFX_GEN12LP_CORE;
+}
+
+unsigned getURBFullWriteMinGranularity() const
+{
+    unsigned overrideValue = IGC_GET_FLAG_VALUE(SetURBFullWriteGranularity);
+    if (overrideValue == 16 || overrideValue == 32)
+    {
+        return overrideValue;
+    }
+    return isProductChildOf(IGFX_XE_HP_SDV) ? 16 : 32; // in bytes
+}
+
+unsigned forceQwAtSrc0ForQwShlWA() const
+{
+    // PVC XT A0: RevID==0X3==REVISION_B
+    return (m_platformInfo.eProductFamily == IGFX_PVC && m_platformInfo.usRevId == REVISION_B);
+}
 
 };
 
