@@ -100,7 +100,8 @@ namespace IGC
     Value* PushAnalysis::addArgumentAndMetadata(llvm::Type* pType, std::string argName, IGC::WIAnalysis::WIDependancy dependency)
     {
         auto pArgInfo = m_pFuncUpgrade.AddArgument(argName, pType);
-        ModuleMetaData* modMD = getAnalysis<MetaDataUtilsWrapper>().getModuleMetaData();
+        ModuleMetaData* modMD = nullptr;
+        modMD = getAnalysis<MetaDataUtilsWrapper>().getModuleMetaData();
         IGC::ArgDependencyInfoMD argDepInfoMD;
         argDepInfoMD.argDependency = dependency;
         modMD->pushInfo.pushAnalysisWIInfos.push_back(argDepInfoMD);
@@ -188,7 +189,7 @@ namespace IGC
         llvm::Instruction* inst,
         int& pushableAddressGrfOffset,
         int& pushableOffsetGrfOffset,
-        unsigned int& offset)
+        unsigned int& pushableOffset)
     {
         if (!llvm::isa<llvm::LoadInst>(inst))
             return false;
@@ -210,6 +211,7 @@ namespace IGC
             pAddress = cast<Instruction>(pAddress)->getOperand(0);
         }
 
+        uint64_t offset = 0;
         SmallVector<Value*, 4> potentialPushableAddresses;
         std::function<void(Value*)> GetPotentialPushableAddresses;
         GetPotentialPushableAddresses = [&potentialPushableAddresses, &offset, &GetPotentialPushableAddresses](
@@ -229,7 +231,7 @@ namespace IGC
             else if (isa<ConstantInt>(pAddress))
             {
                 ConstantInt* pConst = cast<ConstantInt>(pAddress);
-                offset += int_cast<uint>(pConst->getZExtValue());
+                offset += pConst->getZExtValue();
             }
             else
             {
@@ -341,8 +343,10 @@ namespace IGC
             offset = 0;
             GetPotentialPushableAddresses(pAddress);
         }
-        if (potentialPushableAddresses.size() == 1 ||
-            potentialPushableAddresses.size() == 2)
+
+        if (offset <= std::numeric_limits<uint>::max() && // pushableOffset is a 32bit value
+            (potentialPushableAddresses.size() == 1 ||
+             potentialPushableAddresses.size() == 2))
         {
             for (Value* potentialAddress : potentialPushableAddresses)
             {
@@ -357,6 +361,7 @@ namespace IGC
                 }
             }
             IGC_ASSERT(pushableAddressGrfOffset >= 0);
+            pushableOffset = int_cast<uint>(offset);
             return true;
         }
 
@@ -625,16 +630,14 @@ namespace IGC
         if (!isa<LoadInst>(inst) && !isa<LdRawIntrinsic>(inst))
             return false;
 
-        if (inst->getType()->isVectorTy())
+        Type* loadType = inst->getType();
+        Type* elemType = loadType->getScalarType();
+        // TODO: enable promotion of 8 bit, 16 bit and 64 bit values
+        if (!elemType->isFloatTy() &&
+            !elemType->isIntegerTy(32) &&
+            !(elemType->isPointerTy() && GetSizeInBits(elemType) == 32))
         {
-            if (!(cast<VectorType>(inst->getType())->getElementType()->isFloatTy() ||
-                cast<VectorType>(inst->getType())->getElementType()->isIntegerTy(32)))
-                return false;
-        }
-        else
-        {
-            if (!(inst->getType()->isFloatTy() || inst->getType()->isIntegerTy(32) || inst->getType()->isPointerTy()))
-                return false;
+            return false;
         }
 
         if (IsPushableBindlessLoad(
@@ -767,6 +770,17 @@ namespace IGC
             return false;
         }
 
+        if (m_context->m_DriverInfo.WaDisablePushConstantsWithNoPushedAttributes() &&
+            m_context->platform.getWATable().Wa_16011983264 &&
+            m_context->type == ShaderType::HULL_SHADER)
+        {
+            Function* URBReadFunction = GenISAIntrinsic::getDeclaration(m_module, GenISAIntrinsic::GenISA_URBRead);
+            if (GetMaxNumberOfPushedInputs() == 0 ||
+                none_of(URBReadFunction->users(), [](const User* user) { return isa<GenIntrinsicInst>(user); }))
+            {
+                return false;
+            }
+        }
 
         switch (m_context->type)
         {
@@ -852,11 +866,20 @@ namespace IGC
         if (type->isPointerTy())
         {
             size = m_DL->getPointerSizeInBits(type->getPointerAddressSpace());
+            IGC_ASSERT(size == 32 || size == 64);
+        }
+        else if (type->isVectorTy() &&
+            type->getScalarType()->isPointerTy())
+        {
+            size = m_DL->getPointerSizeInBits(
+                type->getScalarType()->getPointerAddressSpace());
+            IGC_ASSERT(size == 32 || size == 64);
+            size = size * int_cast<uint>(cast<IGCLLVM::FixedVectorType>(type)->getNumElements());
         }
         return size;
     }
 
-    unsigned int PushAnalysis::AllocatePushedConstant(
+    void PushAnalysis::AllocatePushedConstant(
         Instruction* load,
         const SimplePushInfo& newChunk,
         const unsigned int maxSizeAllowed)
@@ -864,23 +887,20 @@ namespace IGC
         if (!newChunk.isBindless &&
             newChunk.cbIdx > m_context->m_DriverInfo.MaximumSimplePushBufferID())
         {
-            return 0;
+            return;
         }
         unsigned int size = GetSizeInBits(load->getType()) / 8;
         IGC_ASSERT_MESSAGE(isa<LoadInst>(load) || isa<LdRawIntrinsic>(load),
             "Expected a load instruction");
-        PushInfo& pushInfo = m_context->getModuleMetaData()->pushInfo;
 
-        bool canPromote = false;
-        unsigned int sizeGrown = 0;
         // greedy allocation for now
         // first check if we are already pushing from the buffer
         unsigned int piIndex;
         bool regionFound = false;
 
-        for (piIndex = 0; piIndex < pushInfo.simplePushBufferUsed; piIndex++)
+        for (piIndex = 0; piIndex < numSimplePush; piIndex++)
         {
-            const SimplePushInfo& info = pushInfo.simplePushInfoArr[piIndex];
+            const SimplePushData& info = CollectAllSimplePushInfoArr[piIndex];
             // Stateless load - GRF offsets need to match.
             if (info.isStateless &&
                 newChunk.isStateless &&
@@ -913,7 +933,7 @@ namespace IGC
         }
         if (regionFound)
         {
-            SimplePushInfo& info = pushInfo.simplePushInfoArr[piIndex];
+            SimplePushData& info = CollectAllSimplePushInfoArr[piIndex];
             unsigned int newStartOffset = iSTD::RoundDown(
                 std::min(newChunk.offset, info.offset),
                 getMinPushConstantBufferAlignmentInBytes());
@@ -922,21 +942,16 @@ namespace IGC
                 getMinPushConstantBufferAlignmentInBytes());
             unsigned int newSize = newEndOffset - newStartOffset;
 
-            if (newSize - info.size <= maxSizeAllowed)
+            if (newSize <= maxSizeAllowed)
             {
-                sizeGrown = newSize - info.size;
-                canPromote = true;
                 info.offset = newStartOffset;
                 info.size = newSize;
+                info.Load.push_back(std::make_pair(load, newChunk.offset));
             }
         }
 
-        const unsigned int maxNumberOfPushedBuffers = pushInfo.MaxNumberOfPushedBuffers;
-
         // we couldn't add it to an existing buffer try to add a new one if there is a slot available
-        if (canPromote == false &&
-            maxSizeAllowed > 0 &&
-            pushInfo.simplePushBufferUsed < maxNumberOfPushedBuffers)
+        else
         {
             unsigned int newStartOffset = iSTD::RoundDown(newChunk.offset, getMinPushConstantBufferAlignmentInBytes());
             unsigned int newEndOffset = iSTD::Round(newChunk.offset + size, getMinPushConstantBufferAlignmentInBytes());
@@ -944,11 +959,7 @@ namespace IGC
 
             if (newSize <= maxSizeAllowed)
             {
-                canPromote = true;
-                sizeGrown = newSize;
-
-                piIndex = pushInfo.simplePushBufferUsed;
-                SimplePushInfo& info = pushInfo.simplePushInfoArr[piIndex];
+                SimplePushData& info = CollectAllSimplePushInfoArr[numSimplePush];
                 info.pushableAddressGrfOffset = newChunk.pushableAddressGrfOffset;
                 info.pushableOffsetGrfOffset = newChunk.pushableOffsetGrfOffset;
                 info.cbIdx = newChunk.cbIdx;
@@ -956,20 +967,11 @@ namespace IGC
                 info.isBindless = newChunk.isBindless;
                 info.offset = newStartOffset;
                 info.size = newSize;
-
-                pushInfo.simplePushBufferUsed++;
+                info.Load.push_back(std::make_pair(load, newChunk.offset));
+                numSimplePush++;
             }
         }
-
-        if (canPromote)
-        {
-            // promote the load to be pushed
-            PromoteLoadToSimplePush(
-                load,
-                pushInfo.simplePushInfoArr[piIndex],
-                newChunk.offset);
-        }
-        return sizeGrown;
+        return;
     }
 
     void PushAnalysis::PromoteLoadToSimplePush(Instruction* load, SimplePushInfo& info, unsigned int offset)
@@ -987,7 +989,7 @@ namespace IGC
 
         if (pTypeToPush->isVectorTy())
         {
-            num_elms = (unsigned)cast<VectorType>(pTypeToPush)->getNumElements();
+            num_elms = (unsigned)cast<IGCLLVM::FixedVectorType>(pTypeToPush)->getNumElements();
             pTypeToPush = cast<VectorType>(pTypeToPush)->getElementType();
             llvm::Type* pVecTy = IGCLLVM::FixedVectorType::get(pTypeToPush, num_elms);
             pReplacedInst = llvm::UndefValue::get(pVecTy);
@@ -999,7 +1001,7 @@ namespace IGC
 
         for (unsigned int i = 0; i < num_elms; ++i)
         {
-            uint address = offset + i * ((unsigned int)pScalarTy->getPrimitiveSizeInBits() / 8);
+            uint address = offset + i * (GetSizeInBits(pScalarTy) / 8);
             auto it = info.simplePushLoads.find(address);
             llvm::Value* value = nullptr;
             if (it != info.simplePushLoads.end())
@@ -1007,17 +1009,19 @@ namespace IGC
                 // Value is already getting pushed
                 IGC_ASSERT_MESSAGE((it->second <= m_argIndex), "Function arguments list and metadata are out of sync!");
                 value = m_argList[it->second];
-                if (pTypeToPush != value->getType())
-                    value = CastInst::CreateZExtOrBitCast(value, pTypeToPush, "", load);
             }
             else
             {
                 value = addArgumentAndMetadata(pTypeToPush, VALUE_NAME("cb"), WIAnalysis::UNIFORM_GLOBAL);
-                if (pTypeToPush != value->getType())
-                    value = CastInst::CreateZExtOrBitCast(value, pTypeToPush, "", load);
-
                 info.simplePushLoads.insert(std::make_pair(address, m_argIndex));
             }
+            if (pTypeToPush != value->getType())
+            {
+                IGC_ASSERT(pTypeToPush->isPointerTy() == value->getType()->isPointerTy());
+                value = pTypeToPush->isPointerTy() ?
+                    CastInst::CreatePointerBitCastOrAddrSpaceCast(value, pTypeToPush, "", load) :
+                    CastInst::CreateZExtOrBitCast(value, pTypeToPush, "", load);
+            }
 
             if (load->getType()->isVectorTy())
             {
@@ -1089,13 +1093,55 @@ namespace IGC
                 bool isPushable = IsPushableShaderConstant(instr, info);
                 if(isPushable)
                 {
-                    sizePushed += AllocatePushedConstant(
+                    AllocatePushedConstant(
                         instr,
                         info,
-                        cthreshold - sizePushed); // maxSizeAllowed
+                        cthreshold); // maxSizeAllowed
                 }
             }
         }
+
+
+        PushInfo& pushInfo = m_context->getModuleMetaData()->pushInfo;
+        unsigned int simplePushBufferId = 0;
+        while ((pushInfo.simplePushBufferUsed < pushInfo.MaxNumberOfPushedBuffers) && CollectAllSimplePushInfoArr.size())
+        {
+            unsigned int iter = CollectAllSimplePushInfoArr.begin()->first;
+            SimplePushData info;
+            if (IGC_IS_FLAG_ENABLED(EnableSimplePushSizeBasedOpimization))
+            {
+                for (auto I = CollectAllSimplePushInfoArr.begin(), E = CollectAllSimplePushInfoArr.end(); I != E; I++)
+                {
+                    if (I->second.size > info.size)
+                    {
+                        info = I->second;
+                        iter = I->first;
+                    }
+                }
+            }
+            else
+            {
+                info = CollectAllSimplePushInfoArr[simplePushBufferId];
+                iter = simplePushBufferId;
+            }
+            SimplePushInfo& newChunk = pushInfo.simplePushInfoArr[pushInfo.simplePushBufferUsed];
+            if (sizePushed + info.size <= cthreshold)
+            {
+                newChunk.cbIdx = info.cbIdx;
+                newChunk.isBindless = info.isBindless;
+                newChunk.isStateless = info.isStateless;
+                newChunk.offset = info.offset;
+                newChunk.size = info.size;
+                newChunk.pushableAddressGrfOffset = info.pushableAddressGrfOffset;
+                newChunk.pushableOffsetGrfOffset = info.pushableOffsetGrfOffset;
+                for (auto I = info.Load.begin(), E = info.Load.end(); I != E; I++)
+                    PromoteLoadToSimplePush(I->first, newChunk, I->second);
+                pushInfo.simplePushBufferUsed++;
+                sizePushed += info.size;
+            }
+            CollectAllSimplePushInfoArr.erase(iter);
+            simplePushBufferId++;
+        }
     }
 
     PushConstantMode PushAnalysis::GetPushConstantMode()
@@ -1178,7 +1224,7 @@ namespace IGC
         }
 
         unsigned num_elms =
-            inst->getType()->isVectorTy() ? (unsigned)cast<VectorType>(inst->getType())->getNumElements() : 1;
+            inst->getType()->isVectorTy() ? (unsigned)cast<IGCLLVM::FixedVectorType>(inst->getType())->getNumElements() : 1;
         llvm::Type* pTypeToPush = inst->getType();
         llvm::Value* replaceVector = nullptr;
         unsigned int numberChannelReplaced = 0;
