@@ -1,22 +1,20 @@
-//===-- RegisterPressureEstimate - Estimate Register Pressure------------------===//
-//
-//                     The LLVM Compiler Infrastructure
-//
-// This file is distributed under the University of Illinois Open Source
-// License. See LRCENSE.TXT for details.
-//
-//  Copyright  (C) 2014-2020 Intel Corporation. All rights reserved.
-//
-// The information and source code contained herein is the exclusive
-// property of Intel Corporation and may not be disclosed, examined
-// or reproduced in whole or in part without explicit written authorization
-// from the company.
-//
-//===----------------------------------------------------------------------===//
-//
-// Estimate the register pressure at a program point.
-//
-//===----------------------------------------------------------------------===//
+/*========================== begin_copyright_notice ============================
+
+Copyright (C) 2017-2021 Intel Corporation
+
+SPDX-License-Identifier: MIT
+
+============================= end_copyright_notice ===========================*/
+
+/*========================== begin_copyright_notice ============================
+
+This file is distributed under the University of Illinois Open Source License.
+See LICENSE.TXT for details.
+
+============================= end_copyright_notice ===========================*/
+
+// Estimates the register pressure at a program point.
+
 #include "RegisterPressureEstimate.hpp"
 #include "Compiler/IGCPassSupport.h"
 #include <set>
@@ -27,6 +25,7 @@
 #include <llvm/IR/Intrinsics.h>
 #include <llvm/IR/Function.h>
 #include <llvm/IR/InstIterator.h>
+#include <llvm/Transforms/Utils/Local.h>
 #include "common/LLVMWarningsPop.hpp"
 #include "Probe/Assertion.h"
 
@@ -49,14 +48,16 @@ namespace IGC
     {
         AU.setPreservesAll();
         AU.addRequired<LoopInfoWrapperPass>();
+        AU.addRequired<WIAnalysis>();
     }
 
     bool RegisterPressureEstimate::runOnFunction(Function& F)
     {
+        m_DL = &F.getParent()->getDataLayout();
         m_pFunc = &F;
         LI = &getAnalysis<LoopInfoWrapperPass>().getLoopInfo();
+        WI = &getAnalysis<WIAnalysis>();
         m_available = buildLiveIntervals(true);
-
         return false;
     }
 
@@ -76,6 +77,10 @@ namespace IGC
         {
             Argument* Arg = &(*AI);
             m_pNumbers[Arg] = Num;
+            if (!Arg->use_empty())
+            {
+                getOrCreateLiveRange(Arg);
+            }
         }
 
         // Assign a number to basic blocks and instructions.
@@ -91,7 +96,7 @@ namespace IGC
                 {
                     continue;
                 }
-                else if (isa<PHINode>(Inst))
+                if (isa<PHINode>(Inst))
                 {
                     m_pNumbers[Inst] = BlockNum;
                 }
@@ -99,8 +104,14 @@ namespace IGC
                 {
                     m_pNumbers[Inst] = Num++;
                 }
+                if (!Inst->use_empty())
+                {
+                    getOrCreateLiveRange(Inst);
+                }
             }
         }
+
+        MaxAssignedNumber = Num;
     }
 
     /// Construct a string from a unsigned integer with the intended width.
@@ -112,7 +123,6 @@ namespace IGC
         {
             Str.insert(Str.begin(), Width - Str.size(), ' ');
         }
-
         return Str;
     }
 
@@ -158,18 +168,6 @@ namespace IGC
         return m_pNumbers[pInst];
     }
 
-    unsigned RegisterPressureEstimate::getMaxAssignedNumberForFunction()
-    {
-        auto& BBs = m_pFunc->getBasicBlockList();
-        unsigned maxInstructionNumber = 0;
-        for (auto BI = BBs.begin(), BE = BBs.end(); BI != BE; ++BI)
-        {
-            BasicBlock* pBB = &*BI;
-            maxInstructionNumber = (m_pNumbers[&pBB->back()] > maxInstructionNumber) ? m_pNumbers[&pBB->back()] : maxInstructionNumber;
-        }
-        return maxInstructionNumber;
-    }
-
     unsigned RegisterPressureEstimate::getMaxAssignedNumberForBB(BasicBlock* pBB)
     {
         return m_pNumbers[&pBB->back()];
@@ -280,29 +278,52 @@ namespace IGC
         clear(RemoveLR);
 
         // Assign a number to arguments, basic blocks and instructions.
+        // build the live-range pool.
         assignNumbers();
 
-        // A side data structure to build live intervals.
+        unsigned OverallEstimate = 0;
+        // quick estimate
+        for (auto VI = m_pLiveRanges.begin(), VE = m_pLiveRanges.end();
+            VI != VE; ++VI)
+        {
+            auto V = VI->first;
+            unsigned RangeStart = m_pNumbers[V];
+            unsigned MaxRange = 0;
+            // need to find the last use
+            for (auto UI = V->user_begin(), UE = V->user_end(); UI != UE; ++UI)
+            {
+                Instruction* UseI = dyn_cast<Instruction>(*UI);
+                if (!UseI || isInstructionTriviallyDead(UseI))
+                    continue;
+                unsigned RangeEnd = RangeStart;
+                if (PHINode* PN = dyn_cast<PHINode>(UseI))
+                {
+                    // PHI nodes use the operand in the predecessor block,
+                    // not the block with the PHI.
+                    Use& U = UI.getUse();
+                    unsigned num = PHINode::getIncomingValueNumForOperand(U.getOperandNo());
+                    auto UseBB = PN->getIncomingBlock(num);
+                    RangeEnd = m_pNumbers[&UseBB->back()] + 1;
+                }
+                else
+                {
+                    RangeEnd = m_pNumbers[UseI];
+                }
+                if (RangeEnd > RangeStart && RangeEnd - RangeStart > MaxRange)
+                    MaxRange = RangeEnd - RangeStart;
+                else if (RangeStart > RangeEnd && RangeStart - RangeEnd > MaxRange)
+                    MaxRange = RangeStart - RangeEnd;
+            }
+            OverallEstimate += MaxRange * getValueBytes(V);
+        }
+        OverallEstimate = iSTD::Round(OverallEstimate, SIMD_PRESSURE_MULTIPLIER) /
+            SIMD_PRESSURE_MULTIPLIER;
+        OverallEstimate = OverallEstimate / getMaxAssignedNumberForFunction();
+        if (OverallEstimate > OVERALL_PRESSURE_UPBOUND)
+            return false;
+
         DenseMap<BasicBlock*, std::set<Value*>> BlockLiveMap;
         auto& BBs = m_pFunc->getBasicBlockList();
-
-        for (auto BI = BBs.begin(), BE = BBs.end(); BI != BE; ++BI)
-        {
-            for (auto II = BI->begin(), IE = BI->end(); II != IE; ++II)
-            {
-                Instruction* Inst = &*II;
-                if (isa<DbgInfoIntrinsic>(Inst))
-                {
-                    continue;
-                }
-                if (Inst->use_empty())
-                {
-                    continue;
-                }
-                getOrCreateLiveRange(Inst);
-            }
-        }
-
         // Top level loop to visit each block once in reverse order.
         for (auto BI = BBs.rbegin(), BE = BBs.rend(); BI != BE; ++BI)
         {
@@ -356,7 +377,6 @@ namespace IGC
                     LR->addSegment(BlockNum, End);
                 }
             }
-
             // for each operation op of b in reverse order do
             //     for each output operand opnd of op do
             //         intervals[opnd].setFrom(op.id)
@@ -466,7 +486,7 @@ namespace IGC
         auto Iter = m_pRegisterPressureByInstruction.find(number);
         if (Iter != m_pRegisterPressureByInstruction.end())
         {
-            return Iter->second;
+            return iSTD::Round(Iter->second, SIMD_PRESSURE_MULTIPLIER) / SIMD_PRESSURE_MULTIPLIER;
         }
         else
         {
@@ -485,15 +505,17 @@ namespace IGC
         // Segments are sorted.
         for (auto I = m_pLiveRanges.begin(), E = m_pLiveRanges.end(); I != E; ++I)
         {
+            Value* V = I->first;
+            unsigned int pressure = getValueBytes(V);
             for (auto& Seg : I->second->Segments)
             {
                 for (unsigned number = Seg.Begin; number < Seg.End; number++)
                 {
-                    Value* V = I->first;
-                    m_pRegisterPressureByInstruction[number] = m_pRegisterPressureByInstruction[number] + (unsigned int)V->getType()->getPrimitiveSizeInBits() / 8;
+                    m_pRegisterPressureByInstruction[number] += pressure;
                 }
             }
         }
+        return;
     }
 
     unsigned RegisterPressureEstimate::getRegisterPressure(Instruction* Inst) const
@@ -505,7 +527,7 @@ namespace IGC
             unsigned N = Iter->second;
 
             // Now sum all intervals that contain this location.
-            unsigned Presssure = 0;
+            unsigned Pressure = 0;
 
             // Segments are sorted.
             for (auto I = m_pLiveRanges.begin(), E = m_pLiveRanges.end(); I != E; ++I)
@@ -513,20 +535,19 @@ namespace IGC
                 if (I->second->contains(N))
                 {
                     Value* V = I->first;
-                    Presssure += (unsigned int)V->getType()->getPrimitiveSizeInBits() / 8;
+                    Pressure += getValueBytes(V);
                 }
             }
 
-            return Presssure;
+            return iSTD::Round(Pressure, SIMD_PRESSURE_MULTIPLIER) / SIMD_PRESSURE_MULTIPLIER;
         }
 
         // ignore this instruction.
         return 0;
     }
 
-    unsigned RegisterPressureEstimate::getRegisterPressure() const
+    unsigned RegisterPressureEstimate::getMaxRegisterPressure() const
     {
-        // TODO: Make this more efficient.
         unsigned MaxPressure = 0;
         for (auto BI = m_pFunc->begin(), BE = m_pFunc->end(); BI != BE; ++BI)
         {
@@ -540,7 +561,7 @@ namespace IGC
         return MaxPressure;
     }
 
-    unsigned RegisterPressureEstimate::getRegisterPressure(BasicBlock* BB) const
+    unsigned RegisterPressureEstimate::getMaxRegisterPressure(BasicBlock* BB) const
     {
         unsigned RP = 0;
         for (auto II = BB->begin(), IE = BB->end(); II != IE; ++II)
@@ -555,8 +576,7 @@ namespace IGC
     (bool Detailed,
         const char* msg) const
     {
-        unsigned MaxRP = getRegisterPressure();
-
+        unsigned MaxRP = getMaxRegisterPressure();
         if (Detailed)
         {
             for (inst_iterator I = inst_begin(m_pFunc), E = inst_end(m_pFunc); I != E; ++I)
