@@ -12,8 +12,10 @@ SPDX-License-Identifier: MIT
 #include "Compiler/CISACodeGen/Platform.hpp"
 #include "common/LLVMWarningsPush.hpp"
 #include <llvmWrapper/IR/DerivedTypes.h>
+#include <llvmWrapper/Transforms/Utils/LoopUtils.h>
 #include <llvm/IR/InstIterator.h>
 #include <llvm/IR/Operator.h>
+#include <llvmWrapper/IR/DerivedTypes.h>
 #include "common/LLVMWarningsPop.hpp"
 #include "GenISAIntrinsics/GenIntrinsics.h"
 #include "GenISAIntrinsics/GenIntrinsicInst.h"
@@ -212,7 +214,7 @@ countOperands(Value* V, Value* LHS, Value* RHS) {
         ConstantInt* CI = dyn_cast<ConstantInt>(BO->getOperand(1));
         if (!CI)
             return std::make_tuple(0, 0);
-        int L, R;
+        int L = 0, R = 0;
         std::tie(L, R) = countOperands(BO->getOperand(0), LHS, RHS);
         uint64_t ShAmt = CI->getZExtValue();
         return std::make_tuple((L << ShAmt), (R << ShAmt));
@@ -222,7 +224,7 @@ countOperands(Value* V, Value* LHS, Value* RHS) {
         ConstantInt* CI = dyn_cast<ConstantInt>(BO->getOperand(1));
         if (!CI || CI->getSExtValue() != -1)
             return std::make_tuple(0, 0);
-        int L, R;
+        int L = 0, R = 0;
         std::tie(L, R) = countOperands(BO->getOperand(0), LHS, RHS);
         return std::make_tuple(-L, -R);
     }
@@ -232,9 +234,9 @@ countOperands(Value* V, Value* LHS, Value* RHS) {
 
     if (isa<Constant>(BO->getOperand(1)))
         return countOperands(BO->getOperand(0), LHS, RHS);
-    int L0, L1;
+    int L0 = 0, L1 = 0;
     std::tie(L0, L1) = countOperands(BO->getOperand(0), LHS, RHS);
-    int R0, R1;
+    int R0 = 0, R1 = 0;
     std::tie(R0, R1) = countOperands(BO->getOperand(1), LHS, RHS);
     if (BO->getOpcode() == Instruction::Add)
         return std::make_tuple(L0 + R0, L1 + R1);
@@ -245,14 +247,14 @@ countOperands(Value* V, Value* LHS, Value* RHS) {
 
 static bool isNegatedByLB(Value* V, Value* X, Value* LB) {
     // Check if `V` is calculated as LB - X +/- C, where C is constant.
-    int L, R;
+    int L = 0, R = 0;
     std::tie(L, R) = countOperands(V, LB, X);
     return (L == 1) && (R == -1);
 }
 
 static bool isNegatedBy2UB(Value* V, Value* X, Value* UB) {
     // Check if `V` is calculated as 2UB - X +/- C, where C is constant.
-    int L, R;
+    int L = 0, R = 0;
     std::tie(L, R) = countOperands(V, UB, X);
     return (L == 2) && (R == -1);
 }
@@ -268,8 +270,8 @@ unsigned Simd32ProfitabilityAnalysis::estimateLoopCount_CASE1(Loop* L) {
     if (!L->contains(Br->getSuccessor(0)))
         return LOOPCOUNT_UNKNOWN;
 
-    Value* X, * LB, * UB;
-    bool Signed;
+    Value* X = nullptr, * LB = nullptr, * UB = nullptr;
+    bool Signed = false;
     std::tie(X, LB, UB, Signed) = isOutOfRangeComparison(Br->getCondition());
     if (!X) {
         ICmpInst* Cmp = dyn_cast<ICmpInst>(Br->getCondition());
@@ -362,9 +364,9 @@ unsigned Simd32ProfitabilityAnalysis::estimateLoopCount_CASE1(Loop* L) {
             return LOOPCOUNT_UNKNOWN;
         if (RHS != LB)
             return LOOPCOUNT_UNKNOWN;
-        int L0, R0;
+        int L0 = 0, R0 = 0;
         std::tie(L0, R0) = countOperands(X0, LB, nullptr);
-        int L1, R1;
+        int L1 = 0, R1 = 0;
         std::tie(L1, R1) = countOperands(X1, UB, nullptr);
         if (L0 != 1 || L1 != 2)
             return LOOPCOUNT_UNKNOWN;
@@ -392,7 +394,7 @@ unsigned Simd32ProfitabilityAnalysis::estimateLoopCount_CASE2(Loop* L) {
     SmallVector<BasicBlock*, 8> ExitingBBs;
     L->getExitingBlocks(ExitingBBs);
 
-    Value* Init, * Curr, * Next, * Step;
+    Value* Init = nullptr, * Curr= nullptr, * Next= nullptr, * Step= nullptr;
     std::tie(Init, Curr, Step, Next) = getInductionVariable(L);
     if (!Init || !Curr || !Step || !Next)
         return LOOPCOUNT_UNKNOWN;
@@ -539,7 +541,8 @@ static bool hasSubGroupFunc(const Function& F)
 bool Simd32ProfitabilityAnalysis::runOnFunction(Function& F)
 {
     this->F = &F;
-    CodeGenContext* context = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
+    CodeGenContext* context = nullptr;
+    context = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
     if (context->type == ShaderType::OPENCL_SHADER)
     {
         PDT = &getAnalysis<PostDominatorTreeWrapperPass>().getPostDomTree();
@@ -561,7 +564,7 @@ static bool isPayloadHeader(Value* V) {
     Argument* Arg = dyn_cast<Argument>(V);
     if (!Arg || !Arg->hasName())
         return false;
-    VectorType* VTy = dyn_cast<VectorType>(Arg->getType());
+    IGCLLVM::FixedVectorType* VTy = dyn_cast<IGCLLVM::FixedVectorType>(Arg->getType());
     if (!VTy || VTy->getNumElements() != 8 ||
         !VTy->getElementType()->isIntegerTy(32))
         return false;
@@ -572,7 +575,7 @@ static bool isR0(Value* V) {
     Argument* Arg = dyn_cast<Argument>(V);
     if (!Arg || !Arg->hasName())
         return false;
-    VectorType* VTy = dyn_cast<VectorType>(Arg->getType());
+    IGCLLVM::FixedVectorType* VTy = dyn_cast<IGCLLVM::FixedVectorType>(Arg->getType());
     if (!VTy || VTy->getNumElements() != 8 ||
         !VTy->getElementType()->isIntegerTy(32))
         return false;
@@ -583,7 +586,7 @@ static bool isEnqueuedLocalSize(Value* V) {
     Argument* Arg = dyn_cast<Argument>(V);
     if (!Arg || !Arg->hasName())
         return false;
-    VectorType* VTy = dyn_cast<VectorType>(Arg->getType());
+    IGCLLVM::FixedVectorType* VTy = dyn_cast<IGCLLVM::FixedVectorType>(Arg->getType());
     if (!VTy || VTy->getNumElements() != 3 ||
         !VTy->getElementType()->isIntegerTy(32))
         return false;
@@ -730,13 +733,15 @@ bool Simd32ProfitabilityAnalysis::checkSimd32Profitable(CodeGenContext* ctx)
     // even without simd32; and simd32 would have more visa variables than
     // 64K limit (ocl c99 64 bit PrintHalf/half8.c for example); thus make
     // sense to skip simd32.
+    size_t programSizeLimit = 8000;
     size_t programSize = 0;
     for (Function::iterator FI = F->begin(), FE = F->end(); FI != FE; ++FI)
     {
         BasicBlock* BB = &*FI;
         programSize += BB->size();
     }
-    if (programSize > 8000)
+    ctx->metrics.CollectInstructionCnt(F, programSize, programSizeLimit);
+    if (programSize > programSizeLimit)
     {
         return false;
     }
@@ -749,18 +754,26 @@ bool Simd32ProfitabilityAnalysis::checkSimd32Profitable(CodeGenContext* ctx)
     {
         ThreadGroupSizeMetaDataHandle tgSize = funcInfoMD->second->getThreadGroupSize();
         ThreadGroupSizeMetaDataHandle tgSizeHint = funcInfoMD->second->getThreadGroupSizeHint();
+        const int tgSizeLimit = 16;
+        int tgSizeCal = tgSize->getXDim() * tgSize->getYDim() * tgSize->getZDim();
+        int tgSizeHintCal = tgSizeHint->getXDim() * tgSizeHint->getYDim() * tgSizeHint->getZDim();
 
-        if (ctx->getModuleMetaData()->csInfo.maxWorkGroupSize && ctx->getModuleMetaData()->csInfo.maxWorkGroupSize <= 16)
+        ctx->metrics.CollectThreadGroupSize(F, tgSizeCal, tgSizeLimit);
+        ctx->metrics.CollectThreadGroupSizeHint(F, tgSizeHintCal, tgSizeLimit);
+
+        if (ctx->getModuleMetaData()->csInfo.maxWorkGroupSize && ctx->getModuleMetaData()->csInfo.maxWorkGroupSize <= tgSizeLimit)
             return false;
 
-        if ((tgSize->hasValue() && (tgSize->getXDim() * tgSize->getYDim() * tgSize->getZDim()) <= 16) ||
-            (tgSizeHint->hasValue() && (tgSizeHint->getXDim() * tgSizeHint->getYDim() * tgSizeHint->getZDim()) <= 16)) {
+        if ((tgSize->hasValue() && tgSizeCal <= tgSizeLimit) ||
+            (tgSizeHint->hasValue() && tgSizeHintCal <= tgSizeLimit)) {
             return false;
         }
     }
 
     // WORKAROUND - Skip SIMD32 if subgroup functions are present.
-    if (hasSubGroupFunc(*F)) {
+    bool hasSubGrFunc = hasSubGroupFunc(*F);
+    ctx->metrics.CollectIsSubGroupFuncIn(F, hasSubGrFunc);
+    if (hasSubGrFunc) {
         return false;
     }
 
@@ -772,12 +785,14 @@ bool Simd32ProfitabilityAnalysis::checkSimd32Profitable(CodeGenContext* ctx)
          // FALL THROUGH
     case IGFX_GEN10_CORE:
         if (hasIEEESqrtOrDivFunc(*F)) {
+            ctx->metrics.CollectGen9Gen10WithIEEESqrtDivFunc(F, true);
             return false;
         }
         break;
     default:
         break;
     }
+    ctx->metrics.CollectGen9Gen10WithIEEESqrtDivFunc(F, false);
     // END OF WORKAROUND
 
     // Ok, that's not the case.
@@ -807,7 +822,7 @@ bool Simd32ProfitabilityAnalysis::checkSimd32Profitable(CodeGenContext* ctx)
                 if (Br && Br->isConditional()) {
                     auto ICmp = dyn_cast<ICmpInst>(Br->getCondition());
                     if (ICmp) {
-                        Value* Init, * Curr, * Step, * Next;
+                        Value* Init = nullptr, * Curr = nullptr, * Step= nullptr, * Next = nullptr;
                         std::tie(Init, Curr, Step, Next)
                             = getInductionVariable(loop);
                         if (Init && Curr && Next && Step &&
@@ -846,15 +861,18 @@ bool Simd32ProfitabilityAnalysis::checkSimd32Profitable(CodeGenContext* ctx)
             }
         }
         if (!AllUniform) {
-            switch (estimateLoopCount(loop)) {
+            unsigned int estLoopCnt = estimateLoopCount(loop);
+            switch (estLoopCnt) {
             case LOOPCOUNT_LIKELY_LARGE:
             case LOOPCOUNT_UNKNOWN:
+                ctx->metrics.CollectNonUniformLoop(F, estLoopCnt, loop);
                 return false;
             case LOOPCOUNT_LIKELY_SMALL:
                 break;
             }
         }
     }
+    ctx->metrics.CollectNonUniformLoop(F, LOOPCOUNT_LIKELY_SMALL, nullptr);
 
     return true;
 }
@@ -925,7 +943,6 @@ static bool hasMultipleExits(Loop* L, WIAnalysis* WI) {
 ///          L6
 ///             L7
 /// then it returns {L2, L5}
-///
 static void getNestedLoopsWithMultpleExists(Loop* L, WIAnalysis* WI,
     SmallVectorImpl<Loop*>& Result) {
     if (getNumOfNonUniformExits(L, WI) > 1) {
@@ -944,9 +961,10 @@ static void getNestedLoopsWithMultpleExists(Loop* L, WIAnalysis* WI,
         getNestedLoopsWithMultpleExists(InnerL, WI, Result);
 }
 
+static const float NestedLoopsWithMultipleExits_THRESHOLD = 0.7f;
 
-/// Check if loops with multiple exists dominate the entire function.
-static bool hasNestedLoopsWithMultipleExits(Function* F, LoopInfo* LI,
+/// Return the ratio between loops with multiple exists to other instructions in function.
+static float NestedLoopsWithMultipleExitsRatio(Function* F, LoopInfo* LI,
     WIAnalysis* WI) {
     // Find top level nested loops with multiple non-uniform exists.
     SmallVector<Loop*, 8> Loops;
@@ -968,21 +986,31 @@ static bool hasNestedLoopsWithMultipleExits(Function* F, LoopInfo* LI,
     for (auto& BB : F->getBasicBlockList())
         FuncSize += (unsigned)BB.size();
 
-    bool retVal = false;
     if (FuncSize > 0)
     {
-        retVal = float(LoopSize) / FuncSize >= 0.7f;
+        return float(LoopSize) / FuncSize;
+    }
+    else
+    {
+        return 0.0f;
     }
-
-    return retVal;
 }
 
-static bool hasLongStridedLdStInLoop(Function* F, LoopInfo* LI, WIAnalysis* WI) {
+static const unsigned LongStridedLdStInLoop_THRESHOLD = 3;
+
+typedef struct _LdStInLoop {
+  unsigned LDs = 0;
+  unsigned STs = 0;
+  llvm::Loop* pProblematicLoop = nullptr;
+} LdStInLoop;
+
+static const LdStInLoop LongStridedLdStInLoop(Function* F, LoopInfo* LI, WIAnalysis* WI) {
+    LdStInLoop retVal;
     SmallVector<Loop*, 32> Loops;
     // Collect innermost simple loop.
     for (auto I = LI->begin(), E = LI->end(); I != E; ++I) {
         auto L = *I;
-        if (!L->empty())
+        if (!IGCLLVM::isInnermost(L))
             continue;
         if (L->getNumBlocks() != 2)
             continue;
@@ -1015,27 +1043,65 @@ static bool hasLongStridedLdStInLoop(Function* F, LoopInfo* LI, WIAnalysis* WI)
                 ++STs;
             }
         }
-        if (LDs > 3 || STs > 3)
-            return true;
+        if (LDs > LongStridedLdStInLoop_THRESHOLD ||
+            STs > LongStridedLdStInLoop_THRESHOLD)
+        {
+            retVal.LDs = LDs;
+            retVal.STs = STs;
+            retVal.pProblematicLoop = L;
+            return retVal;
+        }
     }
-    return false;
+    return retVal;
 }
 
 bool Simd32ProfitabilityAnalysis::checkSimd16Profitable(CodeGenContext* ctx) {
-    if ((IGC_GET_FLAG_VALUE(OCLSIMD16SelectionMask) & 0x1) &&
-        getLoopCyclomaticComplexity() >= CYCLOMATIC_COMPLEXITY_THRESHOLD) {
-        return false;
+    if ((IGC_GET_FLAG_VALUE(OCLSIMD16SelectionMask) & 0x1))
+    {
+        int loopCyclomaticComplexity = getLoopCyclomaticComplexity();
+
+        ctx->metrics.CollectLoopCyclomaticComplexity(
+            F,
+            loopCyclomaticComplexity,
+            CYCLOMATIC_COMPLEXITY_THRESHOLD);
+
+        if (loopCyclomaticComplexity >= CYCLOMATIC_COMPLEXITY_THRESHOLD)
+        {
+            return false;
+        }
     }
 
-    if ((IGC_GET_FLAG_VALUE(OCLSIMD16SelectionMask) & 0x2) &&
-        hasNestedLoopsWithMultipleExits(F, LI, WI)) {
-        return false;
+    if (IGC_GET_FLAG_VALUE(OCLSIMD16SelectionMask) & 0x2)
+    {
+        float nestedLoopsWithMultipleExits = NestedLoopsWithMultipleExitsRatio(F, LI, WI);
+
+        ctx->metrics.CollectNestedLoopsWithMultipleExits(
+            F,
+            nestedLoopsWithMultipleExits,
+            NestedLoopsWithMultipleExits_THRESHOLD);
+
+        if (nestedLoopsWithMultipleExits >= NestedLoopsWithMultipleExits_THRESHOLD)
+        {
+            return false;
+        }
     }
 
     // If there's wider vector load/store in a loop, skip SIMD16.
-    if ((IGC_GET_FLAG_VALUE(OCLSIMD16SelectionMask) & 0x4) &&
-        hasLongStridedLdStInLoop(F, LI, WI)) {
-        return false;
+    if (IGC_GET_FLAG_VALUE(OCLSIMD16SelectionMask) & 0x4)
+    {
+        LdStInLoop ldStInLoop = LongStridedLdStInLoop(F, LI, WI);
+
+        ctx->metrics.CollectLongStridedLdStInLoop(
+            F,
+            ldStInLoop.pProblematicLoop,
+            ldStInLoop.LDs,
+            ldStInLoop.STs,
+            LongStridedLdStInLoop_THRESHOLD);
+
+        if (ldStInLoop.pProblematicLoop != nullptr)
+        {
+            return false;
+        }
     }
 
     auto hasDouble = [](Function& F) {
@@ -1053,9 +1119,15 @@ bool Simd32ProfitabilityAnalysis::checkSimd16Profitable(CodeGenContext* ctx) {
     const CPlatform* platform = &ctx->platform;
     if (platform->GetPlatformFamily() == IGFX_GEN9_CORE &&
         platform->getPlatformInfo().eProductFamily == IGFX_GEMINILAKE &&
-        hasDouble(*F)) {
+        hasDouble(*F))
+    {
+        ctx->metrics.CollectIsGeminiLakeWithDoubles(F, false);
         return false;
     }
+    else
+    {
+        ctx->metrics.CollectIsGeminiLakeWithDoubles(F, true);
+    }
 
     return true;
 }
