@@ -138,7 +138,7 @@ namespace
         }
         Instruction* Create(Type* returnType, Value* ptr, unsigned int alignment, bool isVolatile)
         {
-            IRBuilder<> builder(m_inst);
+            IGCLLVM::IRBuilder<> builder(m_inst);
             if (isa<LoadInst>(m_inst))
             {
                 Type* newPtrType = PointerType::get(returnType, ptr->getType()->getPointerAddressSpace());
@@ -161,7 +161,7 @@ namespace
         // Emulates a GEP on a pointer of the scalar type of returnType.
         Value* CreateConstScalarGEP(Type* returnType, Value* ptr, uint32_t offset)
         {
-            IRBuilder<> builder(m_inst);
+            IGCLLVM::IRBuilder<> builder(m_inst);
             if (isa<LoadInst>(m_inst))
             {
                 Type* ePtrType = PointerType::get(returnType->getScalarType(), ptr->getType()->getPointerAddressSpace());
@@ -266,7 +266,7 @@ namespace
         // Emulates a GEP on a pointer of the scalar type of storedType.
         Value* CreateConstScalarGEP(Type* storedType, Value* ptr, uint32_t offset)
         {
-            IRBuilder<> builder(m_inst);
+            IGCLLVM::IRBuilder<> builder(m_inst);
             if (isa<StoreInst>(m_inst))
             {
                 Type* ePtrType = PointerType::get(storedType->getScalarType(), ptr->getType()->getPointerAddressSpace());
@@ -305,8 +305,9 @@ namespace
     public:
         typedef SmallVector<Instruction*, 32> InstWorkVector;
         typedef SmallVector<Value*, 16> ValVector;
-        // map from Vector value to its Component Values
-        typedef DenseMap<Value*, ValVector> V2SMap;
+
+        // vector value -> (split size in bytes -> vector's component values)
+        typedef DenseMap<Value*, DenseMap<uint32_t, ValVector>> V2SMap;
 
         enum class VPConst {
             // If a vector's size is bigger than SPLIT_SIZE, split it into multiple
@@ -314,6 +315,8 @@ namespace
             // With SPLIT_SIZE=32, we have the max vectors as below after this pass:
             //     <32 x i8>, 16xi16, 8xi32, or 4xi64!
             SPLIT_SIZE = 32,              // default, 32 bytes
+            LSC_D64_UNIFORM_SPLIT_SIZE = 512, // LSC transpose 64 x D64
+            LSC_D32_UNIFORM_SPLIT_SIZE = 256, // LSC transpose 64 x D32
             RAW_SPLIT_SIZE = 16
         };
 
@@ -366,6 +369,7 @@ namespace
             uint32_t NElts,
             uint32_t SplitSize,
             SmallVector<std::pair<Type*, uint32_t>, 8>& SplitInfo);
+        bool processScalarLoadStore(Function& F);
 
     private:
         const DataLayout* m_DL;
@@ -485,7 +489,7 @@ void VectorPreProcess::replaceAllVectorUsesWithScalars(Instruction* VI, ValVecto
             I = VI->getParent()->getFirstNonPHI();
         }
         IRBuilder<> Builder(I);
-        VectorType* VTy = cast<VectorType>(VI->getType());
+        IGCLLVM::FixedVectorType* VTy = cast<IGCLLVM::FixedVectorType>(VI->getType());
         Value* newVec = UndefValue::get(VTy);
         for (uint32_t i = 0, e = int_cast<uint32_t>(VTy->getNumElements()); i < e; ++i)
         {
@@ -583,14 +587,50 @@ uint32_t VectorPreProcess::getSplitByteSize(Instruction* I, WIAnalysisRunner& WI
     if (LoadInst* LI = dyn_cast<LoadInst>(I))
     {
         bytes = (uint32_t)VPConst::SPLIT_SIZE;
+        if (WI.isUniform(LI->getPointerOperand()) &&
+            (m_CGCtx->platform.LSCEnabled() || IGC_GET_FLAG_VALUE(UniformMemOpt4OW)))
+        {
+            if (LI->getAlignment() >= 8)
+                bytes = (uint32_t)VPConst::LSC_D64_UNIFORM_SPLIT_SIZE;
+            else if (LI->getAlignment() >= 4)
+                bytes = (uint32_t)VPConst::LSC_D32_UNIFORM_SPLIT_SIZE;
+        }
     }
     else if (StoreInst* SI = dyn_cast<StoreInst>(I))
     {
         bytes = (uint32_t)VPConst::SPLIT_SIZE;
+        Value* Addr = SI->getPointerOperand();
+        Value* Data = SI->getValueOperand();
+        if (m_CGCtx->platform.LSCEnabled() && WI.isUniform(Addr) && WI.isUniform(Data))
+        {
+            if (SI->getAlignment() >= 8)
+                bytes = (uint32_t)VPConst::LSC_D64_UNIFORM_SPLIT_SIZE;
+            else if (SI->getAlignment() >= 4)
+                bytes = (uint32_t)VPConst::LSC_D32_UNIFORM_SPLIT_SIZE;
+        }
     }
     else if (isa<LdRawIntrinsic>(I) || isa<StoreRawIntrinsic>(I))
     {
         bytes = (uint32_t)VPConst::RAW_SPLIT_SIZE;
+        if (EmitPass::shouldGenerateLSCQuery(*m_CGCtx, I) == Tristate::True)
+            bytes = (uint32_t)VPConst::SPLIT_SIZE;
+        else
+        {
+            Type* ValueTy = nullptr;
+            if (StoreRawIntrinsic* SRI = dyn_cast<StoreRawIntrinsic>(I))
+            {
+                ValueTy = SRI->getArgOperand(2)->getType();
+            }
+            else
+            {
+                ValueTy = I->getType();
+            }
+            IGCLLVM::FixedVectorType* vecType = dyn_cast_or_null<IGCLLVM::FixedVectorType>(ValueTy);
+            if (vecType && vecType->getScalarType()->getPrimitiveSizeInBits() == 64)
+            {
+                bytes = 8;  // use QW load/store
+            }
+        }
     }
     else
     {
@@ -626,7 +666,7 @@ bool VectorPreProcess::splitStore(
 {
     Instruction* SI = ASI.getInst();
     Value* StoredVal = ASI.getValueOperand();
-    VectorType* VTy = cast<VectorType>(StoredVal->getType());
+    IGCLLVM::FixedVectorType* VTy = cast<IGCLLVM::FixedVectorType>(StoredVal->getType());
     Type* ETy = VTy->getElementType();
     uint32_t nelts = int_cast<uint32_t>(VTy->getNumElements());
 
@@ -671,7 +711,7 @@ bool VectorPreProcess::splitStore(
         return false;
     }
 
-    ValVector& svals = vecToSubVec[StoredVal];
+    ValVector& svals = vecToSubVec[StoredVal][splitSize];
     if (svals.size() == 0)
     {
         // Need to create splitted values.
@@ -712,7 +752,7 @@ bool VectorPreProcess::splitStore(
         {
             Type* Ty1 = splitInfo[i].first;
             uint32_t len1 = splitInfo[i].second;
-            VectorType* VTy1 = dyn_cast<VectorType>(Ty1);
+            IGCLLVM::FixedVectorType* VTy1 = dyn_cast<IGCLLVM::FixedVectorType>(Ty1);
             for (uint32_t j = 0; j < len1; ++j)
             {
                 Value* subVec;
@@ -749,7 +789,7 @@ bool VectorPreProcess::splitStore(
     {
         Type* Ty1 = splitInfo[i].first;
         uint32_t len1 = splitInfo[i].second;
-        VectorType* VTy1 = dyn_cast<VectorType>(Ty1);
+        IGCLLVM::FixedVectorType* VTy1 = dyn_cast<IGCLLVM::FixedVectorType>(Ty1);
         for (uint32_t j = 0; j < len1; ++j)
         {
             uint32_t vAlign = (uint32_t)MinAlign(Align, (uint32_t)eOffset * EBytes);
@@ -793,7 +833,7 @@ bool VectorPreProcess::splitLoad(
 {
     Instruction* LI = ALI.getInst();
     bool isLdRaw = isa<LdRawIntrinsic>(LI);
-    VectorType* VTy = cast<VectorType>(LI->getType());
+    IGCLLVM::FixedVectorType* VTy = cast<IGCLLVM::FixedVectorType>(LI->getType());
     Type* ETy = VTy->getElementType();
     uint32_t nelts = int_cast<uint32_t>(VTy->getNumElements());
 
@@ -849,13 +889,13 @@ bool VectorPreProcess::splitLoad(
     uint32_t EBytes = int_cast<unsigned int>(m_DL->getTypeAllocSize(ETy));
 
     // Create a map entry for LI
-    ValVector& svals = vecToSubVec[LI];
+    ValVector& svals = vecToSubVec[LI][splitSize];
 
     for (uint32_t i = 0; i < len; ++i)
     {
         Type* Ty1 = splitInfo[i].first;
         uint32_t len1 = splitInfo[i].second;
-        VectorType* VTy1 = dyn_cast<VectorType>(Ty1);
+        IGCLLVM::FixedVectorType* VTy1 = dyn_cast<IGCLLVM::FixedVectorType>(Ty1);
         for (uint32_t j = 0; j < len1; ++j)
         {
             uint32_t vAlign = (uint32_t)MinAlign(Align, eOffset * EBytes);
@@ -923,7 +963,7 @@ bool VectorPreProcess::splitLoadStore(
     Optional<AbstractStoreInst> ASI = AbstractStoreInst::get(Inst);
     IGC_ASSERT_MESSAGE((ALI || ASI), "Inst should be either load or store");
     Type* Ty = ALI ? ALI->getInst()->getType() : ASI->getValueOperand()->getType();
-    VectorType* VTy = dyn_cast<VectorType>(Ty);
+    IGCLLVM::FixedVectorType* VTy = dyn_cast<IGCLLVM::FixedVectorType>(Ty);
     if (!VTy)
     {
         return false;
@@ -981,7 +1021,7 @@ bool VectorPreProcess::splitVector3LoadStore(Instruction* Inst)
     AbstractStoreInst* ASI = optionalASI ? optionalASI.getPointer() : nullptr;
     IGC_ASSERT_MESSAGE((optionalALI || optionalASI), "Inst should be either load or store");
     Type* Ty = ALI ? ALI->getInst()->getType() : ASI->getValueOperand()->getType();
-    VectorType* VTy = dyn_cast<VectorType>(Ty);
+    IGCLLVM::FixedVectorType *VTy = dyn_cast<IGCLLVM::FixedVectorType>(Ty);
     IGC_ASSERT_MESSAGE(nullptr != VTy, "Inst should be a 3-element vector load/store!");
     IGC_ASSERT_MESSAGE(VTy->getNumElements() == 3, "Inst should be a 3-element vector load/store!");
 
@@ -1154,7 +1194,7 @@ void VectorPreProcess::getOrGenScalarValues(
 {
     availBeforeInst = nullptr;
 
-    VectorType* VTy = cast<VectorType>(VecVal->getType());
+    IGCLLVM::FixedVectorType* VTy = cast<IGCLLVM::FixedVectorType>(VecVal->getType());
     if (!VTy)
     {
         scalars[0] = VecVal;
@@ -1279,9 +1319,35 @@ Instruction* VectorPreProcess::simplifyLoadStore(Instruction* Inst)
         if (NBits < 32)
             return Inst;
 
+        BitCastInst* BC = nullptr;
+        Type* DstEltTy = nullptr;
+        // Handle bitcasts patterns like:
+        //
+        // %41 = call <4 x i32> @llvm.genx.GenISA.ldrawvector.indexed.v4i32.p1v4f32(...)
+        // %bc = bitcast <4 x i32> %41 to <4 x float>
+        // %42 = extractelement <4 x float> %bc, i32 0
+        if (Inst->hasOneUse())
+        {
+            BC = dyn_cast<BitCastInst>(*Inst->users().begin());
+            if (BC)
+            {
+                IGCLLVM::FixedVectorType* DstVTy = dyn_cast<IGCLLVM::FixedVectorType>(BC->getType());
+                IGCLLVM::FixedVectorType* SrcVTy = dyn_cast<IGCLLVM::FixedVectorType>(BC->getOperand(0)->getType());
+                if (IGC_IS_FLAG_DISABLED(EnableBitcastedLoadNarrowing) ||
+                    !DstVTy || !SrcVTy || DstVTy->getNumElements() != SrcVTy->getNumElements())
+                {
+                    BC = nullptr;
+                }
+                else
+                {
+                    DstEltTy = DstVTy->getElementType();
+                }
+            }
+        }
+
         SmallVector<ExtractElementInst*, 8> ConstEEIUses;
         unsigned MaxIndex = 0;
-        for (auto U : Inst->users())
+        for (auto U : (BC ? BC : Inst)->users())
         {
             auto EEI = dyn_cast<ExtractElementInst>(U);
             if (!EEI || !isa<ConstantInt>(EEI->getIndexOperand()))
@@ -1293,7 +1359,7 @@ Instruction* VectorPreProcess::simplifyLoadStore(Instruction* Inst)
         }
 
         // All uses are constant EEI.
-        IGC_ASSERT_MESSAGE(ConstEEIUses.size() == Inst->getNumUses(), "out of sync");
+        IGC_ASSERT_MESSAGE(ConstEEIUses.size() == (BC ? BC : Inst)->getNumUses(), "out of sync");
 
         // FIXME: this is to WA an issue that splitLoadStore does not split
         // vectors of size 5, 6, 7.
@@ -1305,7 +1371,7 @@ Instruction* VectorPreProcess::simplifyLoadStore(Instruction* Inst)
         //
         // TODO: further optimize this load into a message with channel masks
         // for cases in which use indices are sparse like {0, 2}.
-        unsigned N = (unsigned)cast<VectorType>(Inst->getType())->getNumElements();
+        unsigned N = (unsigned)cast<IGCLLVM::FixedVectorType>(Inst->getType())->getNumElements();
         if (N == MaxIndex + 1)
             return Inst;
 
@@ -1314,12 +1380,20 @@ Instruction* VectorPreProcess::simplifyLoadStore(Instruction* Inst)
         IRBuilder<> Builder(Inst);
         auto ldrawvec = dyn_cast<LdRawIntrinsic>(Inst);
         bool canSimplifyOneUse = ldrawvec && isa<VectorType>(ldrawvec->getType()) &&
-            ldrawvec->hasOneUse() &&
+            (BC ? BC : Inst)->hasOneUse() &&
             !ConstEEIUses.empty();
 
         bool canSimplifyOneUseZeroIndex = canSimplifyOneUse &&
             cast<ConstantInt>(ConstEEIUses.front()->getIndexOperand())->getZExtValue() == 0;
 
+        // There is a known case where narrowing bitcasted ldrawvector to ldraw
+        // leads to a corruption. We can still simplify a vector load to
+        // a narrow one (e.g. <4 x i32> to <2 x i32> when only 0th elt is used
+        // as a float).
+        // TODO: remove WA when issue is resolved.
+        bool skipSimplifyBitcastedOneUse = canSimplifyOneUse &&
+            BC && IGC_IS_FLAG_DISABLED(EnableBitcastedLoadNarrowingToScalar);
+
         auto simplifyLDVecToLDRaw = [&](bool calc_offset)
         {
             auto EE_user = ConstEEIUses.front();
@@ -1350,28 +1424,44 @@ Instruction* VectorPreProcess::simplifyLoadStore(Instruction* Inst)
                 GenISAIntrinsic::getDeclaration(ldrawvec->getModule(), GenISAIntrinsic::GenISA_ldraw_indexed, types);
             NewLI = Builder.CreateCall(newLdRawFunction, args);
             NewLI->setDebugLoc(EE_user->getDebugLoc());
-            EE_user->replaceAllUsesWith(NewLI);
+            Value* NewBC = nullptr;
+            if (BC)
+            {
+                NewBC = Builder.CreateBitCast(NewLI, DstEltTy);
+            }
+            EE_user->replaceAllUsesWith(BC ? NewBC : NewLI);
             EE_user->eraseFromParent();
+            if (BC) {
+                BC->eraseFromParent();
+            }
         };
 
-        if (canSimplifyOneUseZeroIndex)
+        if (canSimplifyOneUseZeroIndex && !skipSimplifyBitcastedOneUse)
         {
             simplifyLDVecToLDRaw(false);
             return NewLI;
         }
-        else if (canSimplifyOneUse)
+        else if (canSimplifyOneUse && !skipSimplifyBitcastedOneUse)
         {
             simplifyLDVecToLDRaw(true);
             return NewLI;
         }
         else
         {
+            // WA: Do not narrow a bitcasted vector load to 1 elt vector load,
+            // choose at least 2 elts vector.
+            if (canSimplifyOneUseZeroIndex && skipSimplifyBitcastedOneUse)
+            {
+                MaxIndex = 1;
+            }
+
             Type* NewVecTy = FixedVectorType::get(cast<VectorType>(Inst->getType())->getElementType(),
                 MaxIndex + 1);
             NewLI = ALI.Create(NewVecTy);
 
             // Loop and replace all uses.
             SmallVector<Value*, 8> NewEEI(MaxIndex + 1, nullptr);
+            SmallVector<Value*, 8> NewBC(MaxIndex + 1, nullptr);
             for (auto EEI : ConstEEIUses)
             {
                 auto CI = cast<ConstantInt>(EEI->getIndexOperand());
@@ -1379,11 +1469,19 @@ Instruction* VectorPreProcess::simplifyLoadStore(Instruction* Inst)
                 if (NewEEI[Idx] == nullptr)
                 {
                     NewEEI[Idx] = Builder.CreateExtractElement(NewLI, CI);
+                    if (BC)
+                    {
+                        NewBC[Idx] = Builder.CreateBitCast(NewEEI[Idx], DstEltTy);
+                        dyn_cast<BitCastInst>(NewBC[Idx])->setDebugLoc(BC->getDebugLoc());
+                    }
                 }
                 dyn_cast<ExtractElementInst>(NewEEI[Idx])->setDebugLoc(EEI->getDebugLoc());
-                EEI->replaceAllUsesWith(NewEEI[Idx]);
+                EEI->replaceAllUsesWith(BC ? NewBC[Idx] : NewEEI[Idx]);
                 EEI->eraseFromParent();
             }
+            if (BC) {
+                BC->eraseFromParent();
+            }
             IGC_ASSERT_MESSAGE(Inst->use_empty(), "out of sync");
             Inst->eraseFromParent();
             return NewLI;
@@ -1420,7 +1518,7 @@ Instruction* VectorPreProcess::simplifyLoadStore(Instruction* Inst)
     if (NBits < 32)
         return Inst;
 
-    unsigned N = (unsigned)cast<VectorType>(Val->getType())->getNumElements();
+    unsigned N = (unsigned)cast<IGCLLVM::FixedVectorType>(Val->getType())->getNumElements();
     if (auto CV = dyn_cast<ConstantVector>(Val))
     {
         unsigned MaxIndex = 0;
@@ -1499,6 +1597,77 @@ Instruction* VectorPreProcess::simplifyLoadStore(Instruction* Inst)
     return Inst;
 }
 
+// Replace store instructions like
+// store i24 %1, i24 addrspace(3)* %2, align 4
+// or
+// store i48 %1, i48 addrspace(3)* %2, align 4
+//
+// with
+// store <3 x i8> %3, <3 x i8> addrspace(3)* %4, align 4
+// or
+// store <3 x i8> %16, <3 x i16> addrspace(3)* %4, align 4
+//
+// to be split later in this pass.
+// Otherwise later TypeLegalizwe pass replaces these instructions with 3-element store.
+// The same is for i24 and i48 load instruxtions.
+//
+bool VectorPreProcess::processScalarLoadStore(Function& F)
+{
+    InstWorkVector list_delete;
+    for (inst_iterator I = inst_begin(F), E = inst_end(F); I != E; ++I)
+    {
+        Instruction* inst = &*I;
+        if (isa<StoreInst>(inst))
+        {
+            Type* Ty = inst->getOperand(0)->getType();
+            if (Ty->isVectorTy())
+                continue;
+            unsigned bitSize = Ty->getScalarSizeInBits();
+            if (bitSize != 24 && bitSize != 48)
+                continue;
+            IRBuilder<> Builder(inst);
+            Type* newScalTy = bitSize == 24 ? Type::getInt8Ty(inst->getContext()) : Type::getInt16Ty(inst->getContext());
+            Type* newVecTy = IGCLLVM::FixedVectorType::get(newScalTy, 3);
+            Value* val = Builder.CreateBitCast(inst->getOperand(0), newVecTy);
+            Value* ptr = inst->getOperand(1);
+            Type* newPtrType = PointerType::get(newVecTy, ptr->getType()->getPointerAddressSpace());
+            Value* newPtr = Builder.CreateBitCast(ptr, newPtrType);
+            Builder.CreateAlignedStore(val, newPtr, IGCLLVM::getAlign(cast<StoreInst>(inst)->getAlignment()),
+                cast<StoreInst>(inst)->isVolatile());
+            list_delete.push_back(inst);
+        }
+        else if (isa<LoadInst>(inst))
+        {
+            Type* Ty = inst->getType();
+            if (Ty->isVectorTy())
+                continue;
+            unsigned bitSize = Ty->getScalarSizeInBits();
+            if (bitSize != 24 && bitSize != 48)
+                continue;
+            IRBuilder<> Builder(inst);
+            Type* newScalTy = bitSize == 24 ? Type::getInt8Ty(inst->getContext()) : Type::getInt16Ty(inst->getContext());
+            Type* newVecTy = IGCLLVM::FixedVectorType::get(newScalTy, 3);
+            Value* ptr = inst->getOperand(0);
+            Type* newPtrType = PointerType::get(newVecTy, ptr->getType()->getPointerAddressSpace());
+            Value* newPtr = Builder.CreateBitCast(ptr, newPtrType);
+            Value* newVecVal=  Builder.CreateAlignedLoad(newVecTy, newPtr, IGCLLVM::getAlign(cast<LoadInst>(inst)->getAlignment()),
+                cast<LoadInst>(inst)->isVolatile());
+            Value* newVal = Builder.CreateBitCast(newVecVal, Ty);
+            inst->replaceAllUsesWith(newVal);
+            list_delete.push_back(inst);
+        }
+    }
+
+    if (list_delete.empty())
+        return false;
+
+    for (auto i : list_delete)
+    {
+        i->eraseFromParent();
+    }
+    return true;
+}
+
 bool VectorPreProcess::runOnFunction(Function& F)
 {
     bool changed = false;
@@ -1506,6 +1675,8 @@ bool VectorPreProcess::runOnFunction(Function& F)
     m_C = &F.getContext();
     m_CGCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
 
+    changed = processScalarLoadStore(F);
+
     for (inst_iterator I = inst_begin(F), E = inst_end(F); I != E; ++I)
     {
         Instruction* inst = &*I;
@@ -1581,83 +1752,85 @@ bool VectorPreProcess::runOnFunction(Function& F)
                 continue;
             }
             Instruction* LI = ALI.getValue().getInst();
-            ValVector& svals = vecToSubVec[V];
-            if (!LI->use_empty())
-            {
-                ValVector Scalars;
-                IRBuilder<> Builder(LI);
-                for (uint32_t j = 0; j < svals.size(); ++j)
+            for (auto& it : vecToSubVec[V]) {
+                ValVector& svals = it.second;
+                if (!LI->use_empty())
                 {
-                    Type* Ty1 = svals[j]->getType();
-                    VectorType* VTy1 = dyn_cast<VectorType>(Ty1);
-                    if (VTy1) {
-                        for (uint32_t k = 0; k < VTy1->getNumElements(); ++k)
-                        {
-                            Value* S = Builder.CreateExtractElement(
-                                svals[j], Builder.getInt32(k), "split");
-                            Scalars.push_back(S);
-                        }
-                    }
-                    else
+                    ValVector Scalars;
+                    IRBuilder<> Builder(LI);
+                    for (uint32_t j = 0; j < svals.size(); ++j)
                     {
-                        Scalars.push_back(svals[j]);
-
-                        // svals[j] will be no long needed, set it to null
-                        // to prevent double-deleting later
-                        svals[j] = nullptr;
-                    }
-                }
-                // Replace LI and erase LI.
-                replaceAllVectorUsesWithScalars(LI, Scalars);
-
-                // Remove any dead scalars
-                for (uint32_t j = 0; j < Scalars.size(); ++j)
-                {
-                    if (Scalars[j]->use_empty())
-                    {
-                        Instruction* tInst = cast<Instruction>(Scalars[j]);
-                        tInst->eraseFromParent();
-                    }
-                }
-            }
-            else
-            {
-                LI->eraseFromParent();
-            }
-
-            // Remove any dead sub vectors
-            for (uint32_t j = 0; j < svals.size(); ++j)
-            {
-                if (svals[j] == nullptr)
-                {
-                    continue;
-                }
-                Instruction* tInst = cast<Instruction>(svals[j]);
-                if (tInst->use_empty())
-                {
-                    // If this is a 3-element vector load, remove it
-                    // from m_Vector3List as well.
-                    if (isAbstractLoadInst(tInst) && tInst->getType()->isVectorTy() &&
-                        cast<VectorType>(tInst->getType())->getNumElements() == 3)
-                    {
-                        InstWorkVector::iterator
-                            tI = m_Vector3List.begin(),
-                            tE = m_Vector3List.end();
-                        for (; tI != tE; ++tI)
-                        {
-                            Instruction* tmp = *tI;
-                            if (tmp == tInst)
+                        Type* Ty1 = svals[j]->getType();
+                        IGCLLVM::FixedVectorType* VTy1 = dyn_cast<IGCLLVM::FixedVectorType>(Ty1);
+                        if (VTy1) {
+                            for (uint32_t k = 0; k < VTy1->getNumElements(); ++k)
                             {
-                                break;
+                                Value* S = Builder.CreateExtractElement(
+                                    svals[j], Builder.getInt32(k), "split");
+                                Scalars.push_back(S);
                             }
                         }
-                        if (tI != m_Vector3List.end())
+                        else
                         {
-                            m_Vector3List.erase(tI);
+                            Scalars.push_back(svals[j]);
+
+                            // svals[j] will be no long needed, set it to null
+                            // to prevent double-deleting later
+                            svals[j] = nullptr;
                         }
                     }
+                    // Replace LI and erase LI.
+                    replaceAllVectorUsesWithScalars(LI, Scalars);
 
-                    tInst->eraseFromParent();
+                    // Remove any dead scalars
+                    for (uint32_t j = 0; j < Scalars.size(); ++j)
+                    {
+                        if (Scalars[j]->use_empty())
+                        {
+                            Instruction* tInst = cast<Instruction>(Scalars[j]);
+                            tInst->eraseFromParent();
+                        }
+                    }
+                }
+                else
+                {
+                    LI->eraseFromParent();
+                }
+
+                // Remove any dead sub vectors
+                for (uint32_t j = 0; j < svals.size(); ++j)
+                {
+                    if (svals[j] == nullptr)
+                    {
+                        continue;
+                    }
+                    Instruction* tInst = cast<Instruction>(svals[j]);
+                    if (tInst->use_empty())
+                    {
+                        // If this is a 3-element vector load, remove it
+                        // from m_Vector3List as well.
+                        if (isAbstractLoadInst(tInst) && tInst->getType()->isVectorTy() &&
+                            cast<IGCLLVM::FixedVectorType>(tInst->getType())->getNumElements() == 3)
+                        {
+                            InstWorkVector::iterator
+                                tI = m_Vector3List.begin(),
+                                tE = m_Vector3List.end();
+                            for (; tI != tE; ++tI)
+                            {
+                                Instruction* tmp = *tI;
+                                if (tmp == tInst)
+                                {
+                                    break;
+                                }
+                            }
+                            if (tI != m_Vector3List.end())
+                            {
+                                m_Vector3List.erase(tI);
+                            }
+                        }
+
+                        tInst->eraseFromParent();
+                    }
                 }
             }
         }
