@@ -1,28 +1,10 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
+============================= end_copyright_notice ===========================*/
 
 #pragma once
 #include "Compiler/CISACodeGen/CISACodeGen.h"
@@ -53,6 +35,9 @@ namespace IGC
             MESSAGE_A32_BYTE_SCATTERED_RW,
             MESSAGE_A64_UNTYPED_SURFACE_RW,
             MESSAGE_A64_SCATTERED_RW,
+            MESSAGE_A32_QWORD_SCATTERED_RW,
+            MESSAGE_A32_LSC_RW,
+            MESSAGE_A64_LSC_RW
         };
 
         // VECMESSAGEINFO_MAX_LEN:
@@ -67,6 +52,8 @@ namespace IGC
             // SKL+
             // Exception
             //    BDW : use A64_BYTE_SCATTERED_MAX_BYTES_8B
+            //    TGL : no 8DW scattered message.
+            //          use A64_SCATTERED_MAX_BYTES_4DW
             A32_UNTYPED_MAX_BYTES = 16,
             A32_BYTE_SCATTERED_MAX_BYTES = 4,
             A64_UNTYPED_MAX_BYTES = 16,
@@ -74,6 +61,10 @@ namespace IGC
             A64_SCATTERED_MAX_BYTES_4DW = 16,
             A64_BYTE_SCATTERED_MAX_BYTES_8B = 8,
             A64_BYTE_SCATTERED_MAX_BYTES = 4,
+            A32_A64_BYTE_LSC_LOAD_STORE_MAX_BYTES_SIMD32 = 16,
+            A32_A64_BYTE_LSC_LOAD_STORE_MAX_BYTES_SIMD16 = 32,
+            A32_A64_BYTE_LSC_LOAD_STORE_MAX_BYTES = 64,
+            A32_A64_DATA_SIZE_LSC_LOAD_STORE_MAX_BYTES = 64
         };
 
         // Calculated by getInfo().
@@ -92,6 +83,7 @@ namespace IGC
             bool forceByteScatteredRW = false);
 
 
+        void getLSCInfo(llvm::Type* Ty, uint32_t Align, CodeGenContext* ctx, bool useA32, bool transpose);
 
     private:
         const EmitPass* m_emitter;
