@@ -145,6 +145,8 @@ namespace IGC
         IGC_ASSERT(entry->getParent());
         const bool isPositionOnlyShader = (entry->getParent()->getModuleFlag("IGC::PositionOnlyVertexShader") != nullptr);
 
+        pKernelProgram->m_StagingCtx = GetContext()->m_StagingCtx;
+        pKernelProgram->m_RequestStage2 = RequestStage2(GetContext()->m_CgFlag, GetContext()->m_StagingCtx);
         {
             pKernelProgram->simd8 = *ProgramOutput();
         }
