@@ -1,28 +1,10 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
+============================= end_copyright_notice ===========================*/
 
 #include "Compiler/IGCPassSupport.h"
 #include "common/LLVMWarningsPush.hpp"
@@ -70,7 +52,8 @@ namespace IGC
     {
         // VS lowering only applies to entry function. Non-entry funtions
         // are emulation functions that do not need to be lowered!
-        MetaDataUtils* pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
+        MetaDataUtils* pMdUtils = nullptr;
+        pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
         if (!isEntryFunc(pMdUtils, &F))
         {
             return false;
@@ -331,7 +314,7 @@ namespace IGC
                         case XP0:
                         case XP1:
                         case XP2:
-                            IGC_ASSERT(m_context->platform.supportsDrawParametersSGVs() || 
+                            IGC_ASSERT(m_context->platform.supportsDrawParametersSGVs() ||
                                 m_context->m_DriverInfo.UsesVertexBuffersToSendShaderDrawParameters());
                             vertexFetchSGVExtendedParameters.at(usage - XP0) = inst;
                             break;
@@ -355,7 +338,7 @@ namespace IGC
 
             if (baseVertex || baseInstance || drawIndex || vertexId || InstanceId)
             {
-                // Find first free location at the end i.e. after all user inputs. 
+                // Find first free location at the end i.e. after all user inputs.
                 unsigned int drawParametersIndex = (ARRAY_COUNT(m_inputUsed) - 1);
                 for (int index = drawParametersIndex; index >= 0; --index)
                 {
@@ -370,9 +353,9 @@ namespace IGC
                     }
                 }
 
-                /// UMD has to limit the number of user inputs in order 
+                /// UMD has to limit the number of user inputs in order
                 /// to be sure there are 2 free inputs at the end.
-                assert(drawParametersIndex < (ARRAY_COUNT(m_inputUsed) - 8));
+                IGC_ASSERT(drawParametersIndex < (ARRAY_COUNT(m_inputUsed) - 8));
 
                 if (baseVertex || baseInstance)
                 {
@@ -442,6 +425,16 @@ namespace IGC
             }
         }
 
+        for (unsigned int slot = 0;
+            slot < ARRAY_COUNT(m_inputUsed);
+            ++slot)
+        {
+            if (m_inputUsed[slot])
+            {
+                m_vsPropsPass->SetInputSlotUsed(slot);
+            }
+        }
+
         //URB padding to 32Byte offsets
         bool addURBPaddingTo32Bytes = true;
         for (unsigned int i = 0; addURBPaddingTo32Bytes && i < MaxNumOfOutput + m_headerSize.Count(); i++)
@@ -579,8 +572,9 @@ namespace IGC
             index,
             offset
         };
-        unsigned int inIndex = int_cast<unsigned int>(cast<ConstantInt>(offset)->getZExtValue());
-        IGC_ASSERT(inIndex < MaxNumOfUserInputs);
+        unsigned int inIndex = 0;
+        inIndex = int_cast<unsigned int>(cast<ConstantInt>(offset)->getZExtValue());
+        IGC_ASSERT(inIndex < MaxNumOfInputs);
 
         Instruction* urbRead = GenIntrinsicInst::Create(
             GenISAIntrinsic::getDeclaration(m_module, GenISAIntrinsic::GenISA_URBRead),
@@ -600,7 +594,8 @@ namespace IGC
                 Instruction* newExt = ExtractElementInst::Create(urbRead, extIdx, "", elem);
                 if (isa<ConstantInt>(extIdx))
                 {
-                    unsigned int channel = int_cast<unsigned int>(
+                    unsigned int channel = 0;
+                    channel = int_cast<unsigned int>(
                         cast<ConstantInt>(extIdx)->getZExtValue());
                     m_inputUsed[inIndex * 4 + channel] = true;
                 }
@@ -696,6 +691,7 @@ namespace IGC
     }
 
     VertexShaderProperties::VertexShaderProperties() :
+        m_MaxUsedInputSlots(0),
         m_HasVertexID(false),
         m_VID(0),
         m_HasInstanceID(false),
@@ -750,6 +746,12 @@ namespace IGC
         m_vsProps.m_VID = VIDSlot;
     }
 
+    void CollectVertexShaderProperties::SetInputSlotUsed(unsigned int slot)
+    {
+        m_vsProps.m_MaxUsedInputSlots =
+            std::max(m_vsProps.m_MaxUsedInputSlots, slot + 1);
+    }
+
     void CollectVertexShaderProperties::SetShaderDrawParameter(size_t paramIndex, unsigned int slot)
     {
         IGC_ASSERT(paramIndex < ARRAY_COUNT(m_vsProps.m_VertexFetchSGVExtendedParameters.extendedParameters));
