@@ -212,6 +212,10 @@ void WIAnalysisRunner::print(raw_ostream& OS, const Module*) const
                 OS << pred->getName();
             isFirst = false;
         }
+        {
+            auto dep = getCFDependency(BB);
+            OS << "[ " << dep_str[dep] << " ]";
+        }
         OS << "\n";
         for (BasicBlock::iterator it = BB->begin(), ie = BB->end(); it != ie; ++it) {
             Instruction* I = &*it;
@@ -491,7 +495,7 @@ void WIAnalysisRunner::updateArgsDependency(llvm::Function* pF)
     int implicitArgStart = (unsigned)(pF->arg_size()
         - implicitArgs.size()
         - (IsSubroutine ? 0 : m_ModMD->pushInfo.pushAnalysisWIInfos.size()));
-    IGC_ASSERT(implicitArgStart >= 0 && "Function arg size does not match meta data and push args.");
+    IGC_ASSERT_MESSAGE(implicitArgStart >= 0, "Function arg size does not match meta data and push args.");
 
     llvm::Function::arg_iterator ai, ae;
     ai = pF->arg_begin();
@@ -586,9 +590,9 @@ bool WIAnalysis::insideDivergentCF(const Value* val) const
     return Runner.insideDivergentCF(val);
 }
 
-bool WIAnalysis::insideThreadDivergentCF(const Value* val) const
+bool WIAnalysis::insideWorkgroupDivergentCF(const Value* val) const
 {
-    return Runner.insideThreadDivergentCF(val);
+    return Runner.insideWorkgroupDivergentCF(val);
 }
 
 WIAnalysis::WIDependancy WIAnalysisRunner::whichDepend(const Value* val) const
@@ -599,6 +603,10 @@ WIAnalysis::WIDependancy WIAnalysisRunner::whichDepend(const Value* val) const
     {
         return WIAnalysis::UNIFORM_GLOBAL;
     }
+    else if (isa<StaticConstantPatchIntrinsic>(val))
+    {
+        return WIAnalysis::UNIFORM_GLOBAL;
+    }
     auto EL = m_depMap.GetAttributeWithoutCreating(val);
     if (IGC_IS_FLAG_ENABLED(DisableUniformAnalysis))
     {
@@ -636,6 +644,34 @@ bool WIAnalysisRunner::isGlobalUniform(const Value* val) const
     return dep == WIAnalysis::UNIFORM_GLOBAL;
 }
 
+WIAnalysis::WIDependancy WIAnalysisRunner::getCFDependency(const BasicBlock* BB) const
+{
+    auto II = m_ctrlBranches.find(BB);
+    if (II == m_ctrlBranches.end())
+        return WIAnalysis::UNIFORM_GLOBAL;
+
+    WIAnalysis::WIDependancy dep = WIAnalysis::UNIFORM_GLOBAL;
+    for (auto* BI : II->second)
+    {
+        auto newDep = whichDepend(BI);
+        if (depRank(dep) < depRank(newDep))
+            dep = newDep;
+    }
+
+    return dep;
+}
+
+bool WIAnalysisRunner::insideWorkgroupDivergentCF(const Value* val) const
+{
+    if (auto* I = dyn_cast<Instruction>(val))
+    {
+        auto dep = getCFDependency(I->getParent());
+        return depRank(dep) > WIAnalysis::UNIFORM_WORKGROUP;
+    }
+
+    return false;
+}
+
 WIAnalysis::WIDependancy WIAnalysisRunner::getDependency(const Value* val)
 {
     if (m_depMap.GetAttributeWithoutCreating(val) == m_depMap.end())
@@ -777,23 +813,26 @@ void WIAnalysisRunner::calculate_dep(const Value* val)
         // LLVM does not have compile time polymorphisms
         // TODO: to make things faster we may want to sort the list below according
         // to the order of their probability of appearance.
-        if (const BinaryOperator * BI = dyn_cast<BinaryOperator>(inst))         dep = calculate_dep(BI);
-        else if (const CallInst * CI = dyn_cast<CallInst>(inst))                     dep = calculate_dep(CI);
+        if (const BinaryOperator* BI = dyn_cast<BinaryOperator>(inst))              dep = calculate_dep(BI);
+        else if (const CallInst* CI = dyn_cast<CallInst>(inst))                     dep = calculate_dep(CI);
         else if (isa<CmpInst>(inst))                                                dep = calculate_dep_simple(inst);
         else if (isa<ExtractElementInst>(inst))                                     dep = calculate_dep_simple(inst);
-        else if (const GetElementPtrInst * GEP = dyn_cast<GetElementPtrInst>(inst))  dep = calculate_dep(GEP);
+        else if (const GetElementPtrInst* GEP = dyn_cast<GetElementPtrInst>(inst))  dep = calculate_dep(GEP);
         else if (isa<InsertElementInst>(inst))                                      dep = calculate_dep_simple(inst);
         else if (isa<InsertValueInst>(inst))                                        dep = calculate_dep_simple(inst);
-        else if (const PHINode * Phi = dyn_cast<PHINode>(inst))                      dep = calculate_dep(Phi);
+        else if (const PHINode* Phi = dyn_cast<PHINode>(inst))                      dep = calculate_dep(Phi);
         else if (isa<ShuffleVectorInst>(inst))                                      dep = calculate_dep_simple(inst);
         else if (isa<StoreInst>(inst))                                              dep = calculate_dep_simple(inst);
         else if (inst->isTerminator())                                              dep = calculate_dep_terminator(dyn_cast<IGCLLVM::TerminatorInst>(inst));
-        else if (const SelectInst * SI = dyn_cast<SelectInst>(inst))                 dep = calculate_dep(SI);
-        else if (const AllocaInst * AI = dyn_cast<AllocaInst>(inst))                 dep = calculate_dep(AI);
-        else if (const CastInst * CI = dyn_cast<CastInst>(inst))                     dep = calculate_dep(CI);
+        else if (const SelectInst* SI = dyn_cast<SelectInst>(inst))                 dep = calculate_dep(SI);
+        else if (const AllocaInst* AI = dyn_cast<AllocaInst>(inst))                 dep = calculate_dep(AI);
+        else if (const CastInst* CI = dyn_cast<CastInst>(inst))                     dep = calculate_dep(CI);
         else if (isa<ExtractValueInst>(inst))                                       dep = calculate_dep_simple(inst);
-        else if (const LoadInst * LI = dyn_cast<LoadInst>(inst))                     dep = calculate_dep(LI);
-        else if (const VAArgInst * VAI = dyn_cast<VAArgInst>(inst))                  dep = calculate_dep(VAI);
+        else if (const LoadInst* LI = dyn_cast<LoadInst>(inst))                     dep = calculate_dep(LI);
+        else if (const VAArgInst* VAI = dyn_cast<VAArgInst>(inst))                  dep = calculate_dep(VAI);
+#if LLVM_VERSION_MAJOR >= 10
+        else if (inst->getOpcode() == Instruction::FNeg)                            dep = calculate_dep_simple(inst);
+#endif
 
         if (m_func->hasFnAttribute("KMPLOCK"))
         {
@@ -914,14 +953,23 @@ void WIAnalysisRunner::update_cf_dep(const IGCLLVM::TerminatorInst* inst)
                     m_pChangedNew->push_back(it->second);
             }
 
+            // This is an optimization that tries to detect instruction
+            // not really affected by control-flow divergency because
+            // all the sources are outside the region.
+            // However this is only as good as we can get because we
+            // only search limited depth
             if (isRegionInvariant(defi, &br_info, 0))
             {
                 continue;
             }
-            // look at the uses
+            // We need to look at where the use is in order to decide
+            // we should make def to be "random" when loop is not in
+            // LCSSA form because we do not have LCSSA phi-nodes.
+            // 1) if use is in the full-join
+            // 2) if use is even outside the full-join
+            // 3) if use is in partial-join but def is not in partial-join
             Value::use_iterator use_it = defi->use_begin();
             Value::use_iterator use_e = defi->use_end();
-
             for (; use_it != use_e; ++use_it)
             {
                 Instruction* user = dyn_cast<Instruction>((*use_it).getUser());
@@ -930,32 +978,20 @@ void WIAnalysisRunner::update_cf_dep(const IGCLLVM::TerminatorInst* inst)
                 PHINode* phi = dyn_cast<PHINode>(user);
                 if (phi)
                 {
-                    // another place we assume all critical edges have been split and
-                    // phi-move will be placed on the blocks created on those
+                    // another place we assume all critical edges have been
+                    // split and phi-move will be placed on those splitters
                     user_blk = phi->getIncomingBlock(*use_it);
                 }
-                auto canUserBeHoistedToDefBlock = [](Instruction* UI, Instruction* DI)->bool {
-                    for (auto& uo : UI->operands())
-                    {
-                        Value* V = uo.get();
-                        if (V == DI)
-                            continue;
-                        else if (dyn_cast<Constant>(V))
-                            continue;
-                        else
-                            return false;
-                    }
-                    return true;
-                };
-                if ((user_blk == def_blk) ||
-                    (canUserBeHoistedToDefBlock(user, defi) && !phi))
+                if (user_blk == def_blk)
                 {
                     // local def-use, not related to control-dependence
                     continue; // skip
                 }
                 if (user_blk == br_info.full_join ||
-                    br_info.partial_joins.count(user_blk) ||
-                    !br_info.influence_region.count(user_blk))
+                    !br_info.influence_region.count(user_blk) ||
+                    (br_info.partial_joins.count(user_blk) &&
+                     !br_info.partial_joins.count(def_blk))
+                   )
                 {
                     updateDepMap(defi, instDep);
                     // break out of the use loop
@@ -1267,6 +1303,19 @@ WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep(const CallInst* inst)
         GII_id == GenISAIntrinsic::GenISA_bftof ||
         GII_id == GenISAIntrinsic::GenISA_2fto2bf ||
         GII_id == GenISAIntrinsic::GenISA_dual_subslice_id ||
+        GII_id == GenISAIntrinsic::GenISA_hftobf8 ||
+        GII_id == GenISAIntrinsic::GenISA_bf8tohf ||
+        GII_id == GenISAIntrinsic::GenISA_srnd ||
+        GII_id == GenISAIntrinsic::GenISA_ftotf32 ||
+        GII_id == GenISAIntrinsic::GenISA_tf32tof ||
+        GII_id == GenISAIntrinsic::GenISA_GlobalBufferPointer ||
+        GII_id == GenISAIntrinsic::GenISA_LocalBufferPointer ||
+        GII_id == GenISAIntrinsic::GenISA_InlinedData ||
+        GII_id == GenISAIntrinsic::GenISA_TileYOffset ||
+        GII_id == GenISAIntrinsic::GenISA_GetShaderRecordPtr ||
+        GII_id == GenISAIntrinsic::GenISA_URBWrite ||
+        GII_id == GenISAIntrinsic::GenISA_URBRead ||
+        GII_id == GenISAIntrinsic::GenISA_URBReadOutput ||
         GII_id == GenISAIntrinsic::GenISA_getSR0   ||
         GII_id == GenISAIntrinsic::GenISA_getSR0_0 ||
         GII_id == GenISAIntrinsic::GenISA_mul_rtz  ||
@@ -1282,8 +1331,24 @@ WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep(const CallInst* inst)
         GII_id == GenISAIntrinsic::GenISA_hw_thread_id ||
         GII_id == GenISAIntrinsic::GenISA_hw_thread_id_alloca ||
         GII_id == GenISAIntrinsic::GenISA_StackAlloca ||
+        GII_id == GenISAIntrinsic::GenISA_vectorUniform ||
         GII_id == GenISAIntrinsic::GenISA_getR0 ||
-        GII_id == GenISAIntrinsic::GenISA_vectorUniform)
+        GII_id == GenISAIntrinsic::GenISA_getPayloadHeader ||
+        GII_id == GenISAIntrinsic::GenISA_getWorkDim ||
+        GII_id == GenISAIntrinsic::GenISA_getNumWorkGroups ||
+        GII_id == GenISAIntrinsic::GenISA_getLocalSize ||
+        GII_id == GenISAIntrinsic::GenISA_getGlobalSize ||
+        GII_id == GenISAIntrinsic::GenISA_getEnqueuedLocalSize ||
+        GII_id == GenISAIntrinsic::GenISA_getLocalID_X ||
+        GII_id == GenISAIntrinsic::GenISA_getLocalID_Y ||
+        GII_id == GenISAIntrinsic::GenISA_getLocalID_Z ||
+        GII_id == GenISAIntrinsic::GenISA_getPrivateBase ||
+        GII_id == GenISAIntrinsic::GenISA_getPrintfBuffer ||
+        GII_id == GenISAIntrinsic::GenISA_getStageInGridOrigin ||
+        GII_id == GenISAIntrinsic::GenISA_getStageInGridSize ||
+        GII_id == GenISAIntrinsic::GenISA_getSyncBuffer ||
+        GII_id == GenISAIntrinsic::GenISA_GetImplicitBufferPtr ||
+        GII_id == GenISAIntrinsic::GenISA_GetLocalIdBufferPtr)
     {
         switch (GII_id)
         {
@@ -1293,7 +1358,6 @@ WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep(const CallInst* inst)
             // collect the seeds for forcing uniform vectors
             m_forcedUniforms.push_back(inst);
             return WIAnalysis::UNIFORM_THREAD;
-        case GenISAIntrinsic::GenISA_getR0:
         case GenISAIntrinsic::GenISA_getSR0:
         case GenISAIntrinsic::GenISA_getSR0_0:
         case GenISAIntrinsic::GenISA_eu_id:
@@ -1305,6 +1369,25 @@ WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep(const CallInst* inst)
             // Make sure they are UNIFORM_WORKGROUP
             //return WIAnalysis::UNIFORM_WORKGROUP;
             return WIAnalysis::UNIFORM_THREAD;
+        case GenISAIntrinsic::GenISA_GetImplicitBufferPtr:
+        case GenISAIntrinsic::GenISA_GetLocalIdBufferPtr:
+            return WIAnalysis::UNIFORM_THREAD;
+        case GenISAIntrinsic::GenISA_getR0:
+        case GenISAIntrinsic::GenISA_getPayloadHeader:
+        case GenISAIntrinsic::GenISA_getWorkDim:
+        case GenISAIntrinsic::GenISA_getNumWorkGroups:
+        case GenISAIntrinsic::GenISA_getLocalSize:
+        case GenISAIntrinsic::GenISA_getGlobalSize:
+        case GenISAIntrinsic::GenISA_getEnqueuedLocalSize:
+        case GenISAIntrinsic::GenISA_getLocalID_X:
+        case GenISAIntrinsic::GenISA_getLocalID_Y:
+        case GenISAIntrinsic::GenISA_getLocalID_Z:
+        case GenISAIntrinsic::GenISA_getPrivateBase:
+        case GenISAIntrinsic::GenISA_getPrintfBuffer:
+        case GenISAIntrinsic::GenISA_getStageInGridOrigin:
+        case GenISAIntrinsic::GenISA_getStageInGridSize:
+        case GenISAIntrinsic::GenISA_getSyncBuffer:
+            return ImplicitArgs::getArgDep(GII_id);
         }
 
         if (intrinsic_name == llvm_input ||
@@ -1318,21 +1401,120 @@ WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep(const CallInst* inst)
             }
         }
 
+        if (GII_id == GenISAIntrinsic::GenISA_TileYOffset)
+        {
+            IGC_ASSERT(m_CGCtx->type == ShaderType::RAYTRACING_SHADER);
+            auto* Ctx = static_cast<RayDispatchShaderContext*>(m_CGCtx);
+            if (auto Mode = Ctx->knownSIMDSize())
+            {
+                auto* TYI = cast<TileYIntrinsic>(inst);
+                uint32_t TileXDim = TYI->getTileXDim();
+                uint32_t SubtileXDim = TYI->getSubtileXDim();
+                const uint32_t Lanes = numLanes(*Mode);
+                // We currently tile along the x-dim first. If the SIMD size
+                // perfectly divides the x-dim, then the y-dim must be uniform.
+                return (TileXDim % Lanes == 0 &&
+                        (SubtileXDim == 0 || SubtileXDim % Lanes == 0)) ?
+                    WIAnalysis::UNIFORM_THREAD :
+                    WIAnalysis::RANDOM;
+            }
+            else
+            {
+                return WIAnalysis::RANDOM;
+            }
+        }
 
         if (intrinsic_name == llvm_sgv)
         {
-            SGVUsage usage = (SGVUsage)cast<ConstantInt>(inst->getOperand(0))->getZExtValue();
-            if ((usage != VFACE
-                ) &&
-                usage != ACTUAL_COARSE_SIZE_X &&
-                usage != ACTUAL_COARSE_SIZE_Y &&
-                usage != THREAD_GROUP_ID_X &&
-                usage != THREAD_GROUP_ID_Y &&
-                usage != THREAD_GROUP_ID_Z
-                )
+            IGC_ASSERT(isa<SGVIntrinsic>(inst));
+            const SGVIntrinsic* systemValueIntr = cast<SGVIntrinsic>(inst);
+            switch (systemValueIntr->getUsage())
+            {
+            case VFACE: // palygon front/back facing from PS payload
+            case RENDER_TARGET_ARRAY_INDEX: // render target array index from PS payload
+            case VIEWPORT_INDEX: // viewport index from PS payload
+            {
+                IGC_ASSERT(m_CGCtx->type == ShaderType::PIXEL_SHADER);
+                const bool hasMultipolyDispatch =
+                    m_CGCtx->platform.supportDualSimd8PS();
+                return hasMultipolyDispatch ? WIAnalysis::RANDOM : WIAnalysis::UNIFORM_THREAD;
+            }
+            case POSITION_X: // position from VUE header in GS or pixel position X in PS
+            case POSITION_Y: // position from VUE header in GS or pixel position Y in PS
+            case POSITION_Z: // position from VUE header in GS or source depth in PS
+            case POSITION_W: // position from VUE header in GS or source W in PS
+            case PRIMITIVEID: // primitive id payload phase in GS
+            case GS_INSTANCEID: // GS instance id, calculated from URB handles
+            case POINT_WIDTH: // point width in VUE header in GS
+            case INPUT_COVERAGE_MASK: // pixel coverage mask payload phase in PS
+            case SAMPLEINDEX:  // sample index from PS payload
+            case CLIP_DISTANCE: // unused
+            case THREAD_ID_X: // global invocation id X in CS or OCL Kernel
+            case THREAD_ID_Y: // global invocation id Y in CS or OCL Kernel
+            case THREAD_ID_Z: // global invocation id Z in CS or OCL Kernel
+            case THREAD_ID_IN_GROUP_X: // local invocation id X in CS or OCL Kernel
+            case THREAD_ID_IN_GROUP_Y: // local invocation id Y in CS or OCL Kernel
+            case THREAD_ID_IN_GROUP_Z: // local invocation id Z in CS or OCL Kernel
+            case OUTPUT_CONTROL_POINT_ID: // unused
+            case DOMAIN_POINT_ID_X: // domain point U from DS payload
+            case DOMAIN_POINT_ID_Y: // domain point V from DS payload
+            case DOMAIN_POINT_ID_Z: // domain point W from DS payload
+            case VERTEX_ID: // vertex id in VS, delivered as an attribute
+            case REQUESTED_COARSE_SIZE_X: // requested per-subspan coarse pixel size X from PS payload
+            case REQUESTED_COARSE_SIZE_Y: // requested per-subspan coarse pixel size Y from PS payload
+            case CLIP_DISTANCE_X: // DX10 clip distance X from VUE header in GS
+            case CLIP_DISTANCE_Y: // DX10 clip distance Y from VUE header in GS
+            case CLIP_DISTANCE_Z: // DX10 clip distance Z from VUE header in GS
+            case CLIP_DISTANCE_W: // DX10 clip distance W from VUE header in GS
+            case POSITION_X_OFFSET: // pixel position offset X in PS
+            case POSITION_Y_OFFSET: // pixel position offset Y in PS
+            case POINT_COORD_X: // point-sprite coordinate X from PS attributes
+            case POINT_COORD_Y: // point-sprite coordinate Y from PS attributes
             {
                 return WIAnalysis::RANDOM;
             }
+            case MSAA_RATE: // multisample rate from PS payload
+            case DISPATCH_DIMENSION_X: // dispatch size X from MS payload
+            case DISPATCH_DIMENSION_Y: // dispatch size Y from MS payload
+            case DISPATCH_DIMENSION_Z: // dispatch size Z from MS payload
+            case INDIRECT_DATA_ADDRESS: // indirect data address from MS payload
+            case SHADER_TYPE:
+            {
+                return WIAnalysis::UNIFORM_GLOBAL;
+            }
+            case THREAD_GROUP_ID_X: // workgroup id X in CS or OCL Kernel
+            case THREAD_GROUP_ID_Y: // workgroup id Y in CS or OCL Kernel
+            case THREAD_GROUP_ID_Z: // workgroup id Z in CS or OCL Kernel
+            {
+                return WIAnalysis::UNIFORM_WORKGROUP;
+            }
+            case ACTUAL_COARSE_SIZE_X: // actual coarse pixel size X from PS payload
+            case ACTUAL_COARSE_SIZE_Y: // actual coarse pixel size Y from PS payload
+            case THREAD_ID_WITHIN_THREAD_GROUP: // (physical) thread id in thread group from CS payload
+            {
+                return WIAnalysis::UNIFORM_THREAD;
+            }
+            case XP0: // base vertex from VS attributes
+            case XP1: // base instance from VS attributes
+            case XP2: // draw index from VS attributes
+            {
+                if (m_CGCtx->type == ShaderType::TASK_SHADER ||
+                    m_CGCtx->type == ShaderType::MESH_SHADER)
+                {
+                    // XP0 is used for draw index in mesh and task
+                    return WIAnalysis::UNIFORM_GLOBAL;
+                }
+                // Extended parameters are delivered in VS as attributes. Values
+                // are uniform but delivered per-vertex, frontends can use
+                // subgroup operations to get the uniform value.
+                return WIAnalysis::RANDOM;
+            }
+            case NUM_SGV:
+                IGC_ASSERT_MESSAGE(0, "Unexpected value");
+                break;
+            // This switch intentionally has no `default:` case. Whenever a new
+            // SGV type is added this code must be updated.
+            }
         }
         if (intrinsic_name == llvm_getMessagePhaseX ||
             intrinsic_name == llvm_getMessagePhaseXV)
@@ -1393,6 +1575,33 @@ WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep(const CallInst* inst)
             }
         }
 
+        if (intrinsic_name == llvm_URBRead ||
+            intrinsic_name == llvm_URBReadOutput)
+        {
+            if (!m_CGCtx->platform.isProductChildOf(IGFX_DG2))
+            {
+                return WIAnalysis::RANDOM;
+            }
+            if (m_CGCtx->type != ShaderType::TASK_SHADER &&
+                m_CGCtx->type != ShaderType::MESH_SHADER)
+            {
+                return WIAnalysis::RANDOM;
+            }
+        }
+
+        if (intrinsic_name == llvm_URBWrite)
+        {
+            // TODO: enable this for other platforms/shader types if needed
+            if (!m_CGCtx->platform.isProductChildOf(IGFX_DG2))
+            {
+                return WIAnalysis::RANDOM;
+            }
+            if (m_CGCtx->type != ShaderType::TASK_SHADER &&
+                m_CGCtx->type != ShaderType::MESH_SHADER)
+            {
+                return WIAnalysis::RANDOM;
+            }
+        }
 
         // Iterate over all input dependencies. If all are uniform - propagate it.
         // otherwise - return RANDOM
