@@ -1,28 +1,10 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
+============================= end_copyright_notice ===========================*/
 
 #include "Compiler/CISACodeGen/layout.hpp"
 #include "Compiler/CISACodeGen/ShaderCodeGen.hpp"
@@ -210,7 +192,7 @@ bool Layout::tryMovingWrite(llvm::Instruction* write, llvm::Loop* loop, LoopInfo
         llvm::BasicBlock* blk = blocksToMove.back();
 
         // If one (and only one) of the predecessors is in the needed loop, move blocks after it
-        llvm::BasicBlock* insertPoint;
+        llvm::BasicBlock* insertPoint = nullptr;
         int predsInLoop = 0;
         for (pred_iterator predIter = pred_begin(blk), predEnd = pred_end(blk);
             predIter != predEnd; ++predIter)
@@ -367,6 +349,10 @@ static bool HasThreadGroupBarrierInBlock(BasicBlock * blk)
 
 BasicBlock* Layout::getLastReturnBlock(Function& Func)
 {
+    // If Func has any return BB, return the last return BB (may have multiple);
+    // otherwise, return the last BB that has no succ;
+    //     or nullptr if every BB has Succ (infinite looping)
+    BasicBlock* noRetAndNoSucc = nullptr;  // for func that never returns
     Function::BasicBlockListType& bblist = Func.getBasicBlockList();
     for (Function::BasicBlockListType::reverse_iterator RI = bblist.rbegin(),
         RE = bblist.rend();  RI != RE; ++RI)
@@ -374,11 +360,18 @@ BasicBlock* Layout::getLastReturnBlock(Function& Func)
         BasicBlock* bb = &*RI;
         if (succ_begin(bb) == succ_end(bb))
         {
-            return bb;
+            if (isa_and_nonnull<ReturnInst>(bb->getTerminator()))
+            {
+                return bb;
+            }
+            if (!noRetAndNoSucc)
+            {
+                noRetAndNoSucc = bb;
+            }
         }
     }
     // Function does not have a return block
-    return nullptr;
+    return noRetAndNoSucc;
 }
 
 //
