@@ -30,6 +30,8 @@ namespace IGC
         EU_MESSAGE_TARGET_URB = 6,
         EU_MESSAGE_TARGET_THREAD_SPAWNER = 7,
         EU_MESSAGE_TARGET_SFID_VME = 8,
+        EU_MESSAGE_TARGET_SFID_BTD = 7,
+        EU_MESSAGE_TARGET_SFID_RTA = 8,
         EU_MESSAGE_TARGET_DATA_PORT_DATA_CACHE_READ_ONLY = 9,
         EU_GEN7_MESSAGE_TARGET_DATA_PORT_DATA_CACHE = 10,
         EU_GEN7_MESSAGE_TARGET_PIXEL_INTERPOLATOR = 11,
@@ -276,6 +278,10 @@ namespace IGC
     \*****************************************************************************/
     static const unsigned int BINDLESS_BTI = 252;
 
+    /*****************************************************************************\
+    CONST: SSH_BINDLESS_BTI
+    \*****************************************************************************/
+    static const unsigned int SSH_BINDLESS_BTI = 251;
     /*****************************************************************************\
     ENUM: EU_GEN7_DATA_CACHE_MESSAGE_TYPE
     \*****************************************************************************/
@@ -607,6 +613,83 @@ namespace IGC
         } DW0;
     };
 
+    /*****************************************************************************\
+    ENUM: EU_GW_FENCE_PORTS. Spec: Enumeration_GW_FENCE_PORTS.
+    \*****************************************************************************/
+    enum EU_GW_FENCE_PORTS
+    {
+        EU_GW_FENCE_PORTS_None = 0x0,
+        EU_GW_FENCE_PORTS_SLM = 0x1,
+        EU_GW_FENCE_PORTS_UGM = 0x2,
+        EU_GW_FENCE_PORTS_UGML = 0x4,
+        EU_GW_FENCE_PORTS_TGM = 0x8,
+    };
+
+    /*****************************************************************************\
+    STRUCT: URBFenceGen12. Spec: Instruction_URBFence (symbol: MSD_URBFENCE).
+    \*****************************************************************************/
+    struct URBFenceGen12
+    {
+        union _DW0
+        {
+            struct _All
+            {
+                unsigned int       URBOpcode : 4; // [3:0]
+                unsigned int : 16;
+                unsigned int       ResponseLength : 5; // [24:20]
+                unsigned int       MessageLength : 4; // [28:25]
+                unsigned int : 3;
+            } All;
+
+            unsigned int   Value;
+        } DW0;
+    };
+
+    /*****************************************************************************\
+    STRUCT: EOTMessageDescriptorGen12. Spec: Instruction_EOT (symbol: MSD_EOT).
+    \*****************************************************************************/
+    struct EOTMessageDescriptorGen12
+    {
+        union _DW0
+        {
+            struct _All
+            {
+                unsigned int       EOTSubfunction : 3; // [2:0]
+                unsigned int : 9;
+                unsigned int       FenceDataPorts : 4; // [15:12]
+                unsigned int : 4;
+                unsigned int       ResponseLength : 5; // [24:20]
+                unsigned int       MessageLength : 4; // [28:25]
+                unsigned int : 3;
+            } All;
+
+            unsigned int   Value;
+        } DW0;
+    };
+
+    /*****************************************************************************\
+    STRUCT: TraceRayBTDMessageDescriptorGen12
+    \*****************************************************************************/
+    struct TraceRayBTDMessageDescriptorGen12
+    {
+        union _DW0
+        {
+            struct _All
+            {
+                unsigned int                      : 8;
+                unsigned int       SIMDMode       : 1; // [8]
+                unsigned int                      : 5;
+                unsigned int       MessageType    : 4; // [17:14]
+                unsigned int                      : 1;
+                unsigned int       HeaderPresent  : 1; // [19]
+                unsigned int       ResponseLength : 5; // [20:24]
+                unsigned int       MessageLength  : 4; // [28:25]
+                unsigned int : 3;
+            } All;
+
+            unsigned int   Value;
+        } DW0;
+    };
 
     /*****************************************************************************\
     STRUCT: VMEMessageDescriptorGen8_0
@@ -696,6 +779,7 @@ namespace IGC
         EU_URB_OPCODE_ATOMIC_ADD = 6,
         EU_URB_OPCODE_SIMD8_WRITE = 7,
         EU_URB_OPCODE_SIMD8_READ = 8,
+        EU_URB_OPCODE_FENCE = 9,
     };
 
     EU_GEN6_SAMPLER_MESSAGE_TYPE GetSampleMessage(EOPCODE opCode);
@@ -824,5 +908,15 @@ namespace IGC
         const uint32_t regs2rcv
     );
 
+    uint URBFence();
 
-}
\ No newline at end of file
+    uint EOTGateway(
+        const EU_GW_FENCE_PORTS fencePorts);
+
+    uint BindlessThreadDispatch(
+        const uint   messageLength,
+        const uint   SIMDMode,
+        const bool   IsTraceMessage,
+        const bool   IsRayQueryMessage);
+
+}
