@@ -95,6 +95,7 @@ namespace IGC
         STATELESS_A32,
         SSH_BINDLESS,
         SSH_BINDLESS_CONSTANT_BUFFER,
+        SSH_BINDLESS_TEXTURE,
         BUFFER_TYPE_UNKNOWN = 31
     };
 
@@ -114,7 +115,7 @@ namespace IGC
         ADDRESS_SPACE_CONSTANT = 2,
         ADDRESS_SPACE_LOCAL = 3,
         ADDRESS_SPACE_GENERIC = 4,
-        ADDRESS_SPACE_A32 = 5,
+        ADDRESS_SPACE_THREAD_ARG = 5,
 
         ADDRESS_SPACE_GLOBAL_OR_PRIVATE = 20,
 
@@ -145,6 +146,9 @@ namespace IGC
         EATOMIC_FADD,
         EATOMIC_FSUB,
         EATOMIC_FADD64,
+        EATOMIC_FSUB64,
+        EATOMIC_LOAD,
+        EATOMIC_STORE,
         //64 bit
         EATOMIC_IADD64,
         EATOMIC_SUB64,
@@ -229,9 +233,18 @@ namespace IGC
         POSITION_Y_OFFSET,
         RENDER_TARGET_ARRAY_INDEX,
         MSAA_RATE,
+        DISPATCH_DIMENSION_X,
+        DISPATCH_DIMENSION_Y,
+        DISPATCH_DIMENSION_Z,
+        INDIRECT_DATA_ADDRESS,
+        THREAD_ID_WITHIN_THREAD_GROUP,
         XP0,
         XP1,
         XP2,
+        SHADER_TYPE, // For raytracing
+        POINT_COORD_X,
+        POINT_COORD_Y,
+        VIEWPORT_INDEX,
         NUM_SGV,
     };
 
@@ -252,6 +265,51 @@ namespace IGC
     };
 
 
+
+    enum DISPATCH_SHADER_RAY_INFO_TYPE : unsigned int
+    {
+        WORLD_RAY_ORG = 0,
+        WORLD_RAY_DIR = 1,
+        OBJ_RAY_ORG = 2,
+        OBJ_RAY_DIR = 3,
+        RAY_T_MIN = 4,
+        RAY_T_CURRENT = 5,
+        PRIMITIVE_INDEX_TRIANGLE = 6,
+        PRIMITIVE_INDEX_PROCEDURAL = 7,
+        INSTANCE_ID = 8,
+        INSTANCE_INDEX = 9,
+        RAY_FLAGS = 10,
+        OBJECT_TO_WORLD = 11,
+        WORLD_TO_OBJECT = 12,
+        COMMITTED_TRIANGLE_FRONT_FACE = 13,
+        COMMITTED_GEOMETRY_INDEX = 14,
+        COMMITTED_INSTANCE_INDEX = 15,
+        COMMITTED_PRIMITIVE_INDEX = 16,
+        COMMITTED_INSTANCE_ID = 17,
+        COMMITTED_BARYCENTRICS = 18,
+        COMMITTED_OBJECT_TO_WORLD = 19,
+        COMMITTED_WORLD_TO_OBJECT = 20,
+        COMMITTED_OBJECT_RAY_ORG = 21,
+        COMMITTED_OBJECT_RAY_DIR = 22,
+        COMMITTED_INST_CONTRIBUTION_TO_HITGROUP_INDEX = 23,
+        CANDIDATE_TRIANGLE_FRONT_FACE = 24,
+        CANDIDATE_TRIANGLE_T_CURRENT = 25,
+        CANDIDATE_GEOMETRY_INDEX = 26,
+        CANDIDATE_INSTANCE_INDEX = 27,
+        CANDIDATE_PRIMITIVE_INDEX = 28,
+        CANDIDATE_INSTANCE_ID = 29,
+        CANDIDATE_BARYCENTRICS = 30,
+        CANDIDATE_OBJECT_TO_WORLD = 31,
+        CANDIDATE_WORLD_TO_OBJECT = 32,
+        CANDIDATE_OBJECT_RAY_ORG = 33,
+        CANDIDATE_OBJECT_RAY_DIR = 34,
+        CANDIDATE_PROCEDURAL_PRIM_NON_OPAQUE = 35,
+        CANDIDATE_INST_CONTRIBUTION_TO_HITGROUP_INDEX = 36,
+        GEOMETRY_INDEX_TRIANGLE = 37,
+        GEOMETRY_INDEX_PROCEDURAL = 38,
+        RAY_INFO_UNKNOWN = 0xFF
+    };
+
 }
 
 #endif //CODE_GEN_PUBLIC_ENUMS_H_
