@@ -1,28 +1,10 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
+============================= end_copyright_notice ===========================*/
 
 #include "common/LLVMWarningsPush.hpp"
 #include <llvm/Transforms/Utils/Cloning.h>
@@ -339,7 +321,7 @@ void CustomLoopVersioning::hoistSeg2Invariant(Loop* loop,
     {
         IntrinsicInst* intrin = dyn_cast<IntrinsicInst>(UI);
         if (intrin->getIntrinsicID() == Intrinsic::fabs &&
-            intrin->getNumUses() == 1)
+            intrin->hasOneUse())
         {
             intrin_abs = intrin;
             break;
@@ -352,7 +334,7 @@ void CustomLoopVersioning::hoistSeg2Invariant(Loop* loop,
             *intrin_abs->users().begin());
         if (intrin &&
             intrin->getIntrinsicID() == Intrinsic::log2 &&
-            intrin->getNumUses() == 1)
+            intrin->hasOneUse())
         {
             intrin_log2 = intrin;
         }
@@ -364,7 +346,7 @@ void CustomLoopVersioning::hoistSeg2Invariant(Loop* loop,
             *intrin_log2->users().begin());
         if (fmul &&
             fmul->getOpcode() == Instruction::FMul &&
-            fmul->getNumUses() == 1)
+            fmul->hasOneUse())
         {
             unsigned id = fmul->getOperand(0) == intrin_log2 ? 1 : 0;
             // make sure another operand is coming from out of loop
