@@ -1,28 +1,10 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
+============================= end_copyright_notice ===========================*/
 
 #pragma once
 #include "Compiler/CodeGenContextWrapper.hpp"
@@ -42,6 +24,7 @@ namespace llvm
 {
     // Forward declare:
     class SampleIntrinsic;
+    class SamplerLoadIntrinsic;
 }
 
 namespace IGC
@@ -69,6 +52,7 @@ namespace IGC
         }
 
         void visitInstruction(llvm::Instruction& I);
+        void visitUDiv(llvm::BinaryOperator& I);
         void visitAllocaInst(llvm::AllocaInst& I);
         void visitCallInst(llvm::CallInst& C);
         void removeHftoFCast(llvm::Instruction& I);
@@ -81,12 +65,13 @@ namespace IGC
         void visitFPToUIInst(llvm::FPToUIInst& FPUII);
         void visitFPTruncInst(llvm::FPTruncInst& I);
         void visitExtractElementInst(llvm::ExtractElementInst& I);
-        void visitLdptr(llvm::CallInst* inst);
+        void visitLdptr(llvm::SamplerLoadIntrinsic* inst);
+        void visitLdRawVec(llvm::CallInst* inst);
         void visitLoadInst(llvm::LoadInst& I);
         void dp4WithIdentityMatrix(llvm::ExtractElementInst& I);
         bool isIdentityMatrix(llvm::ExtractElementInst& I);
         void visitAnd(llvm::BinaryOperator& I);
-
+        void visitXor(llvm::Instruction& XorInstr);
         //
         // IEEE Floating point arithmetic is not associative.  Any pattern
         // match that changes the order or paramters is unsafe.
@@ -106,10 +91,14 @@ namespace IGC
         void visitBitCast(llvm::BitCastInst& BC);
 
         void matchDp4a(llvm::BinaryOperator& I);
+        void hoistDp3(llvm::BinaryOperator& I);
 
         template <typename MaskType> void matchReverse(llvm::BinaryOperator& I);
     private:
         bool psHasSideEffect;
+
+        bool lower64bto32b(llvm::BinaryOperator& AndInst);
+        llvm::Value* analyzeTreeForTrunc64bto32b(const llvm::Use& OperandUse, llvm::SmallVector<llvm::BinaryOperator*, 8>& OpsToDelete);
     };
 
 #if LLVM_VERSION_MAJOR >= 7
@@ -184,6 +173,7 @@ namespace IGC
         void visitSDiv(llvm::BinaryOperator& I);
         void visitTruncInst(llvm::TruncInst& I);
         void visitBitCastInst(llvm::BitCastInst& I);
+        void visitLoadInst(llvm::LoadInst& I);
 #if LLVM_VERSION_MAJOR >= 10
         void visitFNeg(llvm::UnaryOperator& I);
 #endif
@@ -237,7 +227,7 @@ namespace IGC
 
     private:
         llvm::Module* module;
-        llvm::Constant* replaceShaderConstant(llvm::LoadInst* inst);
+        llvm::Constant* replaceShaderConstant(llvm::Instruction* inst);
         llvm::Constant* ConstantFoldCmpInst(llvm::CmpInst* inst);
         llvm::Constant* ConstantFoldExtractElement(llvm::ExtractElementInst* inst);
         llvm::Constant* ConstantFoldCallInstruction(llvm::CallInst* inst);
@@ -258,5 +248,8 @@ namespace IGC
     llvm::FunctionPass* createBlendToDiscardPass();
     llvm::FunctionPass* createMarkReadOnlyLoadPass();
     llvm::FunctionPass* createLogicalAndToBranchPass();
+    llvm::FunctionPass* createCleanPHINodePass();
+    llvm::FunctionPass* createMergeMemFromBranchOptPass();
+    llvm::FunctionPass* createInsertBranchOptPass();
 
 } // namespace IGC
