@@ -7,13 +7,12 @@ SPDX-License-Identifier: MIT
 ============================= end_copyright_notice ===========================*/
 
 #include "Compiler/DebugInfo/ScalarVISAModule.h"
+#include "Compiler/DebugInfo/Utils.h"
 #include "Compiler/Optimizer/OpenCLPasses/KernelArgs.hpp"
 #include "Compiler/CISACodeGen/ShaderCodeGen.hpp"
 #include "GenISAIntrinsics/GenIntrinsicInst.h"
 #include "common/debug/Debug.hpp"
 
-#include "DebugInfo/DebugInfoUtils.hpp"
-
 #include "common/LLVMWarningsPush.hpp"
 #include "llvm/IR/Function.h"
 #include "llvm/IR/Module.h"
@@ -37,7 +36,7 @@ namespace IGC {
     fullDebugInfo = false;
     lineNumbersOnly = false;
 
-    if (DebugInfoUtils::HasDebugInfo(*ctx->getModule()))
+    if (Utils::HasDebugInfo(*ctx->getModule()))
     {
         bool hasDbgIntrinsic = false;
         bool hasDbgLoc = false;
@@ -88,14 +87,18 @@ namespace IGC {
     return (fullDebugInfo | lineNumbersOnly);
 }
 
-ScalarVisaModule::ScalarVisaModule(CShader* TheShader, llvm::Function *TheFunction)
-  : m_pShader(TheShader), VISAModule(TheFunction) {
+ScalarVisaModule::ScalarVisaModule(CShader* TheShader,
+                                   llvm::Function *TheFunction,
+                                   bool IsPrimary)
+  : m_pShader(TheShader), VISAModule(TheFunction, IsPrimary) {
   UpdateVisaId();
 }
 
-std::unique_ptr<IGC::VISAModule> ScalarVisaModule::BuildNew(CShader* S, llvm::Function *F)
+std::unique_ptr<IGC::VISAModule> ScalarVisaModule::BuildNew(CShader* S,
+                                                            llvm::Function *F,
+                                                            bool IsPrimary)
 {
-    auto* n = new ScalarVisaModule(S, F);
+    auto* n = new ScalarVisaModule(S, F, IsPrimary);
     return std::unique_ptr<IGC::VISAModule>(n);
 }
 
@@ -171,6 +174,16 @@ unsigned ScalarVisaModule::getPointerSize() const {
     return IGC::getPointerSize((llvm::Module &)(*GetModule()));
 }
 
+uint64_t ScalarVisaModule::getTypeSizeInBits(Type* Ty) const
+{
+    IGC_ASSERT(getFunction());
+    // TODO: looks like data layout for function pointers is not set
+    // correctly. According to the current data layout all pointers are of
+    // 64 bits, while vISA/genIsa function pointers are deemed to be 32 bits.
+    // Double-check if this is an issue.
+    return getFunction()->getParent()->getDataLayout().getTypeSizeInBits(Ty);
+}
+
 void ScalarVisaModule::UpdateVisaId()
 {
     auto* Kernel = m_pShader->GetEncoder().GetVISAKernel();
@@ -357,13 +370,13 @@ const Argument* ScalarVisaModule::GetTracedArgument(const Value* pVal, bool isAd
     // If reach this point. Return nullptr.
     return nullptr;
 }
-std::vector<VISAVariableLocation>
+
+VISAVariableLocation
 ScalarVisaModule::GetVariableLocation(const llvm::Instruction* pInst) const
 {
     const Value* pVal = nullptr;
     MDNode* pNode = nullptr;
     bool isDbgDclInst = false;
-    std::vector<VISAVariableLocation> ret;
     if (const DbgDeclareInst * pDbgAddrInst = dyn_cast<DbgDeclareInst>(pInst))
     {
         pVal = pDbgAddrInst->getAddress();
@@ -383,8 +396,7 @@ ScalarVisaModule::GetVariableLocation(const llvm::Instruction* pInst) const
     if (!pVal || isa<UndefValue>(pVal))
     {
         // No debug info value, return empty location!
-        ret.push_back(VISAVariableLocation(this));
-        return ret;
+        return VISAVariableLocation(this);
     }
 
     if (const Constant * pConstVal = dyn_cast<Constant>(pVal))
@@ -392,8 +404,7 @@ ScalarVisaModule::GetVariableLocation(const llvm::Instruction* pInst) const
         if (!isa<GlobalVariable>(pVal) && !isa<ConstantExpr>(pVal))
         {
             IGC_ASSERT_MESSAGE(!isDbgDclInst, "address cannot be immediate!");
-            ret.push_back(VISAVariableLocation(pConstVal, this));
-            return ret;
+            return VISAVariableLocation(pConstVal, this);
         }
     }
 
@@ -428,6 +439,16 @@ ScalarVisaModule::GetVariableLocation(const llvm::Instruction* pInst) const
             {
                 const std::string typeStr = modMD->FuncMD[const_cast<Function*>(curFunc)].m_OpenCLArgBaseTypes[pArgument->getArgNo()];
                 KernelArg::ArgType argType = KernelArg::calcArgType(pArgument, typeStr);
+                if (argType == KernelArg::ArgType::SAMPLER)
+                {
+                    // SAMPLER and NOT_TO_ALLOCATE have same enum values so disambiguate these
+                    auto pr = KernelArg::getBufferType(pArgument, typeStr);
+                    if(!pr.isSampler)
+                    {
+                        // type is actually NOT_TO_ALLOCATE
+                        argType = KernelArg::ArgType::End;
+                    }
+                }
                 FunctionMetaData* funcMD = &modMD->FuncMD[const_cast<Function*>(curFunc)];
                 ResourceAllocMD* resAllocMD = &funcMD->resAllocMD;
                 IGC_ASSERT_MESSAGE(resAllocMD->argAllocMDList.size() == curFunc->arg_size(), "Invalid ArgAllocMDList");
@@ -440,8 +461,7 @@ ScalarVisaModule::GetVariableLocation(const llvm::Instruction* pInst) const
                     break;
                 case KernelArg::ArgType::SAMPLER:
                     IGC_ASSERT_MESSAGE(index < SAMPLER_REGISTER_NUM, "Bad sampler index");
-                    ret.push_back(VISAVariableLocation(SAMPLER_REGISTER_BEGIN + index, this));
-                    return ret;
+                    return VISAVariableLocation(SAMPLER_REGISTER_BEGIN + index, this);
                 case KernelArg::ArgType::IMAGE_1D:
                 case KernelArg::ArgType::IMAGE_1D_BUFFER:
                 case KernelArg::ArgType::IMAGE_2D:
@@ -461,18 +481,15 @@ ScalarVisaModule::GetVariableLocation(const llvm::Instruction* pInst) const
                         // Found write image
                         index = m_pShader->m_pBtiLayout->GetUavIndex(index);
                         IGC_ASSERT_MESSAGE(index < TEXTURE_REGISTER_NUM, "Bad texture index");
-                        ret.push_back(VISAVariableLocation(TEXTURE_REGISTER_BEGIN + index, this));
-                        return ret;
+                        return VISAVariableLocation(TEXTURE_REGISTER_BEGIN + index, this);
                     case SRVResourceType:
                         // Found read image
                         index = m_pShader->m_pBtiLayout->GetTextureIndex(index);
                         IGC_ASSERT_MESSAGE(index < TEXTURE_REGISTER_NUM, "Bad texture index");
-                        ret.push_back(VISAVariableLocation(TEXTURE_REGISTER_BEGIN + index, this));
-                        return ret;
+                        return VISAVariableLocation(TEXTURE_REGISTER_BEGIN + index, this);
                     default:
                         IGC_ASSERT_MESSAGE(0, "Unknown texture resource");
-                        ret.push_back(VISAVariableLocation(this));
-                        return ret;
+                        return VISAVariableLocation(this);
                     }
                 }
             }
@@ -489,8 +506,7 @@ ScalarVisaModule::GetVariableLocation(const llvm::Instruction* pInst) const
             // TODO: Re-enable this assert once -O2 fixes bug where llvm.dbg.declare points
             // to a non-address value.
             //IGC_ASSERT_MESSAGE(0, "DBG declare intrinsic must point to an address");
-            ret.push_back(VISAVariableLocation(this));
-            return ret;
+            return VISAVariableLocation(this);
         }
         pType = pType->getPointerElementType();
     }
@@ -511,7 +527,8 @@ ScalarVisaModule::GetVariableLocation(const llvm::Instruction* pInst) const
             isGlobalAddrSpace = true;
         }
     }
-    else if (pVal->getType()->isPointerTy())
+
+    if (pVal->getType()->isPointerTy())
     {
         unsigned int addrSpace = pVal->getType()->getPointerAddressSpace();
         if (addrSpace == ADDRESS_SPACE_LOCAL)
@@ -526,11 +543,9 @@ ScalarVisaModule::GetVariableLocation(const llvm::Instruction* pInst) const
         unsigned int offset = m_pShader->GetGlobalMappingValue(pValue);
         if (isInSurface)
         {
-            ret.push_back(VISAVariableLocation(surfaceReg, offset, false, isDbgDclInst, 0, false, this));
-            return ret;
+            return VISAVariableLocation(surfaceReg, offset, false, isDbgDclInst, 0, false, this);
         }
-        ret.push_back(VISAVariableLocation(offset, false, isDbgDclInst, 0, false, false, this));
-        return ret;
+        return VISAVariableLocation(offset, false, isDbgDclInst, 0, false, false, this);
     }
 
     // At this point we expect only a register
@@ -541,8 +556,7 @@ ScalarVisaModule::GetVariableLocation(const llvm::Instruction* pInst) const
         pVar = m_pShader->GetDebugInfoData().getMapping(*pInst->getFunction(), pValue);
         if (!pVar)
         {
-            ret.push_back(VISAVariableLocation(this));
-            return ret;
+            return VISAVariableLocation(this);
         }
     }
     else
@@ -551,21 +565,21 @@ ScalarVisaModule::GetVariableLocation(const llvm::Instruction* pInst) const
     IGC_ASSERT_MESSAGE(false == pVar->IsImmediate(), "Do not expect an immediate value at this level");
 
     std::string varName = cast<DIVariable>(pNode)->getName().str();
-    unsigned int reg = 0, reg2 = 0;
     unsigned int vectorNumElements = 0;
 
     switch (pVar->GetVarType()) {
     case EVARTYPE_GENERAL:
+    {
         // We want to attach "Output" attribute to all src variables
         // so that finalizer can extend their liveness to end of
         // the program. This will help debugger examine their
         // values anywhere in the code till they are in scope.
-        reg = m_pShader->GetDebugInfoData().getVISADclId(pVar, 0);
+        unsigned int reg = m_pShader->GetDebugInfoData().getVISADclId(pVar, 0);
         IGC_ASSERT_MESSAGE(reg < GENERAL_REGISTER_NUM, "Bad VISA general register");
 
         if (pType->isVectorTy())
         {
-            vectorNumElements = (unsigned)cast<VectorType>(pType)->getNumElements();
+            vectorNumElements = (unsigned)cast<IGCLLVM::FixedVectorType>(pType)->getNumElements();
         }
         else if (!pVar->IsUniform())
         {
@@ -574,32 +588,32 @@ ScalarVisaModule::GetVariableLocation(const llvm::Instruction* pInst) const
 
         if (isInSurface)
         {
-            ret.push_back(VISAVariableLocation(surfaceReg, GENERAL_REGISTER_BEGIN + reg, true, isDbgDclInst, vectorNumElements, !pVar->IsUniform(), this));
-            return ret;
+            return VISAVariableLocation(surfaceReg, reg, true /*isRegister*/, isDbgDclInst,
+                                        vectorNumElements, !pVar->IsUniform(), this);
         }
-        ret.push_back(VISAVariableLocation(GENERAL_REGISTER_BEGIN + reg, true, isDbgDclInst, vectorNumElements, !pVar->IsUniform(), isGlobalAddrSpace, this));
+        VISAVariableLocation genReg(reg, true /*isRegister*/, isDbgDclInst, vectorNumElements,
+                                    !pVar->IsUniform(), isGlobalAddrSpace, this);
+        // SIMD32 locations can't into one register. See VISAVariableLocation::m_locationSecondReg field description for more information
         if (GetSIMDSize() == 32 && pVar->visaGenVariable[1] && !pVar->IsUniform())
         {
-            reg2 = m_pShader->GetDebugInfoData().getVISADclId(pVar, 1);
-            ret.push_back(VISAVariableLocation(GENERAL_REGISTER_BEGIN + reg2, true, isDbgDclInst, vectorNumElements, !pVar->IsUniform(), isGlobalAddrSpace, this));
+            unsigned int reg2 = m_pShader->GetDebugInfoData().getVISADclId(pVar, 1);
+            genReg.AddSecondReg(reg2);
         }
-        return ret;
+        return genReg;
+    }
     case EVARTYPE_ADDRESS:
     case EVARTYPE_PREDICATE:
     case EVARTYPE_SURFACE:
     case EVARTYPE_SAMPLER:
         // TODO: Handle case where variable is mapped to flag/address register
-        ret.push_back(VISAVariableLocation(this));
-        return ret;
-        break;
+        return VISAVariableLocation(this);
     default:
         IGC_ASSERT_MESSAGE(0, "Unhandled VISA register type!");
         break;
     }
 
     IGC_ASSERT_MESSAGE(0, "Empty variable location");
-    ret.push_back(VISAVariableLocation(this));
-    return ret;
+    return VISAVariableLocation(this);
 }
 
 bool ScalarVisaModule::IsCatchAllIntrinsic(const llvm::Instruction* pInst) const
@@ -663,7 +677,7 @@ void insertOCLMissingDebugConstMetadata(CodeGenContext* ctx)
 
                 if (!GlobalValue::isLocalLinkage(g->getLinkage()) || isConstForThisFunc)
                 {
-                    DebugInfoUtils::UpdateGlobalVarDebugInfo(g, init, &func.getEntryBlock().getInstList().front(), false);
+                    Utils::UpdateGlobalVarDebugInfo(g, init, &func.getEntryBlock().getInstList().front(), false);
                 }
             }
         }
