@@ -32,7 +32,8 @@ class ScalarVisaModule final : public IGC::VISAModule {
 
 public:
 
-    static std::unique_ptr<IGC::VISAModule> BuildNew(CShader* S, llvm::Function *F);
+    static std::unique_ptr<IGC::VISAModule>
+        BuildNew(CShader* S, llvm::Function *F, bool IsPrimary = false);
 
     unsigned int getUnpaddedProgramSize() const override {
         return m_pShader->ProgramOutput()->m_unpaddedProgramSize;
@@ -42,7 +43,7 @@ public:
     }
     unsigned getPrivateBaseReg() const override;
 
-    unsigned getGRFSize() const override {
+    unsigned getGRFSizeInBytes() const override {
       return m_pShader->getGRFSize();
     }
 
@@ -51,6 +52,7 @@ public:
     }
 
     unsigned getPointerSize() const override;
+    uint64_t getTypeSizeInBits(Type*) const override;
 
     llvm::ArrayRef<char> getGenDebug() const override {
       const auto& PO = *m_pShader->ProgramOutput();
@@ -60,8 +62,7 @@ public:
       const auto& PO = *m_pShader->ProgramOutput();
       return llvm::ArrayRef<char>((const char*)PO.m_programBin, PO.m_programSize);
     }
-    std::vector<VISAVariableLocation>
-        GetVariableLocation(const llvm::Instruction* pInst) const override;
+    VISAVariableLocation GetVariableLocation(const llvm::Instruction* pInst) const override;
 
     void UpdateVisaId() override;
     void ValidateVisaId() override;
@@ -116,8 +117,14 @@ public:
 
 private:
     /// @brief Constructor.
-    /// @param m_pShader holds the processed entry point function and generated VISA code.
-    explicit ScalarVisaModule (CShader* TheShader, llvm::Function *TheFunction);
+    /// @param m_pShader holds the Shader object that provides information
+    /// about the properties of the compiled program.
+    /// @param TheFunction holds currently processed llvm IR function
+    /// used to emit vISA code.
+    /// @param IsPrimary indicates if the currently processed function
+    /// is a "primary entry point" to the corresponding Shader object.
+    /// See Source/IGC/DebugInfo/VISAModule.hpp for more details
+    ScalarVisaModule(CShader* TheShader, llvm::Function *TheFunction, bool IsPrimary);
     /// @brief Trace given value to its origin value, searching for LLVM Argument.
     /// @param pVal value to process.
     /// @param isAddress indecates if the value represents an address.
