@@ -1,28 +1,10 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
+============================= end_copyright_notice ===========================*/
 
 #include "common/debug/Debug.hpp"
 #include "common/debug/Dump.hpp"
@@ -107,41 +89,12 @@ bool FixResourcePtr::runOnFunction(llvm::Function& F)
     return m_changed;
 }
 
-
-// Function modifies address space in all BitCast or GEP uses input pointer.
-void FixResourcePtr::FixAddressSpaceInAllUses(Value* ptr, uint newAS, uint oldAS)
-{
-    IGC_ASSERT(newAS != oldAS);
-
-    for (auto UI = ptr->user_begin(), E = ptr->user_end(); UI != E; ++UI)
-    {
-        Instruction* inst = dyn_cast<Instruction>(*UI);
-        PointerType* instType = nullptr;
-        if (BitCastInst * bitCastInst = dyn_cast<BitCastInst>(inst))
-        {
-            instType = dyn_cast<PointerType>(bitCastInst->getType());
-        }
-        else if (GetElementPtrInst * gepInst = dyn_cast<GetElementPtrInst>(inst))
-        {
-            instType = dyn_cast<PointerType>(gepInst->getType());
-        }
-
-        if (instType && instType->getAddressSpace() == oldAS)
-        {
-            Type* eltType = instType->getElementType();
-            PointerType* ptrType = PointerType::get(eltType, newAS);
-            inst->mutateType(ptrType);
-            FixAddressSpaceInAllUses(inst, newAS, oldAS);
-        }
-    }
-}
-
 void FixResourcePtr::RemoveGetBufferPtr(GenIntrinsicInst* bufPtr, Value* bufIdx)
 {
     uint outAS = bufPtr->getType()->getPointerAddressSpace();
     uint origAS = outAS;
     BufferType bufType = (BufferType)(cast<ConstantInt>(bufPtr->getOperand(1))->getZExtValue());
-    uint encodeAS = EncodeAS4GFXResource(*bufIdx, bufType, 0);
+    uint encodeAS = EncodeAS4GFXResource(*bufIdx, bufType);
     if (outAS != encodeAS &&
         (bufType == CONSTANT_BUFFER || bufType == RESOURCE || bufType == UAV))
     {
@@ -252,6 +205,7 @@ Value* FixResourcePtr::GetByteOffset(Instruction* eltPtr)
     IGC_ASSERT(eltPtr->getNumOperands() == 2);
     Value* ptrOp = eltPtr->getOperand(0);
     PointerType* ptrTy = dyn_cast<PointerType>(ptrOp->getType());
+    IGC_ASSERT(ptrTy);
     Value* eltIdx = eltPtr->getOperand(1);
 
     builder->SetInsertPoint(eltPtr);
