@@ -27,12 +27,15 @@ void initializeBreakConstantExprPass(llvm::PassRegistry&);
 void initializeBuiltinCallGraphAnalysisPass(llvm::PassRegistry&);
 void initializeBuiltinsConverterPass(llvm::PassRegistry&);
 void initializeClearTessFactorsPass(llvm::PassRegistry&);
+void initializeCleanPHINodePass(llvm::PassRegistry&);
 void initializeCoalescingEnginePass(llvm::PassRegistry&);
 void initializeCodeGenContextWrapperPass(llvm::PassRegistry&);
 void initializeCodeGenPatternMatchPass(llvm::PassRegistry&);
 void initializeCollectGeometryShaderPropertiesPass(llvm::PassRegistry&);
 void initializeConstantCoalescingPass(llvm::PassRegistry&);
 void initializeCorrectlyRoundedDivSqrtPass(llvm::PassRegistry&);
+void initializeCrossPhaseConstPropPass(llvm::PassRegistry&);
+void initializeCustomLoopVersioningPass(llvm::PassRegistry&);
 void initializeCustomSafeOptPassPass(llvm::PassRegistry&);
 void initializeCustomUnsafeOptPassPass(llvm::PassRegistry&);
 void initializeHoistFMulInLoopPassPass(llvm::PassRegistry&);
@@ -40,12 +43,16 @@ void initializeHandleFRemInstructionsPass(llvm::PassRegistry&);
 void initializeDeSSAPass(llvm::PassRegistry&);
 void initializeDeviceEnqueueFuncsAnalysisPass(llvm::PassRegistry&);
 void initializeDeviceEnqueueFuncsResolutionPass(llvm::PassRegistry&);
+void initializeDisableLoopUnrollOnRetryPass(llvm::PassRegistry&);
 void initializeDynamicTextureFoldingPass(llvm::PassRegistry&);
+void initializeEarlyOutPatternsPass(llvm::PassRegistry&);
 void initializeExtensionArgAnalysisPass(llvm::PassRegistry&);
 void initializeExtensionFuncsAnalysisPass(llvm::PassRegistry&);
 void initializeExtensionFuncsResolutionPass(llvm::PassRegistry&);
+void initializeErrorCheckPass(llvm::PassRegistry&);
 void initializeGenericAddressAnalysisPass(llvm::PassRegistry&);
 void initializeGenericAddressDynamicResolutionPass(llvm::PassRegistry&);
+void initializeGenFDIVEmulationPass(llvm::PassRegistry&);
 void initializeGenIRLoweringPass(llvm::PassRegistry&);
 void initializeGeometryShaderLoweringPass(llvm::PassRegistry&);
 void initializeGEPLoweringPass(llvm::PassRegistry&);
@@ -54,6 +61,7 @@ void initializeGreedyLiveRangeReductionPass(llvm::PassRegistry&);
 void initializeIGCIndirectICBPropagaionPass(llvm::PassRegistry&);
 void initializeGenUpdateCBPass(llvm::PassRegistry&);
 void initializeGenStrengthReductionPass(llvm::PassRegistry&);
+void initializeGenOptLegalizerPass(llvm::PassRegistry&);
 void initializeNanHandlingPass(llvm::PassRegistry&);
 void initializeFixResourcePtrPass(llvm::PassRegistry&);
 void initializeFlattenSmallSwitchPass(llvm::PassRegistry&);
@@ -64,18 +72,28 @@ void initializeEstimateFunctionSizePass(llvm::PassRegistry&);
 void initializeSubroutineInlinerPass(llvm::PassRegistry&);
 void initializeHandleLoadStoreInstructionsPass(llvm::PassRegistry&);
 void initializeIGCConstPropPass(llvm::PassRegistry&);
+void initializeJointMatrixFuncsResolutionPassPass(llvm::PassRegistry&);
 void initializeGatingSimilarSamplesPass(llvm::PassRegistry&);
+void initializeInsertGenericPtrArithmeticMetadataPass(llvm::PassRegistry&);
 void initializeImageFuncResolutionPass(llvm::PassRegistry&);
 void initializeImageFuncsAnalysisPass(llvm::PassRegistry&);
+void initializeImage3dToImage2darrayPass(llvm::PassRegistry&);
 void initializeImplicitGlobalIdPass(llvm::PassRegistry&);
 void initializeCleanImplicitIdsPass(llvm::PassRegistry&);
 void initializeInlineLocalsResolutionPass(llvm::PassRegistry&);
+void initializeInlineUnmaskedFunctionsPassPass(llvm::PassRegistry&);
+void initializeInsertBranchOptPass(llvm::PassRegistry&);
+void initializeLayoutPass(llvm::PassRegistry&);
+void initializeLdShrinkPass(llvm::PassRegistry&);
 void initializeLegalizationPass(llvm::PassRegistry&);
 void initializeLegalizeResourcePointerPass(llvm::PassRegistry&);
 void initializeLegalizeFunctionSignaturesPass(llvm::PassRegistry&);
 void initializeLiveVarsAnalysisPass(llvm::PassRegistry&);
+void initializeLogicalAndToBranchPass(llvm::PassRegistry&);
 void initializeLowerGEPForPrivMemPass(llvm::PassRegistry&);
+void initializeLowerImplicitArgIntrinsicsPass(llvm::PassRegistry&);
 void initializeLowPrecisionOptPass(llvm::PassRegistry&);
+void initializeMarkReadOnlyLoadPass(llvm::PassRegistry&);
 void initializeMetaDataUtilsWrapperInitializerPass(llvm::PassRegistry&);
 void initializeMetaDataUtilsWrapperPass(llvm::PassRegistry&);
 void initializeOpenCLPrintfAnalysisPass(llvm::PassRegistry&);
@@ -92,24 +110,33 @@ void initializeProgramScopeConstantAnalysisPass(llvm::PassRegistry&);
 void initializeProgramScopeConstantResolutionPass(llvm::PassRegistry&);
 void initializePromoteResourceToDirectASPass(llvm::PassRegistry&);
 void initializePromoteStatelessToBindlessPass(llvm::PassRegistry&);
+void initializePruneUnusedArgumentsPass(llvm::PassRegistry&);
 void initializePullConstantHeuristicsPass(llvm::PassRegistry&);
 void initializeScalarizerCodeGenPass(llvm::PassRegistry&);
 void initializeReduceLocalPointersPass(llvm::PassRegistry&);
 void initializeReplaceUnsupportedIntrinsicsPass(llvm::PassRegistry&);
 void initializePreCompiledFuncImportPass(llvm::PassRegistry&);
 void initializePurgeMetaDataUtilsPass(llvm::PassRegistry&);
+void initializeRemoveNonPositionOutputPass(llvm::PassRegistry&);
 void initializeResolveAggregateArgumentsPass(llvm::PassRegistry&);
 void initializeResolveOCLAtomicsPass(llvm::PassRegistry&);
+void initializeResolveSampledImageBuiltinsPass(llvm::PassRegistry&);
 void initializeResourceAllocatorPass(llvm::PassRegistry&);
 void initializeRewriteLocalSizePass(llvm::PassRegistry&);
 void initializeSampleCmpToDiscardPass(llvm::PassRegistry&);
+void initializeSamplerPerfOptPassPass(llvm::PassRegistry&);
 void initializeScalarizeFunctionPass(llvm::PassRegistry&);
 void initializeSimd32ProfitabilityAnalysisPass(llvm::PassRegistry&);
 void initializeSetFastMathFlagsPass(llvm::PassRegistry&);
 void initializeSPIRMetaDataTranslationPass(llvm::PassRegistry&);
+void initializeStatelessToStatefulPass(llvm::PassRegistry&);
 void initializeSubGroupFuncsResolutionPass(llvm::PassRegistry&);
 void initializeTransformUnmaskedFunctionsPassPass(llvm::PassRegistry&);
 void initializeIndirectCallOptimizationPass(llvm::PassRegistry&);
+void initializePromoteInt8TypePass(llvm::PassRegistry&);
+void initializeDpasFuncsResolutionPass(llvm::PassRegistry&);
+void initializeLSCFuncsResolutionPass(llvm::PassRegistry&);
+void initializePrepareLoadsStoresPassPass(llvm::PassRegistry&);
 void initializeVectorBitCastOptPass(llvm::PassRegistry&);
 void initializeVectorPreProcessPass(llvm::PassRegistry&);
 void initializeVectorProcessPass(llvm::PassRegistry&);
@@ -120,6 +147,7 @@ void initializeWIAnalysisPass(llvm::PassRegistry&);
 void initializeWIFuncResolutionPass(llvm::PassRegistry&);
 void initializeWIFuncsAnalysisPass(llvm::PassRegistry&);
 void initializeWorkaroundAnalysisPass(llvm::PassRegistry&);
+void initializeCPSMSAAOMaskWAPass(llvm::PassRegistry&);
 void initializeWAFMinFMaxPass(llvm::PassRegistry&);
 void initializePingPongTexturesAnalysisPass(llvm::PassRegistry&);
 void initializePingPongTexturesOptPass(llvm::PassRegistry&);
@@ -158,3 +186,7 @@ void initializeNamedBarriersResolutionPass(llvm::PassRegistry&);
 void initializeUndefinedReferencesPassPass(llvm::PassRegistry&);
 void initializeBreakdownIntrinsicPassPass(llvm::PassRegistry&);
 void initializeCatchAllLineNumberPass(llvm::PassRegistry&);
+void initializePromoteConstantStructsPass(llvm::PassRegistry&);
+void initializeLowerInvokeSIMDPass(llvm::PassRegistry&);
+void initializeImplicitGIDRestoringPass(llvm::PassRegistry&);
+void initializeResolvePointersComparisonPass(llvm::PassRegistry&);
