@@ -16,10 +16,12 @@ SPDX-License-Identifier: MIT
 #include "InstScalarizer.h"
 #include "InstElementizer.h"
 #include "common/LLVMWarningsPush.hpp"
+#include "llvmWrapper/IR/DerivedTypes.h"
 #include "llvm/ADT/PostOrderIterator.h"
 #include "llvm/IR/CFG.h"
 #include "llvm/Support/Debug.h"
 #include "llvm/Support/raw_ostream.h"
+#include "llvm/Transforms/Utils/Local.h"
 #include "common/LLVMWarningsPop.hpp"
 #include "Compiler/IGCPassSupport.h"
 #include "Probe/Assertion.h"
@@ -36,11 +38,12 @@ TheModule(nullptr), TheFunction(nullptr) {
     initializeTypeLegalizerPass(*PassRegistry::getPassRegistry());
 }
 
-
 void TypeLegalizer::getAnalysisUsage(AnalysisUsage& AU) const {
+    AU.addRequired<DominatorTreeWrapperPass>();
     AU.setPreservesCFG();
 }
 
+
 FunctionPass* createTypeLegalizerPass() { return new TypeLegalizer(); }
 
 #define PASS_FLAG     "igc-type-legalizer"
@@ -52,9 +55,12 @@ IGC_INITIALIZE_PASS_BEGIN(TypeLegalizer, PASS_FLAG, PASS_DESC, PASS_CFG_ONLY, PA
 IGC_INITIALIZE_PASS_END(TypeLegalizer, PASS_FLAG, PASS_DESC, PASS_CFG_ONLY, PASS_ANALYSIS)
 
 #define LEGALIZE_ALL 0
+// IGC supports type i64 natively or by emulation
+const unsigned int MAX_LEGAL_INT_SIZE_IN_BITS = 64;
 
 bool TypeLegalizer::runOnFunction(Function & F) {
     DL = &F.getParent()->getDataLayout();
+    DT = &getAnalysis<DominatorTreeWrapperPass>().getDomTree();
 
     IGC_ASSERT_MESSAGE(DL->isLittleEndian(), "ONLY SUPPORT LITTLE ENDIANNESS!");
 
@@ -96,6 +102,7 @@ bool TypeLegalizer::runOnFunction(Function & F) {
     eraseIllegalInsts();
 
     DL = nullptr;
+    DT = nullptr;
 
     IRB = nullptr;
 
@@ -148,7 +155,7 @@ TypeLegalizer::getTypeLegalizeAction(Type* Ty) const {
     if (isLegalInteger(Width))
         return Legal;
 
-    if (DL->fitsInLegalInteger(Width))
+    if (DL->fitsInLegalInteger(Width) || Width <= MAX_LEGAL_INT_SIZE_IN_BITS)
         return Promote;
 
     return Expand;
@@ -167,7 +174,7 @@ TypeLegalizer::getLegalizeAction(Instruction* I) const {
 }
 
 std::pair<TypeSeq*, LegalizeAction>
-TypeLegalizer::getLegalizedTypes(Type* Ty) {
+TypeLegalizer::getLegalizedTypes(Type* Ty, bool legalizeToScalar) {
     LegalizeAction Act = getTypeLegalizeAction(Ty);
     TypeSeq* TySeq = nullptr;
 
@@ -175,7 +182,7 @@ TypeLegalizer::getLegalizedTypes(Type* Ty) {
     case Legal:
         break;
     case Promote:
-        TySeq = getPromotedTypeSeq(Ty);
+        TySeq = getPromotedTypeSeq(Ty, legalizeToScalar);
         break;
     case Expand:
         TySeq = getExpandedTypeSeq(Ty);
@@ -194,8 +201,8 @@ TypeLegalizer::getLegalizedTypes(Type* Ty) {
     return std::make_pair(TySeq, Act);
 }
 
-TypeSeq* TypeLegalizer::getPromotedTypeSeq(Type* Ty) {
-    IGC_ASSERT(Ty->isIntegerTy());
+TypeSeq* TypeLegalizer::getPromotedTypeSeq(Type* Ty, bool legalizeToScalar) {
+    IGC_ASSERT(Ty->isIntOrIntVectorTy());
     IGC_ASSERT(getTypeLegalizeAction(Ty) == Promote);
 
     TypeMapTy::iterator TMI; bool New;
@@ -206,7 +213,20 @@ TypeSeq* TypeLegalizer::getPromotedTypeSeq(Type* Ty) {
     }
 
     Type* PromotedTy =
-        DL->getSmallestLegalIntType(Ty->getContext(), getTypeSizeInBits(Ty));
+        DL->getSmallestLegalIntType(Ty->getContext(),
+            getTypeSizeInBits(legalizeToScalar ? Ty : Ty->getScalarType()));
+
+    if (!PromotedTy && getTypeSizeInBits(Ty) < MAX_LEGAL_INT_SIZE_IN_BITS)
+    {
+        PromotedTy = Type::getIntNTy(Ty->getContext(), MAX_LEGAL_INT_SIZE_IN_BITS);
+    }
+
+    if (!legalizeToScalar && Ty->isVectorTy())
+    {
+        unsigned Elems = (unsigned)cast<IGCLLVM::FixedVectorType>(Ty)->getNumElements();
+        PromotedTy = IGCLLVM::FixedVectorType::get(PromotedTy, Elems);
+    }
+
     TMI->second.push_back(PromotedTy);
 
     return &TMI->second;
@@ -269,12 +289,13 @@ TypeSeq* TypeLegalizer::getScalarizedTypeSeq(Type* Ty) {
     TypeMapTy::iterator TMI; bool New;
     std::tie(TMI, New) = TypeMap.insert(std::make_pair(Ty, TypeSeq()));
     if (!New) {
-        IGC_ASSERT(TMI->second.size() == cast<VectorType>(Ty)->getNumElements());
-        return &TMI->second;
+      IGC_ASSERT(TMI->second.size() ==
+                 cast<IGCLLVM::FixedVectorType>(Ty)->getNumElements());
+      return &TMI->second;
     }
 
     Type* EltTy = cast<VectorType>(Ty)->getElementType();
-    for (unsigned i = 0, e = (unsigned)cast<VectorType>(Ty)->getNumElements(); i != e; ++i)
+    for (unsigned i = 0, e = (unsigned)cast<IGCLLVM::FixedVectorType>(Ty)->getNumElements(); i != e; ++i)
         TMI->second.push_back(EltTy);
 
     return &TMI->second;
@@ -308,8 +329,6 @@ TypeSeq* TypeLegalizer::getElementizedTypeSeq(Type* Ty) {
 
 std::pair<ValueSeq*, LegalizeAction>
 TypeLegalizer::getLegalizedValues(Value* V, bool isSigned) {
-    IGC_ASSERT(!V->getType()->isVoidTy());
-
     LegalizeAction Act = getTypeLegalizeAction(V->getType());
 
     if (Act == Legal)
@@ -363,8 +382,6 @@ TypeLegalizer::getLegalizedValues(Value* V, bool isSigned) {
 void
 TypeLegalizer::setLegalizedValues(Value* OVal,
     ArrayRef<Value*> LegalizedVals) {
-    IGC_ASSERT(!OVal->getType()->isVoidTy());
-
     ValueMapTy::iterator VMI; bool New;
     std::tie(VMI, New) = ValueMap.insert(std::make_pair(OVal, ValueSeq()));
     IGC_ASSERT(New);
@@ -556,6 +573,8 @@ bool TypeLegalizer::preparePHIs(Function& F) {
                 PHINode* Promoted =
                     PHINode::Create(PromotedTy, PN->getNumIncomingValues(),
                         Twine(Name, getSuffix(Act)), &(*BI));
+                Promoted->setDebugLoc(PN->getDebugLoc());
+                replaceAllDbgUsesWith(*PN, *Promoted, *PN, *DT);
                 setLegalizedValues(PN, Promoted);
                 BI = BasicBlock::iterator(Promoted);
                 break;
