@@ -12,6 +12,7 @@ SPDX-License-Identifier: MIT
 #include "llvmWrapper/IR/Instructions.h"
 #include "llvmWrapper/Analysis/InlineCost.h"
 #include "llvmWrapper/IR/InstrTypes.h"
+#include "llvmWrapper/IR/DerivedTypes.h"
 #include "llvmWrapper/Support/Alignment.h"
 #include "llvm/ADT/ArrayRef.h"
 #include "llvm/ADT/DenseMap.h"
@@ -20,7 +21,7 @@ SPDX-License-Identifier: MIT
 #include "llvm/IR/Argument.h"
 #include "llvm/IR/DataLayout.h"
 #include "llvm/IR/Function.h"
-#include "llvm/IR/IRBuilder.h"
+#include "llvmWrapper/IR/IRBuilder.h"
 #include "llvm/IR/Instruction.h"
 #include "llvm/IR/Instructions.h"
 #include "llvm/IR/IntrinsicInst.h"
@@ -30,6 +31,7 @@ SPDX-License-Identifier: MIT
 #include "llvm/Pass.h"
 #include "llvm/Analysis/TargetFolder.h"
 #include "common/LLVMWarningsPop.hpp"
+#include "Compiler/CISACodeGen/RegisterPressureEstimate.hpp"
 #include "common/Types.hpp"
 #include "GenISAIntrinsics/GenIntrinsicInst.h"
 #include "Probe/Assertion.h"
@@ -49,7 +51,7 @@ namespace IGC {
             Elementize
         };
 
-        typedef IRBuilder<TargetFolder> BuilderType;
+        typedef IGCLLVM::IRBuilder<TargetFolder> BuilderType;
 
         typedef TinyPtrVector<Type*> TypeSeq;
         typedef TinyPtrVector<Value*> ValueSeq;
@@ -69,18 +71,19 @@ namespace IGC {
             friend class InstScalarizer;
             friend class InstElementizer;
 
-            const DataLayout* DL;
-            BuilderType* IRB;
+            const DataLayout* DL = nullptr;
+            DominatorTree* DT = nullptr;;
+            BuilderType* IRB = nullptr;;
 
-            InstLegalChecker* ILC;
-            InstPromoter* IPromoter;
-            InstExpander* IExpander;
-            InstSoftener* ISoftener;
-            InstScalarizer* IScalarizer;
-            InstElementizer* IElementizer;
+            InstLegalChecker* ILC = nullptr;;
+            InstPromoter* IPromoter = nullptr;;
+            InstExpander* IExpander = nullptr;;
+            InstSoftener* ISoftener = nullptr;;
+            InstScalarizer* IScalarizer = nullptr;;
+            InstElementizer* IElementizer = nullptr;;
 
-            Module* TheModule;
-            Function* TheFunction;
+            Module* TheModule = nullptr;;
+            Function* TheFunction = nullptr;;
 
             // Map from illegal type to legalized type(s).
             typedef DenseMap<Type*, TypeSeq> TypeMapTy;
@@ -103,6 +106,7 @@ namespace IGC {
             void getAnalysisUsage(AnalysisUsage& AU) const override;
 
             LLVMContext& getContext() const { return TheModule->getContext(); }
+            DominatorTree& getDominatorTree() const { return *DT; }
             Module* getModule() const { return TheModule; }
             Function* getFunction() const { return TheFunction; }
 
@@ -119,9 +123,9 @@ namespace IGC {
             LegalizeAction getLegalizeAction(Value* V) const;
             LegalizeAction getLegalizeAction(Instruction* I) const;
 
-            std::pair<TypeSeq*, LegalizeAction> getLegalizedTypes(Type* Ty);
+            std::pair<TypeSeq*, LegalizeAction> getLegalizedTypes(Type* Ty, bool legalizeToScalar = false);
 
-            TypeSeq* getPromotedTypeSeq(Type* Ty);
+            TypeSeq* getPromotedTypeSeq(Type* Ty, bool legalizeToScalar = false);
             TypeSeq* getExpandedTypeSeq(Type* Ty);
             TypeSeq* getSoftenedTypeSeq(Type* Ty);
             TypeSeq* getScalarizedTypeSeq(Type* Ty);
@@ -271,7 +275,7 @@ namespace IGC {
                 if (!Ty->isVectorTy())
                     return false;
 
-                unsigned NumElts = (unsigned)cast<VectorType>(Ty)->getNumElements();
+                unsigned NumElts = (unsigned)cast<IGCLLVM::FixedVectorType>(Ty)->getNumElements();
                 Type* EltTy = cast<VectorType>(Ty)->getElementType();
                 const auto& ProfitLengths = getProfitLoadVectorLength(EltTy);
 
