@@ -8,7 +8,9 @@ SPDX-License-Identifier: MIT
 
 #include "Compiler/Optimizer/OpenCLPasses/AggregateArguments/AggregateArguments.hpp"
 #include "Compiler/IGCPassSupport.h"
+#include "llvmWrapper/IR/DerivedTypes.h"
 #include "common/LLVMWarningsPush.hpp"
+#include "llvmWrapper/IR/IRBuilder.h"
 #include "llvm/IR/Function.h"
 #include "common/LLVMWarningsPop.hpp"
 #include "Probe/Assertion.h"
@@ -114,7 +116,7 @@ static uint64_t getNumElements(Type* type)
     {
         return arrayType->getNumElements();
     }
-    if (VectorType * vectorType = dyn_cast<VectorType>(type))
+    if (IGCLLVM::FixedVectorType * vectorType = dyn_cast<IGCLLVM::FixedVectorType>(type))
     {
         return vectorType->getNumElements();
     }
@@ -216,7 +218,7 @@ bool ResolveAggregateArguments::runOnFunction(Function& F)
     m_pFunction = &F;
 
     bool changed = false;
-    IRBuilder<> irBuilder(&F.getEntryBlock(), F.getEntryBlock().begin());
+    IGCLLVM::IRBuilder<> irBuilder(&F.getEntryBlock(), F.getEntryBlock().begin());
 
     Function::arg_iterator argument = F.arg_begin();
     for (; argument != F.arg_end(); ++argument)
@@ -246,7 +248,7 @@ bool ResolveAggregateArguments::runOnFunction(Function& F)
     return changed;
 }
 
-void ResolveAggregateArguments::storeArgument(const Argument* arg, AllocaInst* base, IRBuilder<>& irBuilder)
+void ResolveAggregateArguments::storeArgument(const Argument* arg, AllocaInst* base, IGCLLVM::IRBuilder<>& irBuilder)
 {
     unsigned int startArgNo, endArgNo;
     getImplicitArg(arg->getArgNo(), startArgNo, endArgNo);
