@@ -1,28 +1,10 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
+============================= end_copyright_notice ===========================*/
 
 #include "Compiler/Optimizer/OpenCLPasses/Atomics/ResolveOCLAtomics.hpp"
 #include "Compiler/IGCPassSupport.h"
@@ -44,11 +26,13 @@ using namespace IGC;
 #define PASS_CFG_ONLY false
 #define PASS_ANALYSIS false
 IGC_INITIALIZE_PASS_BEGIN(ResolveOCLAtomics, PASS_FLAG, PASS_DESCRIPTION, PASS_CFG_ONLY, PASS_ANALYSIS)
+IGC_INITIALIZE_PASS_DEPENDENCY(CodeGenContextWrapper)
 IGC_INITIALIZE_PASS_END(ResolveOCLAtomics, PASS_FLAG, PASS_DESCRIPTION, PASS_CFG_ONLY, PASS_ANALYSIS)
 
 char ResolveOCLAtomics::ID = 0;
 
-const llvm::StringRef BUILTIN_GET_LOCAL_LOCK = "__builtin_IB_get_local_lock";
+const llvm::StringRef BUILTIN_GET_LOCAL_LOCK =  "__builtin_IB_get_local_lock";
+const llvm::StringRef BUILTIN_GET_GLOBAL_LOCK = "__builtin_IB_get_global_lock";
 
 ResolveOCLAtomics::ResolveOCLAtomics() : ModulePass(ID)
 {
@@ -56,29 +40,6 @@ ResolveOCLAtomics::ResolveOCLAtomics() : ModulePass(ID)
     initResolveOCLAtomics();
 }
 
-OCLAtomicAttrs ResolveOCLAtomics::genAtomicAttrs(AtomicOp   op,
-                                                 BufferType bufType)
-{
-    //              bufType
-    //                 |
-    //       not used  V   op
-    //       |-------|---|---|
-    //    0 x 0 0 0 0 0 0 0 0
-    return op | (bufType << ATTR_BUFFER_TYPE_SHIFT);
-}
-
-AtomicOp ResolveOCLAtomics::getAtomicOp(StringRef name)
-{
-    OCLAtomicAttrs  attrs = m_AtomicDescMap[name];
-    return (AtomicOp)(attrs & 0xFF);
-}
-
-BufferType ResolveOCLAtomics::getBufType(StringRef name)
-{
-    OCLAtomicAttrs  attrs = m_AtomicDescMap[name];
-    return (BufferType)((attrs >> ATTR_BUFFER_TYPE_SHIFT) & 0xFF);
-}
-
 void ResolveOCLAtomics::initResolveOCLAtomics()
 {
     initOCLAtomicsMap();
@@ -87,31 +48,20 @@ void ResolveOCLAtomics::initResolveOCLAtomics()
 void ResolveOCLAtomics::initOCLAtomicsMap()
 {
 #define DEF_OCL_IGC_ATOMIC(name, op, buf_type) \
-    m_AtomicDescMap[StringRef(name)] = genAtomicAttrs(op, buf_type);
+    m_AtomicDescMap[StringRef(name)] = OCLAtomicAttrs{op, buf_type};
 #include "OCLAtomicsDef.hpp"
 #undef DEF_OCL_IGC_ATOMIC
 }
 
 bool ResolveOCLAtomics::runOnModule(Module& M)
 {
-    m_pModule  = (IGCLLVM::Module*)&M;
+    m_CGCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
+    m_pModule  = static_cast<IGCLLVM::Module*>(&M);
     m_Int32Ty = Type::getInt32Ty(m_pModule->getContext());
 
     llvm::IGCIRBuilder<> builder(M.getContext());
     m_builder = &builder;
 
-    int pointerSize = M.getDataLayout().getPointerSizeInBits();
-    IGC_ASSERT(pointerSize == 64 || pointerSize == 32);
-
-    if (pointerSize == 64)
-    {
-        m_64bitPointer = true;
-    }
-    else
-    {
-        m_64bitPointer = false;
-    }
-
     m_changed = false;
 
     // Visit all call instructions in the function F.
@@ -135,11 +85,15 @@ void ResolveOCLAtomics::visitCallInst(CallInst& callInst)
     if (funcName == BUILTIN_GET_LOCAL_LOCK) {
         processGetLocalLock(callInst);
     }
+    else if (funcName == BUILTIN_GET_GLOBAL_LOCK) {
+        processGetGlobalLock(callInst);
+    }
 
     if (funcName.startswith("__builtin_IB_atomic"))
     {
         IGC_ASSERT_MESSAGE(m_AtomicDescMap.count(funcName), "Unexpected IGC atomic function name.");
-        processOCLAtomic(callInst, getAtomicOp(funcName), getBufType(funcName));
+        const OCLAtomicAttrs& attrs = m_AtomicDescMap[funcName];
+        processOCLAtomic(callInst, attrs.op, attrs.bufType);
         m_changed = true;
     }
 }
@@ -161,7 +115,10 @@ void ResolveOCLAtomics::processOCLAtomic(CallInst& callInst, AtomicOp op, Buffer
         callInst.getOperand(1);
 
     const bool floatArgs = !noSources && src0->getType()->isFloatingPointTy();
-    const bool is64bit = m_64bitPointer && bufType != SLM;
+
+    Value* dstBuffer = callInst.getOperand(0);
+    PointerType* PtrTy = dyn_cast<PointerType>(dstBuffer->getType());
+    const bool is64bit = PtrTy && isA64Ptr(PtrTy, m_CGCtx) && bufType != SLM;
 
     // Cmpxchg intrinsic has 2 sources.
     if (op == EATOMIC_CMPXCHG ||
@@ -202,7 +159,6 @@ void ResolveOCLAtomics::processOCLAtomic(CallInst& callInst, AtomicOp op, Buffer
         }
     }
 
-    Value* dstBuffer = callInst.getOperand(0);
     Value* dst = callInst.getOperand(0);
 
     // We will use 64-bit dst only for 64-bit global pointers.
@@ -304,6 +260,29 @@ void ResolveOCLAtomics::processGetLocalLock(CallInst& callInst)
     m_changed = true;
 }
 
+void ResolveOCLAtomics::processGetGlobalLock(CallInst& callInst)
+{
+    IGC_ASSERT(callInst.getCalledFunction()->getName() == BUILTIN_GET_GLOBAL_LOCK);
+    if (m_globalLock == nullptr) {
+        auto& C = m_pModule->getContext();
+
+        m_globalLock = new GlobalVariable(
+            *m_pModule,
+            Type::getInt32Ty(C),
+            false,
+            GlobalVariable::ExternalLinkage,
+            ConstantInt::get(Type::getInt32Ty(C), 0),
+            "spinlock",
+            nullptr,
+            GlobalValue::NotThreadLocal,
+            ADDRESS_SPACE_GLOBAL);
+    }
+
+    callInst.replaceAllUsesWith(m_globalLock);
+    callInst.eraseFromParent();
+    m_changed = true;
+}
+
 void ResolveOCLAtomics::findLockUsers(Value* V)
 {
     for (User* U : V->users())
