@@ -8,6 +8,7 @@ SPDX-License-Identifier: MIT
 
 #include "Compiler/Optimizer/OpenCLPasses/KernelArgs.hpp"
 #include "AdaptorCommon/ImplicitArgs.hpp"
+#include "llvmWrapper/IR/DerivedTypes.h"
 #include "common/LLVMWarningsPush.hpp"
 #include <llvm/IR/Argument.h>
 #include <llvm/IR/DataLayout.h>
@@ -116,6 +117,36 @@ unsigned int KernelArg::calcElemAllocateSize(const Argument* arg, const DataLayo
     return int_cast<unsigned int>(DL->getTypeAllocSize(arg->getType()->getScalarType()));
 }
 
+// First member of pair is ArgType of buffer.
+// When ArgType is SAMPLER, second member should be true.
+// When ArgType is NOT_TO_ALLOCATE, second member should be false.
+// ArgType enum uses same values for SAMPLER and NOT_TO_ALLOCATE.
+// This function helps disambiguate between the two values.
+KernelArg::BufferArgType KernelArg::getBufferType(const Argument* arg, const StringRef typeStr)
+{
+    if (arg->getType()->getTypeID() != Type::PointerTyID)
+        return { KernelArg::ArgType::SAMPLER, true };
+
+    PointerType* ptrType = cast<PointerType>(arg->getType());
+
+    int address_space = ptrType->getPointerAddressSpace();
+    bool directIdx = false;
+    unsigned int bufId = 0;
+    BufferType bufType = DecodeAS4GFXResource(address_space, directIdx, bufId);
+
+    // Check if this arg is an image
+    if (bufType == BufferType::UAV)
+    {
+        ArgType imgArgType;
+        // Check if argument is image
+        if (isImage(arg, typeStr, imgArgType)) return { imgArgType, false };
+    }
+    else if (bufType == BufferType::SAMPLER)
+        return { KernelArg::ArgType::SAMPLER, true };
+
+    return { KernelArg::ArgType::NOT_TO_ALLOCATE, false };
+}
+
 KernelArg::ArgType KernelArg::calcArgType(const Argument* arg, const StringRef typeStr)
 {
     switch (arg->getType()->getTypeID())
@@ -132,7 +163,7 @@ KernelArg::ArgType KernelArg::calcArgType(const Argument* arg, const StringRef t
         {
 
             Type* type = arg->getType();
-            if (typeStr.equals("queue_t"))
+            if (typeStr.equals("queue_t") || typeStr.equals("spirv.Queue"))
             {
                 return KernelArg::ArgType::PTR_DEVICE_QUEUE;
             }
@@ -185,22 +216,7 @@ KernelArg::ArgType KernelArg::calcArgType(const Argument* arg, const StringRef t
             IGC_ASSERT_MESSAGE(0, "Unrecognized address space");
 #endif
             // This is a buffer. Try to decode this
-            int address_space = ptrType->getPointerAddressSpace();
-            bool directIdx = false;
-            unsigned int bufId = 0;
-            BufferType bufType = DecodeAS4GFXResource(address_space, directIdx, bufId);
-
-            // Check if this arg is an image
-            if (bufType == BufferType::UAV)
-            {
-                ArgType imgArgType;
-                // Check if argument is image
-                if (isImage(arg, typeStr, imgArgType)) return imgArgType;
-            }
-            else if (bufType == BufferType::SAMPLER)
-                return KernelArg::ArgType::SAMPLER;
-
-            return KernelArg::ArgType::NOT_TO_ALLOCATE;
+            return getBufferType(arg, typeStr).type;
         }
     }
     case  Type::IntegerTyID:
@@ -329,9 +345,16 @@ KernelArg::ArgType KernelArg::calcArgType(const ImplicitArg& arg) const
         return KernelArg::ArgType::IMPLICIT_LOCAL_MEMORY_STATELESS_WINDOW_SIZE;
     case ImplicitArg::PRIVATE_MEMORY_STATELESS_SIZE:
         return KernelArg::ArgType::IMPLICIT_PRIVATE_MEMORY_STATELESS_SIZE;
+    case ImplicitArg::RT_STACK_ID:
+        return KernelArg::ArgType::RT_STACK_ID;
+    case ImplicitArg::RT_GLOBAL_BUFFER_POINTER:
+        return KernelArg::ArgType::IMPLICIT_RT_GLOBAL_BUFFER;
     case ImplicitArg::BINDLESS_OFFSET:
         return KernelArg::ArgType::IMPLICIT_BINDLESS_OFFSET;
 
+    case ImplicitArg::IMPLICIT_ARG_BUFFER_PTR:
+        return KernelArg::ArgType::IMPLICIT_ARG_BUFFER;
+
     default:
         return KernelArg::ArgType::NOT_TO_ALLOCATE;
     }
@@ -372,7 +395,7 @@ unsigned int KernelArg::calcAssociatedArgNo(const ImplicitArg& implicitArg, cons
 
 unsigned int KernelArg::getNumComponents() const
 {
-    if (VectorType * vecType = dyn_cast<VectorType>(m_arg->getType()))
+    if (IGCLLVM::FixedVectorType * vecType = dyn_cast<IGCLLVM::FixedVectorType>(m_arg->getType()))
     {
         // Vector
         return int_cast<unsigned int>(vecType->getNumElements());
@@ -659,6 +682,7 @@ std::map<KernelArg::ArgType, iOpenCL::DATA_PARAMETER_TOKEN> initArgTypeTokenMap(
     std::map<KernelArg::ArgType, iOpenCL::DATA_PARAMETER_TOKEN> map
     {
        { KernelArg::ArgType::IMPLICIT_LOCAL_IDS, iOpenCL::DATA_PARAMETER_LOCAL_ID },
+       { KernelArg::ArgType::RT_STACK_ID, iOpenCL::DATA_PARAMETER_RT_STACK_ID },
        { KernelArg::ArgType::IMPLICIT_WORK_DIM, iOpenCL::DATA_PARAMETER_WORK_DIMENSIONS },
        { KernelArg::ArgType::IMPLICIT_NUM_GROUPS, iOpenCL::DATA_PARAMETER_NUM_WORK_GROUPS },
        { KernelArg::ArgType::IMPLICIT_GLOBAL_SIZE, iOpenCL::DATA_PARAMETER_GLOBAL_WORK_SIZE },
@@ -701,7 +725,8 @@ std::map<KernelArg::ArgType, iOpenCL::DATA_PARAMETER_TOKEN> initArgTypeTokenMap(
        { KernelArg::ArgType::IMPLICIT_LOCAL_MEMORY_STATELESS_WINDOW_START_ADDRESS, iOpenCL::DATA_PARAMETER_LOCAL_MEMORY_STATELESS_WINDOW_START_ADDRESS },
        { KernelArg::ArgType::IMPLICIT_LOCAL_MEMORY_STATELESS_WINDOW_SIZE, iOpenCL::DATA_PARAMETER_LOCAL_MEMORY_STATELESS_WINDOW_SIZE },
        { KernelArg::ArgType::IMPLICIT_PRIVATE_MEMORY_STATELESS_SIZE, iOpenCL::DATA_PARAMETER_PRIVATE_MEMORY_STATELESS_SIZE },
-       { KernelArg::ArgType::IMPLICIT_BUFFER_OFFSET, iOpenCL::DATA_PARAMETER_BUFFER_OFFSET }
+       { KernelArg::ArgType::IMPLICIT_BUFFER_OFFSET, iOpenCL::DATA_PARAMETER_BUFFER_OFFSET },
+       { KernelArg::ArgType::IMPLICIT_ARG_BUFFER, iOpenCL::DATA_PARAMETER_IMPL_ARG_BUFFER },
     };
     return map;
 }
@@ -763,6 +788,7 @@ KernelArgsOrder::KernelArgsOrder(InputType layout)
             KernelArg::ArgType::IMPLICIT_PRIVATE_BASE,
             KernelArg::ArgType::IMPLICIT_PRINTF_BUFFER,
             KernelArg::ArgType::IMPLICIT_SYNC_BUFFER,
+            KernelArg::ArgType::IMPLICIT_RT_GLOBAL_BUFFER,
             KernelArg::ArgType::IMPLICIT_BUFFER_OFFSET,
             KernelArg::ArgType::IMPLICIT_WORK_DIM,
             KernelArg::ArgType::IMPLICIT_NUM_GROUPS,
@@ -809,8 +835,11 @@ KernelArgsOrder::KernelArgsOrder(InputType layout)
             KernelArg::ArgType::IMPLICIT_PRIVATE_MEMORY_STATELESS_SIZE,
 
             KernelArg::ArgType::R1,
+            KernelArg::ArgType::RT_STACK_ID,
             KernelArg::ArgType::IMPLICIT_LOCAL_IDS,
 
+            KernelArg::ArgType::IMPLICIT_ARG_BUFFER,
+
             KernelArg::ArgType::STRUCT,
             KernelArg::ArgType::SAMPLER,
             KernelArg::ArgType::IMAGE_1D,
@@ -864,6 +893,7 @@ KernelArgsOrder::KernelArgsOrder(InputType layout)
             KernelArg::ArgType::IMPLICIT_R0,
 
             KernelArg::ArgType::R1,
+            KernelArg::ArgType::RT_STACK_ID,
             KernelArg::ArgType::IMPLICIT_LOCAL_IDS,
 
             KernelArg::ArgType::RUNTIME_VALUE,
@@ -880,6 +910,7 @@ KernelArgsOrder::KernelArgsOrder(InputType layout)
             KernelArg::ArgType::IMPLICIT_PRIVATE_BASE,
             KernelArg::ArgType::IMPLICIT_PRINTF_BUFFER,
             KernelArg::ArgType::IMPLICIT_SYNC_BUFFER,
+            KernelArg::ArgType::IMPLICIT_RT_GLOBAL_BUFFER,
             KernelArg::ArgType::IMPLICIT_BUFFER_OFFSET,
             KernelArg::ArgType::IMPLICIT_WORK_DIM,
             KernelArg::ArgType::IMPLICIT_NUM_GROUPS,
@@ -891,6 +922,8 @@ KernelArgsOrder::KernelArgsOrder(InputType layout)
 
             KernelArg::ArgType::IMPLICIT_BINDLESS_OFFSET,
 
+            KernelArg::ArgType::IMPLICIT_ARG_BUFFER,
+
             KernelArg::ArgType::IMPLICIT_IMAGE_HEIGHT,
             KernelArg::ArgType::IMPLICIT_IMAGE_WIDTH,
             KernelArg::ArgType::IMPLICIT_IMAGE_DEPTH,
