@@ -59,6 +59,7 @@ namespace IGC
 
             IMPLICIT_PRINTF_BUFFER,
             IMPLICIT_SYNC_BUFFER,
+            IMPLICIT_RT_GLOBAL_BUFFER,
 
             IMPLICIT_BUFFER_OFFSET,
 
@@ -107,12 +108,17 @@ namespace IGC
 
             IMPLICIT_LOCAL_IDS,
 
+            // Raytracing args
+            RT_STACK_ID,
+
             // STAGE_IN_GRID runtime values
             IMPLICIT_STAGE_IN_GRID_ORIGIN,
             IMPLICIT_STAGE_IN_GRID_SIZE,
 
             IMPLICIT_BINDLESS_OFFSET,
 
+            IMPLICIT_ARG_BUFFER,
+
             // Argument types that shouldn't be allocated
             NOT_TO_ALLOCATE,
             SAMPLER = NOT_TO_ALLOCATE,
@@ -228,6 +234,17 @@ namespace IGC
         /// @return The kernel argument type of the given explicit argument
         static ArgType calcArgType(const llvm::Argument* arg, const llvm::StringRef typeStr);
 
+        struct BufferArgType
+        {
+            KernelArg::ArgType type = KernelArg::ArgType::End;
+            bool isSampler = false;
+        };
+        /// @brief  Calculates the kernel arg type for buffer
+        /// @param  arg         The explicit kernel argument
+        /// @param  typeStr     The OpenCL type information for the kernel this argument belongs to
+        /// @return Pair of the kernel argument type of the given explicit argument and whether the type is really SAMPLER
+        static BufferArgType getBufferType(const llvm::Argument* arg, const llvm::StringRef typeStr);
+
         /// @brief  Checks whether the given argument is an image
         /// @param  arg           The kernel argument
         /// @param  typeStr       The OpenCL type information for the kernel this argument belongs to
