@@ -13,10 +13,13 @@ SPDX-License-Identifier: MIT
 #include "Compiler/IGCPassSupport.h"
 #include "Compiler/CISACodeGen/GenCodeGenModule.h"
 #include "Compiler/CISACodeGen/LowerGEPForPrivMem.hpp"
+#include "llvmWrapper/IR/DerivedTypes.h"
 #include "common/LLVMWarningsPush.hpp"
 #include "llvmWrapper/IR/DerivedTypes.h"
+#include "llvmWrapper/IR/IRBuilder.h"
 #include "llvm/Transforms/Utils/Local.h"
 #include "llvm/IR/DataLayout.h"
+#include "llvm/IR/DebugInfo.h"
 #include "llvm/IR/DebugInfoMetadata.h"
 #include "llvm/IR/Dominators.h"
 #include "common/LLVMWarningsPop.hpp"
@@ -225,6 +228,7 @@ void PrivateMemoryResolution::getAnalysisUsage(llvm::AnalysisUsage& AU) const
     AU.setPreservesCFG();
     AU.addRequired<MetaDataUtilsWrapper>();
     AU.addRequired<CodeGenContextWrapper>();
+    AU.addRequired<llvm::CallGraphWrapperPass>();
 }
 
 bool PrivateMemoryResolution::safeToUseScratchSpace(llvm::Module& M) const
@@ -251,6 +255,16 @@ bool PrivateMemoryResolution::safeToUseScratchSpace(llvm::Module& M) const
     bool supportsStatelessSpacePrivateMemory = Ctx.m_DriverInfo.supportsStatelessSpacePrivateMemory();
     bool bOCLLegacyStatelessCheck = true;
 
+    if (supportsScratchSpacePrivateMemory) {
+        if (Ctx.type == ShaderType::OPENCL_SHADER) {
+            if (Ctx.platform.hasScratchSurface() && !Ctx.m_DriverInfo.UseScratchSpaceForATSPlus()) {
+                supportsScratchSpacePrivateMemory = Ctx.platform.useScratchSpaceForOCL();
+                //IGC has some legacy cases where stateless private memory must be used. This flag is to remove them. If regression happens, revert it.
+                bOCLLegacyStatelessCheck = !(Ctx.platform.hasScratchSurface() && IGC_IS_FLAG_ENABLED(RemoveLegacyOCLStatelessPrivateMemoryCases));
+            }
+        }
+    }
+
     if (Ctx.allocatePrivateAsGlobalBuffer())
     {
         return false;
@@ -293,6 +307,16 @@ bool PrivateMemoryResolution::safeToUseScratchSpace(llvm::Module& M) const
                         return false;
                     }
                 }
+                if (Ctx.type == ShaderType::OPENCL_SHADER) {
+                    // PtrToInt may be used to test if pointer is null, then we cannot
+                    // distinguish nullptr versus zero offset. This causes a problem
+                    // with an OpenCL3.0 test. see cassian/oclc_address_space_qualifiers
+                    if (PtrToIntInst* IPI = dyn_cast<PtrToIntInst>(&I)) {
+                        if (IPI->getPointerAddressSpace() == ADDRESS_SPACE_PRIVATE) {
+                            return false;
+                        }
+                    }
+                }
             }
         }
 
@@ -329,11 +353,12 @@ bool PrivateMemoryResolution::safeToUseScratchSpace(llvm::Module& M) const
             Ctx.type == ShaderType::DOMAIN_SHADER ||
             Ctx.type == ShaderType::GEOMETRY_SHADER;
 
-        // Start with simd16, which allows the medium size of space per WI
+        //FIXME: Below heuristics is not a clean design. Revisit this!
+        //Start with simd16 or simd32 correspondingly if MinDispatchMode() is 8 or 16, which allows the medium size of space per WI
         // (simd8: largest, simd32, smallest). In doing so, there will be
         // some space left for spilling in simd8 if spilling happens.
         int32_t simd_size = isGeometryStageShader ? numLanes(Ctx.platform.getMinDispatchMode()) :
-            numLanes(SIMDMode::SIMD16);
+            (Ctx.platform.getMinDispatchMode() == SIMDMode::SIMD8 ? numLanes(SIMDMode::SIMD16) : numLanes(SIMDMode::SIMD32));
         const int32_t subGrpSize = funcInfoMD->getSubGroupSize()->getSIMD_size();
         if (subGrpSize > simd_size)
             simd_size = std::min(subGrpSize, static_cast<int32_t>(numLanes(SIMDMode::SIMD32)));
@@ -343,6 +368,11 @@ bool PrivateMemoryResolution::safeToUseScratchSpace(llvm::Module& M) const
         if (groupSize > simd_size)
             simd_size = std::min(groupSize, static_cast<int32_t>(numLanes(SIMDMode::SIMD32)));
 
+        // if one API doesn't support stateless, we should try to use smallest dispatch mode
+        // which can hold more pvt_data to avoid error out.
+        if (Ctx.platform.hasScratchSurface() && Ctx.m_DriverInfo.supportsSeparatingSpillAndPrivateScratchMemorySpace() && !supportsStatelessSpacePrivateMemory)
+            simd_size = numLanes(Ctx.platform.getMinDispatchMode());
+
         unsigned maxScratchSpaceBytes = Ctx.platform.maxPerThreadScratchSpace();
         unsigned scratchSpaceLimitPerWI = maxScratchSpaceBytes / simd_size;
         //
@@ -353,6 +383,18 @@ bool PrivateMemoryResolution::safeToUseScratchSpace(llvm::Module& M) const
         const unsigned int totalPrivateMemPerWI = m_ModAllocaInfo->getTotalPrivateMemPerWI(&F);
 
         if (totalPrivateMemPerWI > scratchSpaceLimitPerWI) {
+            // IGC errors out when we are trying to remove statelesspvtmem of OCL (even though OCl still supports statelesspvtmem).
+            // This assertion tests a scenario where (pvt_mem_usage > 256k) while statelessprivatememory is not supported.
+            IGC_ASSERT_EXIT(bOCLLegacyStatelessCheck);
+
+            if (!supportsStatelessSpacePrivateMemory)
+            {
+                // For XeHP_SDV and above, if any API doesn't support statelesspvtmem, error it out if we find a case where (pvt_mem > 256k).
+                // This assertion found a scenario where (pvt_mem_usage > 256k) while statelessprivatememory is not supported.
+                IGC_ASSERT(0);
+                return true;
+            }
+
             return false;
         }
     }
@@ -418,6 +460,104 @@ bool PrivateMemoryResolution::runOnModule(llvm::Module& M)
         changed |= resolveAllocaInstructions(hasStackCall || hasVLA);
     }
 
+    if (FGA)
+    {
+        auto DL = M.getDataLayout();
+        auto& CG = getAnalysis<CallGraphWrapperPass>().getCallGraph();
+
+        // lambda to recursively calculate the max private memory usage for each call path
+        std::function<uint32_t(Function*)> AnalyzeCGPrivateMemUsage =
+            [&AnalyzeCGPrivateMemUsage, &modMD, &CG, &DL, &Ctx, &M](Function* F)->uint32_t
+        {
+            // Not a valid function, just return 0
+            if (!F || F->isDeclaration())
+                return 0;
+
+            // No function metadata found, return 0
+            auto funcIt = modMD.FuncMD.find(F);
+            if (funcIt == modMD.FuncMD.end())
+                return 0;
+
+            // Stack offsets should be OWORD aligned
+            uint32_t currFuncPrivateMem = (uint32_t)(funcIt->second.privateMemoryPerWI);
+            currFuncPrivateMem = iSTD::Align(currFuncPrivateMem, SIZE_OWORD);
+            CallGraphNode* Node = CG[F];
+
+            // Function has recursion, don't search CG further
+            if (F->hasFnAttribute("hasRecursion"))
+                return currFuncPrivateMem;
+
+            // Reached a leaf, return the private memory used by the current function
+            if (Node->empty())
+                return currFuncPrivateMem;
+
+            SmallSet<Function*, 16> childFuncs;
+            // Collect the list of all direct callees
+            for (auto FI = Node->begin(), FE = Node->end(); FI != FE; ++FI)
+            {
+                if (Function* childF = FI->second->getFunction())
+                {
+                    childFuncs.insert(childF);
+                }
+            }
+
+            // Recursively calculate the private mem usage of all callees
+            uint32_t maxSize = currFuncPrivateMem;
+            for (auto childF : childFuncs)
+            {
+                IGC_ASSERT(childF);
+                // As a conservative measure, assume all stackcall args are stored on private memory
+                uint32_t argSize = 0;
+                for (auto AI = childF->arg_begin(), AE = childF->arg_end(); AI != AE; ++AI)
+                {
+                    // Argument offsets are also OWORD aligned
+                    argSize += iSTD::Align(static_cast<DWORD>(DL.getTypeAllocSize(AI->getType())), SIZE_OWORD);
+                }
+                // Also do it for return value
+                if (!childF->getReturnType()->isVoidTy())
+                {
+                    argSize += iSTD::Align(static_cast<DWORD>(DL.getTypeAllocSize(childF->getReturnType())), SIZE_OWORD);
+                }
+
+                uint32_t stackFrameSize = currFuncPrivateMem + argSize + SIZE_OWORD;
+                uint32_t size = stackFrameSize + AnalyzeCGPrivateMemUsage(childF);
+                maxSize = std::max(maxSize, size);
+            }
+            return maxSize;
+        };
+
+        // Calculate the max private mem used by each function group
+        // by analyzing the call depth. Store this info in the FunctionGroup container.
+        // This info is needed in EmitVISAPass to determine how much private memory to allocate
+        // per SIMD per thread.
+        for (auto GI = FGA->begin(), GE = FGA->end(); GI != GE; ++GI)
+        {
+            FunctionGroup* FG = *GI;
+            Function* pKernel = FG->getHead();
+            uint32_t maxPrivateMem = modMD.FuncMD[pKernel].privateMemoryPerWI;
+
+            if (FG->hasStackCall())
+            {
+                // Analyze call depth for stack memory required
+                maxPrivateMem = AnalyzeCGPrivateMemUsage(pKernel);
+
+                // If indirect calls or recursions exist, add additional 4KB,
+                // and hope we don't run out.
+                if (FG->hasIndirectCall() || FG->hasRecursion())
+                {
+                    maxPrivateMem += (4 * 1024);
+                }
+            }
+            if (FG->hasVariableLengthAlloca())
+            {
+                // Add another 1KB if there are VLAs
+                maxPrivateMem += 1024;
+            }
+            maxPrivateMem = std::max(maxPrivateMem, (uint32_t)(IGC_GET_FLAG_VALUE(ForcePerThreadPrivateMemorySize)));
+            FG->setMaxPrivateMemOnStack((unsigned)maxPrivateMem);
+        }
+    }
+
     if (changed)
     {
         m_pMdUtils->save(M.getContext());
@@ -537,6 +677,11 @@ static void sinkAllocaSingleUse(SmallVectorImpl<AllocaInst*>& Allocas) {
             bool Skip = false;
             SmallVector<Instruction*, 8> UInsts;
             auto UI = dyn_cast<Instruction>(A);
+            // can't sink phi nodes to other BBs
+            // can't sink loads since we don't check for stores on the way
+            if (isa<PHINode>(UI) || UI->mayReadFromMemory())
+                continue;
+
             for (auto U : UI->users()) {
                 auto UUI = dyn_cast<Instruction>(U);
                 //can't sink the use in the same BB where a PHI node exists
@@ -601,7 +746,7 @@ public:
     {
         IGC_ASSERT(nullptr != pLoad);
         IGC_ASSERT(pLoad->isSimple());
-        IRBuilder<> IRB(pLoad);
+        IGCLLVM::IRBuilder<> IRB(pLoad);
         if (isa<Instruction>(pLoad->getPointerOperand()))
         {
             IRB.SetInsertPoint(cast<Instruction>(pLoad->getPointerOperand()));
@@ -618,7 +763,7 @@ public:
             Type* scalarptrTy = PointerType::get(scalarType, pLoad->getPointerAddressSpace());
             IGC_ASSERT(scalarType->getPrimitiveSizeInBits() / 8 == elementSize);
             Value* vec = UndefValue::get(pLoad->getType());
-            auto pLoadVT = cast<VectorType>(pLoad->getType());
+            auto pLoadVT = cast<IGCLLVM::FixedVectorType>(pLoad->getType());
             for (unsigned i = 0, e = (unsigned)pLoadVT->getNumElements(); i < e; ++i)
             {
                 Value* ptr = IRB.CreateIntToPtr(address, scalarptrTy);
@@ -639,7 +784,7 @@ public:
     {
         IGC_ASSERT(nullptr != pStore);
         IGC_ASSERT(pStore->isSimple());
-        IRBuilder<> IRB(pStore);
+        IGCLLVM::IRBuilder<> IRB(pStore);
         if (isa<Instruction>(pStore->getPointerOperand()))
         {
             IRB.SetInsertPoint(cast<Instruction>(pStore->getPointerOperand()));
@@ -657,7 +802,7 @@ public:
             IGC_ASSERT(scalarType->getPrimitiveSizeInBits() / 8 == elementSize);
             Value* vec = pStore->getValueOperand();
 
-            unsigned vecNumElts = (unsigned)cast<VectorType>(vec->getType())->getNumElements();
+            unsigned vecNumElts = (unsigned)cast<IGCLLVM::FixedVectorType>(vec->getType())->getNumElements();
             for (unsigned i = 0; i < vecNumElts; ++i)
             {
                 Value* ptr = IRB.CreateIntToPtr(address, scalarptrTy);
@@ -728,7 +873,7 @@ bool PrivateMemoryResolution::testTransposedMemory(const Type* pTmpType, const T
         }
         else if(pTmpType->isVectorTy())
         {
-            auto pTmpVType = cast<VectorType>(pTmpType);
+            auto pTmpVType = cast<IGCLLVM::FixedVectorType>(pTmpType);
             tmpAllocaSize *= pTmpVType->getNumElements();
             pTmpType = pTmpType->getContainedType(0);
             ok = (nullptr != pTmpType);
@@ -828,12 +973,12 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool privateOnStack)
     // Creates intrinsics that will be lowered in the CodeGen and will handle the simd size
     Function* simdSizeFunc = GenISAIntrinsic::getDeclaration(m_currFunction->getParent(), GenISAIntrinsic::GenISA_simdSize);
 
-    llvm::IRBuilder<> entryBuilder(&*m_currFunction->getEntryBlock().getFirstInsertionPt());
+    IGCLLVM::IRBuilder<> entryBuilder(&*m_currFunction->getEntryBlock().getFirstInsertionPt());
     ImplicitArgs implicitArgs(*m_currFunction, m_pMdUtils);
 
     // Construct an empty DebugLoc.
-    IF_DEBUG_INFO(DebugLoc entryDebugLoc);
-    IF_DEBUG_INFO(entryBuilder.SetCurrentDebugLocation(entryDebugLoc));
+    DebugLoc entryDebugLoc;
+    entryBuilder.SetCurrentDebugLocation(entryDebugLoc);
 
     if (privateOnStack)
     {
@@ -844,8 +989,8 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool privateOnStack)
         for (auto pAI : allocaInsts)
         {
             bool isUniform = pAI->getMetadata("uniform") != nullptr;
-            llvm::IRBuilder<> builder(pAI);
-            IF_DEBUG_INFO(builder.SetCurrentDebugLocation(entryDebugLoc));
+            IGCLLVM::IRBuilder<> builder(pAI);
+            builder.SetCurrentDebugLocation(entryDebugLoc);
 
             // buffer of this private var
             Value* privateBuffer = nullptr;
@@ -905,6 +1050,12 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool privateOnStack)
     // What is the size limit of this scratch memory? If we use >= 128 KB for private data, then we have
     // no space left for later spilling.
     bool useStateless = false;
+
+    if (Ctx.type != ShaderType::OPENCL_SHADER && Ctx.platform.hasScratchSurface()) {
+        useStateless = Ctx.m_DriverInfo.supportsStatelessSpacePrivateMemory();
+    }
+
+    //NOTE: Below if block logic is used either for SSS RW or non-OCL stateless RW
     if (modMD && (modMD->compOpt.UseScratchSpacePrivateMemory || useStateless)) {
         // We want to use this pass to lower alloca instruction
         // to remove some redundant instruction caused by alloca. For original approach,
@@ -918,58 +1069,60 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool privateOnStack)
         Instruction* simdSize = entryBuilder.CreateCall(simdSizeFunc, llvm::None, VALUE_NAME("simdSize"));
 
         Value* privateBase = nullptr;
+        ADDRESS_SPACE scratchMemoryAddressSpace = ADDRESS_SPACE_PRIVATE;
         if (modMD->compOpt.UseScratchSpacePrivateMemory)
         {
-            Argument* r0Arg = implicitArgs.getArgInFunc(*m_currFunction, ImplicitArg::R0);
-            Value* r0_5 = entryBuilder.CreateExtractElement(r0Arg, ConstantInt::get(typeInt32, 5), VALUE_NAME("r0.5"));
-            privateBase = entryBuilder.CreateAnd(r0_5, ConstantInt::get(typeInt32, 0xFFFFFC00), VALUE_NAME("privateBase"));
-        }
-
-        ADDRESS_SPACE scratchMemoryAddressSpace = ADDRESS_SPACE_PRIVATE;
-        if (Ctx.platform.hasScratchSurface())
-        {
-            if (modMD->compOpt.UseScratchSpacePrivateMemory)
+            if (Ctx.platform.hasScratchSurface())
             {
+                // when we use per-thread scratch-surface with SSH bindless
+                // R0_5[32:10] is the offset of the surface-state for scratch
+                // surface slot#0, NOT the offset into the surface.
                 privateBase = entryBuilder.getInt32(0);
             }
-            else if (nullptr == privateBase)
-            {
-                scratchMemoryAddressSpace = ADDRESS_SPACE_GLOBAL;
-                modMD->compOpt.UseStatelessforPrivateMemory = true;
-
-                const uint32_t dwordSizeInBits = 32;
-                const uint32_t pointerSizeInDwords = Ctx.getRegisterPointerSizeInBits(scratchMemoryAddressSpace) / dwordSizeInBits;
-                IGC_ASSERT(pointerSizeInDwords <= 2);
-                llvm::Type* resultType = entryBuilder.getInt32Ty();
-                if (pointerSizeInDwords > 1)
-                {
-                    resultType = IGCLLVM::FixedVectorType::get(resultType, 2);
-                }
-                Function* pFunc = GenISAIntrinsic::getDeclaration(
-                    m_currFunction->getParent(),
-                    GenISAIntrinsic::GenISA_RuntimeValue,
-                    resultType);
-                privateBase = entryBuilder.CreateCall(pFunc, entryBuilder.getInt32(modMD->MinNOSPushConstantSize - pointerSizeInDwords));
-                if (privateBase->getType()->isVectorTy())
-                {
-                    privateBase = entryBuilder.CreateBitCast(privateBase, entryBuilder.getInt64Ty());
-                }
-
-                ConstantInt* totalPrivateMemPerWIValue = ConstantInt::get(typeInt32, totalPrivateMemPerWI);
-                Value* totalPrivateMemPerThread = entryBuilder.CreateMul(simdSize, totalPrivateMemPerWIValue, VALUE_NAME("totalPrivateMemPerThread"));
-
-                Function* pHWTIDFunc = GenISAIntrinsic::getDeclaration(m_currFunction->getParent(), GenISAIntrinsic::GenISA_hw_thread_id_alloca, Type::getInt32Ty(C));
-                llvm::Value* threadId = entryBuilder.CreateCall(pHWTIDFunc);
-                llvm::Value* perThreadOffset = entryBuilder.CreateMul(threadId, totalPrivateMemPerThread, VALUE_NAME("perThreadOffset"));
-                perThreadOffset = entryBuilder.CreateZExt(perThreadOffset, privateBase->getType());
-                privateBase = entryBuilder.CreateAdd(privateBase, perThreadOffset);
+            else
+            {   // the old mechanism
+                Value* r0Val = implicitArgs.getImplicitArgValue(*m_currFunction, ImplicitArg::R0, m_pMdUtils);
+                Value* r0_5 = entryBuilder.CreateExtractElement(r0Val, ConstantInt::get(typeInt32, 5), VALUE_NAME("r0.5"));
+                privateBase = entryBuilder.CreateAnd(r0_5, ConstantInt::get(typeInt32, 0xFFFFFC00), VALUE_NAME("privateBase"));
             }
         }
+        else
+        {
+            scratchMemoryAddressSpace = ADDRESS_SPACE_GLOBAL;
+            modMD->compOpt.UseStatelessforPrivateMemory = true;
+
+            const uint32_t dwordSizeInBits = 32;
+            const uint32_t pointerSizeInDwords = Ctx.getRegisterPointerSizeInBits(scratchMemoryAddressSpace) / dwordSizeInBits;
+            IGC_ASSERT(pointerSizeInDwords <= 2);
+            llvm::Type* resultType = entryBuilder.getInt32Ty();
+            if (pointerSizeInDwords > 1)
+            {
+                resultType = IGCLLVM::FixedVectorType::get(resultType, 2);
+            }
+            Function* pFunc = GenISAIntrinsic::getDeclaration(
+                m_currFunction->getParent(),
+                GenISAIntrinsic::GenISA_RuntimeValue,
+                resultType);
+            privateBase = entryBuilder.CreateCall(pFunc, entryBuilder.getInt32(modMD->MinNOSPushConstantSize - pointerSizeInDwords));
+            if (privateBase->getType()->isVectorTy())
+            {
+                privateBase = entryBuilder.CreateBitCast(privateBase, entryBuilder.getInt64Ty());
+            }
+
+            ConstantInt* totalPrivateMemPerWIValue = ConstantInt::get(typeInt32, totalPrivateMemPerWI);
+            Value* totalPrivateMemPerThread = entryBuilder.CreateMul(simdSize, totalPrivateMemPerWIValue, VALUE_NAME("totalPrivateMemPerThread"));
+
+            Function* pHWTIDFunc = GenISAIntrinsic::getDeclaration(m_currFunction->getParent(), GenISAIntrinsic::GenISA_hw_thread_id_alloca, Type::getInt32Ty(C));
+            llvm::Value* threadId = entryBuilder.CreateCall(pHWTIDFunc);
+            llvm::Value* perThreadOffset = entryBuilder.CreateMul(threadId, totalPrivateMemPerThread, VALUE_NAME("perThreadOffset"));
+            perThreadOffset = entryBuilder.CreateZExt(perThreadOffset, privateBase->getType());
+            privateBase = entryBuilder.CreateAdd(privateBase, perThreadOffset);
+        }
 
         for (auto pAI : allocaInsts)
         {
             bool isUniform = pAI->getMetadata("uniform") != nullptr;
-            llvm::IRBuilder<> builder(pAI);
+            IGCLLVM::IRBuilder<> builder(pAI);
             // Post upgrade to LLVM 3.5.1, it was found that inliner propagates debug info of callee
             // in to the alloca. Further, those allocas are somehow hoisted to the top of program.
             // When those allocas are lowered to below sequence, they result in prologue instructions
@@ -978,7 +1131,7 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool privateOnStack)
             // correctly. So instead, we set DebugLoc for the instructions generated by lowering
             // alloca to mark that they are part of the prologue.
             // Note: As per Amjad, later LLVM version has a fix for this in llvm/lib/Transforms/Utils/InlineFunction.cpp.
-            IF_DEBUG_INFO(builder.SetCurrentDebugLocation(pAI->getDebugLoc()));
+            builder.SetCurrentDebugLocation(pAI->getDebugLoc());
 
             // Get buffer information from the analysis
             unsigned int scalarBufferOffset = m_ModAllocaInfo->getConstBufferOffset(pAI);
@@ -1039,10 +1192,13 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool privateOnStack)
         return true;
     }
 
+    // Only OCL is supposed to reach here.
+    IGC_ASSERT_EXIT(ShaderType::OPENCL_SHADER == Ctx.type);
+
     // Find the implicit argument representing r0 and the private memory base.
-    Argument* r0Arg = implicitArgs.getArgInFunc(*m_currFunction, ImplicitArg::R0);
-    Argument* privateMemArg = implicitArgs.getArgInFunc(*m_currFunction, ImplicitArg::PRIVATE_BASE);
-    // Note: for debugging purposes privateMemArg will be marked as Output to keep its liveness all time
+    Value* r0Val = implicitArgs.getImplicitArgValue(*m_currFunction, ImplicitArg::R0, m_pMdUtils);
+    Value* privateMemPtr = implicitArgs.getImplicitArgValue(*m_currFunction, ImplicitArg::PRIVATE_BASE, m_pMdUtils);
+    // Note: for debugging purposes privateMemPtr will be marked as Output to keep its liveness all time
 
     // Resolve the call
 
@@ -1076,7 +1232,7 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool privateOnStack)
         Value* shlThreadID = entryBuilder.CreateShl(threadId, ConstantInt::get(typeInt32, 1), VALUE_NAME("shlThreadID"));
 
         // FFSID - r0.0 bit 16
-        Value* r0_0 = entryBuilder.CreateExtractElement(r0Arg, ConstantInt::get(typeInt32, 0), VALUE_NAME("r0.0"));
+        Value* r0_0 = entryBuilder.CreateExtractElement(r0Val, ConstantInt::get(typeInt32, 0), VALUE_NAME("r0.0"));
         Value* FFSIDbit = entryBuilder.CreateLShr(r0_0, ConstantInt::get(typeInt32, 16), VALUE_NAME("FFSIDbit"));
         Value* FFSID = entryBuilder.CreateAnd(FFSIDbit, ConstantInt::get(typeInt32, 1), VALUE_NAME("FFSID"));
 
@@ -1088,7 +1244,7 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool privateOnStack)
 
     if (IGC_IS_FLAG_ENABLED(UseOffsetInLocation) &&
         (privateOnStack == false) &&
-        (IGC_GET_FLAG_VALUE(FunctionControl) == FLAG_FCALL_FORCE_INLINE))
+        (IGC::ForceAlwaysInline()))
     {
         IGC_ASSERT_MESSAGE(perThreadOffsetInst, "perThreadOffset will not be marked as Output");
         if (perThreadOffsetInst)
@@ -1109,8 +1265,8 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool privateOnStack)
         // %privateBufferGEP            = getelementptr i8* %privateBase, i32 %totalOffset
         // %privateBuffer               = bitcast i8* %offsettmp1 to <buffer type>
 
-        llvm::IRBuilder<> builder(pAI);
-        IF_DEBUG_INFO(builder.SetCurrentDebugLocation(entryDebugLoc));
+        IGCLLVM::IRBuilder<> builder(pAI);
+        builder.SetCurrentDebugLocation(entryDebugLoc);
         bool isUniform = pAI->getMetadata("uniform") != nullptr;
         // Get buffer information from the analysis
         unsigned int scalarBufferOffset = m_ModAllocaInfo->getConstBufferOffset(pAI);
@@ -1121,7 +1277,7 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool privateOnStack)
         Value* perLaneOffset = isUniform ? builder.getInt32(0) : simdLaneId;
         perLaneOffset = builder.CreateMul(perLaneOffset, ConstantInt::get(typeInt32, bufferSize), VALUE_NAME("perLaneOffset"));
         Value* totalOffset = builder.CreateAdd(bufferOffsetForThread, perLaneOffset, VALUE_NAME(pAI->getName() + ".totalOffset"));
-        Value* privateBufferGEP = builder.CreateGEP(privateMemArg, totalOffset, VALUE_NAME(pAI->getName() + ".privateBufferGEP"));
+        Value* privateBufferGEP = builder.CreateGEP(privateMemPtr, totalOffset, VALUE_NAME(pAI->getName() + ".privateBufferGEP"));
         Value* privateBuffer = builder.CreatePointerCast(privateBufferGEP, pAI->getType(), VALUE_NAME(pAI->getName() + ".privateBuffer"));
 
         auto DbgUses = llvm::FindDbgAddrUses(pAI);
