@@ -46,17 +46,14 @@ bool PrivateMemoryUsageAnalysis::runOnModule(Module& M)
     for (Module::iterator I = M.begin(), E = M.end(); I != E; ++I)
     {
         Function* pFunc = &(*I);
-        // Don't have implicit arg if doing relocation
-        if (pFunc->hasFnAttribute("visaStackCall"))
+        // Skip functions called from function marked with stackcall attribute
+        if (AddImplicitArgs::hasStackCallInCG(pFunc))
         {
             hasStackCall = true;
             continue;
         }
         if (pFunc->isDeclaration())
             continue;
-        // Skip functions called from function marked with IndirectlyCalled attribute
-        if (AddImplicitArgs::hasIndirectlyCalledParent(pFunc))
-            continue;
         if (m_pMDUtils->findFunctionsInfoItem(pFunc) == m_pMDUtils->end_FunctionsInfo())
             continue;
         if (runOnFunction(*pFunc))
@@ -158,4 +155,20 @@ void PrivateMemoryUsageAnalysis::visitAllocaInst(llvm::AllocaInst& AI)
 
 void PrivateMemoryUsageAnalysis::visitBinaryOperator(llvm::BinaryOperator& I)
 {
+    // If we encountered Alloca, then the function uses private memory
+    CodeGenContext* pCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
+    if (pCtx->platform.Enable32BitIntDivRemEmu())
+    {
+        switch (I.getOpcode())
+        {
+        case Instruction::UDiv:
+        case Instruction::URem:
+        case Instruction::SDiv:
+        case Instruction::SRem:
+            m_hasPrivateMem = true;
+            break;
+        default:
+            break;
+        }
+    }
 }
