@@ -1,34 +1,18 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
+============================= end_copyright_notice ===========================*/
 
 #include "AdaptorCommon/ImplicitArgs.hpp"
+#include "AdaptorCommon/AddImplicitArgs.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/ProgramScopeConstants/ProgramScopeConstantResolution.hpp"
 #include "Compiler/CodeGenPublic.h"
 #include "Compiler/IGCPassSupport.h"
 #include "common/LLVMWarningsPush.hpp"
+#include <llvm/ADT/MapVector.h>
 #include <llvm/IR/Function.h>
 #include <llvm/IR/Module.h>
 #include <llvm/IR/Instructions.h>
@@ -74,6 +58,14 @@ static bool needRunConservatively(const Module& M) {
     return false;
 }
 
+static bool isLoweredToRelocation(const GlobalVariable *GV)
+{
+  StringRef name = GV->getName();
+  if (name == "__SubDeviceID" || name == "__MaxHWThreadIDPerSubDevice")
+      return true;
+  return false;
+}
+
 bool ProgramScopeConstantResolution::runOnModule(Module& M)
 {
     LLVMContext& C = M.getContext();
@@ -85,13 +77,14 @@ bool ProgramScopeConstantResolution::runOnModule(Module& M)
         // There are no constants, or no constants are used, so we have nothing to do.
         return false;
     }
-    if (IGC_IS_FLAG_ENABLED(EnableZEBinary) ||
-        modMD->compOpt.EnableZEBinary)
+
+    if (IGC_IS_FLAG_ENABLED(EnableZEBinary) || modMD->compOpt.EnableZEBinary)
     {
         // ZEBinary always relies on relocation, so we can ignore this pass
         return false;
     }
 
+
     if (RunCautiously) {
         if (!needRunConservatively(M))
             return false;
@@ -117,6 +110,10 @@ bool ProgramScopeConstantResolution::runOnModule(Module& M)
         PointerType* ptrType = cast<PointerType>(pGlobalVar->getType());
         IGC_ASSERT_MESSAGE(ptrType, "The type of a global variable must be a pointer type");
 
+        // Do not convert future relocations to implict argument.
+        if (isLoweredToRelocation(pGlobalVar))
+            continue;
+
         // Pointer's address space should be either constant or global
         const unsigned AS = ptrType->getAddressSpace();
 
@@ -165,7 +162,7 @@ bool ProgramScopeConstantResolution::runOnModule(Module& M)
         // Now, go over the users of this constant.
         // First, copy use list, because we will be removing uses.
         std::vector<User*> useVector(pGlobalVar->user_begin(), pGlobalVar->user_end());
-        std::map<Function*, std::map<GlobalVariable*, Value*> > funcToVarSet;
+        llvm::MapVector<Function*, llvm::MapVector<GlobalVariable*, Value*>> funcToVarSet;
 
         for (std::vector<User*>::iterator U = useVector.begin(), UE = useVector.end(); U != UE; ++U)
         {
@@ -177,8 +174,8 @@ bool ProgramScopeConstantResolution::runOnModule(Module& M)
 
             Function* userFunc = user->getParent()->getParent();
 
-            // Don't have implicit arg if doing relocation
-            if (userFunc->hasFnAttribute("visaStackCall"))
+            // Skip functions called from function marked with stackcall attribute
+            if (AddImplicitArgs::hasStackCallInCG(userFunc))
                 continue;
 
             // Skip unused internal functions.
@@ -190,7 +187,12 @@ bool ProgramScopeConstantResolution::runOnModule(Module& M)
 
             ImplicitArgs implicitArgs(*userFunc, mdUtils);
 
+            // Skip if this function does not have the implicit arg
+            if (!implicitArgs.isImplicitArgExist(argType))
+                continue;
+
             // Find the implicit argument representing this constant.
+            IGC_ASSERT_MESSAGE(userFunc->arg_size() >= implicitArgs.size(), "Function arg size does not match meta data args.");
             unsigned int ImplicitArgsBaseIndex = userFunc->arg_size() - implicitArgs.size();
             unsigned int implicitArgIndex = implicitArgs.getArgIndex(argType);
             unsigned int implicitArgIndexInFunc = ImplicitArgsBaseIndex + implicitArgIndex;
