@@ -16,6 +16,7 @@ SPDX-License-Identifier: MIT
 #include "llvm/Config/llvm-config.h"
 #include "llvmWrapper/IR/DerivedTypes.h"
 #include "llvmWrapper/IR/Instructions.h"
+#include "llvmWrapper/IR/IRBuilder.h"
 #include "llvmWrapper/Support/Alignment.h"
 #include "llvmWrapper/Support/TypeSize.h"
 #include "llvm/IR/DerivedTypes.h"
@@ -71,6 +72,12 @@ namespace
         // if the value comes from a bitcast, return the source, otherwise return itself
         Value* SkipBitCast(Value* v) {
             if (BitCastInst* bc = dyn_cast<BitCastInst>(v)) {
+                // Don't skip if this is a pointer cast and the addrspace changed
+                if (v->getType()->isPointerTy() &&
+                    bc->getOperand(0)->getType()->isPointerTy() &&
+                    v->getType()->getPointerAddressSpace() != bc->getOperand(0)->getType()->getPointerAddressSpace()) {
+                    return v;
+                }
                 v = bc->getOperand(0);
             }
             return v;
@@ -165,7 +172,7 @@ Instruction* ReplaceUnsupportedIntrinsics::insertReverseLoop(
     {
         // Remove the unconditional 'br' instruction which will be replaced by a conditional 'br'
         Pre->getTerminator()->eraseFromParent();
-        IRBuilder<> B(Pre);
+        IGCLLVM::IRBuilder<> B(Pre);
         B.SetCurrentDebugLocation(DL);
         // Init the IV
         auto* Init = B.CreateSub(Length, One);
@@ -177,7 +184,7 @@ Instruction* ReplaceUnsupportedIntrinsics::insertReverseLoop(
     Instruction* IV;
     {
         // Loop body's Basic Block
-        IRBuilder<> B(Body);
+        IGCLLVM::IRBuilder<> B(Body);
         B.SetCurrentDebugLocation(DL);
         IV = B.CreateLoad(pIV, "IV");
         // User of function will add more instructions at this point ...
@@ -210,7 +217,7 @@ Instruction* ReplaceUnsupportedIntrinsics::insertLoop(Instruction* Loc, Value* L
     {
         // Remove the unconditional 'br' instruction which will be replaced by a conditional 'br'
         Pre->getTerminator()->eraseFromParent();
-        IRBuilder<> B(Pre);
+        IGCLLVM::IRBuilder<> B(Pre);
         B.SetCurrentDebugLocation(DL);
         ConstantInt* Zero = ConstantInt::get(LengthType, 0);
         // Init the IV
@@ -222,7 +229,7 @@ Instruction* ReplaceUnsupportedIntrinsics::insertLoop(Instruction* Loc, Value* L
     Instruction* IV;
     {
         // Loop body's Basic Block
-        IRBuilder<> B(Body);
+        IGCLLVM::IRBuilder<> B(Body);
         B.SetCurrentDebugLocation(DL);
         IV = B.CreateLoad(pIV, "IV");
         // User of function will add more instructions at this point ...
@@ -238,7 +245,7 @@ Instruction* ReplaceUnsupportedIntrinsics::insertLoop(Instruction* Loc, Value* L
 Value* ReplaceUnsupportedIntrinsics::replicateScalar(
     Value* ScalarVal, Type* Ty, Instruction* InsertBefore)
 {
-    VectorType* VTy = dyn_cast<VectorType>(Ty);
+    IGCLLVM::FixedVectorType* VTy = dyn_cast<IGCLLVM::FixedVectorType>(Ty);
     Type* ETy = VTy ? VTy->getElementType() : Ty;
     uint32_t sBits = (unsigned int)ScalarVal->getType()->getPrimitiveSizeInBits();
     uint32_t nBits = (unsigned int)ETy->getPrimitiveSizeInBits();
@@ -247,7 +254,7 @@ Value* ReplaceUnsupportedIntrinsics::replicateScalar(
     IGC_ASSERT_MESSAGE(nBits <= 64, "Type mismatch in replicateScalar!");
     uint32_t ratio = nBits / sBits;
 
-    IRBuilder<> Builder(InsertBefore);
+    IGCLLVM::IRBuilder<> Builder(InsertBefore);
     Value* NewVal;
     if (ratio > 1)
     {
@@ -458,6 +465,9 @@ void ReplaceUnsupportedIntrinsics::replaceMemcpy(IntrinsicInst* I)
     Value* Src = MC->getRawSource();
     Value* LPCount = MC->getLength();
     uint32_t Align = MC->getDestAlignment();
+    Align = Align != 0 ? Align : 1;
+    uint32_t SrcAlign = MC->getSourceAlignment();
+    SrcAlign = SrcAlign != 0 ? SrcAlign : 1;
     const bool IsVolatile = MC->isVolatile();
     const uint32_t SrcAS = MC->getSourceAddressSpace();
     const uint32_t DstAS = MC->getDestAddressSpace();
@@ -466,7 +476,7 @@ void ReplaceUnsupportedIntrinsics::replaceMemcpy(IntrinsicInst* I)
     Type* TySrcPtrI8 = Type::getInt8PtrTy(C, SrcAS);
     Type* TyDstPtrI8 = Type::getInt8PtrTy(C, DstAS);
 
-    IRBuilder<> Builder(MC);
+    IGCLLVM::IRBuilder<> Builder(MC);
 
     // BaseSize is flag if we want to handle algorithm in general way
     // or want to keep size of base type to further optimizations
@@ -506,6 +516,7 @@ void ReplaceUnsupportedIntrinsics::replaceMemcpy(IntrinsicInst* I)
             // To set alignment correctly
             uint32_t adjust_align = getLargestPowerOfTwo(SZ);
             Align = adjust_align < Align ? adjust_align : Align;
+            SrcAlign = adjust_align < SrcAlign ? adjust_align : SrcAlign;
 
             // If NewCount is less than 6,  don't generate loop.
             // Note that 6 is just an arbitrary number here.
@@ -515,7 +526,7 @@ void ReplaceUnsupportedIntrinsics::replaceMemcpy(IntrinsicInst* I)
                 {
                     Value* tSrc = Builder.CreateConstGEP1_32(vSrc, i);
                     Value* tDst = Builder.CreateConstGEP1_32(vDst, i);
-                    LoadInst* L = Builder.CreateAlignedLoad(tSrc, getAlign(Align), IsVolatile);
+                    LoadInst* L = Builder.CreateAlignedLoad(tSrc, getAlign(SrcAlign), IsVolatile);
                     (void)Builder.CreateAlignedStore(L, tDst, getAlign(Align), IsVolatile);
                 }
             }
@@ -524,10 +535,10 @@ void ReplaceUnsupportedIntrinsics::replaceMemcpy(IntrinsicInst* I)
                 Value* NewLPCount = ConstantInt::get(LPCount->getType(), NewCount);
                 Instruction* IV = insertLoop(MC, NewLPCount, "memcpy");
                 {
-                    IRBuilder<> B(&(*++BasicBlock::iterator(IV)));
+                    IGCLLVM::IRBuilder<> B(&(*++BasicBlock::iterator(IV)));
                     Value* tSrc = B.CreateGEP(vSrc, IV);
                     Value* tDst = B.CreateGEP(vDst, IV);
-                    LoadInst* L = B.CreateAlignedLoad(tSrc, getAlign(Align), IsVolatile);
+                    LoadInst* L = B.CreateAlignedLoad(tSrc, getAlign(SrcAlign), IsVolatile);
                     (void)B.CreateAlignedStore(L, tDst, getAlign(Align), IsVolatile);
                 }
             }
@@ -549,11 +560,12 @@ void ReplaceUnsupportedIntrinsics::replaceMemcpy(IntrinsicInst* I)
             uint32_t SZ = (unsigned int)VecTys[i]->getPrimitiveSizeInBits() / 8;
             uint32_t adjust_align = getLargestPowerOfTwo(SZ);
             Align = adjust_align < Align ? adjust_align : Align;
+            SrcAlign = adjust_align < SrcAlign ? adjust_align : SrcAlign;
             NewSrc = BOfst > 0 ? Builder.CreateConstGEP1_32(Src, BOfst) : Src;
             NewDst = BOfst > 0 ? Builder.CreateConstGEP1_32(Dst, BOfst) : Dst;
             vSrc = Builder.CreateBitCast(SkipBitCast(NewSrc), PointerType::get(VecTys[i], SrcAS), "memcpy_rem");
             vDst = Builder.CreateBitCast(SkipBitCast(NewDst), PointerType::get(VecTys[i], DstAS), "memcpy_rem");
-            LoadInst* L = Builder.CreateAlignedLoad(vSrc, getAlign(Align), IsVolatile);
+            LoadInst* L = Builder.CreateAlignedLoad(vSrc, getAlign(SrcAlign), IsVolatile);
             (void)Builder.CreateAlignedStore(L, vDst, getAlign(Align), IsVolatile);
             BOfst += SZ;
         }
@@ -565,10 +577,10 @@ void ReplaceUnsupportedIntrinsics::replaceMemcpy(IntrinsicInst* I)
         // Fall back to i8 copy
         Instruction* IV = insertLoop(MC, LPCount, "memcpy");
         {
-            IRBuilder<> B(&(*++BasicBlock::iterator(IV)));
+            IGCLLVM::IRBuilder<> B(&(*++BasicBlock::iterator(IV)));
             Value* tSrc = B.CreateGEP(Src, IV);
             Value* tDst = B.CreateGEP(Dst, IV);
-            LoadInst* L = B.CreateAlignedLoad(tSrc, getAlign(Align), IsVolatile);
+            LoadInst* L = B.CreateAlignedLoad(tSrc, getAlign(SrcAlign), IsVolatile);
             (void)B.CreateAlignedStore(L, tDst, getAlign(Align), IsVolatile);
         }
     }
@@ -610,7 +622,7 @@ void ReplaceUnsupportedIntrinsics::replaceMemMove(IntrinsicInst* I)
 
     auto* F = MM->getParent()->getParent();
 
-    IRBuilder<> B(MM);
+    IGCLLVM::IRBuilder<> B(MM);
 
     auto* i8Src = B.CreateBitCast(SkipBitCast(Src), TySrcPtrI8, "memcpy_src");
     auto* i8Dst = B.CreateBitCast(SkipBitCast(Dst), TyDstPtrI8, "memcpy_dst");
@@ -720,7 +732,7 @@ void ReplaceUnsupportedIntrinsics::replaceMemMove(IntrinsicInst* I)
                 auto* NewLPCount = ConstantInt::get(LPCount->getType(), NewCount);
                 Instruction* IV = insertReverseLoop(BBTrue, Post, NewLPCount, "memmmove");
                 {
-                    IRBuilder<> B(&(*++BasicBlock::iterator(IV)));
+                    IGCLLVM::IRBuilder<> B(&(*++BasicBlock::iterator(IV)));
                     Value* tSrc = B.CreateGEP(vSrc, IV);
                     Value* tDst = B.CreateGEP(vDst, IV);
                     LoadInst* L = B.CreateAlignedLoad(tSrc, getAlign(newAlign), IsVolatile);
@@ -744,7 +756,7 @@ void ReplaceUnsupportedIntrinsics::replaceMemMove(IntrinsicInst* I)
             // Fall back to i8 copy
             Instruction* IV = insertReverseLoop(BBTrue, Post, LPCount, "memmove");
             {
-                IRBuilder<> B(&(*++BasicBlock::iterator(IV)));
+                IGCLLVM::IRBuilder<> B(&(*++BasicBlock::iterator(IV)));
                 Value* tSrc = B.CreateGEP(i8Src, IV);
                 Value* tDst = B.CreateGEP(i8Dst, IV);
                 LoadInst* L = B.CreateAlignedLoad(tSrc, getAlign(1), IsVolatile);
@@ -777,7 +789,7 @@ void ReplaceUnsupportedIntrinsics::replaceMemset(IntrinsicInst* I)
     LLVMContext& C = MS->getContext();
     Type* TyPtrI8 = Type::getInt8PtrTy(C, AS);
 
-    IRBuilder<> Builder(MS);
+    IGCLLVM::IRBuilder<> Builder(MS);
 
     ConstantInt* CI = dyn_cast<ConstantInt>(LPCount);
     if (CI)
@@ -821,7 +833,7 @@ void ReplaceUnsupportedIntrinsics::replaceMemset(IntrinsicInst* I)
                 Value* NewLPCount = ConstantInt::get(LPCount->getType(), NewCount);
                 Instruction* IV = insertLoop(MS, NewLPCount, "memset");
                 {
-                    IRBuilder<> B(&(*++BasicBlock::iterator(IV)));
+                    IGCLLVM::IRBuilder<> B(&(*++BasicBlock::iterator(IV)));
                     Value* tDst = B.CreateGEP(vDst, IV);
                     (void)B.CreateAlignedStore(vSrc, tDst, getAlign(Align), IsVolatile);
                 }
@@ -858,7 +870,7 @@ void ReplaceUnsupportedIntrinsics::replaceMemset(IntrinsicInst* I)
         // Fall back to i8 copy
         Instruction* IV = insertLoop(MS, LPCount, "memset");
         {
-            IRBuilder<> B(&(*++BasicBlock::iterator(IV)));
+            IGCLLVM::IRBuilder<> B(&(*++BasicBlock::iterator(IV)));
             Value* tDst = B.CreateGEP(Dst, IV);
             (void)B.CreateAlignedStore(Src, tDst, getAlign(Align), IsVolatile);
         }
@@ -886,20 +898,23 @@ void ReplaceUnsupportedIntrinsics::replaceExpect(IntrinsicInst* MS)
 void ReplaceUnsupportedIntrinsics::replaceFunnelShift(IntrinsicInst* I) {
     IGC_ASSERT(I->getIntrinsicID() == Intrinsic::fshl ||
         I->getIntrinsicID() == Intrinsic::fshr);
-    IRBuilder<> Builder(I);
+    IGCLLVM::IRBuilder<> Builder(I);
     unsigned sizeInBits = I->getArgOperand(0)->getType()->getScalarSizeInBits();
 
     // Don't replace rotate
     if (I->getArgOperand(0) == I->getArgOperand(1) && !I->getType()->isVectorTy() &&
         m_Ctx->platform.supportRotateInstruction())
     {
+        if (m_Ctx->platform.supportQWRotateInstructions() && sizeInBits == 64) {
+            return;
+        }
         if (sizeInBits == 16 || sizeInBits == 32) {
             return;
         }
     }
 
     Value* numBits = Builder.getIntN(sizeInBits, sizeInBits);
-    if (auto IVT = dyn_cast<VectorType>(I->getType())) {
+    if (auto IVT = dyn_cast<IGCLLVM::FixedVectorType>(I->getType())) {
         numBits = ConstantVector::getSplat(IGCLLVM::getElementCount((uint32_t)IVT->getNumElements()), cast<Constant>(numBits));
     }
     auto shiftModulo = Builder.CreateURem(I->getArgOperand(2), numBits);
