@@ -1,28 +1,10 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
+============================= end_copyright_notice ===========================*/
 
 #include "Compiler/Optimizer/OpenCLPasses/SubGroupFuncs/SubGroupFuncsResolution.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/KernelArgs.hpp"
@@ -212,6 +194,7 @@ const std::array<std::pair<std::string, WaveOps>, 13> SubGroupFuncsResolution::m
 };
 
 
+const llvm::StringRef SubGroupFuncsResolution::SUBGROUP_BLOCK_READ = "__builtin_IB_subgroup_block_read_flat";
 
 SubGroupFuncsResolution::SubGroupFuncsResolution(void) : FunctionPass(ID)
 {
@@ -269,10 +252,10 @@ void SubGroupFuncsResolution::CheckSIMDSize(Instruction& I, StringRef msg)
 {
     int32_t simdSize = GetSIMDSize(I.getParent()->getParent());
 
-    if ((simdSize == 32 || m_pCtx->getModuleMetaData()->csInfo.forcedSIMDSize == 32)
-        )
+    if ((simdSize == 32 || m_pCtx->getModuleMetaData()->csInfo.forcedSIMDSize == 32) &&
+        m_pCtx->platform.getGRFSize() != 64)
     {
-        m_pCtx->EmitError(std::string(msg).c_str());
+        m_pCtx->EmitError(std::string(msg).c_str(), &I);
     }
 }
 
@@ -312,6 +295,7 @@ void SubGroupFuncsResolution::mediaBlockRead(llvm::CallInst& CI)
             use->getType());
         Instruction* simdMediaBlockRead = CallInst::Create(simdMediaBlockReadFunc, args, "", &CI);
         use->replaceAllUsesWith(simdMediaBlockRead);
+        updateDebugLoc(&CI, simdMediaBlockRead);
         m_instsToDelete.push_back(bitCast);
         m_instsToDelete.push_back(&CI);
     }
@@ -321,6 +305,7 @@ void SubGroupFuncsResolution::mediaBlockRead(llvm::CallInst& CI)
             GenISAIntrinsic::GenISA_simdMediaBlockRead,
             CI.getType());
         Instruction* simdMediaBlockRead = CallInst::Create(simdMediaBlockReadFunc, args, "", &CI);
+        updateDebugLoc(&CI, simdMediaBlockRead);
         CI.replaceAllUsesWith(simdMediaBlockRead);
         CI.eraseFromParent();
     }
@@ -338,6 +323,7 @@ void SubGroupFuncsResolution::mediaBlockWrite(llvm::CallInst& CI)
         GenISAIntrinsic::GenISA_simdMediaBlockWrite,
         CI.getArgOperand(2)->getType());
     Instruction* simdMediaBlockWrite = CallInst::Create(simdMediaBlockWriteFunc, args, "", &CI);
+    updateDebugLoc(&CI, simdMediaBlockWrite);
 
     CI.replaceAllUsesWith(simdMediaBlockWrite);
     CI.eraseFromParent();
@@ -352,7 +338,7 @@ void SubGroupFuncsResolution::simdBlockRead(llvm::CallInst& CI)
     IGC_ASSERT_MESSAGE(PtrTy, "simdBlockRead has non-pointer type!");
     SmallVector<Value*, 1> args;
     args.push_back(Ptr);
-    SmallVector<Type*, 2>  types;
+    SmallVector<Type*, 3>  types;
     types.push_back(nullptr); types.push_back(nullptr);
     GenISAIntrinsic::ID  genIntrinID = GenISAIntrinsic::GenISA_simdBlockRead;
     ADDRESS_SPACE AS = (ADDRESS_SPACE)PtrTy->getAddressSpace();
@@ -360,7 +346,7 @@ void SubGroupFuncsResolution::simdBlockRead(llvm::CallInst& CI)
     supportLocal = m_pCtx->platform.supportSLMBlockMessage();
     if (AS == ADDRESS_SPACE_LOCAL && !supportLocal)
     {
-        m_pCtx->EmitError("BlockReadLocal not supported!");
+        m_pCtx->EmitError("BlockReadLocal not supported!", &CI);
         return;
     }
 
@@ -404,6 +390,7 @@ void SubGroupFuncsResolution::simdBlockRead(llvm::CallInst& CI)
             genIntrinID,
             types);
         Instruction* simdBlockRead = CallInst::Create(simdBlockReadFunc, args, "", &CI);
+        updateDebugLoc(&CI, simdBlockRead);
         use->replaceAllUsesWith(simdBlockRead);
         m_instsToDelete.push_back(bitCast);
         m_instsToDelete.push_back(&CI);
@@ -415,6 +402,7 @@ void SubGroupFuncsResolution::simdBlockRead(llvm::CallInst& CI)
             genIntrinID,
             types);
         Instruction* simdBlockRead = CallInst::Create(simdBlockReadFunc, args, "", &CI);
+        updateDebugLoc(&CI, simdBlockRead);
         CI.replaceAllUsesWith(simdBlockRead);
         CI.eraseFromParent();
     }
@@ -431,7 +419,7 @@ void SubGroupFuncsResolution::simdBlockWrite(llvm::CallInst& CI)
     supportLocal = m_pCtx->platform.supportSLMBlockMessage();
     if (AS == ADDRESS_SPACE_LOCAL && !supportLocal)
     {
-        m_pCtx->EmitError("BlockWriteLocal not supported!");
+        m_pCtx->EmitError("BlockWriteLocal not supported!", &CI);
         return;
     }
 
@@ -465,6 +453,7 @@ void SubGroupFuncsResolution::simdBlockWrite(llvm::CallInst& CI)
     Function* simdBlockWriteFunc = GenISAIntrinsic::getDeclaration(CI.getCalledFunction()->getParent(),
         GenISAIntrinsic::GenISA_simdBlockWrite, types);
     Instruction* simdBlockWrite = CallInst::Create(simdBlockWriteFunc, args, "", &CI);
+    updateDebugLoc(&CI, simdBlockWrite);
 
     CI.replaceAllUsesWith(simdBlockWrite);
     CI.eraseFromParent();
@@ -504,6 +493,8 @@ void SubGroupFuncsResolution::pushMediaBlockArgs(llvm::SmallVector<llvm::Value*,
 void SubGroupFuncsResolution::subGroupArithmetic(CallInst& CI, WaveOps op, GroupOpType groupType)
 {
     IRBuilder<> IRB(&CI);
+    IRB.SetCurrentDebugLocation(CI.getDebugLoc());
+
     Value* arg = CI.getArgOperand(0);
     // GenISA_Wave* instrinsics do not support i1 type. Handle this with i8 version of instrinsic.
     bool isBoolean = (arg->getType() == IRB.getInt1Ty());
@@ -572,6 +563,7 @@ void SubGroupFuncsResolution::visitCallInst(CallInst& CI)
             // Creates intrinsics that will be lowered in the CodeGen and will handle the sub_group size
             Function* simdSizeFunc = GenISAIntrinsic::getDeclaration(CI.getCalledFunction()->getParent(), GenISAIntrinsic::GenISA_simdSize);
             Instruction* simdSize = CallInst::Create(simdSizeFunc, "simdSize", &CI);
+            updateDebugLoc(&CI, simdSize);
             CI.replaceAllUsesWith(simdSize);
         }
         CI.eraseFromParent();
@@ -584,7 +576,8 @@ void SubGroupFuncsResolution::visitCallInst(CallInst& CI)
         Function* simdLaneIdFunc = GenISAIntrinsic::getDeclaration(CI.getCalledFunction()->getParent(), GenISAIntrinsic::GenISA_simdLaneId);
         Instruction* simdLaneId16 = CallInst::Create(simdLaneIdFunc, "simdLaneId16", &CI);
         Instruction* simdLaneId = ZExtInst::CreateIntegerCast(simdLaneId16, typeInt32, false, "simdLaneId", &CI);
-
+        updateDebugLoc(&CI, simdLaneId16);
+        updateDebugLoc(&CI, simdLaneId);
         CI.replaceAllUsesWith(simdLaneId);
         CI.eraseFromParent();
     }
@@ -597,8 +590,6 @@ void SubGroupFuncsResolution::visitCallInst(CallInst& CI)
         funcName.equals(SubGroupFuncsResolution::SUB_GROUP_SHUFFLE_DF)
         )
     {
-        CheckSIMDSize(CI, "Shuffle not supported in SIMD32");
-
         // Creates intrinsics that will be lowered in the CodeGen and will handle the sub_group_shuffle function
         IRBuilder<> IRB(&CI);
         Value* args[3];
@@ -609,7 +600,7 @@ void SubGroupFuncsResolution::visitCallInst(CallInst& CI)
         Function* simdShuffleFunc = GenISAIntrinsic::getDeclaration(CI.getCalledFunction()->getParent(),
             GenISAIntrinsic::GenISA_WaveShuffleIndex, args[0]->getType());
         Instruction* simdShuffle = CallInst::Create(simdShuffleFunc, args, "simdShuffle", &CI);
-
+        updateDebugLoc(&CI, simdShuffle);
         CI.replaceAllUsesWith(simdShuffle);
         CI.eraseFromParent();
     }
@@ -617,8 +608,6 @@ void SubGroupFuncsResolution::visitCallInst(CallInst& CI)
         funcName.equals(SubGroupFuncsResolution::SUB_GROUP_SHUFFLE_DOWN_US) ||
         funcName.equals(SubGroupFuncsResolution::SUB_GROUP_SHUFFLE_DOWN_UC))
     {
-        CheckSIMDSize(CI, "Shuffle Down not supported in SIMD32");
-
         // Creates intrinsics that will be lowered in the CodeGen and will handle the sub_group_shuffle_down function
         Value* args[3];
         args[0] = CI.getArgOperand(0);
@@ -629,7 +618,7 @@ void SubGroupFuncsResolution::visitCallInst(CallInst& CI)
             GenISAIntrinsic::GenISA_simdShuffleDown,
             args[0]->getType());
         Instruction* simdShuffleDown = CallInst::Create(simdShuffleDownFunc, args, "simdShuffleDown", &CI);
-
+        updateDebugLoc(&CI, simdShuffleDown);
         CI.replaceAllUsesWith(simdShuffleDown);
         CI.eraseFromParent();
     }
@@ -773,20 +762,30 @@ void SubGroupFuncsResolution::visitCallInst(CallInst& CI)
         pushMediaBlockArgs(args, CI);
 
         // The spec requires that the width and height are compile-time constants.
-        if (!isa<ConstantInt>(CI.getOperand(2)))
+        Value* blockWidth = CI.getOperand(2);
+        if (!isa<ConstantInt>(blockWidth))
         {
-            m_pCtx->EmitError("width argument supplied to intel_media_block_read*() must be constant.");
-            return;
+            blockWidth = ValueTracker::track(&CI, 2);
+            if (!blockWidth)
+            {
+                m_pCtx->EmitError("width argument supplied to intel_media_block_read*() must be constant.", &CI);
+                return;
+            }
         }
 
-        if (!isa<ConstantInt>(CI.getOperand(3)))
+        Value* blockHeight = CI.getOperand(3);
+        if (!isa<ConstantInt>(blockHeight))
         {
-            m_pCtx->EmitError("height argument supplied to intel_media_block_read*() must be constant.");
-            return;
+            blockHeight = ValueTracker::track(&CI, 3);
+            if (!blockHeight)
+            {
+                m_pCtx->EmitError("height argument supplied to intel_media_block_read*() must be constant.", &CI);
+                return;
+            }
         }
 
-        args.push_back(CI.getArgOperand(2)); // blockWidth
-        args.push_back(CI.getArgOperand(3)); // blockHeight
+        args.push_back(blockWidth);
+        args.push_back(blockHeight);
 
         Function* MediaBlockReadFunc = GenISAIntrinsic::getDeclaration(
             CI.getCalledFunction()->getParent(),
@@ -799,7 +798,7 @@ void SubGroupFuncsResolution::visitCallInst(CallInst& CI)
 
         CheckMediaBlockInstError(MediaBlockRead, true);
         //Return if any error
-        if (!m_pCtx->oclErrorMessage.empty())
+        if (m_pCtx->HasError())
         {
             return;
         }
@@ -815,20 +814,30 @@ void SubGroupFuncsResolution::visitCallInst(CallInst& CI)
         pushMediaBlockArgs(args, CI);
 
         // The spec requires that the width and height are compile-time constants.
-        if (!isa<ConstantInt>(CI.getOperand(2)))
+        Value* blockWidth = CI.getOperand(2);
+        if (!isa<ConstantInt>(blockWidth))
         {
-            m_pCtx->EmitError("width argument supplied to intel_media_block_write*() must be constant.");
-            return;
+            blockWidth = ValueTracker::track(&CI, 2);
+            if (!blockWidth)
+            {
+                m_pCtx->EmitError("width argument supplied to intel_media_block_write*() must be constant.", &CI);
+                return;
+            }
         }
 
-        if (!isa<ConstantInt>(CI.getOperand(3)))
+        Value* blockHeight = CI.getOperand(3);
+        if (!isa<ConstantInt>(blockHeight))
         {
-            m_pCtx->EmitError("height argument supplied to intel_media_block_write*() must be constant.");
-            return;
+            blockHeight = ValueTracker::track(&CI, 3);
+            if (!blockHeight)
+            {
+                m_pCtx->EmitError("height argument supplied to intel_media_block_write*() must be constant.", &CI);
+                return;
+            }
         }
 
-        args.push_back(CI.getArgOperand(2)); // blockWidth
-        args.push_back(CI.getArgOperand(3)); // blockHeight
+        args.push_back(blockWidth);
+        args.push_back(blockHeight);
         args.push_back(CI.getArgOperand(4)); // pixels
 
         Function* MediaBlockWriteFunc = GenISAIntrinsic::getDeclaration(
@@ -842,7 +851,7 @@ void SubGroupFuncsResolution::visitCallInst(CallInst& CI)
 
         CheckMediaBlockInstError(MediaBlockWrite, false);
         //Return if any error
-        if (!m_pCtx->oclErrorMessage.empty())
+        if (m_pCtx->HasError())
         {
             return;
         }
@@ -862,7 +871,7 @@ void SubGroupFuncsResolution::visitCallInst(CallInst& CI)
 
         Function* MediaBlockRectangleReadFunc = GenISAIntrinsic::getDeclaration(CI.getCalledFunction()->getParent(), GenISAIntrinsic::GenISA_MediaBlockRectangleRead);
         Instruction* MediaBlockRectangleRead = CallInst::Create(MediaBlockRectangleReadFunc, args, "", &CI);
-
+        updateDebugLoc(&CI, MediaBlockRectangleRead);
         CI.replaceAllUsesWith(MediaBlockRectangleRead);
         CI.eraseFromParent();
     }
@@ -893,11 +902,32 @@ void SubGroupFuncsResolution::visitCallInst(CallInst& CI)
     }
     else if (funcName.startswith(SubGroupFuncsResolution::SUB_GROUP_BARRIER))
     {
-        Function* waveBarrier = GenISAIntrinsic::getDeclaration(
-            CI.getCalledFunction()->getParent(),
-            GenISAIntrinsic::GenISA_wavebarrier);
-        CallInst::Create(waveBarrier, "", &CI);
-        CI.eraseFromParent();
+        ModuleMetaData* modMD = getAnalysis<MetaDataUtilsWrapper>().getModuleMetaData();
+
+        // Subgroup barrier is a no-op in HW.
+        // For -O0 a dummy instruction is generated in order to preserve debug info.
+        if (modMD->compOpt.OptDisable)
+        {
+            Function* dummyInst = GenISAIntrinsic::getDeclaration(
+                CI.getCalledFunction()->getParent(),
+                GenISAIntrinsic::GenISA_dummyInst);
+            auto* dummyIntrinsic = cast<GenIntrinsicInst>(CallInst::Create(dummyInst, "", &CI));
+            dummyIntrinsic->setDebugLoc(CI.getDebugLoc());
+            CI.eraseFromParent();
+        }
+        else
+        {
+            Function* waveBarrier = GenISAIntrinsic::getDeclaration(
+                CI.getCalledFunction()->getParent(),
+                GenISAIntrinsic::GenISA_wavebarrier);
+            auto waveBarrierInst = CallInst::Create(waveBarrier, "", &CI);
+            updateDebugLoc(&CI, waveBarrierInst);
+            CI.eraseFromParent();
+        }
+    }
+    else if (funcName.consume_front(SubGroupFuncsResolution::SUBGROUP_BLOCK_READ))
+    {
+        subGroup2DBlockRead(CI, funcName);
     }
     else
     {
@@ -907,6 +937,195 @@ void SubGroupFuncsResolution::visitCallInst(CallInst& CI)
     m_changed = true;
 }
 
+void SubGroupFuncsResolution::subGroup2DBlockRead(llvm::CallInst& CI, llvm::StringRef funcName)
+{
+    IGC::IGCMD::MetaDataUtils* pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
+    IGC::IGCMD::FunctionInfoMetaDataHandle funcInfoMD = pMdUtils->getFunctionsInfoItem(CI.getParent()->getParent());
+    unsigned int subGrpSize = funcInfoMD->getSubGroupSize()->getSIMD_size();
+
+    uint32_t isTranspose = funcName.consume_front("_transpose") ? 1 : 0;
+    uint32_t isVnniTransform = funcName.consume_front("_transform") ? 1 : 0;
+
+    uint32_t elemSize = 0;
+    if (funcName.consume_front("_u8"))
+    {
+        elemSize = 8;
+    }
+    else if (funcName.consume_front("_u16"))
+    {
+        elemSize = 16;
+    }
+    else if (funcName.consume_front("_u32"))
+    {
+        elemSize = 32;
+    }
+
+    if (elemSize == 0)
+    {
+        IGC_ASSERT_MESSAGE(0, "Invalid element size settings for subgroup_block_read_flat.");
+        return;
+    }
+
+    uint32_t tileWidth = 0;
+    uint32_t tileHeight = 0;
+    uint32_t numBlocksV = 2;
+    if (!isTranspose && !isVnniTransform)
+    {
+        // 2x ATile Block Read
+        // __builtin_IB_subgroup_block_read_flat_u8_m1k32v2
+        // __builtin_IB_subgroup_block_read_flat_u8_m2k32v2
+        // __builtin_IB_subgroup_block_read_flat_u8_m4k32v2
+        // __builtin_IB_subgroup_block_read_flat_u8_m8k32v2
+        // __builtin_IB_subgroup_block_read_flat_u16_m1k16v2
+        // __builtin_IB_subgroup_block_read_flat_u16_m2k16v2
+        // __builtin_IB_subgroup_block_read_flat_u16_m4k16v2
+        // __builtin_IB_subgroup_block_read_flat_u16_m8k16v2
+        if (funcName.consume_front("_m1"))
+        {
+            tileHeight = 1;
+        }
+        else if (funcName.consume_front("_m2"))
+        {
+            tileHeight = 2;
+        }
+        else if (funcName.consume_front("_m4"))
+        {
+            tileHeight = 4;
+        }
+        else if (funcName.consume_front("_m8"))
+        {
+            tileHeight = 8;
+        }
+        else
+        {
+            IGC_ASSERT_MESSAGE(0, "Unrecognized m element in __builtin_IB_subgroup_block_read_flat.");
+            return;
+        }
+
+        if (funcName.consume_front("k16"))
+        {
+            tileWidth = 16;
+        }
+        else if (funcName.consume_front("k32"))
+        {
+            tileWidth = 32;
+        }
+        else
+        {
+            IGC_ASSERT_MESSAGE(0, "Unrecognized k element in __builtin_IB_subgroup_block_read_flat.");
+            return;
+        }
+
+        IGC_ASSERT_MESSAGE(funcName.consume_front("v2"), "Unrecognized v element in __builtin_IB_subgroup_block_read_flat.");
+    }
+    else if (isTranspose && !isVnniTransform)
+    {
+        if (elemSize == 32)
+        {
+            // isTranspose, dword elements
+            // __builtin_IB_subgroup_block_read_flat_transpose_u32_k8
+            // can be used as equivalent of:
+            // transpose_transform_u8_k32
+            // transpose_transform_u16_k16
+            numBlocksV = 1;
+            tileHeight = subGrpSize;
+            tileWidth = 8;
+        }
+        else
+        {
+            IGC_ASSERT_MESSAGE(0, "Transpose only supports elemSize 32.");
+            return;
+        }
+    }
+    else if (isVnniTransform && !isTranspose)
+    {
+        numBlocksV = 1;
+        if (elemSize == 8)
+        {
+            // __builtin_IB_subgroup_block_read_flat_transform_u8_k32
+            tileHeight = 32;
+            tileWidth = subGrpSize;
+        }
+        else
+        {
+            // __builtin_IB_subgroup_block_read_flat_transform_u16_k16
+            tileHeight = 16;
+            tileWidth = subGrpSize;
+        }
+    }
+    else
+    {
+        IGC_ASSERT_MESSAGE(0, "Transpose and transform should not be used together.");
+        return;
+    }
+
+    if (tileWidth == 0 || tileHeight == 0)
+    {
+        if (subGrpSize == 0)
+        {
+            IGC_ASSERT_MESSAGE(0, "Invalid tile width / tile height settings for subgroup_block_read_flat because intel_reqd_sub_group_size(16) is not set in the kernel!");
+        }
+        else
+        {
+            IGC_ASSERT_MESSAGE(0, "Invalid tile width / tile height settings for subgroup_block_read_flat.");
+        }
+        return;
+    }
+
+    if (isTranspose && isVnniTransform)
+    {
+        IGC_ASSERT_MESSAGE(0, "Cannot use both hw transpose and hw vnni at the same time for subgroup_block_read_flat.");
+        return;
+    }
+
+    Value* imageResBaseoffset = CI.getArgOperand(0);
+    Value* imageResWidth = CI.getArgOperand(1);
+    Value* imageResHeight = CI.getArgOperand(2);
+    Value* imageResPitch = CI.getArgOperand(3);
+
+    SmallVector<Value*, 12> args;
+    args.push_back(imageResBaseoffset);
+    args.push_back(imageResWidth);
+    args.push_back(imageResHeight);
+    args.push_back(imageResPitch);
+
+    LLVMContext& C = CI.getCalledFunction()->getContext();
+    ConstantInt* constIndex = ConstantInt::get((Type::getInt32Ty(C)), 0);
+    Instruction* xOffset = ExtractElementInst::Create(CI.getArgOperand(4), constIndex, "xOffset", &CI);
+    ConstantInt* constIndex2 = ConstantInt::get((Type::getInt32Ty(C)), 1);
+    Instruction* yOffset = ExtractElementInst::Create(CI.getArgOperand(4), constIndex2, "yOffset", &CI);
+    updateDebugLoc(&CI, xOffset);
+    updateDebugLoc(&CI, yOffset);
+    args.push_back(xOffset);
+    args.push_back(yOffset);
+
+    ConstantInt* elemSizeConstant = ConstantInt::get((Type::getInt32Ty(C)), elemSize);
+    ConstantInt* tileWidthConstant = ConstantInt::get((Type::getInt32Ty(C)), tileWidth);
+    ConstantInt* tileHeightConstant = ConstantInt::get((Type::getInt32Ty(C)), tileHeight);
+    ConstantInt* numBlocksVConstant = ConstantInt::get((Type::getInt32Ty(C)), numBlocksV);
+    ConstantInt* isTransposeConstant = ConstantInt::get((Type::getInt1Ty(C)), isTranspose);
+    ConstantInt* isVnniTransformConstant = ConstantInt::get((Type::getInt1Ty(C)), isVnniTransform);
+    args.push_back(elemSizeConstant);
+    args.push_back(tileWidthConstant);
+    args.push_back(tileHeightConstant);
+    args.push_back(numBlocksVConstant);
+    args.push_back(isTransposeConstant);
+    args.push_back(isVnniTransformConstant);
+
+
+    Function* Block2DReadFunc = GenISAIntrinsic::getDeclaration(
+        CI.getCalledFunction()->getParent(),
+        GenISAIntrinsic::GenISA_LSC2DBlockRead,
+        CI.getCalledFunction()->getReturnType());
+
+    auto* BlockRead = cast<GenIntrinsicInst>(
+        CallInst::Create(Block2DReadFunc, args, "", &CI));
+    BlockRead->setDebugLoc(CI.getDebugLoc());
+
+    CI.replaceAllUsesWith(BlockRead);
+    CI.eraseFromParent();
+}
+
 void SubGroupFuncsResolution::CheckMediaBlockInstError(llvm::GenIntrinsicInst* inst, bool isRead)
 {
     Function* F = inst->getParent()->getParent();
@@ -960,7 +1179,7 @@ void SubGroupFuncsResolution::CheckMediaBlockInstError(llvm::GenIntrinsicInst* i
             raw_string_ostream S(output);
             S << "width for " << builtinPrefix << "*() must be <= " << 32 / typeSize;
             S.flush();
-            m_pCtx->EmitError(output.c_str());
+            m_pCtx->EmitError(output.c_str(), inst);
             return;
         }
 
@@ -971,7 +1190,7 @@ void SubGroupFuncsResolution::CheckMediaBlockInstError(llvm::GenIntrinsicInst* i
             S << "height for " << widthInBytes << " bytes wide "
                 << builtinPrefix << "*() must be <= " << maxRows;
             S.flush();
-            m_pCtx->EmitError(output.c_str());
+            m_pCtx->EmitError(output.c_str(), inst);
             return;
         }
 
@@ -986,7 +1205,7 @@ void SubGroupFuncsResolution::CheckMediaBlockInstError(llvm::GenIntrinsicInst* i
                 S << builtinPrefix << "*() attempt of " << IOSize <<
                     " bytes.  Must be <= " << maxIOSize << " bytes.";
                 S.flush();
-                m_pCtx->EmitError(output.c_str());
+                m_pCtx->EmitError(output.c_str(), inst);
                 return;
             }
         }
@@ -1004,7 +1223,7 @@ void SubGroupFuncsResolution::CheckMediaBlockInstError(llvm::GenIntrinsicInst* i
                 S << builtinPrefix << "_us*() widths must be dual pixel aligned.";
             }
             S.flush();
-            m_pCtx->EmitError(output.c_str());
+            m_pCtx->EmitError(output.c_str(), inst);
             return;
         }
     }
