@@ -1,28 +1,11 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
+============================= end_copyright_notice ===========================*/
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
 #pragma once
 #include "Compiler/CodeGenContextWrapper.hpp"
 #include "Compiler/MetaDataUtilsWrapper.h"
@@ -85,6 +68,7 @@ namespace IGC
         void CheckMediaBlockInstError(llvm::GenIntrinsicInst* inst, bool isRead);
 
         void subGroupArithmetic(llvm::CallInst& CI, WaveOps op, GroupOpType groupType);
+        void subGroup2DBlockRead(llvm::CallInst& CI, llvm::StringRef funcName);
 
         static const llvm::StringRef SUB_GROUP_BARRIER;
         static const llvm::StringRef GET_MAX_SUB_GROUP_SIZE;
@@ -224,7 +208,7 @@ namespace IGC
         static const llvm::StringRef SUB_GROUP_SCAN;
         static const llvm::StringRef SUB_GROUP_CLUSTERED_REDUCE;
 
-
+        static const llvm::StringRef SUBGROUP_BLOCK_READ;
 
     private:
         /// @brief  Container for instructions to be deleted after visiting a function.
