@@ -1,28 +1,11 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
+============================= end_copyright_notice ===========================*/
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
 #pragma once
 
 #include "common/LLVMWarningsPush.hpp"
@@ -30,6 +13,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <llvm/IR/Type.h>
 #include <llvm/IR/Instructions.h>
 #include <llvm/ADT/DenseMap.h>
+#include "llvm/ADT/MapVector.h"
 #include <llvm/ADT/ilist.h>
 #include <llvm/ADT/SetVector.h>
 #include <llvm/ADT/SmallPtrSet.h>
@@ -43,6 +27,8 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include <string>
 #include <sstream>
+#include <set>
+#include <map>
 
 namespace IGC
 {
@@ -65,7 +51,7 @@ namespace IGC
     public:
         static char ID; // Pass identification, replacement for typeid
 
-        ScalarizeFunction(bool scalarizingVectorLDSTType = false);
+        ScalarizeFunction(bool selectiveScalarization = false);
 
         ~ScalarizeFunction();
 
@@ -84,6 +70,8 @@ namespace IGC
 
     private:
 
+        /// @brief select an exclusive set that would not be scalarized
+        void buildExclusiveSet();
         /// @brief main Method for dispatching instructions (according to inst type) for scalarization
         /// @param I instruction to dispatch
         void dispatchInstructionToScalarize(llvm::Instruction* I);
@@ -109,14 +97,6 @@ namespace IGC
         void scalarizeInstruction(llvm::CallInst* CI);
         void scalarizeInstruction(llvm::AllocaInst* CI);
         void scalarizeInstruction(llvm::GetElementPtrInst* CI);
-        void scalarizeInstruction(llvm::LoadInst* CI);
-        void scalarizeInstruction(llvm::StoreInst* CI);
-
-
-        /// @brief Check if worth scalarize Load/Store with given vector type
-        /// @param type vector type of Load/Store (can be NULL)
-        /// @return true if Load/Store worth scalarize, false otherwise
-        bool isScalarizableLoadStoreType(llvm::VectorType* type);
 
         /*! \name Scalarizarion Utility Functions
          *  \{ */
@@ -163,6 +143,11 @@ namespace IGC
         /// @brief Counters for "transpose" statistics
         int m_transposeCtr[llvm::Instruction::OtherOpsEnd];
 
+        /// <summary>
+        /// @brief The instructions we do not want to scalarize
+        /// </summary>
+        std::set<llvm::Value*> m_Excludes;
+
         /// @brief The SCM (scalar conversions map). Per each value - map of its scalar elements
         struct SCMEntry
         {
@@ -194,6 +179,28 @@ namespace IGC
         /// @brief release all allocations of SCM entries
         void releaseAllSCMEntries();
 
+        /// @brief create the dummy function which is called to signify a dummy value
+        inline llvm::Function* getOrCreateDummyFunc(llvm::Type* dummyType, llvm::Module* module) {
+            if (createdDummyFunctions.find(dummyType) == createdDummyFunctions.end()) {
+                llvm::FunctionType* funcType = llvm::FunctionType::get(dummyType, false);
+                llvm::Function* function = llvm::Function::Create(funcType, llvm::Function::InternalLinkage, "", module);
+                createdDummyFunctions[dummyType] = function;
+                return function;
+            }
+            else
+                return createdDummyFunctions[dummyType];
+        }
+
+        /// @brief deletes the memory associated with all the created dynamic Function objects in the map
+        inline void destroyDummyFunc() {
+            for (auto& function : createdDummyFunctions) {
+                if (function.second) {
+                    function.second->eraseFromParent();
+                    function.second = nullptr;
+                }
+            }
+        }
+
         /// @brief An array of available SCMEntry's
         SCMEntry* m_SCMAllocationArray;
 
@@ -224,17 +231,17 @@ namespace IGC
         static llvm::SmallVectorImpl<llvm::Value*>& getArgs(funcRootsVect& FRV) { return FRV.first; }
         static const llvm::SmallVectorImpl<llvm::Value*>& getArgs(const funcRootsVect& FRV) { return FRV.first; }
 
-
-        /// @brief flag to enable/disable scalarizing vector ld/st type.
-        bool m_ScalarizingVectorLDSTType;
+        /// @brief flag for selective scalarization
+        bool m_SelectiveScalarization;
 
         /// @brief This holds DataLayout of processed module
         const llvm::DataLayout* m_pDL;
 
-
+        /// @brief This holds all the created dummy functions throughout the lifetime of the pass, and manages their memory
+        llvm::MapVector<llvm::Type*, llvm::Function*> createdDummyFunctions;
     };
 
 } // namespace IGC
 
 /// By default (no argument given to this function), vector load/store are kept as is.
-extern "C" llvm::FunctionPass* createScalarizerPass(bool scalarizingVectorLDSTType = false);
+extern "C" llvm::FunctionPass* createScalarizerPass(bool selectiveScalarization = false);
