@@ -9,7 +9,7 @@ SPDX-License-Identifier: MIT
 #include "Compiler/Optimizer/OCLBIUtils.h"
 #include "Compiler/Optimizer/ValueTracker.h"
 #include "Compiler/MetaDataApi/MetaDataApi.h"
-#include "DebugInfo/DebugInfoUtils.hpp"
+#include "Compiler/DebugInfo/Utils.h"
 #include "common/LLVMWarningsPush.hpp"
 #include "llvmWrapper/IR/DerivedTypes.h"
 #include "common/LLVMWarningsPop.hpp"
@@ -162,7 +162,7 @@ Value* ValueTracker::handleExtractElement(ExtractElementInst* E)
         {
             auto mask = I->getShuffleMask();
             uint shuffleidx = int_cast<uint>(mask[(uint)idx]);
-            auto vType = dyn_cast<VectorType>(I->getOperand(0)->getType());
+            auto vType = dyn_cast<IGCLLVM::FixedVectorType>(I->getOperand(0)->getType());
             baseValue = (shuffleidx < vType->getNumElements()) ?
                 I->getOperand(0) : I->getOperand(1);
         }
@@ -182,9 +182,9 @@ Value* ValueTracker::handleGlobalVariable(GlobalVariable* G)
     Constant* pSamplerVal = G->getInitializer();
     // Add debug info intrinsic for this variable inside the function using this sampler.
     Instruction* pEntryPoint = &(*m_Function->getEntryBlock().getFirstInsertionPt());
-    IF_DEBUG_INFO(DebugInfoUtils::UpdateGlobalVarDebugInfo(G, pSamplerVal, pEntryPoint, false);)
-        // Found a global sampler, return it.
-        return isa<ConstantStruct>(pSamplerVal) ?
+    Utils::UpdateGlobalVarDebugInfo(G, pSamplerVal, pEntryPoint, false);
+    // Found a global sampler, return it.
+    return isa<ConstantStruct>(pSamplerVal) ?
         pSamplerVal->getAggregateElement(0U) : pSamplerVal;
 }
 
@@ -285,8 +285,9 @@ Value* ValueTracker::findAllocaValue(Value* V, const uint depth)
 
         if (auto* GEP = dyn_cast<GetElementPtrInst>(U))
         {
-            if (!GEP->hasAllConstantIndices())
-                return nullptr;
+            if (!GEP->hasAllConstantIndices()) {
+                continue;
+            }
 
             unsigned numIndices = GEP->getNumIndices();
             if (numIndices > depth + 1)
@@ -324,7 +325,8 @@ Value* ValueTracker::findAllocaValue(Value* V, const uint depth)
         {
             if (CI->getCalledFunction()->getIntrinsicID() == Intrinsic::memcpy)
             {
-                return CI->getOperand(1);
+                // Continue search in current users, handle the memcpy arg in the tracking later.
+                workList.push_back(CI->getOperand(1));
             }
             else if (!CI->getCalledFunction()->isIntrinsic()) // handle user-defined functions
             {
@@ -384,15 +386,21 @@ Value* ValueTracker::findAllocaValue(Value* V, const uint depth)
 // the tree and looks for the alloca that stores a value used as a call instruction parameter.
 // Once alloca is found, the function findAllocaValue is called which is the second step
 // of the algorithm.
-Value* ValueTracker::trackValue(CallInst* CI, const uint index)
+Value* ValueTracker::trackValue(Value* I)
 {
-    Value* baseValue = CI->getOperand(index);
-    auto isFinalValue = [this](auto V) { return callInsts.empty() && (V == nullptr || llvm::isa<Argument>(V) || llvm::isa<ConstantInt>(V)); };
+    Value* baseValue = I;
+    auto isFinalValue = [this](auto V) { return callInsts.empty() && workList.empty() && (V == nullptr || llvm::isa<Argument>(V) || llvm::isa<ConstantInt>(V)); };
 
     while (true)
     {
-        if (isFinalValue(baseValue))
+        if (isFinalValue(baseValue)) {
             return baseValue;
+        }
+        else if (baseValue == nullptr) {
+            if (workList.empty()) return baseValue;
+            baseValue = workList.back();
+            workList.pop_back();
+        }
 
         visitedValues.insert(baseValue);
 
@@ -461,6 +469,34 @@ Value* ValueTracker::trackValue(CallInst* CI, const uint index)
         {
             baseValue = handleConstExpr(I);
         }
+        else if (auto* I = llvm::dyn_cast<PHINode> (baseValue))
+        {
+            if (phiVisited.find(I) != phiVisited.end())
+            {
+                return phiVisited[I];
+            }
+            // For PHINode check if all operands are the same. That allows
+            // to continue tracking, otherwise stop tracking.
+            unsigned num = I->getNumIncomingValues();
+            bool foundFirst = false;
+
+            for (unsigned i = 0; i < num; ++i)
+            {
+                Value* op = trackValue(I->getIncomingValue(i));
+                if (!foundFirst)
+                {
+                    baseValue = op;
+                    foundFirst = true;
+                }
+                else if (op != baseValue)
+                {
+                    baseValue = nullptr;
+                    break;
+                }
+            }
+            phiVisited.insert(std::make_pair(I, baseValue));
+            return baseValue;
+        }
         else
         {
             baseValue = nullptr;
@@ -478,5 +514,6 @@ Value* ValueTracker::track(
     const IGC::ModuleMetaData* pModMD)
 {
     ValueTracker VT(pCallInst->getParent()->getParent(), pMdUtils, pModMD);
-    return VT.trackValue(pCallInst, index);
-}
\ No newline at end of file
+    Value* baseValue = pCallInst->getOperand(index);
+    return VT.trackValue(baseValue);
+}
