@@ -1,28 +1,10 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
+============================= end_copyright_notice ===========================*/
 
 #include "PromoteResourceToDirectAS.h"
 #include "Compiler/IGCPassSupport.h"
@@ -197,7 +179,7 @@ void PromoteResourceToDirectAS::PromoteSamplerTextureToDirectAS(GenIntrinsicInst
         return;
     }
     unsigned bufID = 0;
-    BufferType bufTy;
+    BufferType bufTy = BufferType::BUFFER_TYPE_UNKNOWN;
     BufferAccessType accTy;
     bool needBufferOffset;  // Unused
     bool canPromote = false;
@@ -293,7 +275,7 @@ void PromoteResourceToDirectAS::PromoteSamplerTextureToDirectAS(GenIntrinsicInst
             bufferId = builder.CreateAdd(bufferId, arrayIndex);
         }
 
-        addrSpace = IGC::EncodeAS4GFXResource(*bufferId, bufTy, 0);
+        addrSpace = IGC::EncodeAS4GFXResource(*bufferId, bufTy);
         PointerType* newptrType = PointerType::get(resourcePtr->getType()->getPointerElementType(), addrSpace);
 
         Value* mutePtr = nullptr;
@@ -370,11 +352,20 @@ bool PatchGetElementPtr(const std::vector<Value*>& instList, Type* dstTy, unsign
                 patchedInst = GetElementPtrInst::CreateInBounds(nullptr, patchedInst, gepArgs, "", gepInst);
             else
                 patchedInst = GetElementPtrInst::Create(nullptr, patchedInst, gepArgs, "", gepInst);
+
+            if (GetElementPtrInst* gepPatchedInst = dyn_cast<GetElementPtrInst>(patchedInst))
+            {
+                gepPatchedInst->setDebugLoc(gepInst->getDebugLoc());
+            }
         }
         else if (BitCastInst * cast = dyn_cast<BitCastInst>(inst))
         {
             PointerType* newptrType = PointerType::get(cast->getType()->getPointerElementType(), directAS);
             patchedInst = BitCastInst::Create(Instruction::BitCast, patchedInst, newptrType, "", cast);
+            if (BitCastInst* castPathedInst = dyn_cast<BitCastInst>(patchedInst))
+            {
+                castPathedInst->setDebugLoc(cast->getDebugLoc());
+            }
         }
         else
         {
@@ -540,7 +531,7 @@ void PromoteResourceToDirectAS::PromoteBufferToDirectAS(Instruction* inst, Value
     // Vulkan encodes address space differently, with the reserve bits set.
     // TODO: Investigate how addrspace is encoded in Vulkan,
     // for now skip promoting if it's an address space we dont recognize.
-    if ((addrSpace & 0xFFE00000) != 0x0)
+    if ((addrSpace & 0xFFC00000) != 0x0)
     {
         return;
     }
@@ -565,7 +556,7 @@ void PromoteResourceToDirectAS::PromoteBufferToDirectAS(Instruction* inst, Value
     }
 
     // Get the new direct address space
-    unsigned directAS = IGC::EncodeAS4GFXResource(*builder.getInt32(bufferID), bufType, 0);
+    unsigned directAS = IGC::EncodeAS4GFXResource(*builder.getInt32(bufferID), bufType);
 
     Value* pBuffer = nullptr;
     Type* pBufferType = GetBufferAccessType(inst);
