@@ -1,28 +1,10 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
+============================= end_copyright_notice ===========================*/
 
 #include "Compiler/SPIRMetaDataTranslation.h"
 #include "Compiler/MetaDataApi/SpirMetaDataApi.h"
@@ -32,8 +14,6 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "common/LLVMWarningsPush.hpp"
 #include "llvm/ADT/StringSwitch.h"
 #include "common/LLVMWarningsPop.hpp"
-#include "llvm/IR/InstIterator.h"
-#include <set>
 
 using namespace llvm;
 using namespace IGC;
@@ -81,12 +61,15 @@ void SPIRMetaDataTranslation::WarpFunctionMetadata(Module& M)
     std::set<llvm::Function*> Functions;
     for (uint i = 0; i < opencl_kernels->getNumOperands(); i++)
     {
-        auto pMdNode = opencl_kernels->getOperand(i);
-        if (pMdNode != NULL)
+        if (auto pMdNode = opencl_kernels->getOperand(i))
         {
-            llvm::Function* opFunc =
-                mdconst::dyn_extract<llvm::Function>(pMdNode->getOperand(0));
-            Functions.insert(opFunc);
+            if (pMdNode->getOperand(0))
+            {
+                llvm::Function* opFunc =
+                    mdconst::dyn_extract<llvm::Function>(pMdNode->getOperand(0));
+                Functions.insert(opFunc);
+            }
+
         }
     }
 
@@ -117,44 +100,6 @@ void SPIRMetaDataTranslation::WarpFunctionMetadata(Module& M)
     }
 }
 
-bool SPIRMetaDataTranslation::isDoubleMathFunctionUsed(Module& M)
-{
-    StringRef mathFunctionNamePrefix = "__builtin_spirv_OpenCL_";
-    std::set<StringRef> mathFunctionNames = {
-        "acos", "acosh", "acospi", "asin", "asinh", "asinpi", "atan", "atan2",
-        "atanh", "atanpi", "cbrt", "cos", "cosh", "cospi", "erf", "erfc",
-        "exp", "exp10", "exp2", "expm1", "ln", "log10", "log1p", "log2",
-        "pow", "pown", "powr", "rootn", "sin", "sincos", "sinh", "sinpi",
-        "tan", "tanh", "tanpi"
-    };
-
-    for (Function& F : M)
-    {
-        for (inst_iterator i = inst_begin(&F), e = inst_end(&F); i != e; ++i)
-        {
-            if (auto CI = dyn_cast<CallInst>(&*i)) {
-
-                if (CI->getType()->isDoubleTy() ||
-                    (CI->getType()->isVectorTy() && CI->getType()->getContainedType(0)->isDoubleTy()))
-                {
-                    StringRef functionName = CI->getCalledFunction()->getName();
-
-                    if (functionName.startswith(mathFunctionNamePrefix))
-                    {
-                        functionName = functionName.ltrim(mathFunctionNamePrefix);
-                        functionName = functionName.take_front(functionName.find("_", 0));
-
-                        if (mathFunctionNames.find(functionName) != mathFunctionNames.end()) {
-                            return true;
-                        }
-                    }
-                }
-            }
-        }
-    }
-    return false;
-}
-
 bool SPIRMetaDataTranslation::runOnModule(Module& M)
 {
     WarpFunctionMetadata(M);
@@ -205,6 +150,9 @@ bool SPIRMetaDataTranslation::runOnModule(Module& M)
         IGC::FunctionMetaData& funcMD = modMD->FuncMD[spirKernel->getFunction()];
         fHandle->setType(FunctionTypeMD::KernelFunction);
 
+        if(spirKernel->getFunction() == nullptr)
+            continue;
+
         // Handling Thread Group Size
         SPIRMD::WorkGroupDimensionsMetaDataHandle reqdWorkGroupSize = spirKernel->getRequiredWorkGroupSize();
         if (reqdWorkGroupSize->hasValue())
@@ -233,7 +181,14 @@ bool SPIRMetaDataTranslation::runOnModule(Module& M)
             int simd_size = reqdSubGroupSize->getSIMD_Size();
             if (!((simd_size == 8) || (simd_size == 16) || (simd_size == 32)))
             {
-                getAnalysis<CodeGenContextWrapper>().getCodeGenContext()->EmitError("Unsupported required sub group size");
+                getAnalysis<CodeGenContextWrapper>().getCodeGenContext()->EmitError("Unsupported required sub group size", spirKernel->getFunction());
+                return false;
+            }
+            else if (getAnalysis<CodeGenContextWrapper>().getCodeGenContext()->platform.getPlatformInfo().eProductFamily == IGFX_PVC
+                && simd_size == 8)
+            {
+                getAnalysis<CodeGenContextWrapper>().getCodeGenContext()->EmitError(
+                    "Kernel compiled with required subgroup size 8, which is unsupported on this platform", spirKernel->getFunction());
                 return false;
             }
             sgHandle->setSIMD_size(simd_size);
@@ -398,20 +353,13 @@ bool SPIRMetaDataTranslation::runOnModule(Module& M)
                 break;
             case FAST_RELAXED_MATH:
             case RELAXED_BUILTINS:
-                // Our implementations of double math built-in functions are precise only
-                // if we don't make any fast relaxed math optimizations.
-                // If there is any double-type math function call present in a module, we disable fast relaxed math,
-                // even if "-cl-fast-relaxed-math" build option is specified by the user.
-                if (!isDoubleMathFunctionUsed(M))
-                {
-                    modMD->compOpt.FastRelaxedMath = true;
-                    modMD->compOpt.RelaxedBuiltins = true;
-                }
+                modMD->compOpt.FastRelaxedMath = true;
+                modMD->compOpt.RelaxedBuiltins = true;
                 break;
             case DASH_G: modMD->compOpt.DashGSpecified = true;
                 break;
             case MATCH_SINCOSPI: modMD->compOpt.MatchSinCosPi = true;
-              break;
+                break;
             default:
                 break;
             }
