@@ -81,7 +81,7 @@ bool WorkaroundAnalysis::runOnFunction(Function& F)
     m_pCtxWrapper = &getAnalysis<CodeGenContextWrapper>();
     m_pDataLayout = &F.getParent()->getDataLayout();
 
-    llvm::IGCIRBuilder<> builder(F.getContext());
+    LLVM3DBuilder<> builder(F.getContext(), m_pCtxWrapper->getCodeGenContext()->platform.getPlatformInfo());
     m_builder = &builder;
     m_pModule = F.getParent();
     visit(F);
@@ -340,12 +340,7 @@ void WorkaroundAnalysis::visitCallInst(llvm::CallInst& I)
                 llvm::Instruction* cloneinst = I.clone();
                 cloneinst->insertAfter(&I);
                 m_builder->SetInsertPoint(&(*std::next(BasicBlock::iterator(cloneinst))));
-                Function* SGV = GenISAIntrinsic::getDeclaration(
-                    m_pModule,
-                    GenISAIntrinsic::GenISA_DCL_SystemValue,
-                    m_builder->getFloatTy());
-                Value* inputCoverageFloat = m_builder->CreateCall(SGV, m_builder->getInt32(INPUT_COVERAGE_MASK));
-                Value* inputCoverage = m_builder->CreateBitCast(inputCoverageFloat, m_builder->getInt32Ty());
+                Value* inputCoverage = m_builder->CreateCoverage();
 
                 Value* shiftLeft = m_builder->CreateShl(m_builder->getInt32(1), I.getOperand(1));
                 Value* andWithInputCoverage = m_builder->CreateAnd(shiftLeft, inputCoverage);
@@ -407,6 +402,7 @@ void WorkaroundAnalysis::GatherOffsetWorkaround(SamplerGatherIntrinsic* gatherpo
         return;
     }
     Value* const zero = m_builder->getInt32(0);
+    Value* const zeroFP = llvm::ConstantFP::get(gatherpo->getOperand(0)->getType(), 0);
     const bool hasRef = gatherpo->getIntrinsicID() == GenISAIntrinsic::GenISA_gather4POCptr;
     const uint extraBeginArgsNo = hasRef ? 1 : 0;
 
@@ -424,16 +420,22 @@ void WorkaroundAnalysis::GatherOffsetWorkaround(SamplerGatherIntrinsic* gatherpo
     }
     Value* resource = gatherpo->getTextureValue();
     Value* sampler = gatherpo->getSamplerValue();
+
+    Value* lod = zeroFP;
+
     Function* resInfo =
         GenISAIntrinsic::getDeclaration(m_pModule, GenISAIntrinsic::GenISA_resinfoptr, resource->getType());
     m_builder->SetInsertPoint(gatherpo);
-    Value* info = m_builder->CreateCall2(resInfo, resource, m_builder->getInt32(0));
+    Value* info = m_builder->CreateCall2(resInfo, resource, m_builder->CreateFPToUI(lod, zero->getType()));
+
     std::vector<Value*> arg;
     if (extraBeginArgsNo > 0)
     {
         arg.push_back(gatherpo->getOperand(0));
         IGC_ASSERT(extraBeginArgsNo == 1);
     }
+
+
     arg.push_back(nullptr);                  // u
     arg.push_back(nullptr);                  // v
     arg.push_back(gatherpo->getOperand(4 + extraBeginArgsNo)); // r
@@ -594,4 +596,4 @@ void WAFMinFMax::visitCallInst(CallInst& I)
             break;
         }
     }
-}
\ No newline at end of file
+}
