@@ -1,28 +1,11 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
+============================= end_copyright_notice ===========================*/
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
 #ifndef _BUILTINSFRONTEND_H_
 #define _BUILTINSFRONTEND_H_
 
@@ -40,6 +23,8 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "inc/common/igfxfmid.h"
 #include "common/IGCIRBuilder.h"
 #include "../common/EmUtils.h"
+#include "../../../skuwa/iacm_g10_rev_id.h"
+#include "../../../skuwa/iacm_g11_rev_id.h"
 
 struct genplatform
 {
@@ -51,6 +36,7 @@ public:
     GFXCORE_FAMILY GetPlatformFamily() const { return m_platformInfo->eRenderCoreFamily; }
     bool hasHDCSupportForTypedReads() const { return m_platformInfo->eRenderCoreFamily >= IGFX_GEN10_CORE; }
     bool hasHDCSupportForTypedReadsUnormSnormToFloatConversion() const;
+    bool hasSupportForAllOCLImageFormats() const;
 };
 
 inline bool genplatform::hasHDCSupportForTypedReadsUnormSnormToFloatConversion() const
@@ -58,6 +44,18 @@ inline bool genplatform::hasHDCSupportForTypedReadsUnormSnormToFloatConversion()
     return m_platformInfo->eRenderCoreFamily >= IGFX_GEN10_CORE;
 }
 
+inline bool genplatform::hasSupportForAllOCLImageFormats() const
+{
+    bool isDG2B0Plus = SI_WA_FROM(m_platformInfo->usRevId, ACM_G10_GT_REV_ID_B0);
+    bool isDG2C0Plus = SI_WA_FROM(m_platformInfo->usRevId, ACM_G10_GT_REV_ID_C0);
+    bool isDG2G11EUConfig = GFX_IS_DG2_G11_CONFIG(m_platformInfo->usDeviceID);
+    if ((m_platformInfo->eProductFamily == IGFX_DG2 && isDG2C0Plus) ||
+        (m_platformInfo->eProductFamily == IGFX_DG2 && isDG2G11EUConfig && isDG2B0Plus))
+    {
+        return true;
+    }
+    return m_platformInfo->eRenderCoreFamily >= IGFX_XE_HPC_CORE;
+}
 
 class SampleParamsFromCube
 {
@@ -203,6 +201,7 @@ public:
     llvm::Value* CreateCos(llvm::Value* V);
     llvm::Value* CreateIsNan(llvm::Value *V);
     llvm::Value* CreateCtpop(llvm::Value* V);
+    llvm::Value* CreateFrc(llvm::Value* V);
 
     llvm::Value* CreateCPSRqstCoarseSize(
                     llvm::Value* pSrcVal);
@@ -364,6 +363,10 @@ public:
 
     llvm::Value* Create_SyncThreadGroup();
     llvm::Value* Create_FlushSampler();
+    llvm::Value* Create_LscFence(
+        llvm::Value* SFID,
+        llvm::Value* FenceOp,
+        llvm::Value* Scope);
     llvm::Value* Create_MemoryFence(
         bool commit,
         bool flushRWDataCache,
@@ -383,7 +386,7 @@ public:
         llvm::Value* offsetU,
         llvm::Value* offsetV,
         llvm::Value* offsetW,
-        llvm::Value* minlod = nullptr,
+        llvm::Value* minlod,
         bool feedback_enabled = false,
         llvm::Type* returnType = nullptr);
 
@@ -427,11 +430,10 @@ public:
         llvm::Value* int32_offsetU_358,
         llvm::Value* int32_offsetV_359,
         llvm::Value* int32_offsetW_359,
-        llvm::Value* minlod = nullptr,
+        llvm::Value* minlod,
         bool feedback_enabled = false,
         llvm::Type* returnType = nullptr);
 
-
     llvm::CallInst* Create_SAMPLEDC(
         llvm::Value* float_ref,
         llvm::Value* float_src_u,
@@ -449,6 +451,7 @@ public:
         llvm::Value* int32_offsetU,
         llvm::Value* int32_offsetV,
         llvm::Value* int32_offsetW,
+        bool feedback_enabled = false,
         llvm::Type* returnType = nullptr);
 
     SampleD_DC_FromCubeParams Prepare_SAMPLE_D_DC_Cube_Params(SampleD_DC_FromCubeParams& params);
@@ -481,11 +484,10 @@ public:
         llvm::Value* int32_offsetU,
         llvm::Value* int32_offsetV,
         llvm::Value* int32_offsetW,
-        llvm::Value* minlod = nullptr,
+        llvm::Value* minlod,
         bool feedback_enabled = false,
         llvm::Type* returnType = nullptr);
 
-
     llvm::CallInst* Create_SAMPLEL(
         llvm::Value* float_lod_0,
         llvm::Value* float_address_0,
@@ -511,7 +513,7 @@ public:
         llvm::Value* int32_offsetU,
         llvm::Value* int32_offsetV,
         llvm::Value* int32_offsetR,
-        llvm::Value* minlod = nullptr,
+        llvm::Value* minlod,
         bool feedback_enabled = false,
         llvm::Type* returnType = nullptr);
 
@@ -527,6 +529,7 @@ public:
         llvm::Value* int32_offsetU,
         llvm::Value* int32_offsetV,
         llvm::Value* int32_offsetW,
+        bool feedback_enabled = false,
         llvm::Type* returnType = nullptr);
 
     llvm::CallInst* Create_SAMPLEC_LZ(
@@ -543,7 +546,6 @@ public:
         bool feedback_enabled = false,
         llvm::Type* returnType = nullptr);
 
-
     llvm::CallInst* Create_lod(
         llvm::Value* float_address_0,
         llvm::Value* float_address_1,
@@ -551,9 +553,9 @@ public:
         llvm::Value* float_address_3,
         llvm::Value* int32_textureIdx_356,
         llvm::Value* int32_sampler_357,
+        bool feedback_enabled = false,
         llvm::Type* returnType = nullptr);
 
-
     llvm::CallInst* Create_gather4(
         llvm::Value* float_address_0,
         llvm::Value* float_address_1,
@@ -644,7 +646,8 @@ public:
         llvm::Value* int32_sampler_357,
         llvm::Value* int32_offsetU,
         llvm::Value* int32_offsetV,
-        llvm::Value* int32_srcChannel);
+        llvm::Value* int32_srcChannel,
+        bool feedback_enabled = false);
 
     llvm::Value* Create_gather4PositionOffsetsC(
         llvm::Value* float_reference_0,
@@ -656,7 +659,8 @@ public:
         llvm::Value* int32_sampler_357,
         llvm::Value* int32_offsetU,
         llvm::Value* int32_offsetV,
-        llvm::Value* int32_srcChannel);
+        llvm::Value* int32_srcChannel,
+        bool feedback_enabled = false);
 
     llvm::CallInst* Create_SAMPLEBC(
         llvm::Value* float_ref_value,
@@ -670,6 +674,7 @@ public:
         llvm::Value* int32_offsetU,
         llvm::Value* int32_offsetV,
         llvm::Value* int32_offsetW,
+        bool feedback_enabled = false,
         llvm::Type* returnType = nullptr);
 
     llvm::Value* create_indirectLoad(
@@ -694,21 +699,12 @@ public:
     llvm::Value* CreateF16TOF32(
         llvm::Value* f16_src);
 
-    // Remove once frontends switch to using CreateTypedImageDataConversion()
-    llvm::Value* Create_LdUAVTypedFormatConversion(
-        llvm::Value* pLdUAVTypedResult,
-        IGC::SURFACE_FORMAT format);
-
     IGC::SURFACE_FORMAT GetSubstituteImageFormat(
         const IGC::SURFACE_FORMAT format) const;
 
-    bool SubstituteResourceRequiresDoubleUSize(
-        const IGC::SURFACE_FORMAT format) const;
-
     llvm::Value* CreateImageDataConversion(
         IGC::SURFACE_FORMAT format,
-        llvm::Value* data,
-        llvm::Value* dataHi);
+        llvm::Value* data);
 
     llvm::Value* Create_UBFE(
         llvm::Value* int32_width,
@@ -814,23 +810,6 @@ private:
     llvm::Constant* GetSnormFactor(unsigned bits);
     llvm::Constant* GetUnormFactor(unsigned bits);
 
-    llvm::Value* CreateGen9PlusImageDataConversion(
-        IGC::SURFACE_FORMAT surfaceFormat,
-        llvm::Value* pLdUAVTypedResult);
-
-    llvm::Value* CreateGen8ImageDataConversion(
-        IGC::SURFACE_FORMAT format,
-        llvm::Value* data);
-
-    llvm::Value* CreateGen8ImageDataConversion(
-        IGC::SURFACE_FORMAT format,
-        llvm::Value* data,
-        llvm::Value* dataHi);
-
-    void GetGen8ImageFormatInfo(
-        const IGC::SURFACE_FORMAT format,
-        ImageFormatInfo&          info) const;
-
     bool NeedConversionFor128FormatRead(
         IGC::SURFACE_FORMAT format) const;
 
