@@ -41,8 +41,9 @@ SPDX-License-Identifier: MIT
 namespace {
 struct VcPayloadInfo {
   bool IsValid = false;
-  uint64_t VcOptsOffset = 0;
-  uint64_t IrSize = 0;
+  size_t IRSize = 0;
+  llvm::StringRef VCApiOpts;
+  llvm::StringRef VCInternalOpts;
 };
 
 class BuildDiag {
@@ -63,28 +64,30 @@ private:
 
 static VcPayloadInfo tryExtractPayload(const char *Input, size_t InputSize) {
   // Payload format:
-  // |-vc-codegen|c-str llvm-opts|i64(IR size)|i64(Payload size)|-vc-codegen|
+  // |-vc-payload|api opts|internal opts|i64(IR size)|i64(Payload size)|-vc-payload|
+  // NOTE: <api/internal opts> are c-strings.
   //
   // Should be in sync with:
-  //  Source/IGC/AdaptorOCL/ocl_igc_interface/impl/fcl_ocl_translation_ctx_impl.cpp
+  //    Source/IGC/AdaptorOCL/ocl_igc_interface/impl/fcl_ocl_translation_ctx_impl.cpp
 
-  // Check for availability of "-vc-codegen" marker at the end.
-  const std::string CodegenMarker = "-vc-codegen";
+  // Check for availability of "-vc-payload" marker at the end.
+  const char *PayloadMarker = "-vc-payload";
+  const size_t PayloadMarkerSize = strlen(PayloadMarker);
   // Make sure that we also have a room for 2 i64 size items.
-  if (InputSize < (CodegenMarker.size() + 2 * sizeof(uint64_t)))
+  if (InputSize < (PayloadMarkerSize + 2 * sizeof(uint64_t)))
     return {};
   const char *const InputEnd = Input + InputSize;
-  if (std::memcmp(InputEnd - CodegenMarker.size(), CodegenMarker.data(),
-                  CodegenMarker.size()) != 0)
+  if (std::memcmp(InputEnd - PayloadMarkerSize, PayloadMarker,
+                  PayloadMarkerSize) != 0)
     return {};
 
   // Read IR and Payload sizes. We already ensured that we have the room.
   uint64_t IrSize;
   uint64_t PayloadSize;
   const char *IrSizeBuff =
-      InputEnd - CodegenMarker.size() - 2 * sizeof(uint64_t);
+      InputEnd - PayloadMarkerSize - 2 * sizeof(uint64_t);
   const char *PayloadSizeBuff =
-      InputEnd - CodegenMarker.size() - 1 * sizeof(uint64_t);
+      InputEnd - PayloadMarkerSize - 1 * sizeof(uint64_t);
   memcpy_s(&IrSize, sizeof(IrSize), IrSizeBuff, sizeof(IrSize));
   memcpy_s(&PayloadSize, sizeof(PayloadSize), PayloadSizeBuff,
            sizeof(PayloadSize));
@@ -93,21 +96,25 @@ static VcPayloadInfo tryExtractPayload(const char *Input, size_t InputSize) {
 
   // Search for the start of payload, it should start with "-vc-codegen" marker.
   const char *const IREnd = InputEnd - PayloadSize;
-  if (std::memcmp(IREnd, CodegenMarker.data(), CodegenMarker.size()) != 0)
+  if (std::memcmp(IREnd, PayloadMarker, PayloadMarkerSize) != 0)
     return {};
 
   // Make sure that we have a zero-terminated c-string (vc-options are encoded
   // as such).
-  auto NullPos = std::find(IREnd, InputEnd, 0);
-  if (NullPos == InputEnd)
+  auto ApiOptsStart  = IREnd + PayloadMarkerSize;
+  auto ApiOptsEnd = std::find(ApiOptsStart, InputEnd, 0);
+  if (ApiOptsEnd == InputEnd)
     return {};
+  auto IntOptStart = ApiOptsEnd + 1;
+  auto IntOptEnd = std::find(IntOptStart, InputEnd, 0);
   // Consistency check, see the Payload format.
-  if ((NullPos + 1) != IrSizeBuff)
+  if ((IntOptEnd + 1) != IrSizeBuff)
     return {};
 
   VcPayloadInfo Result;
-  Result.VcOptsOffset = (IREnd + CodegenMarker.size()) - Input;
-  Result.IrSize = IrSize;
+  Result.VCApiOpts = llvm::StringRef(ApiOptsStart);
+  Result.VCInternalOpts = llvm::StringRef(IntOptStart);
+  Result.IRSize = IrSize;
   Result.IsValid = true;
 
   return Result;
@@ -121,6 +128,14 @@ getGenericModuleBuffer(int ResourceID) {
       llvm::LoadBufferFromResource(SS.str().c_str(), "BC")};
 }
 
+template <enum vc::bif::RawKind Kind>
+std::unique_ptr<llvm::MemoryBuffer>
+getVCModuleBufferForArch(llvm::StringRef CPUStr = "") {
+  return llvm::MemoryBuffer::getMemBuffer(
+      vc::bif::getRawDataForArch<Kind>(CPUStr), "",
+      false /* RequiresNullTerminator */);
+}
+
 template <enum vc::bif::RawKind Kind>
 std::unique_ptr<llvm::MemoryBuffer> getVCModuleBuffer() {
   return llvm::MemoryBuffer::getMemBuffer(vc::bif::getRawData<Kind>(), "",
@@ -131,8 +146,17 @@ static void adjustPlatform(const IGC::CPlatform &IGCPlatform,
                            vc::CompileOptions &Opts) {
   auto &PlatformInfo = IGCPlatform.getPlatformInfo();
   unsigned RevId = PlatformInfo.usRevId;
-  Opts.CPUStr = cmc::getPlatformStr(PlatformInfo, /* inout */ RevId);
+  const char *PlatformStr =
+      cmc::getPlatformStr(PlatformInfo, /* inout */ RevId);
+  Opts.CPUStr = PlatformStr ? PlatformStr : "";
   Opts.RevId = RevId;
+  Opts.HasL1ReadOnlyCache = IGCPlatform.hasL1ReadOnlyCache();
+  Opts.HasLocalMemFenceSupress = IGCPlatform.localMemFenceSupress();
+  Opts.HasMultiTile = IGCPlatform.hasMultiTile();
+  Opts.HasL3CacheCoherentCrossTiles = IGCPlatform.L3CacheCoherentCrossTiles();
+  Opts.HasL3FlushOnGPUScopeInvalidate =
+      IGCPlatform.hasL3FlushOnGPUScopeInvalidate();
+  Opts.HasHalfSIMDLSC = IGCPlatform.hasHalfSIMDLSC();
   Opts.WATable = &IGCPlatform.getWATable();
 }
 
@@ -155,7 +179,7 @@ static void adjustFileType(TC::TB_DATA_FORMAT DataFormat,
 
 static void adjustOptLevel(vc::CompileOptions &Opts) {
   if (IGC_IS_FLAG_ENABLED(VCOptimizeNone))
-    Opts.OptLevel = vc::OptimizerLevel::None;
+    Opts.IROptLevel = vc::OptimizerLevel::None;
 }
 
 static void adjustStackCalls(vc::CompileOptions &Opts, BuildDiag &Diag) {
@@ -179,6 +203,21 @@ static void adjustStackCalls(vc::CompileOptions &Opts, BuildDiag &Diag) {
   }
 }
 
+static void adjustDebugStrippingPolicy(vc::CompileOptions &Opts) {
+  int DebugStripFlag = IGC_GET_FLAG_VALUE(StripDebugInfo);
+  switch (DebugStripFlag) {
+  default:
+    Opts.StripDebugInfoCtrl = vc::DebugInfoStripControl::None;
+    break;
+  case FLAG_DEBUG_INFO_STRIP_ALL:
+    Opts.StripDebugInfoCtrl = vc::DebugInfoStripControl::All;
+    break;
+  case FLAG_DEBUG_INFO_STRIP_NONLINE:
+    Opts.StripDebugInfoCtrl = vc::DebugInfoStripControl::NonLine;
+    break;
+  }
+}
+
 // Overwrite binary format option for backward compatibility with
 // environment variable approach.
 static void adjustBinaryFormat(vc::BinaryKind &Binary) {
@@ -186,11 +225,50 @@ static void adjustBinaryFormat(vc::BinaryKind &Binary) {
     Binary = vc::BinaryKind::ZE;
 }
 
+template <typename T> T deriveDefaultableFlagValue(int Flag) {
+  switch (Flag) {
+  default:
+    return T::Default;
+  case DEFAULTABLE_FLAG_ENABLE:
+    return T::Enable;
+  case DEFAULTABLE_FLAG_DISABLE:
+    return T::Disable;
+  }
+}
+
 static void adjustTransformationsAndOptimizations(vc::CompileOptions &Opts) {
   if (IGC_IS_FLAG_ENABLED(VCLocalizeAccUsage))
     Opts.ForceLiveRangesLocalizationForAccUsage = true;
   if (IGC_IS_FLAG_ENABLED(VCDisableNonOverlappingRegionOpt))
     Opts.ForceDisableNonOverlappingRegionOpt = true;
+  if (IGC_IS_FLAG_ENABLED(VCSaveStackCallLinkage))
+    Opts.SaveStackCallLinkage = true;
+  if (IGC_IS_FLAG_ENABLED(VCDirectCallsOnly))
+    Opts.DirectCallsOnly = true;
+  if (IGC_IS_FLAG_ENABLED(DisableEuFusion))
+    Opts.DisableEUFusion = true;
+  if (IGC_IS_FLAG_ENABLED(DebugInfoValidation))
+    Opts.ForceDebugInfoValidation = true;
+  if (IGC_IS_FLAG_ENABLED(EnableL3FlushForGlobal))
+    Opts.HasL3FlushForGlobal = true;
+  if (IGC_IS_FLAG_ENABLED(EnableGPUFenceScopeOnSingleTileGPUs))
+    Opts.HasGPUFenceScopeOnSingleTileGPUs = true;
+
+  if (unsigned LoopUnrollThreshold = IGC_GET_FLAG_VALUE(VCLoopUnrollThreshold))
+    Opts.ForceLoopUnrollThreshold = LoopUnrollThreshold;
+  if (IGC_IS_FLAG_ENABLED(VCIgnoreLoopUnrollThresholdOnPragma))
+    Opts.IgnoreLoopUnrollThresholdOnPragma = true;
+
+  Opts.NoOptFinalizerMode =
+      deriveDefaultableFlagValue<vc::NoOptFinalizerControl>(
+          IGC_GET_FLAG_VALUE(VCNoOptFinalizerControl));
+
+  Opts.DisableLRCoalescingMode =
+      deriveDefaultableFlagValue<vc::DisableLRCoalescingControl>(
+          IGC_GET_FLAG_VALUE(VCDisableLRCoalescingControl));
+  Opts.DisableExtraCoalescingMode =
+      deriveDefaultableFlagValue<vc::DisableExtraCoalescingControl>(
+          IGC_GET_FLAG_VALUE(VCDisableExtraCoalescing));
 }
 
 static void adjustDumpOptions(vc::CompileOptions &Opts) {
@@ -211,6 +289,7 @@ static void adjustOptions(const IGC::CPlatform &IGCPlatform,
   adjustBinaryFormat(Opts.Binary);
   adjustDumpOptions(Opts);
   adjustStackCalls(Opts, Diag);
+  adjustDebugStrippingPolicy(Opts);
 
   adjustTransformationsAndOptimizations(Opts);
 }
@@ -312,11 +391,21 @@ parseOptions(vc::ShaderDumper &Dumper, llvm::StringRef ApiOptions,
     dumpInputData(Dumper, ApiOptions, InternalOptions, Input, /*IsRaw=*/true);
   });
 
-  auto NewPathPayload = tryExtractPayload(Input.data(), Input.size());
-  if (NewPathPayload.IsValid) {
-    ApiOptions = "-vc-codegen";
-    InternalOptions = Input.data() + NewPathPayload.VcOptsOffset;
-    Input = Input.take_front(static_cast<size_t>(NewPathPayload.IrSize));
+  std::string ApiOptionsHolder;
+  std::string InternalOptionsHolder;
+  auto LegacyPayload = tryExtractPayload(Input.data(), Input.size());
+  if (LegacyPayload.IsValid) {
+    ApiOptionsHolder = "-vc-codegen";
+    ApiOptionsHolder.append(" ");
+    ApiOptionsHolder.append(LegacyPayload.VCApiOpts.str());
+    ApiOptions = ApiOptionsHolder;
+
+    InternalOptionsHolder = InternalOptions.str();
+    InternalOptionsHolder.append(" ");
+    InternalOptionsHolder.append(LegacyPayload.VCInternalOpts.str());
+    InternalOptions = InternalOptionsHolder;
+
+    Input = Input.take_front(static_cast<size_t>(LegacyPayload.IRSize));
   }
 
   std::string AuxApiOptions;
@@ -353,7 +442,7 @@ bool fillPrintfData(vc::ExternalData &ExtData) {
 }
 
 static llvm::Optional<vc::ExternalData>
-fillExternalData(vc::BinaryKind Binary) {
+fillExternalData(vc::BinaryKind Binary, llvm::StringRef CPUStr) {
   vc::ExternalData ExtData;
   ExtData.OCLGenericBIFModule =
       getGenericModuleBuffer(OCL_BC);
@@ -377,9 +466,13 @@ fillExternalData(vc::BinaryKind Binary) {
     break;
   }
   ExtData.VCEmulationBIFModule =
-      getVCModuleBuffer<vc::bif::RawKind::Emulation>();
+      getVCModuleBufferForArch<vc::bif::RawKind::Emulation>(CPUStr);
   if (!ExtData.VCEmulationBIFModule)
     return {};
+  ExtData.VCSPIRVBuiltinsBIFModule =
+      getVCModuleBuffer<vc::bif::RawKind::SPIRVBuiltins>();
+  if (!ExtData.VCSPIRVBuiltinsBIFModule)
+    return {};
   return std::move(ExtData);
 }
 
@@ -397,12 +490,43 @@ static void dumpPlatform(const vc::CompileOptions &Opts, PLATFORM Platform,
 
   Os << "NEO passed: DisplayCore = " << Core << ", RenderCore = " << RenderCore
      << ", Product = " << Product << ", Revision = " << RevId << "\n";
-  Os << "IGC translated into: " << Opts.CPUStr << ", " << Opts.RevId << "\n";
+  Os << "IGC translated into: "
+     << (Opts.CPUStr.empty() ? "(empty)" : Opts.CPUStr) << ", " << Opts.RevId
+     << "\n";
 
   Dumper.dumpText(Os.str(), "platform.be.txt");
 #endif
 }
 
+static bool textRelocationsMatch(const vc::CMKernel &K) {
+  auto &Info = K.getProgramOutput();
+  return Info.m_relocs.size() == Info.m_funcRelocationTableEntries;
+}
+
+static void validateCMProgramForOCLBin(const vc::CGen8CMProgram &CMProgram) {
+  IGC_ASSERT_MESSAGE(
+      CMProgram.m_programInfo->m_GlobalPointerAddressRelocAnnotation.globalReloc
+          .empty(),
+      "global section relocations aren't supported for oclbin");
+  IGC_ASSERT_MESSAGE(
+      CMProgram.m_programInfo->m_GlobalPointerAddressRelocAnnotation
+          .globalConstReloc.empty(),
+      "constant section relocations aren't supported for oclbin");
+  // FIXME: Relocations in indirect functions are unsupported for oclbin. They
+  // are supported for zebin. So zebin and oclbin data is compared here to
+  // diagnose the issue.
+  // FIXME: It is possible to have a legal case where the number of relocations
+  // for zebin and oclbin differs in future. The check must be updated in
+  // this case. For now number of relocations is expected to match or to be 0
+  // for oclbin and not 0 for zebin in case of relocations in indirect
+  // functions.
+  IGC_ASSERT_MESSAGE(llvm::all_of(CMProgram.m_kernels,
+                                  [](const std::unique_ptr<vc::CMKernel> &K) {
+                                    return textRelocationsMatch(*K);
+                                  }),
+                     "some text relocations are lost for oclbin");
+}
+
 std::error_code vc::translateBuild(const TC::STB_TranslateInputArgs *InputArgs,
                                    TC::STB_TranslateOutputArgs *OutputArgs,
                                    TC::TB_DATA_FORMAT InputDataFormatTemp,
@@ -437,16 +561,19 @@ std::error_code vc::translateBuild(const TC::STB_TranslateInputArgs *InputArgs,
   adjustOptions(IGCPlatform, InputDataFormatTemp, Opts, Diag);
 
   // here we have Opts set and can dump what we got from runtime and how
-  // we understood it
+  // we understood it. We need to do it before output error on unknown platform
   dumpPlatform(Opts, IGCPlatform.getPlatformInfo(), *Dumper);
 
+  if (Opts.CPUStr.empty())
+    llvm::report_fatal_error("vc::translateBuild: unsupported platform");
+
   if (IGC_IS_FLAG_ENABLED(ShaderOverride))
     Opts.ShaderOverrider =
         vc::createVC_IGCShaderOverrider(Hash, IGCPlatform.getPlatformInfo());
 
   Opts.Dumper = std::move(Dumper);
 
-  auto ExtData = fillExternalData(Opts.Binary);
+  auto ExtData = fillExternalData(Opts.Binary, Opts.CPUStr);
   if (!ExtData)
     return getError(vc::make_error_code(vc::errc::bif_load_fail),
                     OutputArgs);
@@ -470,7 +597,8 @@ std::error_code vc::translateBuild(const TC::STB_TranslateInputArgs *InputArgs,
     auto &CompileResult = std::get<vc::ocl::CompileOutput>(Res);
     vc::CGen8CMProgram CMProgram{IGCPlatform.getPlatformInfo(), IGCPlatform.getWATable()};
     vc::createBinary(CMProgram, CompileResult);
-    CMProgram.CreateKernelBinaries();
+    validateCMProgramForOCLBin(CMProgram);
+    CMProgram.CreateKernelBinaries(Opts);
     Util::BinaryStream ProgramBinary;
     CMProgram.GetProgramBinary(ProgramBinary, CompileResult.PointerSizeInBytes);
     llvm::StringRef BinaryRef{ProgramBinary.GetLinearPointer(),
