@@ -12,8 +12,8 @@ SPDX-License-Identifier: MIT
 //
 //===----------------------------------------------------------------------===//
 
-#ifndef LLVM_GENX_OPTS_H
-#define LLVM_GENX_OPTS_H
+#ifndef VC_GENXOPTS_GENXOPTS_H
+#define VC_GENXOPTS_GENXOPTS_H
 
 namespace llvm {
 
@@ -70,6 +70,15 @@ void initializeGenXImportOCLBiFPass(PassRegistry &);
 ModulePass *createGenXBTIAssignmentPass();
 void initializeGenXBTIAssignmentPass(PassRegistry &);
 
+ModulePass *createGenXTranslateSPIRVBuiltinsPass();
+void initializeGenXTranslateSPIRVBuiltinsPass(PassRegistry &);
+
+ModulePass *createGenXTrampolineInsertionPass();
+void initializeGenXTrampolineInsertionPass(PassRegistry &);
+
+ModulePass *createGenXLinkageCorruptorPass();
+void initializeGenXLinkageCorruptorPass(PassRegistry &);
+
 } // End llvm namespace
 
-#endif
+#endif // VC_GENXOPTS_GENXOPTS_H
