@@ -22,19 +22,22 @@ SPDX-License-Identifier: MIT
 
 #include "IGC/common/StringMacros.hpp"
 
-#include <algorithm>
 #include "GenX.h"
 #include "GenXAlignmentInfo.h"
+#include "GenXRegionUtils.h"
 #include "GenXBaling.h"
-#include "GenXRegion.h"
+
 #include "llvm/IR/Constants.h"
 #include "llvm/IR/Function.h"
 #include "llvm/IR/Instructions.h"
 #include "llvm/IR/Intrinsics.h"
 #include "llvm/Support/Debug.h"
-#include <set>
+
 #include "Probe/Assertion.h"
 
+#include <algorithm>
+#include <set>
+
 using namespace llvm;
 using namespace genx;
 
@@ -160,6 +163,14 @@ Alignment AlignmentInfo::get(Value *V)
             } else
               A = Alignment::getUnknown();
             break;
+          case Instruction::Or:
+            if (auto *CI0 = dyn_cast<ConstantInt>(Op0)) {
+              A = A1.logicalOr(CI0);
+            } else if (auto *CI1 = dyn_cast<ConstantInt>(Op1)) {
+              A = A0.logicalOr(CI1);
+            } else
+              A = Alignment::getUnknown();
+            break;
           default:
             A = Alignment::getUnknown();
             break;
@@ -200,7 +211,7 @@ Alignment AlignmentInfo::get(Value *V)
           // Handle the case of reading a scalar from element of a vector, as
           // a trunc from i32 to i16 is lowered to a bitcast to v2i16 then a
           // rdregion.
-          Region R(WorkInst, BaleInfo());
+          Region R = makeRegionFromBaleInfo(WorkInst, BaleInfo());
           if (!R.Indirect && (R.NumElements == 1))
             A = getFromInstMap(WorkInst->getOperand(0));
           else
@@ -384,9 +395,10 @@ Alignment Alignment::mul(Alignment Other) const
 }
 
 /***********************************************************************
- * logicalAnd : logical and two alignments. Only constant int supported.
+ * logicalOp : Helped Function for alignment calculating of logical
+ * 'AND' and 'OR'.
  */
-Alignment Alignment::logicalAnd(ConstantInt *CI) const {
+Alignment Alignment::logicalOp(ConstantInt *CI, SelectFunction F) const {
   IGC_ASSERT(!isUncomputed() && CI);
   // If value doesn't fit into unsigned then be conservative and pretend
   // that alignement is unknown
@@ -396,11 +408,25 @@ Alignment Alignment::logicalAnd(ConstantInt *CI) const {
     return Alignment::getUnknown();
   unsigned UVal = static_cast<unsigned>(std::abs(Val));
   unsigned ValLSB = countTrailingZeros(UVal, ZB_Width);
-  // Chop off constant bits according to maximum log align
-  unsigned NewLogAlign = std::max(ValLSB, LogAlign);
+  // Chop off constant bits according to log align
+  unsigned NewLogAlign = F(ValLSB, LogAlign);
   return Alignment(NewLogAlign, UVal & ((1 << NewLogAlign) - 1));
 }
 
+/***********************************************************************
+ * logicalAnd : logical and two alignments. Only constant int supported.
+ */
+Alignment Alignment::logicalAnd(ConstantInt *CI) const {
+  return logicalOp(CI, std::max<unsigned>);
+}
+
+/***********************************************************************
+ * logicalOr : logical or two alignments. Only constant int supported.
+ */
+Alignment Alignment::logicalOr(ConstantInt *CI) const {
+  return logicalOp(CI, std::min<unsigned>);
+}
+
 /***********************************************************************
  * getFromInstMap : get the alignment of a value, direct from InstMap if
  * found else return Unknown, Alignment(0, 0)
