@@ -9,7 +9,7 @@ SPDX-License-Identifier: MIT
 //
 /// GenXBaling
 /// ----------
-/// 
+///
 /// GenX instruction baling is the process of determining which LLVM instructions
 /// can be combined into a single vISA instruction. Such a group of LLVM
 /// instructions is known as a *bale*. A bale typically has a *main instruction*
@@ -20,127 +20,127 @@ SPDX-License-Identifier: MIT
 ///
 /// Bale example
 /// ^^^^^^^^^^^^
-/// 
+///
 /// .. image:: GenXDesign_bale.png
-/// 
+///
 /// This example shows a bale that is pretty much as complicated as you can get in
 /// a single bale. Each small blue box is an LLVM IR instruction, with arrows showing
 /// how each one is used. Other than the *bale head* instruction at the top, an
 /// instruction in a bale has only one use, which is within the bale.
-/// 
+///
 /// The baling pass
 /// ^^^^^^^^^^^^^^^
 ///
 /// GenX instruction baling happens in two parts:
-/// 
+///
 /// 1. The GenXBaling pass sets up a map to give each Instruction
 ///    a *BaleInfo*, which contains a field giving the role the instruction
 ///    plays in its enclosing bale (main instruction, rdregion, etc), and a
 ///    bit vector where a bit is set if the corresponding operand of the
 ///    instruction is another instruction that is baled in (part of the same
 ///    bale).
-/// 
+///
 ///    GenXBaling is in fact two slightly different passes run at two different
 ///    times:
-/// 
+///
 ///    * The GenXFuncBaling pass (a FunctionPass) runs before GenXLegalization,
 ///      which uses it but invalidates it as it changes the code. This is known
 ///      as *first baling*.
-/// 
+///
 ///    * The GenXGroupBaling pass (a FunctionGroupPass) runs after GenXLiveness.
 ///      From GenXLiveness, baling information remains valid through to
 ///      GenXCisaBuilder, since any code changes made (such as adding
 ///      copies where coalescing fails) either do not invalidate the analysis,
 ///      or the pass making the change also updates the baling analysis.
-/// 
+///
 ///    The GenXBaling pass also detects where an instruction is baled in to
 ///    another, but the instruction has other uses too. In this case it clones the
 ///    instruction. Thus we end up with any baled in instruction having only
 ///    one use (with an exception for goto/join -- see below).
-/// 
+///
 ///    Thus the GenXBaling pass is not a pure analysis, as it can modify the
 ///    code.
-/// 
+///
 /// 2. Using the map set up by the GenXBaling analysis, several functions are
 ///    provided for use by other passes:
-/// 
+///
 ///    * getBaleInfo()/setBaleInfo() allow another pass to directly inspect and modify
 ///      the baling info for an instruction. The BaleInfo for an instruction gives:
-/// 
+///
 ///      - Type, the role of the instruction in the bale (e.g. it is a rdregion);
 ///      - a bitmap of which operands are baled into it, together with methods
 ///        for getting and setting the bit for a particular operand.
-/// 
+///
 ///    * getBaleParent() returns the instruction that the given instruction is
 ///      baled into, if any
-/// 
+///
 ///    * isBaled() says whether the given instruction is baled into anything
-/// 
+///
 ///    * getBaleHead() returns the instruction at the head of the bale that the
 ///      given instruction is baled into, which is the same as the given instruction
 ///      if it is not baled into anything.
-/// 
+///
 ///    * buildBale() takes a head instruction (one for which isBaled is false) and
 ///      fills out a Bale struct with a vector of BaleInst structs for all the
 ///      instructions in the bale, where each BaleInst contains a pointer to the
 ///      instruction and its BaleInfo struct (as in getBaleInfo()/setBaleInfo()).
-/// 
+///
 /// Criteria for baling
 /// ^^^^^^^^^^^^^^^^^^^
-/// 
+///
 /// GenXBaling implements the criteria for baling, i.e. when different LLVM IR
 /// instructions can be combined into the same vISA instruction:
-/// 
+///
 /// * A rdregion with a variable index can bale in an add constant (where the
 ///   constant is splatted if vector) that generates the index. In second baling,
 ///   the constant add is in fact a ``llvm.genx.add.addr`` intrinsic, because that
 ///   is what GenXCategory converted it to.
-/// 
+///
 /// * GenXBaling is where an instruction gets recognized as a modifier, for example
 ///   subtract from 0 is a negate modifier. The instruction is left as it is, and
 ///   its modifier equivalent (e.g. ``BaleInfo::NEGMOD``) is set up in the
 ///   instruction's BaleInfo.
-/// 
+///
 /// * SExt/ZExt are also treated as modifiers, although not always balable. See
 ///   below.
-/// 
+///
 /// * A modifier can bale in an rdregion.
-/// 
+///
 /// * A modifier can bale in another modifier in some circumstances.
-/// 
+///
 /// * In particular, SExt/ZExt normally cannot bale in another modifier, but they
 ///   are allowed to bale in an abs modifier as a bodge to fix a problem where
 ///   the LLVM IR generated for ``cm_abs`` does not properly represent its
 ///   semantics. See ``dc93b907 GenXBaling: bodge to work around cm_abs problems``.
-/// 
+///
 /// * A main instruction can bale a modifier or rdregion into each operand in some
 ///   circumstances:
-/// 
+///
 ///   - Some ALU intrinsics have region requirements, e.g. oword aligned,
 ///     contiguous. GenXBaling enforces those requirements by only baling in an
 ///     rdregion that satifies them, but only in second baling. First baling does
 ///     the baling anyway, as we want GenXLegalization to consider the instructions
 ///     as one bale as it might legalize in a way that makes the region legal for
 ///     the instruction.
-/// 
+///
 ///   - Baling an SExt/ZExt in is how we represent a vISA instruction such as
 ///     ``add`` with a result type different to operand type. The two operands can
 ///     have different types too in Gen, but vISA insists they are the same (if not
 ///     constant). So:
-/// 
+///
 ///     1. In first baling, we allow SExt/ZExt from different types to be baled in
 ///        to the two operands. This tends to make GenXLegalization legalize them
 ///        to the same vector width as the main instruction.
-/// 
+///
 ///     2. In second baling, we do not allow SExt/ZExt from different types (or one
 ///        SExt/ZExt where the other operand does not have one) to be baled in. This
 ///        yields a legal vISA instruction, but having done (1) also allows the
 ///        finalizer to fold the extend into the instruction.
-/// 
+///
 ///   - A raw operand (of a send or shared function intrinsic) has its own
 ///     restrictions -- it can bale in a rdregion, but the region has to be
 ///     contiguous and GRF aligned.
-/// 
+///
 ///   - There is special case code for where send or a shared function intrinsic
 ///     has a ``TWOADDR`` raw operand, one that does not appear as a vISA operand
 ///     in its own right but is implicitly the same register as the result. The
@@ -149,57 +149,57 @@ SPDX-License-Identifier: MIT
 ///     region parameters and the same "old value" input. This represents where a
 ///     send or shared function intrinsic does a predicated partial write, and the
 ///     place it does the partial write to is a region in a vISA register.
-/// 
+///
 /// * ``llvm.genx.sat`` represents floating point saturation, and is a modifier that
 ///   is different to the other modifiers because it is not a source modifier. A
 ///   saturate can bale in a main instruction or modifier or rdregion.
-/// 
+///
 /// * A wrregion can do the following baling:
-///   
+///
 ///   - It can bale a main instruction (subject to region restrictions in second
 ///     baling), a saturate, a modifier or a rdregion into its "new value" input.
-/// 
+///
 ///   - Like rdregion, it can bale a constant add into its index operand.
-/// 
+///
 /// * Anything with a predicate (wrregion, select, send, all/any, some shared
 ///   function intrinsics) can bale in a predicate not, and any of those things,
 ///   including the not, can bale in an rdpredregion to represent using e.g. an M3
 ///   flag to use only part of the predicate. However predicate baling is not
 ///   done in first baling, as GenXLegalization does not want to consider the
 ///   operations together.
-/// 
+///
 /// * Anything with a scalar i1 condition (select, br) can bale in an all/any.
-/// 
+///
 /// Baling of goto/join into br
 /// ^^^^^^^^^^^^^^^^^^^^^^^^^^^
-/// 
+///
 /// The goto and join intrinsics have multiple return values, returned in a single
 /// struct. One of the return values is the scalar i1 !any value that is then used
 /// in a conditional branch.
-/// 
+///
 /// In second baling, we want the goto/join, the extractvalue of the !any
 /// result, and the conditional branch to be baled together, so we can generate
 /// a single goto/join instruction.
-/// 
+///
 /// However the struct result of the goto/join has other uses, the extractvalues of
 /// the other results. Thus, in this special case, we have a bale where the
 /// goto/join instruction inside the bale has uses other than the inside-bale use.
 /// This needs special case code to handle in GenXBaling.
-/// 
+///
 /// In the future it may be worth considering a generalization of this idea of a
 /// bale that is not a strict tree of instructions, so that we can use LLVM IR
 /// to model Gen instructions with a general result and a flag result. Currently
 /// we cannot do that, which means:
-/// 
+///
 /// 1. we cannot represent addc properly;
-/// 
+///
 /// 2. we cannot represent any combined arithmetic-and-set-flags instruction,
 ///    although that is not too much of a problem as the jitter derives such an
 ///    instruction by folding a cmp into an arithmetic instruction.
-/// 
+///
 /// Alignment requirements
 /// ^^^^^^^^^^^^^^^^^^^^^^
-/// 
+///
 /// An additional function of the second baling pass is that, when it bales a
 /// raw result intrinsic into a wrregion, it marks the wrregion's LiveRange as
 /// needing to be 32 aligned, and when it bales a rdregion into a raw operand in
@@ -215,8 +215,10 @@ SPDX-License-Identifier: MIT
 #include "FunctionGroup.h"
 #include "GenX.h"
 #include "GenXAlignmentInfo.h"
-#include "GenXRegion.h"
 #include "GenXSubtarget.h"
+
+#include "vc/Utils/GenX/Region.h"
+
 #include "IgnoreRAUWValueMap.h"
 #include "Probe/Assertion.h"
 #include "llvm/ADT/Hashing.h"
@@ -336,6 +338,9 @@ public:
   // hash : set hash code for bale. Must be called before using comparison
   // operators.
   void hash();
+  hash_code getHash() const {
+    return Hash;
+  }
   // Comparison operators. Two bales are equivalent if they compute the same
   // value, that is, they have the same opcodes in the instructions, the
   // instructions are baled together in the same way, and the operands coming
@@ -367,7 +372,7 @@ class GenXBaling {
   typedef llvm::ValueMap<const Value*, genx::BaleInfo,
                          IgnoreRAUWValueMapConfig<const Value *>>
       InstMap_t;
-  GenXSubtarget *ST;
+  const GenXSubtarget *ST;
   InstMap_t InstMap;
   struct NeedClone {
     Instruction *Inst;
@@ -383,14 +388,13 @@ class GenXBaling {
   SmallVector<CallInst *, 4> TwoAddrSends;
 protected:
   BalingKind Kind;
-  DominatorTree *DT;
+  const DominatorTree *DT;
   GenXLiveness *Liveness; // only in group baling
 public:
   genx::AlignmentInfo AlignInfo;
 public:
-  explicit GenXBaling(BalingKind BKind, GenXSubtarget *Subtarget)
-      : Kind(BKind), ST(Subtarget),
-        Liveness(nullptr) {}
+  explicit GenXBaling(BalingKind BKind, const GenXSubtarget *Subtarget)
+      : Kind(BKind), ST(Subtarget), Liveness(nullptr), DT(nullptr) {}
   // clear : clear out the analysis
   void clear() { InstMap.clear(); }
   // processFunction : process one Function
@@ -406,13 +410,13 @@ public:
   void setBaleInfo(const Instruction *Inst, genx::BaleInfo BI);
   // isBaled : test whether all uses of an instruction would be baled in to
   // users
-  bool isBaled(Instruction *Inst) { return getBaleParent(Inst); }
+  bool isBaled(const Instruction *Inst) const { return getBaleParent(Inst); }
   // getBaleParent : return the instruction baled into, 0 if none
-  Instruction *getBaleParent(Instruction *Inst);
+  Instruction *getBaleParent(const Instruction *Inst) const;
   // unbale : unbale an instruction from its bale parent
   void unbale(Instruction *Inst);
   // getBaleHead : return the head of the bale containing this instruction
-  Instruction *getBaleHead(Instruction *Inst);
+  Instruction *getBaleHead(Instruction *Inst) const;
   // buildBale : build Bale from head instruction. B assumed empty on entry
   void buildBale(Instruction *Inst, genx::Bale *B, bool IncludeAddr = false) const;
   // store : store updated BaleInfo for Instruction (used to unbale by
@@ -434,7 +438,7 @@ public:
   static bool isBalableIndexOr(Value *V);
   // isBalableNewValueIntoWrr: check whether the new val operand can
   // be baled into wrr instruction
-  bool isBalableNewValueIntoWrr(Value *V, const genx::Region &WrrR);
+  bool isBalableNewValueIntoWrr(Value *V, const Region &WrrR);
 
   static bool isHighCostBaling(uint16_t Type, Instruction *Inst);
   // Debug dump/print
@@ -452,13 +456,14 @@ private:
   bool processPredicate(Instruction *Inst, unsigned OperandNum);
   void processSat(Instruction *Inst);
   void processRdRegion(Instruction *Inst);
+  void processRdPredRegion(Instruction *Inst);
   void processInlineAsm(Instruction *Inst);
   void processExtractValue(ExtractValueInst *EV);
   void processFuncPointer(Instruction *Inst);
   void processRdWrPredefReg(Instruction *Inst);
   void processMainInst(Instruction *Inst, int IntrinID);
   bool processSelectToPredicate(SelectInst *SI);
-  // helper func for buildBale
+  // Helper func for buildBale
   void buildBaleSub(Instruction *Inst, genx::Bale *B, bool IncludeAddr) const;
   void processBranch(BranchInst *Branch);
   void processTwoAddrSend(CallInst *CI);
@@ -471,8 +476,9 @@ private:
   bool operandCanBeBaled(Instruction *Inst, unsigned OperandNum, int ModType,
                          unsigned ArgInfoBits);
 
-  bool  isRegionOKForIntrinsic(unsigned ArgInfoBits, const genx::Region &R,
+  bool  isRegionOKForIntrinsic(unsigned ArgInfoBits, const Region &R,
                          bool CanSplitBale);
+  bool isSafeToMove(Instruction *Op, Instruction *From, Instruction *To);
 
   // Cleanup and optimization before do baling on a function.
   bool prologue(Function *F);
@@ -484,7 +490,8 @@ private:
 class GenXFuncBaling : public FunctionPass, public GenXBaling {
 public:
   static char ID;
-  explicit GenXFuncBaling(BalingKind Kind = BalingKind::BK_Legalization, GenXSubtarget *ST = nullptr)
+  explicit GenXFuncBaling(BalingKind Kind = BalingKind::BK_Legalization,
+                          const GenXSubtarget *ST = nullptr)
       : FunctionPass(ID), GenXBaling(Kind, ST) {}
   StringRef getPassName() const override {
     return "GenX instruction baling analysis for a function";
@@ -510,27 +517,29 @@ void initializeGenXFuncBalingPass(PassRegistry &);
 //----------------------------------------------------------------------
 // The GenXGroupBaling analysis pass
 // (used for the second baling just before GenXLiveRanges)
-class GenXGroupBaling : public FunctionGroupPass, public GenXBaling {
+class GenXGroupBaling : public FGPassImplInterface,
+                        public IDMixin<GenXGroupBaling>,
+                        public GenXBaling {
 public:
-  static char ID;
-  explicit GenXGroupBaling(BalingKind Kind = BalingKind::BK_Legalization, GenXSubtarget *ST = nullptr)
-      : FunctionGroupPass(ID), GenXBaling(Kind, ST) {}
-  virtual StringRef getPassName() const {
+  explicit GenXGroupBaling(BalingKind Kind = BalingKind::BK_Legalization,
+                           const GenXSubtarget *ST = nullptr)
+      : GenXBaling(Kind, ST) {}
+  static StringRef getPassName() {
     return "GenX instruction baling analysis for a function group";
   }
-  void getAnalysisUsage(AnalysisUsage &AU) const override;
-  bool runOnFunctionGroup(FunctionGroup &FG);
-  // createPrinterPass : get a pass to print the IR, together with the GenX
-  // specific analyses
-  virtual Pass *createPrinterPass(raw_ostream &O,
-                                  const std::string &Banner) const {
-    return createGenXGroupPrinterPass(O, Banner);
-  }
+  void releaseMemory() override { clear(); }
+  static void getAnalysisUsage(AnalysisUsage &AU);
+  bool runOnFunctionGroup(FunctionGroup &FG) override;
   // processFunctionGroup : process all the Functions in a FunctionGroup
   bool processFunctionGroup(FunctionGroup *FG);
 };
-void initializeGenXGroupBalingPass(PassRegistry &);
+void initializeGenXGroupBalingWrapperPass(PassRegistry &);
+using GenXGroupBalingWrapper = FunctionGroupWrapperPass<GenXGroupBaling>;
 
 } // end namespace llvm
 
+template <> struct std::hash<llvm::genx::Bale> {
+  std::size_t operator()(const llvm::genx::Bale &B) const { return B.getHash(); }
+};
+
 #endif // GENXBALING_H
