@@ -24,6 +24,9 @@ SPDX-License-Identifier: MIT
 #include "llvm/IR/Constants.h"
 #include "llvm/IR/Instructions.h"
 #include "llvm/IR/Intrinsics.h"
+
+#include "llvmWrapper/IR/DerivedTypes.h"
+
 #include "Probe/Assertion.h"
 
 using namespace llvm;
@@ -46,28 +49,10 @@ using namespace llvm;
 // of near-identical code in the intrinsics look-up table and also to
 // aid readability.
 
-const GenXIntrinsicInfo::DescrElementType GenXIntrinsicInfo::Table[] = {
-
 // Region access intrinsics do not appear in this table
-
+const GenXIntrinsicInfo::TableType GenXIntrinsicInfo::Table = {
 #include "GenXIntrinsicInfoTable.inc"
-
-    END};
-
-GenXIntrinsicInfo::GenXIntrinsicInfo(unsigned IntrinId) : Args(0) {
-  const auto *p = Table;
-  for (;;) {
-    if (*p == END)
-      break; // intrinsic not found; leave Args pointing at END field
-    if (IntrinId == *p++)
-      break;
-    // Scan past the rest of this entry.
-    while (*p++ != END)
-      ;
-  }
-  // We have found the right entry.
-  Args = p;
-}
+};
 
 // Get the category and modifier for an arg idx (-1 means return value).
 // The returned ArgInfo struct contains just the short read from the table,
@@ -75,13 +60,11 @@ GenXIntrinsicInfo::GenXIntrinsicInfo(unsigned IntrinId) : Args(0) {
 GenXIntrinsicInfo::ArgInfo GenXIntrinsicInfo::getArgInfo(int Idx) {
   // Read through the fields in the table to find the one with the right
   // arg index...
-  for (const auto *p = Args; *p; p++) {
-    ArgInfo AI(*p);
+  for (auto AI : getInstDesc())
     if (AI.isRealArgOrRet() && AI.getArgIdx() == Idx)
       return AI;
-  }
   // Field with requested arg index was not found.
-  return END;
+  return ArgInfo{};
 }
 
 // Return the starting point of any trailing null (zero) arguments
@@ -89,33 +72,20 @@ GenXIntrinsicInfo::ArgInfo GenXIntrinsicInfo::getArgInfo(int Idx) {
 // this will always return the number of operands to the call (ie, there
 // is no trailing null zone), even if there are some trailing nulls.
 unsigned GenXIntrinsicInfo::getTrailingNullZoneStart(CallInst *CI) {
-  unsigned TrailingNullStart = CI->getNumArgOperands();
-
-  const auto *p = Args;
-  for (; *p; p++) {
-    ArgInfo AI(*p);
-    if (AI.getCategory() == ARGCOUNT)
-      break;
+  auto AI =
+      std::find_if(getInstDesc().begin(), getInstDesc().end(),
+                   [](auto Arg) { return Arg.getCategory() == ARGCOUNT; });
+  if (AI == getInstDesc().end())
+    return CI->getNumOperands();
+  unsigned BaseArg = AI->getArgIdx();
+  unsigned TrailingNullStart = BaseArg;
+  for (unsigned Idx = BaseArg; Idx < CI->getNumArgOperands(); ++Idx) {
+    if (auto CA = dyn_cast<Constant>(CI->getArgOperand(Idx)))
+      if (CA->isNullValue())
+        continue;
+    TrailingNullStart = Idx + 1;
   }
-
-  if (*p) {
-    ArgInfo ACI(*p);
-    unsigned BaseArg = ACI.getArgIdx();
-
-    TrailingNullStart = BaseArg;
-    for (unsigned Idx = BaseArg; Idx < CI->getNumArgOperands(); ++Idx) {
-      if (auto CA = dyn_cast<Constant>(CI->getArgOperand(Idx))) {
-        if (CA->isNullValue())
-          continue;
-      }
-      TrailingNullStart = Idx + 1;
-    }
-
-    if (TrailingNullStart < BaseArg + ACI.getArgCountMin())
-      TrailingNullStart = BaseArg + ACI.getArgCountMin();
-  }
-
-  return TrailingNullStart;
+  return std::max(TrailingNullStart, BaseArg + AI->getArgCountMin());
 }
 
 /***********************************************************************
@@ -125,9 +95,9 @@ unsigned GenXIntrinsicInfo::getTrailingNullZoneStart(CallInst *CI) {
  * Return:  bit N set if execution size 1<<N is allowed
  */
 unsigned GenXIntrinsicInfo::getExecSizeAllowedBits() {
-  for (const auto *p = Args; *p; p++) {
-    if (!(*p & GENERAL)) {
-      switch (*p & CATMASK) {
+  for (auto AI : getInstDesc()) {
+    if (!AI.isGeneral()) {
+      switch (AI.getCategory()) {
       case EXECSIZE:
         return 0x3f;
       case EXECSIZE_GE2:
@@ -154,8 +124,7 @@ bool GenXIntrinsicInfo::getPredAllowed() {
   // Simply search the intrinsic description for an IMPLICITPRED
   // entry. Not very efficient, but the situations where this
   // check is needed are expected to be infrequent.
-  for (const auto *p = getInstDesc(); *p; ++p) {
-    ArgInfo AI(*p);
+  for (auto AI : getInstDesc()) {
     if (AI.getCategory() == IMPLICITPRED)
       return true;
   }
@@ -168,6 +137,19 @@ unsigned GenXIntrinsicInfo::getOverridedExecSize(CallInst *CI,
   auto CalledF = CI->getCalledFunction();
   IGC_ASSERT(CalledF);
   auto ID = GenXIntrinsic::getGenXIntrinsicID(CalledF);
+  if (isGenXIntrinsic(ID)) {
+    // We check the lsc intrinsic by name or we need to introduce a range of lsc
+    // intrinsics.
+    if (ID == GenXIntrinsic::genx_lsc_load2d_stateless ||
+        ID == GenXIntrinsic::genx_lsc_store2d_stateless)
+      return 1;
+
+    StringRef Name = CalledF->getName();
+    StringRef LSCPrefix = "llvm.genx.lsc.";
+    if (Name.startswith(LSCPrefix))
+      // Check if this is a block message from predicate type.
+      return !CI->getArgOperand(0)->getType()->isVectorTy();
+  }
 
   switch (ID) {
   default:
@@ -176,7 +158,7 @@ unsigned GenXIntrinsicInfo::getOverridedExecSize(CallInst *CI,
   case GenXIntrinsic::genx_gather4_scaled2:
   case GenXIntrinsic::genx_gather4_masked_scaled2:
   case GenXIntrinsic::genx_gather4_masked_scaled2_predef_surface:
-    return cast<VectorType>(
+    return cast<IGCLLVM::FixedVectorType>(
                CI->getArgOperand(4 /*address operand idx*/)->getType())
         ->getNumElements();
   case GenXIntrinsic::genx_raw_send:
@@ -186,7 +168,8 @@ unsigned GenXIntrinsicInfo::getOverridedExecSize(CallInst *CI,
     return 16;
   case GenXIntrinsic::genx_subb:
   case GenXIntrinsic::genx_addc:
-    if (auto *VT = dyn_cast<VectorType>(CI->getOperand(0)->getType()))
+    if (auto *VT =
+            dyn_cast<IGCLLVM::FixedVectorType>(CI->getOperand(0)->getType()))
       return VT->getNumElements();
     return 1;
   case GenXIntrinsic::genx_dpas:
