@@ -21,6 +21,7 @@ SPDX-License-Identifier: MIT
 #include "GenX.h"
 #include "GenXVisa.h"
 #include "Probe/Assertion.h"
+#include <unordered_map>
 
 #define GENX_ITR_CATVAL(v) ((v) << CATBASE)
 #define GENX_ITR_FLAGENUM(o, v) ((v) << ((o) + FLAGBASE))
@@ -28,14 +29,9 @@ SPDX-License-Identifier: MIT
 #define GENX_ITR_FLAGVAL(o) GENX_ITR_FLAGENUM(o, 1)
 
 namespace llvm {
-  class CallInst;
+class CallInst;
 
 class GenXIntrinsicInfo {
-public:
-  typedef uint32_t DescrElementType;
-private:
-  const DescrElementType *Args;
-  static const DescrElementType Table[];
 public:
   enum {
     // General format of intrinsic descriptor words:
@@ -71,7 +67,6 @@ public:
     OPNDMASK = ~(CATMASK | FLAGMASK),
 
     // A field that does not contain an operand number or literal value:
-    END =                   0, // end of instruction description
     IMPLICITPRED =          GENX_ITR_CATVAL(0x01), // implicit predication field
     NULLRAW =               GENX_ITR_CATVAL(0x02), // null raw operand
     ISBARRIER =             GENX_ITR_CATVAL(0x03), // intrinsic is barrier: suppress nobarrier attribute
@@ -116,31 +111,32 @@ public:
     Z_PREDICATE = PREDICATE | PREDICATE_ZEROED,
     SAMPLER =               GENX_ITR_CATVAL(0x25), // sampler operand
     SURFACE =               GENX_ITR_CATVAL(0x26), // surface operand
-    VME =                   GENX_ITR_CATVAL(0x27), // vme operand
     // byte height of media 2D block, inferred from the width operand
     // pointed at and the size of the return type or final operand type
-    MEDIAHEIGHT =           GENX_ITR_CATVAL(0x28),
+    MEDIAHEIGHT =           GENX_ITR_CATVAL(0x27),
     // predication control field from explicit predicate arg
-    PREDICATION =           GENX_ITR_CATVAL(0x29),
+    PREDICATION =           GENX_ITR_CATVAL(0x28),
     // chmask field in load/sample, with exec size bit
-    SAMPLECHMASK =          GENX_ITR_CATVAL(0x2a),
+    SAMPLECHMASK =          GENX_ITR_CATVAL(0x29),
     // does not appear in the vISA output, but needs to be two address
     // coalesced with result
-    TWOADDR =               GENX_ITR_CATVAL(0x2b),
-    CONSTVI1ASI32 =         GENX_ITR_CATVAL(0x2c), // constant vXi1 written as i32 (used in setp)
-    RAW =                   GENX_ITR_CATVAL(0x2d), // raw operand or result,
+    TWOADDR =               GENX_ITR_CATVAL(0x2a),
+    CONSTVI1ASI32 =         GENX_ITR_CATVAL(0x2b), // constant vXi1 written as i32 (used in setp)
+    RAW =                   GENX_ITR_CATVAL(0x2c), // raw operand or result,
       // Raw descriptor flags, 3 bits used
       RAW_UNSIGNED =        GENX_ITR_FLAGVAL(0),   // raw operand/result must be unsigned
       RAW_SIGNED =          GENX_ITR_FLAGVAL(1),   // raw operand/result must be signed
       RAW_NULLALLOWED =     GENX_ITR_FLAGVAL(2),   // raw operand or result can be null (V0)
     URAW =                  RAW | RAW_UNSIGNED,
     SRAW =                  RAW | RAW_SIGNED,
-    EXECSIZE_NOMASK =       GENX_ITR_CATVAL(0x2e), // execution size with NoMask
+    EXECSIZE_NOMASK =       GENX_ITR_CATVAL(0x2d), // execution size with NoMask
 
     // A general operand
-    GENERAL =               GENX_ITR_CATVAL(0x30),
+    GENERAL =               GENX_ITR_CATVAL(0x2e),
     // A general operand with compile-time signedness choosing
     GENERAL_CTSIGN =        GENERAL,
+    // A predefined surface operand
+    PREDEF_SURFACE =        GENX_ITR_CATVAL(0x2f),
     // Modifiers for destination or source, 7 bits used
     UNSIGNED =              GENX_ITR_FLAGVAL(0), // int type forced to unsigned
     SIGNED =                GENX_ITR_FLAGVAL(1), // int type forced to signed
@@ -170,13 +166,11 @@ public:
     MODIFIER_LOGIC =        GENX_ITR_FLAGENUM(8, 2), // src modifier: logic
     MODIFIER_EXTONLY =      GENX_ITR_FLAGENUM(8, 3), // src modifier: extend only
     DIRECTONLY =            GENX_ITR_FLAGVAL(10), // indirect region not allowed
-    PREDEF_SURFACE =        GENX_ITR_CATVAL(0x31), // predefined surface operand
   };
   struct ArgInfo {
     unsigned Info;
-    // Default constructor, used in GenXBaling to construct an ArgInfo that
-    // represents an arg of a non-call instruction.
-    ArgInfo() : Info(GENERAL) {}
+    // Construct argument with empty info.
+    ArgInfo() : Info(0) {}
     // Construct from a field read from the intrinsics info table.
     ArgInfo(unsigned Info) : Info(Info) {}
     // getCategory : return field category
@@ -267,7 +261,6 @@ public:
         case PREDICATION:
         case SURFACE:
         case SAMPLER:
-        case VME:
           return true;
         default: break;
       }
@@ -281,26 +274,17 @@ public:
       return Info & MODIFIER;
     }
   };
-  // GenXIntrinsics::iterator : iterate through the fields
-  class iterator {
-    const DescrElementType *p;
-  public:
-    iterator(const DescrElementType *p) : p(p) {}
-    iterator &operator++() { ++p; if (*p == END) p = 0; return *this; }
-    ArgInfo operator*() { return ArgInfo(*p); }
-    bool operator!=(iterator i) { return p != i.p; }
-  };
-  iterator begin() {
-    IGC_ASSERT_MESSAGE(isNotNull(), "iterating an intrinsic without info");
-    return iterator(Args);
-  }
-  iterator end() { return iterator(0); }
+
+  using DescrType = llvm::SmallVector<ArgInfo, 8>;
+
   // Construct a GenXIntrinsicInfo for a particular intrinsic
-  GenXIntrinsicInfo(unsigned IntrinId);
-  bool isNull() const { return *getInstDesc() == GenXIntrinsicInfo::END; }
+  GenXIntrinsicInfo(unsigned IntrinId) { InfoIt = Table.find(IntrinId); }
+  bool isNull() const { return InfoIt == Table.cend(); }
   bool isNotNull() const { return !isNull(); }
   // Return instruction description.
-  const DescrElementType *getInstDesc() const { return Args; }
+  ArrayRef<ArgInfo> getInstDesc() const {
+    return isNull() ? ArrayRef<ArgInfo>{} : ArrayRef<ArgInfo>{InfoIt->second};
+  }
   // Get the category and modifier for an arg idx
   ArgInfo getArgInfo(int Idx);
   // Get the trailing null zone, if any.
@@ -314,6 +298,11 @@ public:
   // Get The overrided execution size or 0.
   static unsigned getOverridedExecSize(CallInst *CI,
                                        const GenXSubtarget *ST = nullptr);
+
+private:
+  using TableType = std::unordered_map<unsigned, DescrType>;
+  static const TableType Table;
+  TableType::const_iterator InfoIt;
 };
 
 } // namespace llvm
