@@ -29,6 +29,9 @@ SPDX-License-Identifier: MIT
 #include "GenXIntrinsics.h"
 #include "GenXLiveness.h"
 #include "GenXNumbering.h"
+
+#include "vc/Support/BackendConfig.h"
+
 #include "llvm/IR/BasicBlock.h"
 #include "llvm/IR/Function.h"
 #include "llvm/IR/Instructions.h"
@@ -42,20 +45,17 @@ using namespace genx;
 
 namespace {
 
-class GenXLiveRanges : public FunctionGroupPass {
+class GenXLiveRanges : public FGPassImplInterface,
+                       public IDMixin<GenXLiveRanges> {
   FunctionGroup *FG = nullptr;
   GenXBaling *Baling = nullptr;
   GenXLiveness *Liveness = nullptr;
+
 public:
-  static char ID;
-  explicit GenXLiveRanges() : FunctionGroupPass(ID) { }
-  virtual StringRef getPassName() const { return "GenX live ranges analysis"; }
-  void getAnalysisUsage(AnalysisUsage &AU) const;
-  bool runOnFunctionGroup(FunctionGroup &FG);
-  // createPrinterPass : get a pass to print the IR, together with the GenX
-  // specific analyses
-  virtual Pass *createPrinterPass(raw_ostream &O, const std::string &Banner) const
-  { return createGenXGroupPrinterPass(O, Banner); }
+  explicit GenXLiveRanges() {}
+  static StringRef getPassName() { return "GenX live ranges analysis"; }
+  static void getAnalysisUsage(AnalysisUsage &AU);
+  bool runOnFunctionGroup(FunctionGroup &FG) override;
 
 private:
   void buildLiveRanges();
@@ -71,44 +71,56 @@ private:
  * the LiveRange for a value that does not need it because it is a baled
  * in instruction.
  */
-bool testDuplicates(const llvm::GenXLiveness& Liveness) {
+bool testDuplicates(/*const*/ llvm::GenXLiveness& Liveness) {
+  if (std::any_of(Liveness.begin(), Liveness.end(),
+                  [](const auto &X) { return X.second == nullptr; })) {
+    llvm::errs() << "Liveness contains NULL live ranges\n";
+    return false;
+  }
 
-  auto testRange = [](const std::pair<llvm::genx::SimpleValue, llvm::genx::LiveRange *>& X) {
-    const llvm::genx::LiveRange *const LR = X.second;
-    IGC_ASSERT(nullptr != LR);
-    const bool HasSegment = (0 < LR->size());
-    IGC_ASSERT(HasSegment);
-    return HasSegment;
+  auto testRange = [](const auto &X) {
+    auto HasSegment = (X.second->size() > 0);
+    auto UndefVal = isa<UndefValue>(X.first.getValue());
+    return !(HasSegment || UndefVal);
   };
 
-  const bool Result = std::all_of(Liveness.begin(), Liveness.end(), testRange);
-  IGC_ASSERT(Result);
-  return Result;
+  auto WrongLR = std::find_if(Liveness.begin(), Liveness.end(), testRange);
+  if (WrongLR != Liveness.end()) {
+    llvm::errs() << "Bad live range\n";
+    WrongLR->second->print(llvm::errs(), true);
+    llvm::errs() << "\nWhole liveness:\n";
+    Liveness.dump();
+    llvm::errs() << "\n";
+    return false;
+  }
+  return true;
 }
 
 } // end anonymous namespace
 
-namespace llvm { void initializeGenXLiveRangesPass(PassRegistry &); }
-char GenXLiveRanges::ID = 0;
-INITIALIZE_PASS_BEGIN(GenXLiveRanges, "GenXLiveRanges", "GenXLiveRanges", false, false)
-INITIALIZE_PASS_DEPENDENCY(GenXGroupBaling)
-INITIALIZE_PASS_DEPENDENCY(GenXLiveness)
-INITIALIZE_PASS_DEPENDENCY(GenXNumbering)
+namespace llvm {
+void initializeGenXLiveRangesWrapperPass(PassRegistry &);
+using GenXLiveRangesWrapper = FunctionGroupWrapperPass<GenXLiveRanges>;
+} // namespace llvm
+INITIALIZE_PASS_BEGIN(GenXLiveRangesWrapper, "GenXLiveRangesWrapper",
+                      "GenXLiveRangesWrapper", false, false)
+INITIALIZE_PASS_DEPENDENCY(GenXGroupBalingWrapper)
+INITIALIZE_PASS_DEPENDENCY(GenXLivenessWrapper)
+INITIALIZE_PASS_DEPENDENCY(GenXNumberingWrapper)
 INITIALIZE_PASS_DEPENDENCY(FunctionGroupAnalysis)
-INITIALIZE_PASS_END(GenXLiveRanges, "GenXLiveRanges", "GenXLiveRanges", false, false)
+INITIALIZE_PASS_END(GenXLiveRangesWrapper, "GenXLiveRangesWrapper",
+                    "GenXLiveRangesWrapper", false, false)
 
-FunctionGroupPass *llvm::createGenXLiveRangesPass()
-{
-  initializeGenXLiveRangesPass(*PassRegistry::getPassRegistry());
-  return new GenXLiveRanges();
+ModulePass *llvm::createGenXLiveRangesWrapperPass() {
+  initializeGenXLiveRangesWrapperPass(*PassRegistry::getPassRegistry());
+  return new GenXLiveRangesWrapper();
 }
 
-void GenXLiveRanges::getAnalysisUsage(AnalysisUsage &AU) const
-{
-  FunctionGroupPass::getAnalysisUsage(AU);
+void GenXLiveRanges::getAnalysisUsage(AnalysisUsage &AU) {
   AU.addRequired<GenXGroupBaling>();
   AU.addRequired<GenXLiveness>();
   AU.addRequired<GenXNumbering>();
+  AU.addRequired<GenXBackendConfig>();
   AU.addRequired<FunctionGroupAnalysis>();
   AU.setPreservesAll();
 }
@@ -119,8 +131,10 @@ void GenXLiveRanges::getAnalysisUsage(AnalysisUsage &AU) const
 bool GenXLiveRanges::runOnFunctionGroup(FunctionGroup &ArgFG)
 {
   FG = &ArgFG;
+  const auto &BC = getAnalysis<GenXBackendConfig>();
   Baling = &getAnalysis<GenXGroupBaling>();
   Liveness = &getAnalysis<GenXLiveness>();
+  Liveness->setNoCoalescingMode(BC.disableLiveRangesCoalescing());
   Liveness->setBaling(Baling);
   Liveness->setNumbering(&getAnalysis<GenXNumbering>());
   // Build the live ranges.
@@ -159,7 +173,7 @@ void GenXLiveRanges::buildLiveRanges()
     // Build live ranges for args.
     for (auto fi = Func->arg_begin(), fe = Func->arg_end(); fi != fe; ++fi)
       Liveness->buildLiveRange(&*fi);
-    if (i != FG->begin() && !Func->getReturnType()->isVoidTy()) {
+    if (!Func->getReturnType()->isVoidTy()) {
       // Build live range(s) for unified return value.
       Liveness->buildLiveRange(Liveness->getUnifiedRet(Func));
     }
@@ -179,7 +193,7 @@ void GenXLiveRanges::buildLiveRanges()
           // result is marked RAW_NULLALLOWED. If so, don't create a live range,
           // so no register gets allocated.
           if (Inst->use_empty()) {
-            unsigned IID = GenXIntrinsic::getAnyIntrinsicID(Inst);
+            unsigned IID = vc::getAnyIntrinsicID(Inst);
             switch (IID) {
               case GenXIntrinsic::not_any_intrinsic:
               case GenXIntrinsic::genx_rdregioni:
