@@ -31,17 +31,15 @@ SPDX-License-Identifier: MIT
 #include "GenX.h"
 #include "GenXBaling.h"
 #include "GenXConstants.h"
-#include "GenXRegion.h"
 #include "GenXSubtarget.h"
 #include "GenXTargetMachine.h"
 #include "GenXUtil.h"
 #include "GenXVectorDecomposer.h"
-#include "vc/Utils/General/BreakConst.h"
+#include "vc/Utils/GenX/BreakConst.h"
 
 #include "llvm/Analysis/TargetLibraryInfo.h"
 #include "llvm/CodeGen/TargetPassConfig.h"
 #include "llvm/IR/Constants.h"
-#include "llvm/IR/Dominators.h"
 #include "llvm/IR/Function.h"
 #include "llvm/IR/Instructions.h"
 #include "llvm/IR/Intrinsics.h"
@@ -59,16 +57,17 @@ namespace {
 
 // GenXPostLegalization : post-legalization pass
 class GenXPostLegalization : public FunctionPass {
-  DominatorTree *DT = nullptr;
   VectorDecomposer VD;
   const DataLayout *DL = nullptr;
   const GenXSubtarget *ST = nullptr;
 public:
   static char ID;
   explicit GenXPostLegalization() : FunctionPass(ID) { }
-  virtual StringRef getPassName() const { return "GenX post-legalization pass"; }
-  void getAnalysisUsage(AnalysisUsage &AU) const;
-  bool runOnFunction(Function &F);
+  StringRef getPassName() const override {
+    return "GenX post-legalization pass";
+  }
+  void getAnalysisUsage(AnalysisUsage &AU) const override;
+  bool runOnFunction(Function &F) override;
 };
 
 } // end namespace llvm
@@ -76,19 +75,17 @@ public:
 
 char GenXPostLegalization::ID = 0;
 namespace llvm { void initializeGenXPostLegalizationPass(PassRegistry &); }
-INITIALIZE_PASS_BEGIN(GenXPostLegalization, "GenXPostLegalization", "GenXPostLegalization", false, false)
-INITIALIZE_PASS_DEPENDENCY(DominatorTreeWrapperPass)
-INITIALIZE_PASS_END(GenXPostLegalization, "GenXPostLegalization", "GenXPostLegalization", false, false)
+INITIALIZE_PASS_BEGIN(GenXPostLegalization, "GenXPostLegalization",
+                      "GenXPostLegalization", false, false)
+INITIALIZE_PASS_END(GenXPostLegalization, "GenXPostLegalization",
+                    "GenXPostLegalization", false, false)
 
-FunctionPass *llvm::createGenXPostLegalizationPass()
-{
+FunctionPass *llvm::createGenXPostLegalizationPass() {
   initializeGenXPostLegalizationPass(*PassRegistry::getPassRegistry());
   return new GenXPostLegalization;
 }
 
-void GenXPostLegalization::getAnalysisUsage(AnalysisUsage &AU) const
-{
-  AU.addRequired<DominatorTreeWrapperPass>();
+void GenXPostLegalization::getAnalysisUsage(AnalysisUsage &AU) const {
   AU.addRequired<TargetPassConfig>();
   AU.setPreservesCFG();
 }
@@ -102,16 +99,15 @@ bool GenXPostLegalization::runOnFunction(Function &F)
   ST = &getAnalysis<TargetPassConfig>()
             .getTM<GenXTargetMachine>()
             .getGenXSubtarget();
-  DT = &getAnalysis<DominatorTreeWrapperPass>().getDomTree();
 
   bool Modified = false;
-  Modified |= vc::breakConstantExprs(&F);
+  Modified |= vc::breakConstantExprs(&F, vc::LegalizationStage::Legalized);
 
   for (Function::iterator fi = F.begin(), fe = F.end(); fi != fe; ++fi) {
     BasicBlock *BB = &*fi;
     for (BasicBlock::iterator bi = BB->begin(), be = BB->end(); bi != be; ++bi) {
       Instruction *Inst = &*bi;
-      switch (GenXIntrinsic::getAnyIntrinsicID(Inst)) {
+      switch (vc::getAnyIntrinsicID(Inst)) {
       default:
         // Lower non-simple constant operands.
         Modified |= loadNonSimpleConstants(Inst, *ST, *DL, nullptr);
@@ -135,11 +131,13 @@ bool GenXPostLegalization::runOnFunction(Function &F)
     }
   }
   // Run the vector decomposer for this function.
-  Modified |= VD.run(DT);
+  Modified |= VD.run(*DL);
   // Cleanup region reads and writes.
-  Modified |= simplifyRegionInsts(&F, DL);
+  Modified |= simplifyRegionInsts(&F, DL, ST);
   // Cleanup redundant global loads.
   Modified |= cleanupLoads(&F);
+  // Cleanup constant loads.
+  Modified |= cleanupConstantLoads(&F);
   // Legalize constants in return.
   for (auto FI = F.begin(), FE = F.end(); FI != FE; ++FI) {
     BasicBlock *BB = &*FI;
