@@ -19,7 +19,6 @@ SPDX-License-Identifier: MIT
 
 #include "GenX.h"
 #include "GenXModule.h"
-#include "GenXRegion.h"
 #include "GenXUtil.h"
 #include "GenXVisa.h"
 
@@ -52,12 +51,12 @@ using namespace genx;
 static cl::opt<std::size_t> SingleAllocaLimitOpt(
     "vc-promote-array-single-alloca-limit",
     cl::desc("max size of a sindle promoted alloca in bytes"),
-    cl::init(96 * defaultGRFWidth), cl::Hidden);
+    cl::init(96 * defaultGRFByteSize), cl::Hidden);
 
 static cl::opt<std::size_t>
     TotalAllocaLimitOpt("vc-promote-array-total-alloca-limit",
                         cl::desc("max total size of promoted allocas in bytes"),
-                        cl::init(256 * defaultGRFWidth), cl::Hidden);
+                        cl::init(256 * defaultGRFByteSize), cl::Hidden);
 
 namespace {
 
@@ -81,18 +80,24 @@ class DiagnosticInfoPromoteArray : public DiagnosticInfo {
 private:
   std::string Description;
 
+  static const int KindID;
+
+  static int getKindID() { return KindID; }
+
 public:
   // Initialize from description
   DiagnosticInfoPromoteArray(const Twine &Desc,
                              DiagnosticSeverity Severity = DS_Error)
-      : DiagnosticInfo(llvm::getNextAvailablePluginDiagnosticKind(), Severity),
-        Description(Desc.str()) {}
+      : DiagnosticInfo(getKindID(), Severity), Description(Desc.str()) {}
 
   void print(DiagnosticPrinter &DP) const override {
     DP << "GenXPromoteArray: " << Description;
   }
 };
 
+const int DiagnosticInfoPromoteArray::KindID =
+    llvm::getNextAvailablePluginDiagnosticKind();
+
 class TransposeHelper {
 public:
   TransposeHelper(bool vectorIndex, const llvm::DataLayout *DL)
@@ -142,16 +147,14 @@ public:
 
   ~TransformPrivMem() {}
 
-  virtual llvm::StringRef getPassName() const override {
-    return "TransformPrivMem";
-  }
+  llvm::StringRef getPassName() const override { return "TransformPrivMem"; }
 
   void getAnalysisUsage(AnalysisUsage &AU) const override {
     AU.addRequired<GenXBackendConfig>();
     AU.setPreservesCFG();
   }
 
-  virtual bool runOnFunction(llvm::Function &F) override;
+  bool runOnFunction(llvm::Function &F) override;
 
   void visitAllocaInst(llvm::AllocaInst &I);
 
@@ -421,7 +424,7 @@ static bool CheckPtrToIntCandidate(PtrToIntInst *PTI) {
   for (auto *MemOp : BinOp->users()) {
     if (!isa<CallInst>(MemOp))
       return false;
-    auto IID = GenXIntrinsic::getAnyIntrinsicID(MemOp);
+    auto IID = vc::getAnyIntrinsicID(MemOp);
     if (IID != GenXIntrinsic::genx_svm_gather &&
         IID != GenXIntrinsic::genx_svm_scatter)
       return false;
@@ -445,12 +448,12 @@ static bool CheckPtrToIntCandidate(PtrToIntInst *PTI) {
 
     IGC_ASSERT(isa<ConstantInt>(NumBlocks));
     if (cast<ConstantInt>(NumBlocks)->getZExtValue() ||
-        cast<VectorType>(Input->getType())->getNumElements() >
-            cast<VectorType>(Pred->getType())
-                ->getNumElements() ||
+        cast<IGCLLVM::FixedVectorType>(Input->getType())->getNumElements() >
+            cast<IGCLLVM::FixedVectorType>(Pred->getType())->getNumElements() ||
         (isa<VectorType>(MemOp->getType()) &&
-         cast<VectorType>(Pred->getType())->getNumElements() <
-             cast<VectorType>(MemOp->getType())->getNumElements()))
+         cast<IGCLLVM::FixedVectorType>(Pred->getType())->getNumElements() <
+             cast<IGCLLVM::FixedVectorType>(MemOp->getType())
+                 ->getNumElements()))
       return false;
   }
   return true;
@@ -512,7 +515,7 @@ static bool CheckAllocaUsesInternal(Instruction *I) {
       // Not a candidate.
       return false;
     } else if (IntrinsicInst *intr = dyn_cast<IntrinsicInst>(*use_it)) {
-      auto IID = GenXIntrinsic::getAnyIntrinsicID(intr);
+      auto IID = vc::getAnyIntrinsicID(intr);
       if (IID == llvm::Intrinsic::lifetime_start ||
           IID == llvm::Intrinsic::lifetime_end ||
           IID == GenXIntrinsic::genx_gather_private ||
@@ -677,7 +680,8 @@ void TransposeHelper::handleBCInst(BitCastInst &BC, GenericVectorIndex Idx) {
       SrcDerefTy->getScalarSizeInBits() / DstDerefTy->getScalarSizeInBits();
   Value *Scale = nullptr;
   if (Idx.Index->getType()->isVectorTy()) {
-    auto Width = cast<VectorType>(Idx.Index->getType())->getNumElements();
+    auto Width =
+        cast<IGCLLVM::FixedVectorType>(Idx.Index->getType())->getNumElements();
     Scale = ConstantVector::getSplat(IGCLLVM::getElementCount(Width),
                                      IRB.getInt32(ElementSize));
   } else
@@ -725,7 +729,7 @@ void TransposeHelper::handleAllocaSources(Instruction &Inst,
     } else if (PHINode *pPhi = llvm::dyn_cast<PHINode>(User)) {
       handlePHINode(pPhi, Idx, Inst.getParent());
     } else if (IntrinsicInst *IntrInst = dyn_cast<IntrinsicInst>(User)) {
-      auto IID = GenXIntrinsic::getAnyIntrinsicID(IntrInst);
+      auto IID = vc::getAnyIntrinsicID(IntrInst);
       if (IID == llvm::Intrinsic::lifetime_start ||
           IID == llvm::Intrinsic::lifetime_end)
         IntrInst->eraseFromParent();
@@ -755,7 +759,8 @@ void TransposeHelper::handleGEPInst(GetElementPtrInst *GEP,
   for (auto OI = GEP->op_begin() + 1, E = GEP->op_end(); OI != E; ++OI) {
     Value *GEPIdx = *OI;
     if (GEPIdx->getType()->isVectorTy()) {
-      auto Width = cast<VectorType>(GEPIdx->getType())->getNumElements();
+      auto Width =
+          cast<IGCLLVM::FixedVectorType>(GEPIdx->getType())->getNumElements();
       if (Width > 1) {
         if (IdxWidth <= 1)
           IdxWidth = Width;
@@ -820,7 +825,8 @@ void TransposeHelper::handleGEPInst(GetElementPtrInst *GEP,
         auto ElementSize =
             m_pDL->getTypeAllocSize(Ty) / Idx.getElementSizeInBytes();
         if (GEPIdx->getType()->isVectorTy()) {
-          IGC_ASSERT(cast<VectorType>(GEPIdx->getType())->getNumElements() == IdxWidth);
+          IGC_ASSERT(cast<IGCLLVM::FixedVectorType>(GEPIdx->getType())
+                         ->getNumElements() == IdxWidth);
           NewIdx = IRB.CreateZExtOrTrunc(GEPIdx, pScalarizedIdx->getType());
           NewIdx = IRB.CreateMul(NewIdx, ConstantVector::getSplat(
                                              IGCLLVM::getElementCount(IdxWidth),
@@ -837,7 +843,8 @@ void TransposeHelper::handleGEPInst(GetElementPtrInst *GEP,
   if (!Idx.Index->getType()->isVectorTy() && IdxWidth <= 1) {
     pScalarizedIdx = IRB.CreateAdd(pScalarizedIdx, Idx.Index);
   } else if (Idx.Index->getType()->isVectorTy()) {
-    IGC_ASSERT(cast<VectorType>(Idx.Index->getType())->getNumElements() == IdxWidth);
+    IGC_ASSERT(cast<IGCLLVM::FixedVectorType>(Idx.Index->getType())
+                   ->getNumElements() == IdxWidth);
     pScalarizedIdx = IRB.CreateAdd(pScalarizedIdx, Idx.Index);
   } else {
     auto SplatIdx = IRB.CreateVectorSplat(IdxWidth, Idx.Index);
@@ -868,7 +875,8 @@ void TransposeHelper::handlePHINode(PHINode *pPhi, GenericVectorIndex Idx,
 template <typename FolderT = ConstantFolder>
 Instruction *loadAndCastVector(AllocaInst &VecAlloca, Type &CastTo,
                                IRBuilder<FolderT> &IRB) {
-  auto *LoadVecAlloca = IRB.CreateLoad(&VecAlloca);
+  auto *LoadVecAlloca =
+      IRB.CreateLoad(VecAlloca.getType()->getPointerElementType(), &VecAlloca);
   auto *AllocatedElemTy = LoadVecAlloca->getType()->getScalarType();
   bool IsFuncPointer =
       CastTo.isPointerTy() && CastTo.getPointerElementType()->isFunctionTy();
@@ -942,7 +950,8 @@ void TransposeHelperPromote::handleLoadInst(LoadInst *pLoad,
     // %v0 = extractelement <32 x float> %w, i32 %idx
     // %v1 = extractelement <32 x float> %w, i32 %idx+1
     // replace all uses of %v with <%v0, %v1>
-    auto Len = cast<VectorType>(pLoad->getType())->getNumElements();
+    auto Len =
+        cast<IGCLLVM::FixedVectorType>(pLoad->getType())->getNumElements();
     Value *Result = UndefValue::get(pLoad->getType());
     for (unsigned i = 0; i < Len; ++i) {
       Value *VectorIdx = ConstantInt::get(pScalarizedIdx->getType(), i);
@@ -1006,7 +1015,8 @@ void TransposeHelperPromote::handleStoreInst(StoreInst *pStore,
     // %v1 = extractelement <2 x float> %v, i32 1
     // %w1 = insertelement <32 x float> %w0, float %v1, i32 %idx+1
     // store <32 x float> %w1, <32 x float>* %ptr1
-    auto Len = cast<VectorType>(pStoreVal->getType())->getNumElements();
+    auto Len =
+        cast<IGCLLVM::FixedVectorType>(pStoreVal->getType())->getNumElements();
     for (unsigned i = 0; i < Len; ++i) {
       Value *VectorIdx = ConstantInt::get(ScalarizedIdx.Index->getType(), i);
       auto *Val = IRB.CreateExtractElement(pStoreVal, VectorIdx);
@@ -1029,9 +1039,11 @@ void TransposeHelperPromote::handlePrivateGather(IntrinsicInst *pInst,
                                           Value *pScalarizedIdx) {
   IRBuilder<> IRB(pInst);
   IGC_ASSERT(pInst->getType()->isVectorTy());
-  Value *pLoadVecAlloca = IRB.CreateLoad(pVecAlloca);
-  auto N = cast<VectorType>(pInst->getType())->getNumElements();
-  auto ElemType = cast<VectorType>(pInst->getType())->getElementType();
+  Value *pLoadVecAlloca = IRB.CreateLoad(
+      pVecAlloca->getType()->getPointerElementType(), pVecAlloca);
+  auto *InstTy = cast<IGCLLVM::FixedVectorType>(pInst->getType());
+  auto N = InstTy->getNumElements();
+  auto ElemType = InstTy->getElementType();
 
   // A vector load
   // %v = <2 x float> gather %pred, %ptr, %offset, %old_value
@@ -1106,10 +1118,12 @@ void TransposeHelperPromote::handlePrivateScatter(llvm::IntrinsicInst *pInst,
   // Add Store instruction to remove list
   IRBuilder<> IRB(pInst);
   llvm::Value *pStoreVal = pInst->getArgOperand(3);
-  llvm::Value *pLoadVecAlloca = IRB.CreateLoad(pVecAlloca);
+  llvm::Value *pLoadVecAlloca = IRB.CreateLoad(
+      pVecAlloca->getType()->getPointerElementType(), pVecAlloca);
   IGC_ASSERT(pStoreVal->getType()->isVectorTy());
-  auto N = cast<VectorType>(pStoreVal->getType())->getNumElements();
-  auto ElemType = cast<VectorType>(pStoreVal->getType())->getElementType();
+  auto *StoreValTy = cast<IGCLLVM::FixedVectorType>(pStoreVal->getType());
+  auto N = StoreValTy->getNumElements();
+  auto ElemType = StoreValTy->getElementType();
   // A vector scatter
   // scatter %pred, %ptr, %offset, %newvalue
   // becomes
@@ -1169,9 +1183,9 @@ void TransposeHelperPromote::handlePrivateScatter(llvm::IntrinsicInst *pInst,
 void TransposeHelperPromote::handleLLVMGather(IntrinsicInst *pInst,
   Value *pScalarizedIdx) {
   IRBuilder<> IRB(pInst);
-  IGC_ASSERT(pInst->getType()->isVectorTy());
-  auto N = cast<VectorType>(pInst->getType())->getNumElements();
-  auto ElemType = cast<VectorType>(pInst->getType())->getElementType();
+  auto *InstTy = cast<IGCLLVM::FixedVectorType>(pInst->getType());
+  auto N = InstTy->getNumElements();
+  auto ElemType = InstTy->getElementType();
   Value *LoadVecAlloca = loadAndCastVector(*pVecAlloca, *ElemType, IRB);
 
   // A vector load
@@ -1300,14 +1314,16 @@ void TransposeHelperPromote::handleSVMGather(IntrinsicInst *pInst,
 
   // part of this is taken from handleLLVMGather above
   IRBuilder<> IRB(pInst);
-  llvm::Value *pLoadVecAlloca = IRB.CreateLoad(pVecAlloca);
+  llvm::Value *pLoadVecAlloca = IRB.CreateLoad(
+      pVecAlloca->getType()->getPointerElementType(), pVecAlloca);
   Region R(pInst);
   R.Mask = pInst->getArgOperand(0);
   R.Indirect = IRB.CreateTrunc(
       pScalarizedIdx,
       IGCLLVM::FixedVectorType::get(
           IntegerType::getInt16Ty(pInst->getContext()),
-          cast<VectorType>(pScalarizedIdx->getType())->getNumElements()),
+          cast<IGCLLVM::FixedVectorType>(pScalarizedIdx->getType())
+              ->getNumElements()),
       "");
   R.Width = 1;
   R.Stride = 0;
@@ -1335,14 +1351,16 @@ void TransposeHelperPromote::handleSVMScatter(IntrinsicInst *pInst,
                                               Value *pScalarizedIdx) {
   IRBuilder<> IRB(pInst);
   Value *pStoreVal = pInst->getArgOperand(3);
-  Value *pLoadVecAlloca = IRB.CreateLoad(pVecAlloca);
+  Value *pLoadVecAlloca = IRB.CreateLoad(
+      pVecAlloca->getType()->getPointerElementType(), pVecAlloca);
   Region R(pStoreVal);
   R.Mask = pInst->getArgOperand(0);
   R.Indirect = IRB.CreateTrunc(
       pScalarizedIdx,
       IGCLLVM::FixedVectorType::get(
           IntegerType::getInt16Ty(pInst->getContext()),
-          cast<VectorType>(pScalarizedIdx->getType())->getNumElements()),
+          cast<IGCLLVM::FixedVectorType>(pScalarizedIdx->getType())
+              ->getNumElements()),
       "");
   R.Width = 1;
   R.Stride = 0;
